@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *    $Id: block_subs.c,v 1.7 2008/03/24 08:58:30 guru Exp $
+ *    $Id: block_subs.c,v 1.11 2009/09/09 23:27:01 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -49,18 +49,14 @@ int BLK_compare_sub (const void *point_1, const void *point_2, int item)
 	p1 = (struct BLK_DATA *)point_1;
 	p2 = (struct BLK_DATA *)point_2;
 
-	if (p1->i < p2->i)
-		return (-1);
-	else if (p1->i > p2->i)
-		return (1);
-	else {
-		if (p1->a[item] < p2->a[item])
-			return (-1);
-		else if (p1->a[item] > p2->a[item])
-			return (1);
-		else
-			return (0);
-	}
+	/* First sort on bin index i */
+	if (p1->i < p2->i) return (-1);
+	if (p1->i > p2->i) return (+1);
+	/* OK, comparing values in the same bin */
+	if (p1->a[item] < p2->a[item]) return (-1);
+	if (p1->a[item] > p2->a[item]) return (+1);
+	/* Values are the same, return 0 */
+	return (0);
 }
 
 #if defined(BLOCKMEAN)
@@ -81,7 +77,7 @@ void * NEW () {	/* Allocate and initialize a new control structure */
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 #if defined(BLOCKMEDIAN)
-	C->T.quartile = 0.5;
+	C->T.quantile = 0.5;
 #endif
 	return ((void *)C);
 }
