@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *    $Id: blockmean.c,v 1.62 2009/01/09 04:02:32 guru Exp $
+ *    $Id: blockmean.c,v 1.67 2009/09/09 23:27:01 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -71,14 +71,11 @@ int main (int argc, char **argv)
 
 	double	weight, weighted_z, *in, wesn[4], out[7], iw;
 
-	int	i, j, n_expected_fields, n_fields, n_req, n_out, w_col;
-	int	n_files = 0, fno, n_args;
-	
+	GMT_LONG	i, j, n_expected_fields, n_fields, n_req, n_out, w_col;
+	GMT_LONG	n_files = 0, fno, n_args, ij, n_blocks;
 	GMT_LONG	n_cells_filled, n_read, n_lost, n_pitched, *np = NULL;
 	
-	size_t ij, n_blocks;
-
-	char	modifier, line[BUFSIZ], format[BUFSIZ];
+	char	modifier, line[BUFSIZ], format[BUFSIZ], *not_used = NULL;
 	
 	struct GRD_HEADER h;
 
@@ -174,7 +171,7 @@ int main (int argc, char **argv)
 		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-C Output center of block and mean z-value.  [Default outputs mean x-y location]\n");
+		fprintf (stderr, "\t-C Output center of block and mean z-value.  [Default outputs (mean x, mean y) location]\n");
 		fprintf (stderr, "\t-E Extend output with st.dev (s), low (l), and high (h) value per block, i,e,\n");
 		fprintf (stderr, "\t   output (x,y,z,s,l,h[,w]) [Default outputs (x,y,z[,w]); see -W regarding w.\n");
 		fprintf (stderr, "\t-F Offsets registration so block edges are on gridlines (pixel reg.).  [Default: grid reg.]\n");
@@ -189,7 +186,7 @@ int main (int argc, char **argv)
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "\t   Default is 3 [6 with -E] columns (or 4 [7] if -W is set).\n");
+		fprintf (stderr, "\t   Default is 3 columns (or 4 if -W is set).\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
 		GMT_explain_option ('f');
@@ -215,14 +212,14 @@ int main (int argc, char **argv)
 	n_req = (Ctrl->W.weighted[GMT_IN]) ? 4 : 3;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
 	if (GMT_io.binary[GMT_IN] && n_req > GMT_io.ncol[GMT_IN]) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %d columns\n", GMT_program, n_req);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %ld columns\n", GMT_program, n_req);
 		error++;
 	}
 	if (error) exit (EXIT_FAILURE);
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -308,7 +305,7 @@ int main (int argc, char **argv)
 
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
-				GMT_fgets (line, BUFSIZ, fp);
+				not_used = GMT_fgets (line, BUFSIZ, fp);
 				line[strlen(line)-1] = 0;
 				if (first && GMT_io.io_header[GMT_OUT]) (Ctrl->W.weighted[GMT_OUT] && !(Ctrl->W.weighted[GMT_IN])) ? fprintf (GMT_stdout, "%s weights\n", line) : fprintf (GMT_stdout, "%s\n", line);
 			}
@@ -318,13 +315,11 @@ int main (int argc, char **argv)
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line ", GMT_program, n_fields,  n_expected_fields);
-				PRINT_SIZE_T (stderr, n_read);
-				fprintf (stderr, "\n");
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 
-			if (GMT_is_dnan (in[2])) continue;	/* Skip when z = NaN */
+			if (GMT_is_dnan (in[GMT_Z])) continue;	/* Skip when z = NaN */
 
 			n_read++;
 
@@ -341,21 +336,21 @@ int main (int argc, char **argv)
 			/* OK, this point is inside and will be used */
 			
 			weight = (Ctrl->W.weighted[GMT_IN]) ? in[3] : 1.0;
-			weighted_z = in[2] * weight;
+			weighted_z = in[GMT_Z] * weight;
 			ij = GMT_IJ (j, i, h.nx);		/* 64-bit safe 1-D index */
 			if (use_xy) {
 				xy[ij].a[BLK_X] += (in[GMT_X]*weight);
 				xy[ij].a[BLK_Y] += (in[GMT_Y]*weight);
 			}
 			if (Ctrl->E.active) {	/* Add up sum(w*z^2) and n for weighted stdev and keep track of min,max */
-				slh[ij].a[BLK_S] += (weighted_z * in[2]);
+				slh[ij].a[BLK_S] += (weighted_z * in[GMT_Z]);
 				if (Ctrl->W.weighted[GMT_IN]) np[ij]++;
 				if (zw[ij].a[BLK_W] == 0) {	/* Initialize low,high */
 					slh[ij].a[BLK_L] = +DBL_MAX;
 					slh[ij].a[BLK_H] = -DBL_MAX;
 				}
-				if (in[2] < slh[ij].a[BLK_L]) slh[ij].a[BLK_L] = in[2];
-				if (in[2] > slh[ij].a[BLK_H]) slh[ij].a[BLK_H] = in[2];
+				if (in[GMT_Z] < slh[ij].a[BLK_L]) slh[ij].a[BLK_L] = in[GMT_Z];
+				if (in[GMT_Z] > slh[ij].a[BLK_H]) slh[ij].a[BLK_H] = in[GMT_Z];
 			}
 			zw[ij].a[BLK_W] += weight;
 			zw[ij].a[BLK_Z] += weighted_z;
@@ -389,10 +384,10 @@ int main (int argc, char **argv)
 			out[GMT_Y] = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
 		}
 		if (Ctrl->S.mode) {
-			out[2] = (Ctrl->S.mode == 2) ? zw[ij].a[BLK_W] : zw[ij].a[BLK_Z];
+			out[GMT_Z] = (Ctrl->S.mode == 2) ? zw[ij].a[BLK_W] : zw[ij].a[BLK_Z];
 		}
 		else
-			out[2] = zw[ij].a[BLK_Z] * iw;
+			out[GMT_Z] = zw[ij].a[BLK_Z] * iw;
 		if (Ctrl->E.active) {
 			if (Ctrl->W.weighted[GMT_IN]) {
 				out[3] = (np[ij] > 1) ? d_sqrt ((zw[ij].a[BLK_W] * slh[ij].a[BLK_S] - zw[ij].a[BLK_Z] * zw[ij].a[BLK_Z]) \
