@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *    $Id: blockmedian.c,v 1.52 2008/03/24 08:58:30 guru Exp $
+ *    $Id: blockmedian.c,v 1.63 2009/09/09 23:27:01 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -20,7 +20,8 @@
    blockmedian.c
    Takes lon, lat, data, [weight] on GMT_stdin or file and writes out one value
    per cell, where cellular region is bounded by West East South North and
-   cell dimensions are delta_x, delta_y.
+   cell dimensions are delta_x, delta_y.  While the word median is used in most
+   places, we actually compute a quantile [which by default is 50%, i.e., the median].
       
    Author: 	Walter H. F. Smith
    Date:	28 June, 1988
@@ -34,8 +35,9 @@
    Version:	4 01-MAR-2003 by PW
    Version:	4.1: 14-SEP-2005 by PW, Added enhanced -I
    Version	4.1.2: 24-MAR-2006 by PW: No longer defines global variables. Use double to hold data.
-   		4-APR-2006 by PW: Added -E for L1 scale, low, and high value, and -T to set quartile
+   		4-APR-2006 by PW: Added -E for L1 scale, low, and high value, and -T to set quantile
 			Also implemented size_t counters to be 64-bit compatible.
+		5-MAR-2008 by PW: Added -Eb to produce box-and-whisker output (all 5 quantiles)
 */
 
 #define BLOCKMEDIAN
@@ -46,29 +48,29 @@
 int main (int argc, char **argv)
 {
 
-	BOOLEAN	error = FALSE, nofile = TRUE, done = FALSE, first = TRUE;
-	/* Default value for go_quickly = FALSE for backward compatibility with 3.0  */
+	BOOLEAN	error = FALSE, nofile = TRUE, done = FALSE, first = TRUE, box_and_whisker = FALSE;
 
 	FILE *fp = NULL;
 
-	double	*in, out[7], wesn[4], weight, *z_tmp = NULL;
+	double	*in, out[7], wesn[4], quantile[3] = {0.25, 0.5, 0.75}, extra[3], weight, *z_tmp = NULL;
 
-	int	i, ix, iy, fno, n_files = 0, n_args, n_req, w_col;
-	int	n_expected_fields, n_fields, n_out, go_quickly = 0;
+	GMT_LONG	i, ix, iy, fno, n_files = 0, n_args, n_req, w_col;
+	GMT_LONG	n_expected_fields, n_fields, n_out, n_quantiles = 1, go_quickly = 0;
+	/* Default value for go_quickly = 0 for backward compatibility with GMT 3.0  */
 
-	GMT_LONG	nz, n_read, n_pitched, n_cells_filled, n_lost, index;
+	GMT_LONG	nz, n_read, n_cells_filled, n_lost, index;
 	GMT_LONG	first_in_cell, first_in_new_cell;
+	GMT_LONG	n_alloc = 0, nz_alloc = 0, n_pitched;
 	
-	size_t	n_alloc, nz_alloc;
-	
-	char	modifier, buffer[BUFSIZ], format[BUFSIZ];
+	char	modifier, buffer[BUFSIZ], format[BUFSIZ], *not_used = NULL;
 
 	struct GRD_HEADER h;
 
-	struct BLK_DATA *data;
+	struct BLK_DATA *data = NULL;
 	struct BLOCKMEDIAN_CTRL *Ctrl;
 	
-	void median_output (struct GRD_HEADER *h, GMT_LONG first_in_cell, GMT_LONG first_in_new_cell, double weight_sum, double *xx, double *yy, double *zz, int go_quickly, double quartile, struct BLK_DATA *data);
+	void median_output (struct GRD_HEADER *h, GMT_LONG first_in_cell, GMT_LONG first_in_new_cell, double weight_sum, double *out, 
+		double *extra, GMT_LONG go_quickly, double *quantile, GMT_LONG n_quantiles, struct BLK_DATA *data);
 
 	argc = GMT_begin (argc, argv);
 
@@ -98,7 +100,8 @@ int main (int argc, char **argv)
 					Ctrl->C.active = TRUE;
 					break;
 				case 'E':
-					Ctrl->E.active = TRUE;		/* Extended report with standard deviation, min, and max in cols 4-6 */
+					Ctrl->E.active = TRUE;				/* Extended report with standard deviation, min, and max in cols 4-6 */
+					if (argv[i][2] == 'b') Ctrl->E.mode = 1;	/* Extended report with min, 25%, 75% and max in cols 4-7 */
 					break;
 				case 'I':
 					Ctrl->I.active = TRUE;
@@ -117,11 +120,11 @@ int main (int argc, char **argv)
 					Ctrl->F.active = TRUE;
 					break;
 				case 'Q':
-					Ctrl->Q.active = TRUE;		/* Extended report with standard deviation, min, and max in cols 4-6 */
+					Ctrl->Q.active = TRUE;		/* Get median z and (x,y) of that point */
 					break;
 				case 'T':
 					Ctrl->T.active = TRUE;		/* Extended report with standard deviation, min, and max in cols 4-6 */
-					Ctrl->T.quartile = atof (&argv[i][2]);
+					Ctrl->T.quantile = atof (&argv[i][2]);
 					break;
 				case 'W':
 					Ctrl->W.active = TRUE;
@@ -154,22 +157,23 @@ int main (int argc, char **argv)
 		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-C Output center of block and median z-value  [Default is median location (but see -Q)]\n");
+		fprintf (stderr, "\t-C Output center of block as location  [Default is (median x, median y), but see -Q]\n");
 		fprintf (stderr, "\t-E Extend output with L1 scale (s), low (l), and high (h) value per block, i,e,\n");
 		fprintf (stderr, "\t   output (x,y,z,s,l,h[,w]) [Default outputs (x,y,z[,w]); see -W regarding w.\n");
+		fprintf (stderr, "\t   Use -Eb for box-and-whisker output (x,y,z,l,25%%q,75%%q,h[,w])\n");
 		fprintf (stderr, "\t-F Offsets registration so block edges are on gridlines (pixel reg.).  [Default: grid reg.]\n");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-Q Quicker; get median z and x,y at that z.  [Default gets median x, median y, median z.]\n");
-		fprintf (stderr, "\t-T Set quartile (0 < q < 1) to report [Default is 0.5 which is the median z]\n");
+		fprintf (stderr, "\t-T Set quantile (0 < q < 1) to report [Default is 0.5 which is the median of z]\n");
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t-W sets Weight options.\n");
-		fprintf (stderr, "\t-  -Wi reads Weighted Input (4 cols: x,y,z,w) but writes only (x,y,z[,s,l,h]) Output.\n");
-		fprintf (stderr, "\t   -Wo reads unWeighted Input (3 cols: x,y,z) but reports sum (x,y,z[,s,l,h],w) Output.\n");
+		fprintf (stderr, "\t   -Wi reads Weighted Input (4 cols: x,y,z,w) but skips w on output.\n");
+		fprintf (stderr, "\t   -Wo reads unWeighted Input (3 cols: x,y,z) but weight sum on output.\n");
 		fprintf (stderr, "\t   -W with no modifier has both weighted Input and Output; Default is no weights used.\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "\t   Default is 3 [6 with -E] columns (or 4 [7] if -W is set).\n");
+		fprintf (stderr, "\t   Default is 3 columns (or 4 if -W is set).\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
 		GMT_explain_option ('f');
@@ -177,17 +181,19 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (Ctrl->C.active && go_quickly == 1) {
-		fprintf (stderr, "%s: GMT WARNING:  -C overrides -Q\n", GMT_program);
+		fprintf (stderr, "%s: GMT WARNING: -C overrides -Q\n", GMT_program);
 		go_quickly = 0;
 	}
 
 	if (!project_info.region_supplied) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR: Must specify -R option\n", GMT_program);
 		error++;
 	}
-	if (Ctrl->T.quartile <= 0.0 || Ctrl->T.quartile >= 1.0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  0 < q < 1 for quartile in -T [0.5]\n", GMT_program);
+	if (Ctrl->T.quantile <= 0.0 || Ctrl->T.quantile >= 1.0) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR: 0 < q < 1 for quantile in -T [0.5]\n", GMT_program);
 		error++;
 	}
 	if (Ctrl->I.xinc <= 0.0 || Ctrl->I.yinc <= 0.0) {
@@ -201,7 +207,7 @@ int main (int argc, char **argv)
 	n_req = (Ctrl->W.weighted[GMT_IN]) ? 4 : 3;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
 	if (GMT_io.binary[GMT_IN] && n_req > GMT_io.ncol[GMT_IN]) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %d columns\n", GMT_program, n_req);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %ld columns\n", GMT_program, n_req);
 		error++;
 	}
 
@@ -209,7 +215,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -223,6 +229,13 @@ int main (int argc, char **argv)
 
 	go_quickly = (Ctrl->Q.active) ? 1 : 0;	
 	if (Ctrl->C.active) go_quickly = 2;	/* Flag used in output calculation */
+	if (Ctrl->E.active && Ctrl->E.mode == 1) {
+		n_quantiles = 3;
+		box_and_whisker = TRUE;
+	}
+	else
+		quantile[0] = Ctrl->T.quantile;
+		
 	h.xy_off = 0.5 * h.node_offset;		/* Use to calculate mean location of block */
 
 	if (gmtdefs.verbose) {
@@ -231,8 +244,6 @@ int main (int argc, char **argv)
 	}
 
 	n_read = n_pitched = 0;
-        n_alloc = GMT_CHUNK;
-        data = (struct BLK_DATA *) GMT_memory (VNULL, n_alloc, sizeof(struct BLK_DATA), GMT_program);
 
 	GMT_set_xy_domain (wesn, &h);	/* May include some padding if gridline-registered */
 
@@ -265,7 +276,7 @@ int main (int argc, char **argv)
 
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
-				GMT_fgets (buffer, BUFSIZ, fp);
+				not_used = GMT_fgets (buffer, BUFSIZ, fp);
 				buffer[strlen(buffer)-1] = 0;
 				if (first && GMT_io.io_header[GMT_OUT]) (Ctrl->W.weighted[GMT_OUT] && !(Ctrl->W.weighted[GMT_IN])) ? fprintf (GMT_stdout, "%s weights\n", buffer) : fprintf (GMT_stdout, "%s\n", buffer);
 			}
@@ -275,13 +286,11 @@ int main (int argc, char **argv)
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line ", GMT_program, n_fields,  n_expected_fields);
-				PRINT_SIZE_T (stderr, n_read);
-				fprintf (stderr, "\n");
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 
-			if (GMT_is_dnan (in[2])) continue;	/* Skip when z = NaN */
+			if (GMT_is_dnan (in[GMT_Z])) continue;	/* Skip when z = NaN */
 
 			n_read++;
 
@@ -295,37 +304,38 @@ int main (int argc, char **argv)
 
 			index = GMT_IJ (iy, ix, h.nx);		/* 64-bit safe 1-D index */
 
+			if (n_pitched == n_alloc) n_alloc = GMT_alloc_memory ((void **)&data, n_pitched, n_alloc, sizeof (struct BLK_DATA), GMT_program);
 			data[n_pitched].i = index;
 			data[n_pitched].a[BLK_W] = ((Ctrl->W.weighted[GMT_IN]) ? in[3] : 1.0);
 			if (!Ctrl->C.active) {
 				data[n_pitched].a[BLK_X] = in[GMT_X];
 				data[n_pitched].a[BLK_Y] = in[GMT_Y];
 			}
-			data[n_pitched].a[BLK_Z] = in[2];
+			data[n_pitched].a[BLK_Z] = in[GMT_Z];
 
 			n_pitched++;
-			if (n_pitched == (GMT_LONG)n_alloc) {
-				n_alloc <<= 1;
-				data = (struct BLK_DATA *) GMT_memory ((void *)data, n_alloc, sizeof (struct BLK_DATA), GMT_program);
-			}
 		}
 		if (fp != GMT_stdin) GMT_fclose(fp);
 
 	}
-
-	n_alloc = (size_t)n_pitched;
-	data = (struct BLK_DATA *) GMT_memory ((void *)data, n_alloc, sizeof (struct BLK_DATA), GMT_program);
+	if (n_pitched < n_alloc) n_alloc = GMT_alloc_memory ((void **)&data, 0, n_pitched, sizeof (struct BLK_DATA), GMT_program);
 	
-	n_lost = n_read - n_pitched;
-	if (gmtdefs.verbose) fprintf(stderr,"%s: N read: %ld N used: %ld N outside_area: %ld\n", GMT_program, n_read, n_pitched, n_lost);
+	if (n_read == 0) {	/* Blank/empty input files */
+		if (gmtdefs.verbose) fprintf (stderr, "%s: No data records found; no output produced", GMT_program);
+		exit (EXIT_SUCCESS);
+	}
+	if (n_pitched == 0) {	/* No points inside region */
+		if (gmtdefs.verbose) fprintf (stderr, "%s: No data points found inside the region; no output produced", GMT_program);
+		exit (EXIT_SUCCESS);
+	}
+	n_lost = n_read - (GMT_LONG)n_pitched;
+	if (gmtdefs.verbose) fprintf(stderr,"%s: N read: %ld N used: %ld N outside_area: %ld\n", GMT_program, n_read, (GMT_LONG)n_pitched, n_lost);
 
 	/* Ready to go. */
 
 	n_out = (Ctrl->W.weighted[GMT_OUT]) ? 4 : 3;
-	if (Ctrl->E.active) n_out += 3;
+	if (Ctrl->E.active) n_out += 3 + (int)box_and_whisker;	/* So, 3, 6, or 7, plus 1 extra if -W */
 	w_col = n_out - 1;	/* Weights always reported in last output column */
-	nz_alloc = GMT_CHUNK;
-	if (Ctrl->E.active) z_tmp = (double *) GMT_memory (VNULL, nz_alloc, sizeof (double), GMT_program);	/* tmp array for L1 scale calculation */
 
 	/* Sort on index and Z value */
 
@@ -333,32 +343,40 @@ int main (int argc, char **argv)
 
 	/* Find n_in_cell and write appropriate output  */
 
-	first_in_cell = 0;
-	n_cells_filled = 0;
+	first_in_cell = n_cells_filled = nz = 0;
 	while (first_in_cell < n_pitched) {
 		weight = data[first_in_cell].a[BLK_W];
-		if (Ctrl->E.active) z_tmp[0] = data[first_in_cell].a[BLK_Z];
-		nz = 1;
+		if (Ctrl->E.active) {
+			if (nz == nz_alloc) nz_alloc = GMT_alloc_memory ((void **)&z_tmp, nz, nz_alloc, sizeof (double), GMT_program);
+			z_tmp[0] = data[first_in_cell].a[BLK_Z];
+			nz = 1;
+		}
 		first_in_new_cell = first_in_cell + 1;
 		while ( (first_in_new_cell < n_pitched) && (data[first_in_new_cell].i == data[first_in_cell].i) ) {
 			weight += data[first_in_new_cell].a[BLK_W];
 			if (Ctrl->E.active) {	/* Must get a temporary copy of the sorted z array */
-				z_tmp[nz] = data[first_in_new_cell].a[BLK_Z];
-				nz++;
-				if (nz == (GMT_LONG)nz_alloc) {
-					nz_alloc <<= 1;
-					z_tmp = (double *) GMT_memory ((void *)z_tmp, nz_alloc, sizeof (double), GMT_program);
-				}
+				if (nz == nz_alloc) nz_alloc = GMT_alloc_memory ((void **)&z_tmp, nz, nz_alloc, sizeof (double), GMT_program);
+				z_tmp[nz++] = data[first_in_new_cell].a[BLK_Z];
 			}
 			first_in_new_cell++;
 		}
-		median_output(&h, first_in_cell, first_in_new_cell, weight, &out[GMT_X], &out[GMT_Y], &out[2], go_quickly, Ctrl->T.quartile, data);
-		if (Ctrl->E.active) {
+		
+		/* Now we have weight sum [and copy of z in case of -E]; now calculate the quantile(s): */
+		
+		median_output (&h, first_in_cell, first_in_new_cell, weight, out, extra, go_quickly, quantile, n_quantiles, data);
+		
+		if (box_and_whisker) {	/* Need 7 items: x, y, median, min, 25%, 75%, max [,weight] */
+			out[3] = z_tmp[0];	/* 0% quantile (min value) */
+			out[4] = extra[0];	/* 25% quantile */
+			out[5] = extra[2];	/* 75% quantile */
+			out[6] = z_tmp[nz-1];	/* 100% quantile (max value) */
+		}
+		else if (Ctrl->E.active) {	/* Need 6 items: x, y, median, MAD, min, max [,weight] */
 			out[4] = z_tmp[0];	/* Low value */
 			out[5] = z_tmp[nz-1];	/* High value */
-			/* Turn z_tmp into absolute deviations from the median (out[2]) */
+			/* Turn z_tmp into absolute deviations from the median (out[GMT_Z]) */
 			if (nz > 1) {
-				for (index = 0; index < nz; index++) z_tmp[index] = fabs (z_tmp[index] - out[2]);
+				for (index = 0; index < nz; index++) z_tmp[index] = fabs (z_tmp[index] - out[GMT_Z]);
 				qsort ((void *)z_tmp, (size_t)nz, sizeof (double), GMT_comp_double_asc);
 				out[3] = (nz%2) ? z_tmp[nz/2] : 0.5 * (z_tmp[(nz-1)/2] + z_tmp[nz/2]);
 				out[3] *= 1.4826;	/* This will be L1 MAD-based scale */
@@ -377,26 +395,96 @@ int main (int argc, char **argv)
 	if (gmtdefs.verbose) fprintf(stderr,"%s: N_cells_filled: %ld\n", GMT_program, n_cells_filled);
 
 	GMT_free ((void *)data);
-	if (Ctrl->E.active) GMT_free ((void *)z_tmp);
+	GMT_free ((void *)z_tmp);
 
 	Free_Blockmedian_Ctrl (Ctrl);	/* Deallocate control structure */
 
-    GMT_end (argc, argv);
+	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-void median_output (struct GRD_HEADER *h, GMT_LONG first_in_cell, GMT_LONG first_in_new_cell, double weight_sum, double *xx, double *yy, double *zz, int go_quickly, double quartile, struct BLK_DATA *data)
+void median_output (struct GRD_HEADER *h, GMT_LONG first_in_cell, GMT_LONG first_in_new_cell, double weight_sum, double *out, double *extra, 
+	GMT_LONG go_quickly, double *quantile, GMT_LONG n_quantiles, struct BLK_DATA *data)
+{
+	double	weight_half, weight_count;
+	GMT_LONG index, n_in_cell, index1;
+	GMT_LONG k, k_for_xy;
+
+	/* Remember: Data are already sorted on z for each cell */
+
+	/* Step 1: Find the n_quantiles requested (typically only one unless -Eb was used) */
+	
+	n_in_cell = first_in_new_cell - first_in_cell;
+	index = first_in_cell;
+	weight_count = data[first_in_cell].a[BLK_W];
+	k_for_xy = (n_quantiles == 3) ? 1 : 0;	/* If -Eb is set get get median location, else same as for z (unless -Q) */
+	for (k = 0; k < n_quantiles; k++) {
+		
+		weight_half  = quantile[k] * weight_sum;	/* Normally, quantile will be 0.5 (i.e., median), hence the name of the variable */
+
+		/* Determine the point where we hit the desired quantile */
+	
+		while (weight_count < weight_half) weight_count += data[++index].a[BLK_W];	/* Wind up until weight_count hits the mark */
+
+		if ( weight_count == weight_half ) {
+			index1 = index + 1;
+			extra[k] = 0.5 * (data[index].a[BLK_Z] + data[index1].a[BLK_Z]);
+			if (k == k_for_xy && go_quickly == 1) {	/* Only get x,y at the z-quantile if requested [-Q] */
+				out[GMT_X] = 0.5 * (data[index].a[BLK_X] + data[index1].a[BLK_X]);
+				out[GMT_Y] = 0.5 * (data[index].a[BLK_Y] + data[index1].a[BLK_Y]);
+			}
+		}
+		else {
+			extra[k] = data[index].a[BLK_Z];
+			if (k == k_for_xy && go_quickly == 1) {	/* Only get x,y at the z-quantile if requested [-Q] */
+				out[GMT_X] = data[index].a[BLK_X];
+				out[GMT_Y] = data[index].a[BLK_Y];
+			}
+		}
+	}
+	out[GMT_Z] = extra[k_for_xy];	/* The desired quantile is passed via z */
+	
+	if (go_quickly == 1) return;	/* Already have everything requested so we return */
+
+	if (go_quickly == 2) {	/* Return center of block instead of computing a representative location */
+		GMT_LONG i, j;
+		j = data[index].i / ((GMT_LONG)h->nx);
+		i = data[index].i % ((GMT_LONG)h->nx);
+		out[GMT_X] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
+		out[GMT_Y] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
+		return;
+	}
+	
+	/* We get here when we need separate quantile calculations for both x and y locations */
+	
+	weight_half = quantile[k_for_xy] * weight_sum;	/* We want the same quantile for locations as was used for z */
+	
+	if (n_in_cell > 2) qsort((void *)&data[first_in_cell], (size_t)n_in_cell, sizeof (struct BLK_DATA), BLK_compare_x);
+	index = first_in_cell;
+	weight_count = data[first_in_cell].a[BLK_W];
+	while (weight_count < weight_half) weight_count += data[++index].a[BLK_W];
+	out[GMT_X] = ( weight_count == weight_half ) ?  0.5 * (data[index].a[BLK_X] + data[index + 1].a[BLK_X]) : data[index].a[BLK_X];
+
+	if (n_in_cell > 2) qsort((void *)&data[first_in_cell], (size_t)n_in_cell, sizeof (struct BLK_DATA), BLK_compare_y);
+	index = first_in_cell;
+	weight_count = data[first_in_cell].a[BLK_W];
+	while (weight_count < weight_half) weight_count += data[++index].a[BLK_W];
+	out[GMT_Y] = ( weight_count == weight_half ) ? 0.5 * (data[index].a[BLK_Y] + data[index + 1].a[BLK_Y]) : data[index].a[BLK_Y];
+}
+
+#ifdef USEOLDSTUFF
+void median_output (struct GRD_HEADER *h, GMT_LONG first_in_cell, GMT_LONG first_in_new_cell, double weight_sum, double *xx, double *yy, double *zz, GMT_LONG go_quickly, double quantile, struct BLK_DATA *data)
 {
 	double	weight_half, weight_count;
 	GMT_LONG index, n_in_cell, index1;
 
-	weight_half  = quartile * weight_sum;	/* Normally, quartile will be 0.5 hence the name of the variable */
+	weight_half  = quantile * weight_sum;	/* Normally, quantile will be 0.5 hence the name of the variable */
 	n_in_cell = first_in_new_cell - first_in_cell;
 
 	/* Data are already sorted on z  */
 
-	/* Determine the point where we hit the desired quartile */
+	/* Determine the point where we hit the desired quantile */
 	
 	index = first_in_cell;
 	weight_count = data[first_in_cell].a[BLK_W];
@@ -418,7 +506,6 @@ void median_output (struct GRD_HEADER *h, GMT_LONG first_in_cell, GMT_LONG first
 
 	/* Now get median x and median y if quick x and quick y not wanted:  */
 
-	weight_half = 0.5 * weight_sum;	/* We want the median location */
 	if (go_quickly == 1) return;
 
 	if (go_quickly == 2) {	/* Get center of block */
@@ -432,6 +519,7 @@ void median_output (struct GRD_HEADER *h, GMT_LONG first_in_cell, GMT_LONG first
 	
 	/* We get here when we need median x,y locations */
 	
+	weight_half = 0.5 * weight_sum;	/* We want the median location */
 	if (n_in_cell > 2) qsort((void *)&data[first_in_cell], (size_t)n_in_cell, sizeof (struct BLK_DATA), BLK_compare_x);
 	index = first_in_cell;
 	weight_count = data[first_in_cell].a[BLK_W];
@@ -456,5 +544,5 @@ void median_output (struct GRD_HEADER *h, GMT_LONG first_in_cell, GMT_LONG first
 	else
 		*yy = data[index].a[BLK_Y];
 }
-
+#endif
 #include "block_subs.c"
