@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *    $Id: blockmode.c,v 1.53 2008/03/24 08:58:30 guru Exp $
+ *    $Id: blockmode.c,v 1.62 2009/09/09 23:27:01 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -50,22 +50,19 @@ int main (int argc, char **argv)
 
 	double	*in, out[7], wesn[4], i_n_in_cell, weight, *z_tmp = NULL;
 
-	int	i, j, ix, iy, fno, n_files = 0, n_args, n_req;
-	int	n_expected_fields, n_fields, n_out, w_col;
-	
+	GMT_LONG i, j, ix, iy, fno, n_files = 0, n_args, n_req;
+	GMT_LONG n_expected_fields, n_fields, n_out, w_col;
 	GMT_LONG index, first_in_cell, first_in_new_cell, n_lost, n_read;
-	GMT_LONG n_pitched, n_cells_filled, n_in_cell, nz;
+	GMT_LONG n_cells_filled, n_in_cell, nz;
+	GMT_LONG n_alloc = 0, nz_alloc = 0, n_pitched;
 
-	size_t n_alloc, nz_alloc;
-
-	char	modifier, buffer[BUFSIZ], format[BUFSIZ];
+	char	modifier, buffer[BUFSIZ], format[BUFSIZ], *not_used = NULL;
 
 	struct GRD_HEADER h;
-
-	struct BLK_DATA *data;
+	struct BLK_DATA *data = NULL;
 	struct BLOCKMODE_CTRL *Ctrl;
 	
-	double weighted_mode (struct BLK_DATA *d, double wsum, GMT_LONG n, int k);
+	double weighted_mode (struct BLK_DATA *d, double wsum, GMT_LONG n, GMT_LONG k);
 	
 	argc = GMT_begin (argc, argv);
 
@@ -155,13 +152,13 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-Q Quicker; get mode z and mean x,y.  [Default gets mode x, mode y, mode z.]\n");
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t-W sets Weight options.\n");
-		fprintf (stderr, "\t-  -Wi reads Weighted Input (4 cols: x,y,z,w) but writes only (x,y,z[,s,l,h]) Output.\n");
+		fprintf (stderr, "\t   -Wi reads Weighted Input (4 cols: x,y,z,w) but writes only (x,y,z[,s,l,h]) Output.\n");
 		fprintf (stderr, "\t   -Wo reads unWeighted Input (3 cols: x,y,z) but reports sum (x,y,z[,s,l,h],w) Output.\n");
 		fprintf (stderr, "\t   -W with no modifier has both weighted Input and Output; Default is no weights used.\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "\t   Default is 3 [6 with -E] columns (or 4 [7] if -W is set).\n");
+		fprintf (stderr, "\t   Default is 3 columns (or 4 if -W is set).\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
 		GMT_explain_option ('f');
@@ -169,6 +166,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (Ctrl->C.active && Ctrl->Q.active) {
 		fprintf (stderr, "%s: GMT WARNING:  -C overrides -Q\n", GMT_program);
 		Ctrl->Q.active = FALSE;
@@ -188,7 +187,7 @@ int main (int argc, char **argv)
 	n_req = (Ctrl->W.weighted[GMT_IN]) ? 4 : 3;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
 	if (GMT_io.binary[GMT_IN] && n_req > GMT_io.ncol[GMT_IN]) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %d columns\n", GMT_program, n_req);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %ld columns\n", GMT_program, n_req);
 		error++;
 	}
 
@@ -196,7 +195,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -216,8 +215,6 @@ int main (int argc, char **argv)
 	}
 
 	n_read = n_pitched = 0;
-        n_alloc = GMT_CHUNK;
-        data = (struct BLK_DATA *) GMT_memory (VNULL, n_alloc, sizeof(struct BLK_DATA), GMT_program);
 
 	GMT_set_xy_domain (wesn, &h);	/* May include some padding if gridline-registered */
 		
@@ -250,7 +247,7 @@ int main (int argc, char **argv)
 
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
-				GMT_fgets (buffer, BUFSIZ, fp);
+				not_used = GMT_fgets (buffer, BUFSIZ, fp);
 				buffer[strlen(buffer)-1] = 0;
 				if (first && GMT_io.io_header[GMT_OUT]) (Ctrl->W.weighted[GMT_OUT] && !(Ctrl->W.weighted[GMT_IN])) ? fprintf (GMT_stdout, "%s weights\n", buffer) : fprintf (GMT_stdout, "%s\n", buffer);
 			}
@@ -260,13 +257,11 @@ int main (int argc, char **argv)
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line ", GMT_program, n_fields,  n_expected_fields);
-				PRINT_SIZE_T (stderr, n_read);
-				fprintf (stderr, "\n");
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 
-			if (GMT_is_dnan (in[2])) continue;	/* Skip when z = NaN */
+			if (GMT_is_dnan (in[GMT_Z])) continue;	/* Skip when z = NaN */
 
 			n_read++;
 
@@ -280,50 +275,54 @@ int main (int argc, char **argv)
 
 			index = GMT_IJ (iy, ix, h.nx);		/* 64-bit safe 1-D index */
 
+			if (n_pitched == n_alloc) n_alloc = GMT_alloc_memory ((void **)&data, n_pitched, n_alloc, sizeof (struct BLK_DATA), GMT_program);
 			data[n_pitched].i = index;
 			if (mode_xy) {
 				data[n_pitched].a[BLK_X] = in[GMT_X];
 				data[n_pitched].a[BLK_Y] = in[GMT_Y];
 			}
-			data[n_pitched].a[BLK_Z] = in[2];
+			data[n_pitched].a[BLK_Z] = in[GMT_Z];
 			data[n_pitched].a[BLK_W] = (Ctrl->W.weighted[GMT_IN]) ? in[3] : 1.0;
 
 			n_pitched++;
-			if (n_pitched == (GMT_LONG)n_alloc) {
-				n_alloc <<= 1;
-				data = (struct BLK_DATA *) GMT_memory ((void *)data, n_alloc, sizeof(struct BLK_DATA), GMT_program);
-			}
 		}
 		if (fp != GMT_stdin) GMT_fclose(fp);
 
 	}
+	n_alloc = GMT_alloc_memory ((void **)&data, 0, n_pitched, sizeof (struct BLK_DATA), GMT_program);
 
-	n_alloc = (size_t)n_pitched;
-	data = (struct BLK_DATA *) GMT_memory ((void *)data, n_alloc, sizeof(struct BLK_DATA), GMT_program);
-	n_lost = n_read - n_pitched;
+	if (n_read == 0) {	/* Blank/empty input files */
+		if (gmtdefs.verbose) fprintf (stderr, "%s: No data records found; no output produced", GMT_program);
+		exit (EXIT_SUCCESS);
+	}
+	if (n_pitched == 0) {	/* No points inside region */
+		if (gmtdefs.verbose) fprintf (stderr, "%s: No data points found inside the region; no output produced", GMT_program);
+		exit (EXIT_SUCCESS);
+	}
+	n_lost = n_read - (GMT_LONG)n_pitched;
 	
-	if (gmtdefs.verbose) fprintf(stderr,"%s: N read: %ld N used: %ld N outside_area: %ld\n", GMT_program, n_read, n_pitched, n_lost);
+	if (gmtdefs.verbose) fprintf(stderr,"%s: N read: %ld N used: %ld N outside_area: %ld\n", GMT_program, n_read, (GMT_LONG)n_pitched, n_lost);
 
 	/* Ready to go. */
 
 	n_out = (Ctrl->W.weighted[GMT_OUT]) ? 4 : 3;
 	if (Ctrl->E.active) n_out += 3;
 	w_col = n_out - 1;	/* Weights always reported in last output column */
-	nz_alloc = GMT_CHUNK;
-	if (Ctrl->E.active) z_tmp = (double *) GMT_memory (VNULL, nz_alloc, sizeof (double), GMT_program);	/* tmp array for L1 scale calculation */
 
 	/* Sort on index and Z value */
 
-	qsort((void *)data, (size_t)n_pitched, sizeof (struct BLK_DATA), BLK_compare_index_z);
+	qsort((void *)data, n_pitched, sizeof (struct BLK_DATA), BLK_compare_index_z);
 
 	/* Find n_in_cell and write appropriate output  */
 
-	first_in_cell = 0;
-	n_cells_filled = 0;
+	first_in_cell = n_cells_filled = nz = 0;
 	while (first_in_cell < n_pitched) {
 		weight = data[first_in_cell].a[BLK_W];
-		if (Ctrl->E.active) z_tmp[0] = data[first_in_cell].a[BLK_Z];
-		nz = 1;
+		if (Ctrl->E.active) {
+			if (nz == nz_alloc) nz_alloc = GMT_alloc_memory ((void **)&z_tmp, nz, nz_alloc, sizeof (double), GMT_program);
+			z_tmp[0] = data[first_in_cell].a[BLK_Z];
+			nz = 1;
+		}
 		if (Ctrl->C.active) {
 			j = data[first_in_cell].i / ((GMT_LONG)h.nx);
 			i = data[first_in_cell].i % ((GMT_LONG)h.nx);
@@ -342,18 +341,15 @@ int main (int argc, char **argv)
 				out[GMT_Y] += data[first_in_new_cell].a[BLK_Y];
 			}
 			if (Ctrl->E.active) {	/* Must get a temporary copy of the sorted z array */
+				if (nz == nz_alloc) nz_alloc = GMT_alloc_memory ((void **)&z_tmp, nz, nz_alloc, sizeof (double), GMT_program);
 				z_tmp[nz] = data[first_in_new_cell].a[BLK_Z];
 				nz++;
-				if (nz == (GMT_LONG)nz_alloc) {
-					nz_alloc <<= 1;
-					z_tmp = (double *) GMT_memory ((void *)z_tmp, nz_alloc, sizeof (double), GMT_program);
-				}
 			}
 			first_in_new_cell++;
 		}
 		n_in_cell = first_in_new_cell - first_in_cell;
 		if (n_in_cell > 2) {	/* data are already sorted on z; get z mode  */
-			out[2] = weighted_mode (&data[first_in_cell], weight, n_in_cell, 2);
+			out[GMT_Z] = weighted_mode (&data[first_in_cell], weight, n_in_cell, 2);
 			if (Ctrl->Q.active) {
 				i_n_in_cell = 1.0 / n_in_cell;
 				out[GMT_X] *= i_n_in_cell;
@@ -369,7 +365,7 @@ int main (int argc, char **argv)
 		}
 		else if (n_in_cell == 2) {
 			if (data[first_in_cell].a[BLK_W] > data[first_in_cell+1].a[BLK_W]) {
-				out[2] = data[first_in_cell].a[BLK_Z];
+				out[GMT_Z] = data[first_in_cell].a[BLK_Z];
 				if (Ctrl->Q.active) {
 					out[GMT_X] *= 0.5;
 					out[GMT_Y] *= 0.5;
@@ -380,7 +376,7 @@ int main (int argc, char **argv)
 				}
 			}
 			else if (data[first_in_cell].a[BLK_W] < data[first_in_cell+1].a[BLK_W]) {
-				out[2] = data[first_in_cell+1].a[BLK_Z];
+				out[GMT_Z] = data[first_in_cell+1].a[BLK_Z];
 				if (Ctrl->Q.active) {
 					out[GMT_X] *= 0.5;
 					out[GMT_Y] *= 0.5;
@@ -395,18 +391,18 @@ int main (int argc, char **argv)
 					out[GMT_X] *= 0.5;
 					out[GMT_Y] *= 0.5;
 				}
-				out[2] = 0.5 * (data[first_in_cell].a[BLK_Z] + data[first_in_cell+1].a[BLK_Z]);
+				out[GMT_Z] = 0.5 * (data[first_in_cell].a[BLK_Z] + data[first_in_cell+1].a[BLK_Z]);
 			}
 		}
 		else
-			out[2] = data[first_in_cell].a[BLK_Z];
+			out[GMT_Z] = data[first_in_cell].a[BLK_Z];
 
 		if (Ctrl->E.active) {
 			out[4] = z_tmp[0];	/* Low value */
 			out[5] = z_tmp[nz-1];	/* High value */
-			/* Turn z_tmp into absolute deviations from the mode (out[2]) */
+			/* Turn z_tmp into absolute deviations from the mode (out[GMT_Z]) */
 			if (nz > 1) {
-				for (index = 0; index < nz; index++) z_tmp[index] = fabs (z_tmp[index] - out[2]);
+				for (index = 0; index < nz; index++) z_tmp[index] = fabs (z_tmp[index] - out[GMT_Z]);
 				qsort ((void *)z_tmp, (size_t)nz, sizeof (double), GMT_comp_double_asc);
 				out[3] = (nz%2) ? z_tmp[nz/2] : 0.5 * (z_tmp[(nz-1)/2] + z_tmp[nz/2]);
 				out[3] *= 1.4826;	/* This will be LMS MAD-based scale */
@@ -425,16 +421,16 @@ int main (int argc, char **argv)
 	if (gmtdefs.verbose) fprintf(stderr,"%s: N_cells_filled: %ld\n", GMT_program, n_cells_filled);
 
 	GMT_free ((void *)data);
-	if (Ctrl->E.active) GMT_free ((void *)z_tmp);
+	GMT_free ((void *)z_tmp);
 
 	Free_Blockmode_Ctrl (Ctrl);	/* Deallocate control structure */
 
-    GMT_end (argc, argv);
+	GMT_end (argc, argv);
 
 	exit (EXIT_SUCCESS);
 }
 
-double	weighted_mode (struct BLK_DATA *d, double wsum, GMT_LONG n, int k)
+double	weighted_mode (struct BLK_DATA *d, double wsum, GMT_LONG n, GMT_LONG k)
 {
 	/* Estimate mode by finding a maximum in the estimated
 	   pdf of weighted data.  Estimate the pdf as the finite 
@@ -463,7 +459,7 @@ double	weighted_mode (struct BLK_DATA *d, double wsum, GMT_LONG n, int k)
 	nh = n / 2;
 	top = wsum;
 
-	while (j-i > nh) {
+	while ((j-i) > nh) {
 		topi = top - d[i].a[BLK_W];
 		topj = top - d[j].a[BLK_W];
 		bottomi = d[j].a[k] - d[i+1].a[k];
