@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdraster.c,v 1.52 2008/03/24 08:58:32 guru Exp $
+ *	$Id: grdraster.c,v 1.59 2009/05/13 21:06:43 guru Exp $
  *
- *    Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /* grdraster.c -- read a rasterfile and extract a region as a grid file.
@@ -30,6 +30,7 @@
  *		27 Jun 2005   PW: Allow modifiers G(ographic) and C(artesian) in registration flag.
  *		19 Sep 2005   PW: Allow optional H<bytes> at end of line for skipping headers.
  *		07 Sep 2006   PW: Only do 360 wrap if geographic grid.
+ *		27 Apr 2009   PW: Added sanity check on file size (actual vs computed from grdraster.info)
  */
 
 #include "gmt.h"
@@ -51,12 +52,14 @@ struct GRDRASTER_INFO {
 	GMT_LONG	nglobal;	/* If not 0, ras is global and i%nglobal makes it periodic  */
 	GMT_LONG	nanflag;
 	GMT_LONG	nanset;		/* True if raster uses nanflag to signal NaN  */
-	GMT_LONG	skip;		/* Skip this number of header bytes when opeing file  */
+	GMT_LONG	skip;		/* Skip this number of header bytes when opening file  */
 	char	type;
 	BOOLEAN swap_me;	/* TRUE if data set need to be swapped */
 	BOOLEAN geo;		/* TRUE if we believe x/y is lon/lat, FALSE otherwise */
 };
 
+GMT_LONG get_byte_size (char type);
+
 int main (int argc, char **argv)
 {
 	BOOLEAN xyz_out = FALSE;
@@ -72,7 +75,7 @@ int main (int argc, char **argv)
 	
 	GMT_LONG	i, j, k, ksize = 0, iselect, nselected = 0, imult, jmult, nrasters, ij_offset;
 	GMT_LONG	irasstart, jrasstart, n_nan, iras, jras, ij, ijras, jseek, nmask = 0;
-	GMT_LONG	error = FALSE, firstread;
+	GMT_LONG	error = FALSE, firstread, nm;
 	
 	float	*grd = NULL, *floatrasrow = NULL;
 	
@@ -181,6 +184,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, NULL, &Ctrl->I.active);
+
 	/* Check that arguments were valid:  */
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
@@ -361,31 +366,17 @@ int main (int argc, char **argv)
 		for (i = 0; i < h.nx; i++) x[i] = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
 		ij_offset = 0;
 	} else {	/* Need entire grid */
-		grd = (float *)GMT_memory(VNULL, (size_t)(h.nx*h.ny), sizeof(float), GMT_program);
+		nm = GMT_get_nm (h.nx, h.ny);
+		grd = (float *)GMT_memory(VNULL, (size_t)nm, sizeof(float), GMT_program);
 		ij_offset = h.nx;
 	}
 	
-	if (myras.type == 'b') {
-		/* Need to read the whole thing:  */
+	ksize = get_byte_size (myras.type);
+	if (ksize == 0) {	/* Bits; Need to read the whole thing:  */
 		nmask = (GMT_LONG)ceil (myras.h.nx * myras.h.ny * 0.125);
 		ubuffer = (unsigned char *)GMT_memory(VNULL, (size_t)nmask, (size_t)1, GMT_program);
 	}
-	else {
-		/* Need to read by rows, and convert each row to float:  */
-		switch (myras.type) {
-			case 'c':
-			case 'u':
-				ksize = 1;
-				break;
-			case 'd':
-			case 'h':
-			case 'i':
-				ksize = 2;
-				break;
-			case 'l':
-				ksize = 4;
-				break;
-		}
+	else {	/* Need to read by rows, and convert each row to float:  */
 		buffer = GMT_memory (VNULL, (size_t)myras.h.nx, (size_t)ksize, GMT_program);
 		floatrasrow = (float *)GMT_memory(VNULL, (size_t)myras.h.nx, sizeof(float), GMT_program);
 	}
@@ -656,17 +647,19 @@ GMT_LONG	load_rasinfo(struct GRDRASTER_INFO **ras, char endian)
 
 	Return 0 if cannot read files correctly, or nrasters if successful.  */
 
-	GMT_LONG	i, j, length, stop_point, nfound = 0;
+	GMT_LONG	i, j, length, stop_point, nfound = 0, ksize = 0;
 	size_t	n_alloc;
+	off_t expected_size;
 	double	global_lon, lon_tol;
 	char	buf[GRD_REMARK_LEN], dir[GRD_REMARK_LEN], *l;
 	FILE	*fp;
 	struct GRDRASTER_INFO *rasinfo;
+	struct STAT F;
 
 	/* Find and open the file grdraster.info */
 
 	if (!(GMT_getdatapath("grdraster.info", dir) || GMT_getsharepath("dbase", "grdraster", ".info", dir))) {
-		fprintf(stderr, "%s: ERROR cannot find file grdraster.info", GMT_program);
+		fprintf(stderr, "%s: ERROR cannot find file grdraster.info\n", GMT_program);
 		return (0);
 	}
 
@@ -1037,6 +1030,17 @@ GMT_LONG	load_rasinfo(struct GRDRASTER_INFO **ras, char endian)
 		j = irint( (rasinfo[nfound].h.y_max - rasinfo[nfound].h.y_min)/rasinfo[nfound].h.y_inc);
 		rasinfo[nfound].h.ny = (rasinfo[nfound].h.node_offset) ? j : j + 1;
 		
+		if ((ksize = get_byte_size (rasinfo[nfound].type)) == 0)
+			expected_size = (off_t)(ceil (GMT_get_nm (rasinfo[nfound].h.nx, rasinfo[nfound].h.ny) * 0.125) + rasinfo[nfound].skip);
+		else
+			expected_size = (off_t)(GMT_get_nm (rasinfo[nfound].h.nx, rasinfo[nfound].h.ny) * ksize + rasinfo[nfound].skip);
+		if (STAT (rasinfo[nfound].h.remark, &F)) {	/* Inquiry about file failed somehow */
+			fprintf (stderr, "%s: Warning: Unable to stat file %s\n", GMT_program, rasinfo[nfound].h.remark);
+		}
+		else if (F.st_size != expected_size) {
+			fprintf (stderr, "%s: Actual size of file %s [%ld] differs from expected [%ld]. Verify file and its grdraster.info details.\n", GMT_program, rasinfo[nfound].h.remark, (GMT_LONG)F.st_size, (GMT_LONG)expected_size);
+			exit (EXIT_FAILURE);
+		}
 		nfound++;
 		
 		if ((size_t)nfound == n_alloc) {
@@ -1051,6 +1055,33 @@ GMT_LONG	load_rasinfo(struct GRDRASTER_INFO **ras, char endian)
 	return (nfound);
 }
 
+GMT_LONG get_byte_size (char type) {
+	/* Return byte size of each item, or 0 if bits */
+	int ksize;
+	switch (type) {
+		case 'b':
+			ksize = 0;
+			break;
+		case 'c':
+		case 'u':
+			ksize = 1;
+			break;
+		case 'd':
+		case 'h':
+		case 'i':
+			ksize = 2;
+			break;
+		case 'l':
+			ksize = 4;
+			break;
+		default:
+			fprintf (stderr, "%s: ERROR: Invalid data type [%c]\n", GMT_program, type);
+			exit (EXIT_FAILURE);
+			break;
+	}
+	return (ksize);
+}
+
 void *New_Grdraster_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDRASTER_CTRL *C;
 	
