@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *    $Id: filter1d.c,v 1.52 2009/01/09 04:02:32 guru Exp $
+ *    $Id: filter1d.c,v 1.58 2009/09/09 23:27:01 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ struct FILTER1D_CTRL {
 		BOOLEAN active;
 		char filter;	/* Character codes for the filter */
 		double width;
-		int mode;
+		GMT_LONG mode;
 		char *file;	/* Character codes for the filter */
 	} F;
 	struct I {	/* -I<ignoreval> */
@@ -62,7 +62,7 @@ struct FILTER1D_CTRL {
 	} L;
 	struct N {	/* -N<n_cols>/<t_col> */
 		BOOLEAN active;
-		int col, ncols;
+		GMT_LONG col, ncols;
 	} N;
 	struct Q {	/* -Q<factor> */
 		BOOLEAN active;
@@ -100,21 +100,21 @@ struct FILTER1D_INFO {	/* Control structure for all aspects of the filter setup
 	BOOLEAN out_at_time;	/* TRUE when output is required at evenly spaced intervals */
 	BOOLEAN f_operator;	/* TRUE if custom weights coefficients sum to zero */
 
-	int	*n_this_col;		/* Pointer to array of counters [one per column]  */
-	int	*n_left;		/* Pointer to array of counters [one per column]  */
-	int	*n_right;		/* Pointer to array of counters [one per column]  */
+	GMT_LONG	*n_this_col;		/* Pointer to array of counters [one per column]  */
+	GMT_LONG	*n_left;		/* Pointer to array of counters [one per column]  */
+	GMT_LONG	*n_right;		/* Pointer to array of counters [one per column]  */
 	GMT_LONG	n_rows;			/* Number of rows of input  */
-	int	n_cols;			/* Number of columns of input  */
-	int	t_col;			/* Column of time abscissae (independent variable)  */
-	int	n_f_wts;		/* Number of filter weights  */
-	int	half_n_f_wts;		/* Half the number of filter weights  */
+	GMT_LONG	n_cols;			/* Number of columns of input  */
+	GMT_LONG	t_col;			/* Column of time abscissae (independent variable)  */
+	GMT_LONG	n_f_wts;		/* Number of filter weights  */
+	GMT_LONG	half_n_f_wts;		/* Half the number of filter weights  */
 	GMT_LONG	n_row_alloc;		/* Number of rows of data to allocate  */
 	GMT_LONG	n_work_alloc;		/* Number of rows of workspace to allocate  */
-	int	filter_type;		/* Flag indicating desired filter type  */
-	int	kind;				/* -1 skip +ve, +1 skip -ve, else use all  [for the l|L|u|U filter] */
-	int	way;			/* -1 find minimum, +1 find maximum  [for the l|L|u|U filter] */
-	int	mode_selection;
-	int	n_multiples;
+	GMT_LONG	filter_type;		/* Flag indicating desired filter type  */
+	GMT_LONG	kind;			/* -1 skip +ve, +1 skip -ve, else use all  [for the l|L|u|U filter] */
+	GMT_LONG	way;			/* -1 find minimum, +1 find maximum  [for the l|L|u|U filter] */
+	GMT_LONG	mode_selection;
+	GMT_LONG	n_multiples;
 	
 	double	*f_wt;			/* Pointer for array of filter coefficients  */
 	double	*min_loc;		/* Pointer for array of values, one per [column]  */
@@ -147,10 +147,10 @@ int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE;
 	
-	char	c, txt_a[GMT_TEXT_LEN], txt_b[GMT_TEXT_LEN];
+	char	c, txt_a[GMT_TEXT_LEN], txt_b[GMT_TEXT_LEN], *not_used = NULL;
 	char	buffer[BUFSIZ];		/* Used to scan input lines  */
 	
-	int	i, n_fields, n_expected_fields;
+	GMT_LONG	i, n_fields, n_expected_fields;
 	
 	double	last_time, new_time, *in;
 	
@@ -159,10 +159,10 @@ int main (int argc, char **argv)
 	struct FILTER1D_INFO F;
 	struct FILTER1D_CTRL *Ctrl;
 
-	int	load_data_and_check_extrema (struct FILTER1D_INFO *F);	/* Does just what it says  */
-	int	set_up_filter (struct FILTER1D_INFO *F);		/* Creates filter weights or reads them from file and sets start & stop  */
-	int	do_the_filter (struct FILTER1D_INFO *F);		/* Does the job  */
-	int	allocate_space (struct FILTER1D_INFO *F);		/* Does just what it says  */
+	GMT_LONG	load_data_and_check_extrema (struct FILTER1D_INFO *F);	/* Does just what it says  */
+	GMT_LONG	set_up_filter (struct FILTER1D_INFO *F);		/* Creates filter weights or reads them from file and sets start & stop  */
+	GMT_LONG	do_the_filter (struct FILTER1D_INFO *F);		/* Does the job  */
+	GMT_LONG	allocate_space (struct FILTER1D_INFO *F);		/* Does just what it says  */
 	void	free_space (struct FILTER1D_INFO *F);			/* Does just what it says  */
 	void	allocate_more_data_space (struct FILTER1D_INFO *F);	/* Does just what it says  */
 	void load_parameters (struct FILTER1D_INFO *C, struct FILTER1D_CTRL *Ctrl);
@@ -240,7 +240,7 @@ int main (int argc, char **argv)
 					Ctrl->L.value = atof (&argv[i][2]);
 					break;
 				case 'N':
-					if (sscanf (&argv[i][2], "%d/%d", &Ctrl->N.ncols, &Ctrl->N.col) != 2) {
+					if (sscanf (&argv[i][2], "%ld/%ld", &Ctrl->N.ncols, &Ctrl->N.col) != 2) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -N option:  Syntax is -N<ncol>/<tcol>\n", GMT_program);
 						error++;
 					}
@@ -372,7 +372,7 @@ int main (int argc, char **argv)
 		error++;
 	}
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < F.n_cols) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %d fields\n", GMT_program, F.n_cols);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %ld fields\n", GMT_program, F.n_cols);
 		error++;
 	}
 	if (GMT_io.binary[GMT_IN] && !Ctrl->N.active && GMT_io.ncol[GMT_IN] == 0) {
@@ -384,7 +384,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	load_parameters (&F, Ctrl);	/* Pass parameters from Control structure to Filter structure */
@@ -438,7 +438,7 @@ int main (int argc, char **argv)
 			break;
 	}
 
-	GMT_io.skip_if_NaN[0] = GMT_io.skip_if_NaN[1] = FALSE;	/* Turn off default GMT NaN-handling */
+	GMT_io.skip_if_NaN[GMT_X] = GMT_io.skip_if_NaN[GMT_Y] = FALSE;	/* Turn off default GMT NaN-handling */
 	GMT_io.skip_if_NaN[F.t_col] = TRUE;			/* ... But disallow NaN in "time" column */
 
 #ifdef SET_IO_MODE
@@ -461,7 +461,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.io_header[GMT_IN]) {
 		for (i = 0; i < GMT_io.n_header_recs; i++) {
-			GMT_fgets (buffer, BUFSIZ, fp_in);
+			not_used = GMT_fgets (buffer, BUFSIZ, fp_in);
 			if (GMT_io.io_header[GMT_OUT]) fputs (buffer, GMT_stdout);
 		}
 	}
@@ -483,7 +483,7 @@ int main (int argc, char **argv)
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, F.n_rows);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, F.n_rows);
 			exit (EXIT_FAILURE);
 		}
 
@@ -506,7 +506,7 @@ int main (int argc, char **argv)
 			if (new_time < last_time) {
 				fprintf (stderr, "%s: Error! Time decreases at line # %ld\n", GMT_program, F.n_rows);
 				fprintf (stderr, "\tUse UNIX utility sort and then try again.\n");
-				return(-1);
+				exit (EXIT_FAILURE);
 			}
 			last_time = new_time;
 		}
@@ -547,7 +547,7 @@ int main (int argc, char **argv)
 
 	free_space (&F);
 
-	if (F.n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %d multiple modes found\n", GMT_program, F.n_multiples);
+	if (F.n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %ld multiple modes found\n", GMT_program, F.n_multiples);
 
 	Free_Filter1d_Ctrl (Ctrl);	/* Deallocate control structure */
 
@@ -556,9 +556,9 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-int set_up_filter (struct FILTER1D_INFO *F)
+GMT_LONG set_up_filter (struct FILTER1D_INFO *F)
 {
-	int	i, i1, i2;
+	GMT_LONG	i, i1, i2;
 	double	t_0, t_1, time, w_sum;
 	char buffer[BUFSIZ];
 	BOOLEAN	normalize = FALSE;
@@ -595,14 +595,14 @@ int set_up_filter (struct FILTER1D_INFO *F)
 		F->half_width = F->half_n_f_wts * F->dt;
 		F->filter_width = 2.0 * F->half_width;
 		F->f_wt = (double *) GMT_memory ((void *)F->f_wt, (size_t)F->n_f_wts, sizeof (double), GMT_program);
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d filter weights from file.\n", GMT_program, F->n_f_wts);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Read %ld filter weights from file.\n", GMT_program, F->n_f_wts);
 	}
 	else if (F->filter_type <= FILTER1D_CONVOLVE) {
 		get_weight[FILTER1D_BOXCAR] = (PFD)boxcar_weight;
 		get_weight[FILTER1D_COS_ARCH] = (PFD)cosine_weight;
 		get_weight[FILTER1D_GAUSSIAN] = (PFD)gaussian_weight;
 		F->half_width = 0.5 * F->filter_width;
-		F->half_n_f_wts = (int)floor (F->half_width / F->dt);
+		F->half_n_f_wts = (GMT_LONG)floor (F->half_width / F->dt);
 		F->n_f_wts = 2 * F->half_n_f_wts + 1;
 
 		F->f_wt = (double *) GMT_memory (VNULL, (size_t)F->n_f_wts, sizeof(double), GMT_program);
@@ -653,18 +653,18 @@ int set_up_filter (struct FILTER1D_INFO *F)
 	return(0);
 }
 
-int do_the_filter (struct FILTER1D_INFO *F)
+GMT_LONG do_the_filter (struct FILTER1D_INFO *F)
 {
 	GMT_LONG	i_row, left, right, n_l, n_r, iq, i_f_wt;
 	GMT_LONG	i_t_output = 0, n_in_filter, n_for_call, n_good_ones;
-	int i_col;
+	GMT_LONG i_col;
 	double	time, delta_time, *outval, wt, val, med, scl, small;
 	BOOLEAN	*good_one;		/* Pointer to array of logicals [one per column]  */
 	double	*wt_sum;		/* Pointer for array of weight sums [each column]  */
 	double	*data_sum;		/* Pointer for array of data * weight sums [columns]  */
 
-	BOOLEAN	lack_check (struct FILTER1D_INFO *F, int i_col, GMT_LONG left, GMT_LONG right);	/* Tests for lack of data condition  (optional)  */
-	void	get_robust_estimates (struct FILTER1D_INFO *F, int j, GMT_LONG n, int both);	/* Does just what it says  */
+	BOOLEAN	lack_check (struct FILTER1D_INFO *F, GMT_LONG i_col, GMT_LONG left, GMT_LONG right);	/* Tests for lack of data condition  (optional)  */
+	void	get_robust_estimates (struct FILTER1D_INFO *F, GMT_LONG j, GMT_LONG n, GMT_LONG both);	/* Does just what it says  */
 
 	outval = (double *)GMT_memory (VNULL, (size_t)F->n_cols, sizeof (double), GMT_program);
 	good_one = (BOOLEAN *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(BOOLEAN), GMT_program);
@@ -740,11 +740,7 @@ int do_the_filter (struct FILTER1D_INFO *F)
 					if (!(good_one[i_col])) continue;
 					n_l = F->n_left[i_col];
 					n_r = F->n_right[i_col];
-#ifdef __LP64__
-					if ((((double)labs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) good_one[i_col] = FALSE;
-#else
-					if ((((double)abs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) good_one[i_col] = FALSE;
-#endif
+					if ((((double)GMT_abs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) good_one[i_col] = FALSE;
 				}
 			}
 			if ( (F->filter_type > FILTER1D_CONVOLVE) && F->check_q) {
@@ -819,11 +815,7 @@ int do_the_filter (struct FILTER1D_INFO *F)
 				if (F->check_asym && !(F->robust) ) {
 					n_l = F->n_left[i_col];
 					n_r = F->n_right[i_col];
-#ifdef __LP64__
-					if ((((double)labs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) {
-#else
-					if ((((double)abs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) {
-#endif
+					if ((((double)GMT_abs(n_l - n_r))/(n_l + n_r)) > F->sym_coeff) {
 						good_one[i_col] = FALSE;
 						continue;
 					}
@@ -865,17 +857,17 @@ int do_the_filter (struct FILTER1D_INFO *F)
 	return (0);
 }
 
-int allocate_space (struct FILTER1D_INFO *F)
+GMT_LONG allocate_space (struct FILTER1D_INFO *F)
 {
-	int	i;
+	GMT_LONG	i;
 
 	F->data = (double **) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(double *), GMT_program);
 	for (i = 0; i < F->n_cols; i++) F->data[i] = (double *) GMT_memory (VNULL, (size_t)F->n_row_alloc, sizeof(double), GMT_program);
 
-	F->n_this_col = (int *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(int), GMT_program);
+	F->n_this_col = (GMT_LONG *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(GMT_LONG), GMT_program);
 
-	if (F->check_asym) F->n_left = (int *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(int), GMT_program);
-	if (F->check_asym) F->n_right = (int *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(int), GMT_program);
+	if (F->check_asym) F->n_left = (GMT_LONG *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(GMT_LONG), GMT_program);
+	if (F->check_asym) F->n_right = (GMT_LONG *) GMT_memory (VNULL, (size_t)F->n_cols, sizeof(GMT_LONG), GMT_program);
 
 	if (F->robust || (F->filter_type > FILTER1D_CONVOLVE) ) {	/* Then we need workspace  */
 
@@ -895,14 +887,14 @@ int allocate_space (struct FILTER1D_INFO *F)
 
 void allocate_more_data_space (struct FILTER1D_INFO *F)
 {
-	int	i;
+	GMT_LONG	i;
 
 	for (i = 0; i < F->n_cols; i++) F->data[i] = (double *) GMT_memory ((void *)F->data[i], (size_t)F->n_row_alloc, sizeof(double), GMT_program);
 }
 
 void allocate_more_work_space (struct FILTER1D_INFO *F)
 {
-	int	i;
+	GMT_LONG	i;
 
 	for (i = 0; i < F->n_cols; i++) F->work[i] = (double *) GMT_memory ((void *)F->work[i], (size_t)F->n_work_alloc, sizeof(double), GMT_program);
 }
@@ -910,7 +902,7 @@ void allocate_more_work_space (struct FILTER1D_INFO *F)
 
 void free_space (struct FILTER1D_INFO *F)
 {
-	int i;
+	GMT_LONG i;
 	if (F->robust || (F->filter_type > FILTER1D_CONVOLVE) ) {	
 		for (i = 0; i < F->n_cols; i++)	GMT_free ((void *)F->work[i]);
 		GMT_free ((void *)F->work);
@@ -931,7 +923,7 @@ void free_space (struct FILTER1D_INFO *F)
 	if (F->n_f_wts) GMT_free ((void *)F->f_wt);
 }
 
-BOOLEAN	lack_check (struct FILTER1D_INFO *F, int i_col, GMT_LONG left, GMT_LONG right)
+BOOLEAN	lack_check (struct FILTER1D_INFO *F, GMT_LONG i_col, GMT_LONG left, GMT_LONG right)
 {
 	GMT_LONG	last_row, this_row;
 	BOOLEAN	lacking = FALSE;
@@ -956,7 +948,7 @@ BOOLEAN	lack_check (struct FILTER1D_INFO *F, int i_col, GMT_LONG left, GMT_LONG
 	return (lacking);
 }
 
-void get_robust_estimates (struct FILTER1D_INFO *F, int j, GMT_LONG n, int both)
+void get_robust_estimates (struct FILTER1D_INFO *F, GMT_LONG j, GMT_LONG n, GMT_LONG both)
 {
 	GMT_LONG i, n_smooth;
 	BOOLEAN sort_me = TRUE;
