@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_bcr.c,v 1.7 2009/01/09 04:02:32 guru Exp $
+ *	$Id: gmt_bcr.c,v 1.9 2009/09/09 23:27:01 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -78,7 +78,7 @@
 #define GMT_WITH_NO_PS
 #include "gmt.h"
 
-void GMT_bcr_init (struct GRD_HEADER *grd, int *pad, int interpolant, double threshold, struct GMT_BCR *bcr)
+void GMT_bcr_init (struct GRD_HEADER *grd, GMT_LONG *pad, GMT_LONG interpolant, double threshold, struct GMT_BCR *bcr)
 {
 	/* Initialize interpolant and threshold */
 	bcr->interpolant = interpolant;
