@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_grd.h,v 1.32 2008/04/02 15:47:08 remko Exp $
+ *	$Id: gmt_grd.h,v 1.38 2009/09/09 23:27:02 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -47,13 +47,14 @@ struct GRD_HEADER {
 	int ny;				/* Number of rows */
 	int node_offset;		/* 0 for node grids, 1 for pixel grids */
 /* This section is flexible. It is not copied to any grid header */
-	int type;			/* Grid format */
+	GMT_LONG type;			/* Grid format */
 	char name[GMT_LONG_TEXT];	/* Actual name of the file after any ?<varname> and =<stuff> has been removed */
 	char varname[GRD_VARNAME_LEN];	/* NetCDF: variable name */
 	int y_order;			/* NetCDF: 1 if S->N, -1 if N->S */
 	int z_id;			/* NetCDF: id of z field */
 	int ncid;			/* NetCDF: file ID */
 	int t_index[3];			/* NetCDF: index of higher coordinates */
+	int xy_dim[2];			/* NetCDF: dimension order of x and y; normally {1, 0} */
 	double nan_value;		/* Missing value as stored in grid file */
 	double xy_off;			/* 0.0 (node_offset == 0) or 0.5 ( == 1) */
 /* The following elements should not be changed. They are copied verbatim to the native grid header */
@@ -104,11 +105,15 @@ struct GRD_HEADER {
 #define GMT_i_to_x(i,x0,x1,dx,off,nx) (((i) == ((nx)-1)) ? (x1) - (off) * (dx) : (x0) + ((i) + (off)) * (dx))
 #define GMT_j_to_y(j,y0,y1,dy,off,ny) (((j) == ((ny)-1)) ? (y0) + (off) * (dy) : (y1) - ((j) + (off)) * (dy))
 
-/* These macros calculate the number of nodes in x or y  or the increment dx, dy*/
+/* These macros calculate the number of nodes in x or y or the increment dx, dy*/
 
 #define GMT_get_n(min,max,inc,off) ((GMT_LONG)irint (((max) - (min)) / (inc)) + 1 - (off))
 #define GMT_get_inc(min,max,n,off) (((max) - (min)) / ((n) + (off) - 1))
 
+/* 64-bit-safe macro to return the number of points in the grid given dimensions */
+
+#define GMT_get_nm(nx,ny) (((GMT_LONG)(nx)) * ((GMT_LONG)(ny)))
+
 /* Calculate 1-D index a[ij] corresponding to 2-D array a[row][col], with 64-bit precision.
  * Use GMT_IJbc when array is padded by BC rows/cols, else use GMT_IJ */
 
