@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_grdio.c,v 1.119 2009/01/09 04:02:32 guru Exp $
+ *	$Id: gmt_grdio.c,v 1.131 2009/09/09 23:27:02 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -63,28 +63,28 @@
 #define STAT stat
 #endif
 
-int GMT_grdformats[GMT_N_GRD_FORMATS][2] = {
+GMT_LONG GMT_grdformats[GMT_N_GRD_FORMATS][2] = {
 #include "gmt_grdformats.h"
 };
 
 void GMT_grd_do_scaling (float *grid, GMT_LONG nm, double scale, double offset);
 void GMT_grd_get_units (struct GRD_HEADER *header);
 void GMT_grd_set_units (struct GRD_HEADER *header);
-int GMT_is_nc_grid (struct GRD_HEADER *header);
-int GMT_is_native_grid (struct GRD_HEADER *header);
-int GMT_is_ras_grid (struct GRD_HEADER *header);
-int GMT_is_srf_grid (struct GRD_HEADER *header);
-int GMT_is_mgg2_grid (struct GRD_HEADER *header);
-int GMT_is_agc_grid (struct GRD_HEADER *header);
+GMT_LONG GMT_is_nc_grid (struct GRD_HEADER *header);
+GMT_LONG GMT_is_native_grid (struct GRD_HEADER *header);
+GMT_LONG GMT_is_ras_grid (struct GRD_HEADER *header);
+GMT_LONG GMT_is_srf_grid (struct GRD_HEADER *header);
+GMT_LONG GMT_is_mgg2_grid (struct GRD_HEADER *header);
+GMT_LONG GMT_is_agc_grid (struct GRD_HEADER *header);
 
 /* GENERIC I/O FUNCTIONS FOR GRIDDED DATA FILES */
 
-int GMT_read_grd_info (char *file, struct GRD_HEADER *header)
+GMT_LONG GMT_read_grd_info (char *file, struct GRD_HEADER *header)
 {	/* file:	File name
 	 * header:	grid structure header
 	 */
 
-	int err;
+	GMT_LONG err;
 	double scale, offset, nan_value;
 
 	/* Initialize grid information */
@@ -108,12 +108,12 @@ int GMT_read_grd_info (char *file, struct GRD_HEADER *header)
 	return (GMT_NOERROR);
 }
 
-int GMT_write_grd_info (char *file, struct GRD_HEADER *header)
+GMT_LONG GMT_write_grd_info (char *file, struct GRD_HEADER *header)
 {	/* file:	File name
 	 * header:	grid structure header
 	 */
 
-	int err;
+	GMT_LONG err;
 
  	GMT_err_trap (GMT_grd_get_format (file, header, FALSE));
 
@@ -129,7 +129,7 @@ int GMT_write_grd_info (char *file, struct GRD_HEADER *header)
 	return ((*GMT_io_writeinfo[header->type]) (header));
 }
 
-int GMT_update_grd_info (char *file, struct GRD_HEADER *header)
+GMT_LONG GMT_update_grd_info (char *file, struct GRD_HEADER *header)
 {	/* file:	- IGNORED -
 	 * header:	grid structure header
 	 */
@@ -140,7 +140,7 @@ int GMT_update_grd_info (char *file, struct GRD_HEADER *header)
 	return ((*GMT_io_updateinfo[header->type]) (header));
 }
 
-int GMT_read_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
+GMT_LONG GMT_read_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, GMT_LONG *pad, BOOLEAN complex)
 {	/* file:	- IGNORED -
 	 * header:	grid structure header
 	 * grid:	array with final grid
@@ -151,7 +151,7 @@ int GMT_read_grd (char *file, struct GRD_HEADER *header, float *grid, double w,
 	 *		for imaginary parts when processed by grdfft etc.
 	 */
 
-	int err;
+	GMT_LONG err;
 	GMT_LONG nm;
 
 	GMT_err_trap ((*GMT_io_readgrd[header->type]) (header, grid, w, e, s, n, pad, complex));
@@ -164,7 +164,7 @@ int GMT_read_grd (char *file, struct GRD_HEADER *header, float *grid, double w,
 	return (GMT_NOERROR);
 }
 
-int GMT_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
+GMT_LONG GMT_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, GMT_LONG *pad, BOOLEAN complex)
 {	/* file:	File name
 	 * header:	grid structure header
 	 * grid:	array with final grid
@@ -175,7 +175,7 @@ int GMT_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w,
 	 *		for imaginary parts when processed by grdfft etc.
 	 */
 
-	int err;
+	GMT_LONG err;
 	GMT_LONG nm;
 
 	GMT_err_trap (GMT_grd_get_format (file, header, FALSE));
@@ -195,18 +195,18 @@ int GMT_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w,
 
 void GMT_expand_filename (char *file, char *fname)
 {
-	int i, length, f_length, found, start;
+	GMT_LONG i, length, f_length, found, start;
 
 	if (gmtdefs.gridfile_shorthand) {	/* Look for matches */
-		f_length = (int) strlen (file);
+		f_length = (GMT_LONG) strlen (file);
 		for (i = found = 0; !found && i < GMT_n_file_suffix; i++) {
-			length = (int) strlen (GMT_file_suffix[i]);
+			length = (GMT_LONG) strlen (GMT_file_suffix[i]);
 			start = f_length - length;
 			found = (start < 0) ? FALSE : !strncmp (&file[start], GMT_file_suffix[i], (size_t)length);
 		}
 		if (found) {	/* file ended in a recognized shorthand extension */
 			i--;
-			sprintf (fname, "%s=%d/%g/%g/%g", file, GMT_file_id[i], GMT_file_scale[i], GMT_file_offset[i], GMT_file_nan[i]);
+			sprintf (fname, "%s=%ld/%g/%g/%g", file, GMT_file_id[i], GMT_file_scale[i], GMT_file_offset[i], GMT_file_nan[i]);
 		}
 		else
 			strcpy (fname, file);
@@ -215,7 +215,7 @@ void GMT_expand_filename (char *file, char *fname)
 		strcpy (fname, file);
 }
 
-int GMT_grd_get_format (char *file, struct GRD_HEADER *header, BOOLEAN magic)
+GMT_LONG GMT_grd_get_format (char *file, struct GRD_HEADER *header, BOOLEAN magic)
 {
 	/* This functions does a couple of things:
 	 * 1. It tries to determine what kind of grid file this is. If a file is openeed for
@@ -231,7 +231,7 @@ int GMT_grd_get_format (char *file, struct GRD_HEADER *header, BOOLEAN magic)
 	 *    by seaching in current dir and the various GMT_*DIR paths.
 	 */
 	
-	int i = 0, val, j;
+	GMT_LONG i = 0, val, j;
 	char code[GMT_TEXT_LEN], tmp[BUFSIZ];
 
 	GMT_expand_filename (file, header->name);	/* May append a suffix to header->name */
@@ -275,9 +275,9 @@ int GMT_grd_get_format (char *file, struct GRD_HEADER *header, BOOLEAN magic)
 		if (GMT_is_agc_grid (header) >= 0) return (GMT_NOERROR);
 		return (GMT_GRDIO_UNKNOWN_FORMAT);	/* No supported format found */
 	}
-	else {			/* Writing: get format type, scale, offset and missing value from gmtdefs.grid_format */
+	else {			/* Writing: get format type, scale, offset and missing value from gmtdefs.gridfile_format */
 		if (sscanf (header->name, "%[^?]?%s", tmp, header->varname) > 1) strcpy (header->name, tmp);    /* Strip off variable name */
-		sscanf (gmtdefs.grid_format, "%[^/]/%lf/%lf/%lf", code, &header->z_scale_factor, &header->z_add_offset, &header->nan_value);
+		sscanf (gmtdefs.gridfile_format, "%[^/]/%lf/%lf/%lf", code, &header->z_scale_factor, &header->z_add_offset, &header->nan_value);
 		val = GMT_grd_format_decoder (code);
 		if (val < 0) return (val);
 		header->type = val;
@@ -286,7 +286,7 @@ int GMT_grd_get_format (char *file, struct GRD_HEADER *header, BOOLEAN magic)
 	return (GMT_NOERROR);
 }
 
-int GMT_grd_data_size (int format, double *nan_value)
+GMT_LONG GMT_grd_data_size (GMT_LONG format, double *nan_value)
 {
 	/* Determine size of data type and set NaN value, if not yet done so (integers only) */
 
@@ -316,18 +316,18 @@ int GMT_grd_data_size (int format, double *nan_value)
 	}
 }
 
-int GMT_grd_format_decoder (const char *code)
+GMT_LONG GMT_grd_format_decoder (const char *code)
 {
 	/* Returns the integer grid format ID that goes with the specified 2-character code */
 
-	int id;
+	GMT_LONG id;
 
 	if (isdigit ((int)code[0])) {	/* File format number given, convert directly */
 		id = atoi (code);
  		if (id < 0 || id >= GMT_N_GRD_FORMATS) return (GMT_GRDIO_UNKNOWN_ID);
 	}
 	else {	/* Character code given */
-		int i, group;
+		GMT_LONG i, group;
 		for (i = group = 0, id = -1; id < 0 && i < GMT_N_GRD_FORMATS; i++) {
 			if (GMT_grdformats[i][0] == (short)code[0]) {
 				group = code[0];
@@ -362,11 +362,11 @@ void GMT_grd_do_scaling (float *grid, GMT_LONG nm, double scale, double offset)
  * Date:	20 April 1998
  */
 
-int GMT_grd_RI_verify (struct GRD_HEADER *h, int mode)
+GMT_LONG GMT_grd_RI_verify (struct GRD_HEADER *h, GMT_LONG mode)
 {
 	/* mode - 0 means we are checking an existing grid, mode = 1 means we test a new -R -I combination */
 
-	int error = 0;
+	GMT_LONG error = 0;
 
 	if (!strcmp (GMT_program, "grdedit")) return (GMT_NOERROR);	/* Separate handling in grdedit to allow grdedit -A */
 
@@ -407,12 +407,15 @@ int GMT_grd_RI_verify (struct GRD_HEADER *h, int mode)
 	return (GMT_NOERROR);
 }
 
-int GMT_grd_prep_io (struct GRD_HEADER *header, double *w, double *e, double *s, double *n, GMT_LONG *width, GMT_LONG *height, GMT_LONG *first_col, GMT_LONG *last_col, GMT_LONG *first_row, GMT_LONG *last_row, int **index)
+GMT_LONG GMT_grd_prep_io (struct GRD_HEADER *header, double *w, double *e, double *s, double *n, GMT_LONG *width, GMT_LONG *height, GMT_LONG *first_col, GMT_LONG *last_col, GMT_LONG *first_row, GMT_LONG *last_row, GMT_LONG **index)
 {
-	/* Determines which rows and columns to extract, and if it is
-	 * a grid that is periodic and wraps around and returns indices. */
+	/* Determines which rows and columns to extract to extract from a grid, based on w,e,s,n.
+	 * This routine first rounds the w,e,s,n boundaries to the nearest gridlines or pixels,
+	 * then determines the first and last columns and rows, and the width and height of the subset (in cells).
+	 * The routine also returns and array of the x-indices in the source grid to be used in the target (subset) grid.
+	 */
 
-	int one_or_zero, i, *k;
+	GMT_LONG one_or_zero, i, *k;
 	BOOLEAN geo = FALSE;
 	double small = 0.1, half_or_zero, x;
 
@@ -437,7 +440,7 @@ int GMT_grd_prep_io (struct GRD_HEADER *header, double *w, double *e, double *s,
 
 		one_or_zero = (header->node_offset) ? 0 : 1;
 
-		/* Make sure w,e,s,n are proper multiples of dx,dy away from w, s */
+		/* Make sure w,e,s,n are proper multiples of x_inc,y_inc away from x_min,y_min */
 
 		GMT_err_pass (GMT_adjust_loose_wesn (w, e, s, n, header), header->name);
 
@@ -452,20 +455,22 @@ int GMT_grd_prep_io (struct GRD_HEADER *header, double *w, double *e, double *s,
 		*last_col  = (GMT_LONG)ceil  ((*e - header->x_min) / header->x_inc - small) - 1 + one_or_zero;
 		*first_row = (GMT_LONG)floor ((header->y_max - *n) / header->y_inc + small);
 		*last_row  = (GMT_LONG)ceil  ((header->y_max - *s) / header->y_inc - small) - 1 + one_or_zero;
-
+#if 0
 		if ((*last_col - *first_col + 1) > *width) (*last_col)--;
 		if ((*last_row - *first_row + 1) > *height) (*last_row)--;
 		if ((*last_col - *first_col + 1) > *width) (*first_col)++;
 		if ((*last_row - *first_row + 1) > *height) (*first_row)++;
+#endif
 	}
 
-	k = (int *) GMT_memory (VNULL, (size_t)(*width), sizeof (int), "GMT_grd_prep_io");
+	k = (GMT_LONG *) GMT_memory (VNULL, *width, sizeof (GMT_LONG), "GMT_grd_prep_io");
 	if (geo) {
+		small = 0.1 * header->x_inc;
 		for (i = 0; i < (*width); i++) {
-			x = *w + (i + half_or_zero) * header->x_inc;
-			if ((header->x_min - x) / header->x_inc > small)
+			x = GMT_i_to_x (i, *w, *e, header->x_inc, half_or_zero, *width);
+			if (header->x_min - x > small)
 				x += 360.0;
-			else if ((x - header->x_max) / header->x_inc > small)
+			else if (x - header->x_max > small)
 				x -= 360.0;
 			k[i] = GMT_x_to_i (x, header->x_min, header->x_inc, half_or_zero, header->nx);
 		}
@@ -491,7 +496,7 @@ void GMT_decode_grd_h_info (char *input, struct GRD_HEADER *h) {
 	and after GMT_grd_init() has been called.
 */
 	char ptr[BUFSIZ], sep[] = "/";
-	int entry = 0, pos = 0;
+	GMT_LONG entry = 0, pos = 0;
 
 	if (input[0] != input[strlen(input)-1]) {}
 	else if (input[0] == '=') {}
@@ -544,7 +549,7 @@ void GMT_decode_grd_h_info (char *input, struct GRD_HEADER *h) {
 	return;
 }
 
-int GMT_open_grd (char *file, struct GMT_GRDFILE *G, char mode)
+GMT_LONG GMT_open_grd (char *file, struct GMT_GRDFILE *G, char mode)
 {
 	/* Assumes header contents is already known.  For writing we
 	 * assume that the header has already been written.  We fill
@@ -553,11 +558,11 @@ int GMT_open_grd (char *file, struct GMT_GRDFILE *G, char mode)
 	 * grdraster-type files.
 	 */
 
-	int r_w, err;
+	GMT_LONG r_w, err;
 	int cdf_mode[3] = { NC_NOWRITE, NC_WRITE, NC_WRITE};
 	char *bin_mode[3] = { "rb", "rb+", "wb"};
 	BOOLEAN header = TRUE, magic = TRUE;
-	EXTERN_MSC int GMT_nc_grd_info (struct GRD_HEADER *header, char job);
+	EXTERN_MSC GMT_LONG GMT_nc_grd_info (struct GRD_HEADER *header, char job);
 
 	if (mode == 'r' || mode == 'R') {	/* Open file for reading */
 		if (mode == 'R') header = FALSE;
@@ -609,7 +614,7 @@ int GMT_open_grd (char *file, struct GMT_GRDFILE *G, char mode)
 	else	/* All other */
 		G->n_byte = G->header.nx * G->size;
 
-	G->v_row = (void *) GMT_memory (VNULL, (size_t)G->n_byte, (size_t)1, GMT_program);
+	G->v_row = (void *) GMT_memory (VNULL, G->n_byte, (size_t)1, GMT_program);
 
 	G->row = 0;
 	G->auto_advance = TRUE;	/* Default is to read sequential rows */
@@ -625,13 +630,13 @@ void GMT_close_grd (struct GMT_GRDFILE *G)
 		GMT_fclose (G->fp);
 }
 
-int GMT_read_grd_row (struct GMT_GRDFILE *G, int row_no, float *row)
+GMT_LONG GMT_read_grd_row (struct GMT_GRDFILE *G, GMT_LONG row_no, float *row)
 {	/* Reads the entire row vector form the grdfile
 	 * If row_no is negative it is interpreted to mean that we want to
 	 * fseek to the start of the abs(row_no) record and no reading takes place.
 	 */
 
-	int i, err;
+	GMT_LONG i, err;
 
 	if (GMT_grdformats[G->header.type][0] == 'c') {		/* Get one NetCDF row, old format */
 		if (row_no < 0) {	/* Special seek instruction */
@@ -670,15 +675,15 @@ int GMT_read_grd_row (struct GMT_GRDFILE *G, int row_no, float *row)
 	return (GMT_NOERROR);
 }
 
-int GMT_write_grd_row (struct GMT_GRDFILE *G, int row_no, float *row)
+GMT_LONG GMT_write_grd_row (struct GMT_GRDFILE *G, GMT_LONG row_no, float *row)
 {	/* Writes the entire row vector to the grdfile */
 
-	int i, size, err;
+	GMT_LONG i, size, err;
 	void *tmp;
 
 	size = GMT_grd_data_size (G->header.type, &G->header.nan_value);
 
-	tmp = (void *) GMT_memory (VNULL, (size_t)G->header.nx, (size_t)size, "GMT_write_grd_row");
+	tmp = (void *) GMT_memory (VNULL, (GMT_LONG)G->header.nx, (size_t)size, "GMT_write_grd_row");
 
 	GMT_grd_do_scaling (row, (GMT_LONG)G->header.nx, G->scale, G->offset);
 	for (i = 0; i < G->header.nx; i++) if (GMT_is_fnan (row[i]) && G->check) row[i] = (float)G->header.nan_value;
@@ -705,7 +710,7 @@ void GMT_grd_init (struct GRD_HEADER *header, int argc, char **argv, BOOLEAN upd
 {	/* GMT_grd_init initializes a grd header to default values and copies the
 	 * command line to the header variable command.
 	 * update = TRUE if we only want to update command line */
-	int i, len;
+	GMT_LONG i, len;
 
 	if (update)	/* Only clean the command history */
 		memset ((void *)header->command, 0, (size_t)GRD_COMMAND_LEN);
@@ -717,10 +722,13 @@ void GMT_grd_init (struct GRD_HEADER *header, int argc, char **argv, BOOLEAN upd
 		header->type			= -1;
 		header->y_order			= 1;
 		header->z_id			= -1;
+		header->z_min			= GMT_d_NaN;
+		header->z_max			= GMT_d_NaN;
 		header->nan_value		= GMT_d_NaN;
 		strcpy (header->x_units, "x");
 		strcpy (header->y_units, "y");
 		strcpy (header->z_units, "z");
+		for (i = 0; i < 3; i++) header->t_index[i] = -1;
 	}
 
 	/* Always update command line history */
@@ -743,24 +751,27 @@ void GMT_grd_shift (struct GRD_HEADER *header, float *grd, double shift)
 	/* Rotate geographical, global grid in e-w direction
 	 * This function will shift a grid by shift degrees */
 
-	GMT_LONG i, j, k, ij, nc, nx1, n_shift, width, n_warn = 0;
+	GMT_LONG i, j, k, ij, nc, n_shift, width, n_warn = 0;
 	float *tmp;
 
-	tmp = (float *) GMT_memory (VNULL, (size_t)header->nx, sizeof (float), "GMT_grd_shift");
-
 	n_shift = irint (shift / header->x_inc);
-	nx1 = header->nx - 1;
-	width = (header->node_offset) ? header->nx : nx1;
+	width = irint (360.0 / header->x_inc);
+	if (width > header->nx) {
+		fprintf (stderr, "%s: Error: can not rotate grid, too small\n", GMT_program);
+		return;
+	}
+
+	tmp = (float *) GMT_memory (VNULL, (GMT_LONG)header->nx, sizeof (float), "GMT_grd_shift");
+
 	nc = header->nx * sizeof (float);
 
 	for (j = ij = 0; j < header->ny; j++, ij += header->nx) {
-		if (!header->node_offset && grd[ij] != grd[ij+nx1]) n_warn++;
+		if (width < header->nx && grd[ij] != grd[ij+width]) n_warn++;
 		for (i = 0; i < header->nx; i++) {
 			k = (i - n_shift) % width;
 			if (k < 0) k += width;
 			tmp[k] = grd[ij+i];
 		}
-		if (!header->node_offset) tmp[width] = tmp[0];
 		memcpy ((void *)&grd[ij], (void *)tmp, (size_t)nc);
 	}
 
@@ -817,113 +828,127 @@ BOOLEAN GMT_grd_is_global (struct GRD_HEADER *h)
 
 #define GMT_region_is_global ((fabs (project_info.e - project_info.w - 360.0) < GMT_SMALL && project_info.degree[0]))
 
-int GMT_grd_setregion (struct GRD_HEADER *h, double *xmin, double *xmax, double *ymin, double *ymax)
+GMT_LONG GMT_grd_setregion (struct GRD_HEADER *h, double *xmin, double *xmax, double *ymin, double *ymax, int interpolant)
 {
-	/* GMT_grd_setregion determines what wesn should be passed to GMT_read_grd.
-	 * It does so by using project_info.w,e,s,n which have been set correctly
-	 * by map_setup. */
+	/* GMT_grd_setregion determines what w,e,s,n should be passed to GMT_read_grd.
+	 * It does so by using project_info.w,e,s,n which have been set correctly by map_setup.
+	 * Use interpolant to indicate if (and how) the grid is interpolated after this call.
+	 * This determines possible extension of the grid to allow interpolation (without padding).
+	 *
+	 * Here are some considerations about the boundary we need to match, assuming the grid is gridline oriented:
+	 * - When the output is to become pixels, the outermost point has to be beyond 0.5 cells inside the region
+	 * - When linear interpolation is needed afterwards, the outermost point needs to be on the region edge or beyond
+	 * - When the grid is pixel oriented, the limits need to go outward by another 0.5 cells
+	 * - When the region is global, do not extend the longitudes outward (otherwise you create wrap-around issues)
+	 * So to determine the boundary, we go inward from there.
+	 */
 
-	BOOLEAN region_global, grid_global;
-	double shift_x = 0.0, x_range;
+	BOOLEAN grid_global;
+	double shift_x, x_range, off;
 
 	/* First make an educated guess whether the grid and region are geographical and global */
 	grid_global = GMT_grd_is_global (h);
-	region_global = GMT_region_is_global;
+
+	switch (interpolant) {
+		case BCR_BILINEAR:
+			off = 0.0;
+			break;
+		case BCR_BSPLINE:
+		case BCR_BICUBIC:
+			off = 1.5;
+			break;
+		default:
+			off = -0.5;
+			break;
+	}
+	if (h->node_offset) off += 0.5;
+	*ymin = project_info.s - off * h->y_inc, *ymax = project_info.n + off * h->y_inc;
+	if (GMT_region_is_global) off = 0.0;
+	*xmin = project_info.w - off * h->x_inc, *xmax = project_info.e + off * h->x_inc;
 
 	if (!project_info.region && !GMT_IS_RECT_GRATICULE) {	/* Used -R... with oblique boundaries - return entire grid */
-		BOOLEAN N_outside, S_outside;
-		N_outside = GMT_outside (0.0, +90.0);	/* North pole outside map boundary? */
-		S_outside = GMT_outside (0.0, -90.0);	/* South pole outside map boundary? */
-		/* Note: while h->xy_off might be 0.5 (pixel) or 0 (gridline), the w/e boundaries are always "gridline" hence we pass 0 as xy_off */
-		if (N_outside && S_outside) {	/* No polar complications, return extreme coordinates */
-			if (project_info.e < h->x_min)	/* Make adjustments so project_info.[w,e] jives with h->x_min|x_max */
-				shift_x = 360.0;
-			else if (project_info.w > h->x_max)
-				shift_x = -360.0;
-			*xmin = GMT_i_to_x (GMT_x_to_i (project_info.w + shift_x, h->x_min, h->x_inc, 0.0, h->nx), h->x_min, h->x_max, h->x_inc, 0.0, h->nx);
-			*xmax = GMT_i_to_x (GMT_x_to_i (project_info.e + shift_x, h->x_min, h->x_inc, 0.0, h->nx), h->x_min, h->x_max, h->x_inc, 0.0, h->nx);
-			*ymin = GMT_j_to_y (GMT_y_to_j (project_info.s, h->y_min, h->y_inc, 0.0, h->ny), h->y_min, h->y_max, h->y_inc, 0.0, h->ny);
-			*ymax = GMT_j_to_y (GMT_y_to_j (project_info.n, h->y_min, h->y_inc, 0.0, h->ny), h->y_min, h->y_max, h->y_inc, 0.0, h->ny);
-			/* Make sure we dont exceed grid domain (which can happen if project_info.w|e exceeds the grid limits) */
-			if (*xmin < h->x_min && !grid_global) *xmin = h->x_min;
-			if (*xmax > h->x_max && !grid_global) *xmax = h->x_max;
-			if (*ymin < h->y_min) *ymin = h->y_min;
-			if (*ymax > h->y_max) *ymax = h->y_max;
-		}
-		else if (!N_outside) {	/* North pole included, need all longitudes but restrict latitudes */
-			*xmin = h->x_min;	*xmax = h->x_max;
-			*ymin = GMT_j_to_y (GMT_y_to_j (project_info.s, h->y_min, h->y_inc, 0.0, h->ny), h->y_min, h->y_max, h->y_inc, 0.0, h->ny);
-			*ymax = h->y_max;
-			if (*ymin < h->y_min) *ymin = h->y_min;
-		}
-		else {			/* South pole included, need all longitudes but restrict latitudes */
-			*xmin = h->x_min;	*xmax = h->x_max;
-			*ymin = h->y_min;
-			*ymax = GMT_j_to_y (GMT_y_to_j (project_info.n, h->y_min, h->y_inc, 0.0, h->ny), h->y_min, h->y_max, h->y_inc, 0.0, h->ny);
-			if (*ymax > h->y_max) *ymax = h->y_max;
-		}
+		if (*xmax < h->x_min)	/* Make adjustments so project_info.[w,e] jives with h->x_min|x_max */
+			shift_x = 360.0;
+		else if (*xmin > h->x_max)
+			shift_x = -360.0;
+		else
+			shift_x = 0.0;
+
+		*xmin = h->x_min + irint ((*xmin - h->x_min + shift_x) / h->x_inc) * h->x_inc;
+		*xmax = h->x_max + irint ((*xmax - h->x_min + shift_x) / h->x_inc) * h->x_inc;
+		*ymin = h->y_min + irint ((*ymin - h->y_min) / h->y_inc) * h->y_inc;
+		*ymax = h->y_max + irint ((*ymax - h->y_min) / h->y_inc) * h->y_inc;
+
+		/* Make sure we do not exceed grid domain (which can happen if project_info.w|e exceeds the grid limits) */
+		if (*xmin < h->x_min && !grid_global) *xmin = h->x_min;
+		if (*xmax > h->x_max && !grid_global) *xmax = h->x_max;
+		if (*ymin < h->y_min) *ymin = h->y_min;
+		if (*ymax > h->y_max) *ymax = h->y_max;
+
+		/* If North or South pole are within the map boundary, we need all longitudes but restrict latitudes */
+		if (!GMT_outside(0.0, +90.0)) *xmin = h->x_min, *xmax = h->x_max, *ymax = h->y_max;
+		if (!GMT_outside(0.0, -90.0)) *xmin = h->x_min, *xmax = h->x_max, *ymin = h->y_min;
 		return (0);
 	}
 
 	/* First set and check latitudes since they have no complications */
-	*ymin = MAX (h->y_min, h->y_min + floor ((project_info.s - h->y_min) / h->y_inc + GMT_SMALL) * h->y_inc);
-	*ymax = MIN (h->y_max, h->y_min + ceil  ((project_info.n - h->y_min) / h->y_inc - GMT_SMALL) * h->y_inc);
+	*ymin = MAX (h->y_min, h->y_min + floor ((*ymin - h->y_min) / h->y_inc + GMT_SMALL) * h->y_inc);
+	*ymax = MIN (h->y_max, h->y_min + ceil  ((*ymax - h->y_min) / h->y_inc - GMT_SMALL) * h->y_inc);
 
 	if (*ymax <= *ymin) {	/* Grid must be outside chosen -R */
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Your grid y's or latitudes appear to be outside the map region and will be skipped.\n", GMT_program);
 		return (1);
 	}
 
-	if (GMT_io.in_col_type[0] != GMT_IS_LON) {	/* Regular Cartesian stuff is easy... */
-		*xmin = MAX (h->x_min, h->x_min + floor ((project_info.w - h->x_min) / h->x_inc + GMT_SMALL) * h->x_inc);
-		*xmax = MIN (h->x_max, h->x_min + ceil  ((project_info.e - h->x_min) / h->x_inc - GMT_SMALL) * h->x_inc);
-		if (*xmax <= *xmin) {	/* Grid is outside chosen -R */
-			if (gmtdefs.verbose) fprintf (stderr, "%s: Your grid x-range appear to be outside the plot region and will be skipped.\n", GMT_program);
-			return (1);
+	/* Periodic grid with 360 degree range is easy */
+
+	if (grid_global) {
+		*xmin = h->x_min + floor ((*xmin - h->x_min) / h->x_inc + GMT_SMALL) * h->x_inc;
+		*xmax = h->x_min + ceil  ((*xmax - h->x_min) / h->x_inc - GMT_SMALL) * h->x_inc;
+		/* For the odd chance that xmin or xmax are outside the region: bring them in */
+		if (*xmax - *xmin >= 360.0) {
+			while (*xmin < project_info.w) *xmin += h->x_inc;
+			while (*xmax > project_info.e) *xmax -= h->x_inc;
 		}
 		return (0);
 	}
 
-	/* OK, longitudes are trickier and we must make sure grid and region is on the same page as far as +-360 degrees go */
+	/* Shift a geographic grid 360 degrees up or down to maximize the amount of longitude range */
 
-	if (grid_global) {	/* Periodic grid with 360 degree range is easy */
-		*xmin = (h->x_min + (ceil  ((project_info.w - h->x_min) / h->x_inc - GMT_SMALL)) * h->x_inc);
-		*xmax = (h->x_min + (floor ((project_info.e - h->x_min) / h->x_inc + GMT_SMALL)) * h->x_inc);
-		return (0);
+	if (GMT_io.in_col_type[0] == GMT_IS_LON) {
+		x_range = MIN (*xmin, h->x_max) - MAX (*xmax, h->x_min);
+		if (MIN (*xmin, h->x_max + 360.0) - MAX (*xmax, h->x_min + 360.0) > x_range)
+			shift_x = 360.0;
+		else if (MIN (*xmin, h->x_max - 360.0) - MAX (*xmax, h->x_min - 360.0) > x_range)
+			shift_x = -360.0;
+		else
+			shift_x = 0.0;
+		h->x_min += shift_x;
+		h->x_max += shift_x;
 	}
 
-	/* Shift the grid 360 degrees up or down to maximize the amount of longitude range */
-
-	x_range = MIN (project_info.e, h->x_max) - MAX (project_info.w, h->x_min);
-
-	if (MIN (project_info.e, h->x_max + 360.0) - MAX (project_info.w, h->x_min + 360.0) > x_range)
-		shift_x = 360.0;
-	else if (MIN (project_info.e, h->x_max - 360.0) - MAX (project_info.w, h->x_min - 360.0) > x_range)
-		shift_x = -360.0;
-	else
-		shift_x = 0.0;
-
-	h->x_min += shift_x;
-	h->x_max += shift_x;
-	*xmin = MAX (h->x_min, h->x_min + ceil  ((project_info.w - h->x_min) / h->x_inc - GMT_SMALL) * h->x_inc);
-	*xmax = MIN (h->x_max, h->x_min + floor ((project_info.e - h->x_min) / h->x_inc + GMT_SMALL) * h->x_inc);
-	while (*xmin <= -360) *xmin += 360.0;
-	while (*xmax <= -360) *xmax += 360.0;
+	*xmin = MAX (h->x_min, h->x_min + floor ((*xmin - h->x_min) / h->x_inc + GMT_SMALL) * h->x_inc);
+	*xmax = MIN (h->x_max, h->x_min + ceil  ((*xmax - h->x_min) / h->x_inc - GMT_SMALL) * h->x_inc);
 
 	if (*xmax <= *xmin) {	/* Grid is outside chosen -R in longitude */
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Your grid longitudes appear to be outside the map region and will be skipped.\n", GMT_program);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Your grid x's or longitudes appear to be outside the map region and will be skipped.\n", GMT_program);
 		return (1);
 	}
 	return (0);
 }
 
-int GMT_adjust_loose_wesn (double *w, double *e, double *s, double *n, struct GRD_HEADER *header)
+GMT_LONG GMT_adjust_loose_wesn (double *w, double *e, double *s, double *n, struct GRD_HEADER *header)
 {
-	/* used to ensure that sloppy w,e,s,n values are rounded to proper multiples */
+	/* Used to ensure that sloppy w,e,s,n values are rounded to the gridlines or pixels in the referenced grid.
+	 * Upon entry, the boundaries w,e,s,n are given as a rough approximation of the actual subset needed.
+	 * The routine will limit the boundaries to the grids region and round w,e,s,n to the nearest gridline or
+	 * pixel boundaries (depending on the grid orientation).
+	 * Warnings are produced when the w,e,s,n boundaries are adjusted, so this routine is currently not
+	 * intended to throw just any values at it (although one could).
+	 */
 	
-	int i;
-	BOOLEAN global;
-	double half_or_zero, val, start, dx, small, i_d;
+	BOOLEAN global, error = FALSE;
+	double half_or_zero, val, dx, small;
 	
 	half_or_zero = (header->node_offset) ? 0.5 : 0.0;
 
@@ -938,37 +963,6 @@ int GMT_adjust_loose_wesn (double *w, double *e, double *s, double *n, struct GR
 			/* Everything is seemingly OK */
 			break;
 	}
-	global = GMT_grd_is_global (header);
-	if (!(GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (*w, *e))) {    /* Do this unless a 360 longitude wrap */
-		small = GMT_SMALL * header->x_inc;
-		start = (GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (*w, header->x_min)) ? *w : header->x_min;
-
-		i_d = (*w - header->x_min) / header->x_inc;
-		if (i_d < 0 && !global) i_d = 0;
-		i = irint(i_d);
-		val = header->x_min + i * header->x_inc;
-
-		dx = fabs (*w - val);
-		if (GMT_io.in_col_type[0] == GMT_IS_LON) dx = fmod (dx, 360.0);
-		if (dx > small) {
-			*w = val;
-			(void) fprintf (stderr, "%s: GMT WARNING: (w-x_min) must equal (NX + eps) * x_inc), where NX is an integer and |eps| <= %g.\n", GMT_program, GMT_SMALL);
-			(void) fprintf (stderr, "%s: GMT WARNING: w reset to %g\n", GMT_program, *w);
-		}
-
-		i_d = (*e - header->x_min) / header->x_inc;
-		i = irint(i_d);
-		val = header->x_min + i * header->x_inc;
-
-		dx = fabs (*e - val);
-		if (GMT_io.in_col_type[0] == GMT_IS_LON) dx = fmod (dx, 360.0);
-		if (dx > GMT_SMALL) {
-			*e = val;
-			(void) fprintf (stderr, "%s: GMT WARNING: (e-x_min) must equal (NX + eps) * x_inc), where NX is an integer and |eps| <= %g.\n", GMT_program, GMT_SMALL);
-			(void) fprintf (stderr, "%s: GMT WARNING: e reset to %g\n", GMT_program, *e);
-		}
-	}
-	
 	switch (GMT_minmaxinc_verify (*s, *n, header->y_inc, GMT_SMALL)) {	/* Check if range is compatible with y_inc */
 		case 3:
 			return (GMT_GRDIO_BAD_YINC);
@@ -980,24 +974,50 @@ int GMT_adjust_loose_wesn (double *w, double *e, double *s, double *n, struct GR
 			/* Everything is OK */
 			break;
 	}
+	global = GMT_grd_is_global (header);
+
+	if (!global) {
+		if (*w < header->x_min) { *w = header->x_min; error = TRUE; }
+		if (*e > header->x_max) { *e = header->x_max; error = TRUE; }
+	}
+	if (*s < header->y_min) { *s = header->y_min; error = TRUE; }
+	if (*n > header->y_max) { *n = header->y_max; error = TRUE; }
+	if (error) fprintf (stderr, "%s: Warning: Subset exceeds data domain. Subset reduced to common region.\n", GMT_program);
+	error = FALSE;
+
+	if (!(GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (*w, *e) && global)) {    /* Do this unless a 360 longitude wrap */
+		small = GMT_SMALL * header->x_inc;
+
+		val = header->x_min + irint((*w - header->x_min) / header->x_inc) * header->x_inc;
+		dx = fabs (*w - val);
+		if (GMT_io.in_col_type[0] == GMT_IS_LON) dx = fmod (dx, 360.0);
+		if (dx > small) {
+			*w = val;
+			(void) fprintf (stderr, "%s: GMT WARNING: (w - x_min) must equal (NX + eps) * x_inc), where NX is an integer and |eps| <= %g.\n", GMT_program, GMT_SMALL);
+			(void) fprintf (stderr, "%s: GMT WARNING: w reset to %g\n", GMT_program, *w);
+		}
+
+		val = header->x_min + irint((*e - header->x_min) / header->x_inc) * header->x_inc;
+		dx = fabs (*e - val);
+		if (GMT_io.in_col_type[0] == GMT_IS_LON) dx = fmod (dx, 360.0);
+		if (dx > GMT_SMALL) {
+			*e = val;
+			(void) fprintf (stderr, "%s: GMT WARNING: (e - x_min) must equal (NX + eps) * x_inc), where NX is an integer and |eps| <= %g.\n", GMT_program, GMT_SMALL);
+			(void) fprintf (stderr, "%s: GMT WARNING: e reset to %g\n", GMT_program, *e);
+		}
+	}
+
 	/* Check if s,n are a multiple of y_inc offset from y_min - if not adjust s, n */
 	small = GMT_SMALL * header->y_inc;
 
-	i_d = (*s - header->y_min) / header->y_inc;
-	if (i_d < 0) i_d = 0;
-	i = irint(i_d);
-	val = header->y_min + i * header->y_inc;
-
+	val = header->y_min + irint((*s - header->y_min) / header->y_inc) * header->y_inc;
 	if (fabs (*s - val) > small) {
 		*s = val;
 		(void) fprintf (stderr, "%s: GMT WARNING: (s - y_min) must equal (NY + eps) * y_inc), where NY is an integer and |eps| <= %g.\n", GMT_program, GMT_SMALL);
 		(void) fprintf (stderr, "%s: GMT WARNING: s reset to %g\n", GMT_program, *s);
 	}
 
-	i_d = (*n - header->y_min) / header->y_inc;
-	i = irint(i_d);
-	val = header->y_min + i * header->y_inc;
-
+	val = header->y_min + irint((*n - header->y_min) / header->y_inc) * header->y_inc;
 	if (fabs (*n - val) > small) {
 		*n = val;
 		(void) fprintf (stderr, "%s: GMT WARNING: (n - y_min) must equal (NY + eps) * y_inc), where NY is an integer and |eps| <= %g.\n", GMT_program, GMT_SMALL);
@@ -1011,7 +1031,7 @@ void GMT_grd_set_units (struct GRD_HEADER *header)
 	/* Set unit strings for grid coordinates x, y and z based on
 	   output data types for columns 0, 1, and 2.
 	*/
-	int i;
+	GMT_LONG i;
 	char *string[3], unit[GRD_UNIT_LEN], date[GMT_CALSTRING_LENGTH], clock[GMT_CALSTRING_LENGTH];
 
 	/* Copy pointers to unit strings */
@@ -1073,7 +1093,7 @@ void GMT_grd_get_units (struct GRD_HEADER *header)
 	   grid coordinates x, y and z.
 	   When "Time": transform the data scale and offset to match the current time system.
 	*/
-	int i;
+	GMT_LONG i;
 	char string[3][GRD_UNIT_LEN], *units;
 	double scale = 1.0, offset = 0.0;
 	struct GMT_TIME_SYSTEM time_system;
@@ -1098,12 +1118,12 @@ void GMT_grd_get_units (struct GRD_HEADER *header)
 		/* Change name of variable and unit to lower case for comparison */
 		GMT_str_tolower (string[i]);
 
-		if (!strncmp (string[i], "longitude", (size_t)9) || strstr (string[i], "degrees_e")) {
+		if ((!strncmp (string[i], "longitude", (size_t)9) || strstr (string[i], "degrees_e")) && (header->x_min > -360.0 && header->x_max <= 360.0)) {
 			/* Input data type is longitude */
 			GMT_io.in_col_type[i] = GMT_IS_LON;
 			project_info.degree[i] = TRUE;
 		}
-		else if (!strncmp (string[i], "latitude", (size_t)8) || strstr (string[i], "degrees_n")) {
+		else if ((!strncmp (string[i], "latitude", (size_t)8) || strstr (string[i], "degrees_n")) && (header->y_min >= -90.0 && header->y_max <= 90.0)) {
 			/* Input data type is latitude */
 			GMT_io.in_col_type[i] = GMT_IS_LAT;
 			project_info.degree[i] = TRUE;
@@ -1143,21 +1163,7 @@ void GMT_grd_get_units (struct GRD_HEADER *header)
 	}
 }
 
-#define GMT_IMG_MINLON		0.0
-#define GMT_IMG_MAXLON		360.0
-#define GMT_IMG_MINLAT_72	-72.0059773539
-#define GMT_IMG_MAXLAT_72	+72.0059773539
-#define GMT_IMG_MINLAT_80	-80.738
-#define GMT_IMG_MAXLAT_80	+80.738
-#define GMT_IMG_NLON_1M		21600	/* At 1 min resolution */
-#define GMT_IMG_NLON_2M		10800	/* At 2 min resolution */
-#define GMT_IMG_NLAT_1M_72	12672	/* At 1 min resolution */
-#define GMT_IMG_NLAT_1M_80	17280	/* At 1 min resolution */
-#define GMT_IMG_NLAT_2M_72	6336	/* At 1 min resolution */
-#define GMT_IMG_NLAT_2M_80	8640	/* At 1 min resolution */
-#define GMT_IMG_ITEMSIZE	2	/* Size of 2 byte short ints */
-
-int GMT_read_img (char *imgfile, struct GRD_HEADER *grd, float **grid, double w, double e, double s, double n, double scale, int mode, double lat, BOOLEAN init)
+GMT_LONG GMT_read_img (char *imgfile, struct GRD_HEADER *grd, float **grid, double w, double e, double s, double n, double scale, GMT_LONG mode, double lat, BOOLEAN init)
 {
 	/* Function that reads an entire Sandwell/Smith Mercator grid and stores it like a regular
 	 * GMT grid.  If init is TRUE we also initialize the Mercator projection.  Lat should be 0.0
@@ -1231,7 +1237,7 @@ int GMT_read_img (char *imgfile, struct GRD_HEADER *grd, float **grid, double w,
 	grd->nx = GMT_get_n (grd->x_min, grd->x_max, grd->x_inc, grd->node_offset);
 	grd->ny = GMT_get_n (grd->y_min, grd->y_max, grd->y_inc, grd->node_offset);
 	mx = grd->nx + GMT_pad[0] + GMT_pad[2];	my = grd->ny + GMT_pad[1] + GMT_pad[3];
-	*grid = (float *) GMT_memory (VNULL, (size_t)(mx * my), sizeof (float), GMT_program);
+	*grid = (float *) GMT_memory (VNULL, (mx * my), sizeof (float), GMT_program);
 	grd->xy_off = 0.5;
 
 	n_cols = (min == 1) ? GMT_IMG_NLON_1M : GMT_IMG_NLON_2M;		/* Number of columns (10800 or 21600) */
@@ -1240,10 +1246,10 @@ int GMT_read_img (char *imgfile, struct GRD_HEADER *grd, float **grid, double w,
 	n_skip = (GMT_LONG)floor ((project_info.ymax - grd->y_max) / grd->y_inc);	/* Number of rows clearly above y_max */
 	if (GMT_fseek (fp, (long)(n_skip * n_cols * GMT_IMG_ITEMSIZE), SEEK_SET)) return (GMT_GRDIO_SEEK_FAILED);
 
-	i2 = (short int *) GMT_memory (VNULL, (size_t)n_cols, sizeof (short int), GMT_program);
+	i2 = (short int *) GMT_memory (VNULL, n_cols, sizeof (short int), GMT_program);
 	for (j = 0; j < grd->ny; j++) {	/* Read all the rows, offset by 2 boundary rows and cols */
 		ij = (j + GMT_pad[3]) * mx + GMT_pad[0];
-		GMT_fread ((void *)i2, sizeof (short int), (size_t)n_cols, fp);
+		if (GMT_fread ((void *)i2, sizeof (short int), (size_t)n_cols, fp) != (size_t)n_cols)  return (GMT_GRDIO_READ_FAILED);	/* Get one row */
 		for (i = 0, k = first_i; i < grd->nx; i++) {	/* Process this row's values */
 			switch (mode) {
 				case 0:	/* No encoded track flags, do nothing */
@@ -1272,8 +1278,8 @@ struct GMT_GRID *GMT_create_grid (char *arg)
 {	/* Allocates space for a new grid container.  No space allocated for the float grid itself */
 	struct GMT_GRID * G;
 
-	G = (struct GMT_GRID *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMT_GRID), arg);
-	G->header = (struct GRD_HEADER *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRD_HEADER), arg);
+	G = (struct GMT_GRID *) GMT_memory (VNULL, (GMT_LONG)1, sizeof (struct GMT_GRID), arg);
+	G->header = (struct GRD_HEADER *) GMT_memory (VNULL, (GMT_LONG)1, sizeof (struct GRD_HEADER), arg);
 
 	return (G);
 }
