@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_vector.c,v 1.25 2009/02/06 23:00:08 guru Exp $
+ *	$Id: gmt_vector.c,v 1.32 2009/09/09 23:27:02 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 #define MAX_SWEEPS 50
 
-int	GMT_jacobi (double *a, GMT_LONG *n, GMT_LONG *m, double *d, double *v, double *b, double *z, int *nrots) {
+GMT_LONG	GMT_jacobi (double *a, GMT_LONG *n, GMT_LONG *m, double *d, double *v, double *b, double *z, GMT_LONG *nrots) {
 /*
  *
  * Find eigenvalues & eigenvectors of a square symmetric matrix by Jacobi's
@@ -280,7 +280,7 @@ int	GMT_jacobi (double *a, GMT_LONG *n, GMT_LONG *m, double *d, double *v, doubl
 }
 
 #ifdef OBSOLETE
-int	GMT_jacobi_old (double *a, int *n, int *m, double *d, double *v, double *b, double *z, int *nrots)
+GMT_LONG	GMT_jacobi_old (double *a, GMT_LONG *n, GMT_LONG *m, double *d, double *v, double *b, double *z, GMT_LONG *nrots)
 /*
  *
  * Find eigenvalues & eigenvectors of a square symmetric matrix by Jacobi's
@@ -309,7 +309,7 @@ int	GMT_jacobi_old (double *a, int *n, int *m, double *d, double *v, double *b,
    	       	/* Returned.  number of Givens rotations performed.  */
 
 {
-	int	ip, iq, nsweeps, i, j, k;
+	GMT_LONG	ip, iq, nsweeps, i, j, k;
 	double	sum, threshold, g, h, t, theta, c, s, tau, p;
 
 
@@ -470,7 +470,7 @@ int	GMT_jacobi_old (double *a, int *n, int *m, double *d, double *v, double *b,
 }
 #endif
 
-void GMT_gauss (double *a, double *vec, int n_in, int nstore_in, double test, int *ierror, int itriag)
+void GMT_gauss (double *a, double *vec, GMT_LONG n_in, GMT_LONG nstore_in, double test, GMT_LONG *ierror, GMT_LONG itriag)
 {
  
 /* subroutine gauss, by william menke */
@@ -486,16 +486,16 @@ void GMT_gauss (double *a, double *vec, int n_in, int nstore_in, double test, in
 /*      itriag          (sent)                  matrix triangularized only*/
 /*                                               on TRUE useful when solving*/
 /*                                               multiple systems with same a */
-        static int l1;
-        GMT_LONG *line, *isub, iet, ieb, i = 0, j, k, l, j2, n, nstore;
+        static GMT_LONG l1;
+        GMT_LONG *line, *isub, i = 0, j, k, l, j2, n, nstore;
+	GMT_LONG iet, ieb;
         double big, testa, b, sum;
 
         iet=0;  /* initial error flags, one for triagularization*/
         ieb=0;  /* one for backsolving */
-	n = (GMT_LONG)n_in;
-	nstore = (GMT_LONG)nstore_in;
-	line = (GMT_LONG *) GMT_memory (VNULL, (size_t)n, sizeof (GMT_LONG), "GMT_gauss");
-	isub = (GMT_LONG *) GMT_memory (VNULL, (size_t)n, sizeof (GMT_LONG), "GMT_gauss");
+	n = n_in;
+	nstore = nstore_in;
+	(void)GMT_alloc_memory2 ((void **)&line, (void **)&isub, n, 0, sizeof (GMT_LONG), "GMT_gauss");
 
 /* triangularize the matrix a*/
 /* replacing the zero elements of the triangularized matrix */
@@ -647,41 +647,41 @@ void GMT_cross3v (double *a, double *b, double *c)
 	c[2] = a[0] * b[1] - a[1] * b[0];
 }
 
-void GMT_geo_to_cart (double *alat, double *alon, double *a, int rads)
+void GMT_geo_to_cart (double lat, double lon, double *a, BOOLEAN degrees)
 {
-	/* Convert geographic latitude and longitude (alat, alon)
-	   to a 3-vector of unit length (a).  rads = TRUE if we
-	   need to convert alat, alon from degrees to radians  */
+	/* Convert geographic latitude and longitude (lat, lon)
+	   to a 3-vector of unit length (a). If degrees = TRUE,
+	   input coordinates are in degrees, otherwise in radian */
 
 	double clat, clon, slon;
 
-	if (rads) {
-		*alat *= D2R;
-		*alon *= D2R;
+	if (degrees) {
+		lat *= D2R;
+		lon *= D2R;
 	}
-	sincos (*alat, &a[2], &clat);
-	sincos (*alon, &slon, &clon);
+	sincos (lat, &a[2], &clat);
+	sincos (lon, &slon, &clon);
 	a[0] = clat * clon;
 	a[1] = clat * slon;
 }
 
-void GMT_cart_to_geo (double *alat, double *alon, double *a, int rads)
+void GMT_cart_to_geo (double *lat, double *lon, double *a, BOOLEAN degrees)
 {
 	/* Convert a 3-vector (a) of unit length into geographic
-	   coordinates (alat, alon).  rads = TRUE if we want the
-	   lat and lon converted from radians into degrees.  */
+	   coordinates (lat, lon). If degrees = TRUE, the output coordinates
+	   are in degrees, otherwise in radian. */
 
-	if(rads) {
-		*alat = R2D * d_asin(a[2]);
-		*alon = R2D * d_atan2(a[1], a[0]);
+	if (degrees) {
+		*lat = d_asind (a[2]);
+		*lon = d_atan2d (a[1], a[0]);
 	}
 	else {
-		*alat = d_asin(a[2]);
-		*alon = d_atan2(a[1],a[0]);
+		*lat = d_asin (a[2]);
+		*lon = d_atan2 (a[1], a[0]);
 	}
 }
 
-GMT_LONG GMT_fix_up_path (double **a_lon, double **a_lat, GMT_LONG n, double step, int mode)
+GMT_LONG GMT_fix_up_path (double **a_lon, double **a_lat, GMT_LONG n, double step, GMT_LONG mode)
 {
 	/* Takes pointers to a list of <n> lon/lat pairs (in degrees) and adds
 	 * auxiliary points if the great circle distance between two given points exceeds
@@ -697,7 +697,7 @@ GMT_LONG GMT_fix_up_path (double **a_lon, double **a_lat, GMT_LONG n, double ste
 	 * inches of map projection.
 	 */
       
-	GMT_LONG i, j, n_tmp, n_step = 0, n_alloc;
+	GMT_LONG i, j, n_tmp, n_step = 0, n_alloc = 0;
 	BOOLEAN meridian;
 	double *lon_tmp, *lat_tmp, *old;
 	double a[3], b[3], x[3], *lon, *lat;
@@ -706,83 +706,64 @@ GMT_LONG GMT_fix_up_path (double **a_lon, double **a_lat, GMT_LONG n, double ste
 	lon = *a_lon;
 	lat = *a_lat;
 	
-	n_alloc = n;
-	lon_tmp = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-	lat_tmp = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-	
-	GMT_geo_to_cart (&lat[0], &lon[0], a, TRUE);
+	GMT_geo_to_cart (lat[0], lon[0], a, TRUE);
+	n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, 1, 0, sizeof (double), "GMT_fix_up_path");
 	lon_tmp[0] = lon[0];	lat_tmp[0] = lat[0];
 	n_tmp = 1;
 	if (step <= 0.0) step = 1.0;
 	
 	for (i = 1; i < n; i++) {
 		
-		GMT_geo_to_cart (&lat[i],&lon[i], b, TRUE);
+		GMT_geo_to_cart (lat[i], lon[i], b, TRUE);
 
 		if (mode == 1) {	/* First follow meridian, then parallel */
-			theta = fabs(lon[i]-lon[i-1]) * cos(lat[i-1]);
-			n_step = irint (theta * R2D / step);
+			theta = fabs(lon[i]-lon[i-1]) * cosd(lat[i-1]);
+			n_step = irint (theta / step);
 			for (j = 1; j < n_step; j++) {
 				c = j / (double)n_step;
+				if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
 				lon_tmp[n_tmp] = lon[i-1] * (1 - c) + lon[i] * c;
 				lat_tmp[n_tmp] = lat[i-1];
 				n_tmp++;
-				if (n_tmp == n_alloc) {
-					n_alloc <<= 1;
-					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-				}
 			}
 			theta = fabs(lat[i]-lat[i-1]);
-			n_step = irint (theta * R2D / step);
+			n_step = irint (theta / step);
 			for (j = 0; j < n_step; j++) {	/* Start at 0 to make sure corner point is saved */
 				c = j / (double)n_step;
+				if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
 				lon_tmp[n_tmp] = lon[i];
 				lat_tmp[n_tmp] = lat[i-1] * (1 - c) + lat[i] * c;
 				n_tmp++;
-				if (n_tmp == n_alloc) {
-					n_alloc <<= 1;
-					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-				}
 			}
 		}
 
 		else if (mode == 2) {	/* First follow parallel, then meridian */
 			theta = fabs(lat[i]-lat[i-1]);
-			n_step = irint (theta * R2D / step);
+			n_step = irint (theta / step);
 			for (j = 1; j < n_step; j++) {
 				c = j / (double)n_step;
+				if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
 				lon_tmp[n_tmp] = lon[i-1];
 				lat_tmp[n_tmp] = lat[i-1] * (1 - c) + lat[i] * c;
 				n_tmp++;
-				if (n_tmp == n_alloc) {
-					n_alloc <<= 1;
-					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-				}
 			}
-			theta = fabs(lon[i]-lon[i-1]) * cos(lat[i]);
-			n_step = irint (theta * R2D / step);
+			theta = fabs(lon[i]-lon[i-1]) * cosd(lat[i]);
+			n_step = irint (theta / step);
 			for (j = 0; j < n_step; j++) {	/* Start at 0 to make sure corner point is saved */
 				c = j / (double)n_step;
+				if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
 				lon_tmp[n_tmp] = lon[i-1] * (1 - c) + lon[i] * c;
 				lat_tmp[n_tmp] = lat[i];
 				n_tmp++;
-				if (n_tmp == n_alloc) {
-					n_alloc <<= 1;
-					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-				}
 			}
 		}
 
 		/* Follow great circle */
-		else if ((theta = d_acos (GMT_dot3v (a, b))) == M_PI) {	/* trouble, no unique great circle */
+		else if ((theta = d_acosd (GMT_dot3v (a, b))) == 180.0) {	/* trouble, no unique great circle */
 			if (gmtdefs.verbose) fprintf (stderr, "%s: GMT Warning: Two points in input list are antipodal - no resampling taken place!\n", GMT_program);
 		}
 
-		else if ((n_step = irint (theta * R2D / step)) > 1) {	/* Must insert (n_step - 1) points, i.e. create n_step intervals */
+		else if ((n_step = irint (theta / step)) > 1) {	/* Must insert (n_step - 1) points, i.e. create n_step intervals */
 			fraction = 1.0 / (float) n_step;
 			minlon = MIN(lon[i-1],lon[i]);
 			maxlon = MAX(lon[i-1],lon[i]);
@@ -794,32 +775,23 @@ GMT_LONG GMT_fix_up_path (double **a_lon, double **a_lat, GMT_LONG n, double ste
 				x[1] = a[1] * d + b[1] * c;
 				x[2] = a[2] * d + b[2] * c;
 				GMT_normalize3v (x);
-				GMT_cart_to_geo (&lat_tmp[n_tmp], &lon_tmp[n_tmp], x, FALSE);
+				if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
+				GMT_cart_to_geo (&lat_tmp[n_tmp], &lon_tmp[n_tmp], x, TRUE);
 				if (meridian)
 					lon_tmp[n_tmp] = minlon;
 				else if (lon_tmp[n_tmp] < minlon)
-					lon_tmp[n_tmp] += TWO_PI;
+					lon_tmp[n_tmp] += 360.0;
 				else if (lon_tmp[n_tmp] > maxlon)
-					lon_tmp[n_tmp] -= TWO_PI;
+					lon_tmp[n_tmp] -= 360.0;
 				n_tmp++;
-				if (n_tmp == n_alloc) {
-					n_alloc <<= 1;
-					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-				}
 			}
 		}
+		if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
 		lon_tmp[n_tmp] = lon[i];	lat_tmp[n_tmp] = lat[i];
 		n_tmp++;
-		if (n_tmp == n_alloc) {
-			n_alloc <<= 1;
-			lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-			lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-		}
 		a[0] = b[0];	a[1] = b[1];	a[2] = b[2];
 	}
-	lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_tmp, sizeof (double), "GMT_fix_up_path");
-	lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_tmp, sizeof (double), "GMT_fix_up_path");
+	n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, 0, n_tmp, sizeof (double), "GMT_fix_up_path");
 	
 	old = lon;
 	lon = lon_tmp;
@@ -827,16 +799,12 @@ GMT_LONG GMT_fix_up_path (double **a_lon, double **a_lat, GMT_LONG n, double ste
 	old = lat;
 	lat = lat_tmp;
 	GMT_free ((void *) old);
-	for (i = 0; i < n_tmp; i++) {
-		lon[i] *= R2D;
-		lat[i] *= R2D;
-	}
 	*a_lon = lon;
 	*a_lat = lat;
 	return (n_tmp);
 }		
 
-int GMT_chol_dcmp (double *a, double *d, double *cond, GMT_LONG nr, GMT_LONG n) {
+GMT_LONG GMT_chol_dcmp (double *a, double *d, double *cond, GMT_LONG nr, GMT_LONG n) {
 
 	/* Given a, a symmetric positive definite matrix
 	of size n, and row dimension nr, compute a lower
@@ -892,7 +860,7 @@ int GMT_chol_dcmp (double *a, double *d, double *cond, GMT_LONG nr, GMT_LONG n)
 	return (0);
 }
 
-void GMT_chol_recover (double *a, double *d, GMT_LONG nr, GMT_LONG n, int nerr, int donly) {
+void GMT_chol_recover (double *a, double *d, GMT_LONG nr, GMT_LONG n, GMT_LONG nerr, GMT_LONG donly) {
 
 	/* Given a, a symmetric positive definite matrix of row dimension nr,
 	and size n >= abs(nerr), one uses GMT_chol_dcmp() to attempt to find
@@ -929,7 +897,7 @@ void GMT_chol_recover (double *a, double *d, GMT_LONG nr, GMT_LONG n, int nerr,
 	
 	GMT_LONG	kbad, i, j, ii, ij, ji, nrp1;
 	
-	kbad = abs(nerr) - 1;
+	kbad = GMT_abs(nerr) - 1;
 	nrp1 = nr + 1;
 	
 	for (i = 0, ii = 0; i <= kbad; i++, ii += nrp1) {
