@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtconvert.c,v 1.54 2009/01/09 04:02:33 guru Exp $
+ *	$Id: gmtconvert.c,v 1.66 2009/09/09 23:27:02 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -41,11 +41,11 @@ struct GMTCONVERT_CTRL {
 	} D;
 	struct E {	/* -E */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 	} E;
 	struct F {	/* -F<cols> */
 		BOOLEAN active;
-		int *col;
+		GMT_LONG *col;
 	} F;
 	struct L {	/* -L */
 		BOOLEAN active;
@@ -56,24 +56,24 @@ struct GMTCONVERT_CTRL {
 	struct N {	/* -N */
 		BOOLEAN active;
 	} N;
-	struct S {	/* -S\"search string\" */
+	struct S {	/* -S[~]\"search string\" */
 		BOOLEAN active;
+		BOOLEAN inverse;
 		char *pattern;
 	} S;
 };
 
 int main (int argc, char **argv)
 {
-	int pos, start, stop, fno, n_files = 0, n_out = 0, n_tot_cols = 0;
-	
-	GMT_LONG i, j, k, n_total_read = 0, n_hdr_delay = 0;
+	GMT_LONG pos, start, stop, fno, n_files = 0, n_out = 0, n_tot_cols = 0;
+	GMT_LONG i, j, k, n_total_read = 0, n_hdr_delay = 0, n_out_seg = 0;
 	GMT_LONG n_rev_alloc = 0, n_rev = 0, n_rev_recs = 0, error = 0, n_seg = 0, n_in_seg = 0, n_nan, out_seg;
 	
-	BOOLEAN on, ascii_in_bin_out;
+	BOOLEAN on, ascii_in_bin_out, match = FALSE;
 
 	double	*in, *out = VNULL, *R_out = (double *)NULL;
 
-	char record_str[BUFSIZ], ptr[BUFSIZ], header[BUFSIZ], filename[BUFSIZ];
+	char record_str[BUFSIZ], ptr[BUFSIZ], header[BUFSIZ], filename[BUFSIZ], *not_used = NULL;
 	
 	FILE *fp_out;
 
@@ -93,9 +93,12 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'H':
+				case 'M':
 				case 'V':
 				case 'b':
 				case 'f':
+				case 'g':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
@@ -124,7 +127,7 @@ int main (int argc, char **argv)
 					Ctrl->F.active = TRUE;
 					while ((GMT_strtok (&argv[i][2], ",", &pos, ptr))) {	/* Process all tokens */
 						if (strchr (ptr, '-'))	/* Range of columns given. e.g., 7-9 */
-							sscanf (ptr, "%d-%d", &start, &stop);
+							sscanf (ptr, "%ld-%ld", &start, &stop);
 						else if (isdigit ((int)ptr[0]))	/* Just a single column, e.g., 13 */
 							start = stop = atoi (ptr);
 						else {				/* Badness! */
@@ -141,15 +144,14 @@ int main (int argc, char **argv)
 				case 'I':
 					Ctrl->I.active = TRUE;
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
 					break;
 				case 'S':
-					Ctrl->S.pattern = strdup (&argv[i][2]);
 					Ctrl->S.active = TRUE;
+					k = (argv[i][2] == '\\' && strlen (argv[i]) > 3 && argv[i][3] == '~') ? 3 : 2;	/* Special escape if pattern starts with ~ */
+					if (argv[i][2] == '~') Ctrl->S.inverse = TRUE;
+					Ctrl->S.pattern = strdup (&argv[i][k+Ctrl->S.inverse]);
 					break;
 				default:
 					error = TRUE;
@@ -164,7 +166,7 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "gmtconvert %s - Convert format, Paste, and/or Extract columns from table data\n\n", GMT_VERSION);
 		fprintf(stderr,"usage:	gmtconvert [files] [-A] [-D[<template>]] [-E[f|l]] [-F<cols>] [%s] [-I]\n", GMT_H_OPT);
-		fprintf(stderr,"\t[-L] [%s] [-N] [-S\"search string\"] [-V] [%s]\n\t[%s] [%s]\n\n", GMT_M_OPT, GMT_t_OPT, GMT_b_OPT, GMT_f_OPT);
+		fprintf(stderr,"\t[-L] [-N] [-S[~]\"search string\"] [-V] [%s]\n\t[%s] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_b_OPT, GMT_f_OPT, GMT_g_OPT, GMT_m_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -178,16 +180,19 @@ int main (int argc, char **argv)
 		fprintf(stderr,"\t-F Give comma-separated list of desired columns or ranges (0 is first column) [Default is all]\n");
 		GMT_explain_option ('H');
 		fprintf (stderr,"\t-I Invert order of rows, i.e., outputting in reverse order\n");
-		fprintf (stderr, "\t-L Output all multisegment headers only, no data records (requires -M and ASCII data)\n");
-		GMT_explain_option ('M');
+		fprintf (stderr, "\t-L Output all multisegment headers only, no data records (requires -m and ASCII data)\n");
 		fprintf (stderr, "\t-N Skip output records where all fields == NaN [Default writes all records]\n");
 		fprintf (stderr, "\t-S Only output segments whose segment headers contains the pattern \"string\".\n");
-		fprintf (stderr, "\t   (Requires -M and ASCII data) [All segments]\n");
+		fprintf (stderr, "\t   Use -S~\"string\" to output segment that DO NOT contain this pattern.\n");
+		fprintf (stderr, "\t   If your pattern begins with ~, escape it with \\~.\n");
+		fprintf (stderr, "\t   (-S requires -m and ASCII data) [All segments]\n");
 		GMT_explain_option ('V');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		GMT_explain_option ('o');
 		GMT_explain_option ('f');
+		GMT_explain_option ('g');
+		GMT_explain_option ('m');
 		exit (EXIT_FAILURE);
 	}
 
@@ -211,7 +216,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -223,15 +228,20 @@ int main (int argc, char **argv)
 	if (Ctrl->D.active && !Ctrl->D.name) Ctrl->D.name = strdup ("gmtconvert_segment_%d.d");
 	out_seg = -1;
 
+	/* When NAN_RECORDS = pass and no multi-segment output, turn off default NaN handling */
+	if (gmtdefs.nan_is_gap && !GMT_io.multi_segments[GMT_OUT]) {
+		for (i = 0; i < 2; i++) GMT_io.skip_if_NaN[i] = FALSE;
+	}
+
 	if (Ctrl->A.active) {	/* Do paste and probably cut as well */
-		int *n_expected_fields, *n_fields, all_set = 0;
+		GMT_LONG *n_expected_fields, *n_fields, all_set = 0;
 		GMT_LONG n_alloc = GMT_SMALL_CHUNK;
 		double *val;
 		FILE **fp = NULL;
 
 		fp = (FILE **) GMT_memory (VNULL, (size_t)n_files, sizeof (FILE *), GMT_program);
-		n_expected_fields = (int *) GMT_memory (VNULL, (size_t)n_files, sizeof (int), GMT_program);
-		n_fields = (int *) GMT_memory (VNULL, (size_t)n_files, sizeof (int), GMT_program);
+		n_expected_fields = (GMT_LONG *) GMT_memory (VNULL, n_files, sizeof (GMT_LONG), GMT_program);
+		n_fields = (GMT_LONG *) GMT_memory (VNULL, n_files, sizeof (GMT_LONG), GMT_program);
 
 		for (fno = 1, k = 0; fno < argc; fno++) {	/* Loop over input files, if any */
 			if (argv[fno][0] == '-') continue;
@@ -260,7 +270,7 @@ int main (int argc, char **argv)
 		if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {	/* Only ascii data can have header records */
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
 				for (k = 0; k < n_files; k++) {
-					GMT_fgets (record_str, BUFSIZ, fp[k]);
+					not_used = GMT_fgets (record_str, BUFSIZ, fp[k]);
 					if (k < (n_files-1)) record_str[strlen(record_str)-1] = '\0';
 					if (k == 0)
 						strcpy (header, record_str);
@@ -301,19 +311,19 @@ int main (int argc, char **argv)
 			}
 			out = (double *) GMT_memory (VNULL, sizeof (double), (size_t)n_out, GMT_program);
 			if (n_tot_cols < n_out) {
-				fprintf (stderr, "%s: Columns requested (%d) exceed actual (%d) - aborting\n", GMT_program, n_out, n_tot_cols);
+				fprintf (stderr, "%s: Columns requested (%ld) exceed actual (%ld) - aborting\n", GMT_program, n_out, n_tot_cols);
 				exit (EXIT_FAILURE);
 			}
 		}
-		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+		while (!GMT_REC_IS_EOF) {	/* Not yet EOF */
 
-			while (GMT_io.status & GMT_IO_SEGMENT_HEADER && !(GMT_io.status & GMT_IO_EOF)) {	/* As long as we have read a segment header */
+			while (GMT_REC_IS_NEW_SEGMENT && !GMT_REC_IS_EOF) {	/* As long as we have read a segment header */
 				strcpy (GMT_io.segment_header, header);
 				if (ascii_in_bin_out && n_out == 0)
 					n_hdr_delay++;
 				else if (Ctrl->D.active) {
 					out_seg++;
-					sprintf (filename, Ctrl->D.name, out_seg);
+					sprintf (filename, Ctrl->D.name, (int)out_seg);
 					if (fp_out != NULL) GMT_fclose (fp_out);
 					if ((fp_out = GMT_fopen (filename, GMT_io.w_mode)) == NULL ) {
 						fprintf (stderr, "%s: Error creating file %s\n", GMT_program, filename);
@@ -324,7 +334,7 @@ int main (int argc, char **argv)
 					GMT_write_segmentheader (fp_out, n_out);
 				for (k = 0; k < n_files; k++) {	/* Get next record */
 					n_fields[k] = GMT_input (fp[k],  &n_expected_fields[k], &in);
-					if (! GMT_io.status & GMT_IO_SEGMENT_HEADER) {	/* Data record */
+					if (!GMT_REC_IS_NEW_SEGMENT) {	/* Data record */
 						if (! all_set) {	/* Very first data record */
 							all_set = 1;
 							n_tot_cols += n_expected_fields[k];
@@ -358,13 +368,13 @@ int main (int argc, char **argv)
 				}
 				out = (double *) GMT_memory (VNULL, (size_t)n_out, sizeof (double), GMT_program);
 				if (n_tot_cols < n_out) {
-					fprintf (stderr, "%s: Columns requested (%d) exceed actual (%d) - aborting\n", GMT_program, n_out, n_tot_cols);
+					fprintf (stderr, "%s: Columns requested (%ld) exceed actual (%ld) - aborting\n", GMT_program, n_out, n_tot_cols);
 					exit (EXIT_FAILURE);
 				}
 			}
-			while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
-				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields[0], n_expected_fields[0], n_total_read);
+			while (!GMT_REC_IS_NEW_SEGMENT && !GMT_REC_IS_EOF) {	/* Keep going until FALSE or = 2 segment header */
+				if (GMT_REC_IS_ERROR) {
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields[0], n_expected_fields[0], n_total_read);
 					exit (EXIT_FAILURE);
 				}
 
@@ -390,7 +400,7 @@ int main (int argc, char **argv)
 
 				for (k = j = 0; k < n_files; k++) {	/* Get next record */
 					n_fields[k] = GMT_input (fp[k], &n_expected_fields[k], &in);
-					if (GMT_io.status & GMT_IO_SEGMENT_HEADER) {	/* ANother segment header */
+					if (GMT_REC_IS_SEG_HEADER) {	/* Another segment header */
 						if (k < (n_files-1)) GMT_io.segment_header[strlen(GMT_io.segment_header)-1] = '\0';
 						if (k == 0)
 							strcpy (header, GMT_io.segment_header);
@@ -406,10 +416,10 @@ int main (int argc, char **argv)
 
 		for (k = 0; k < n_files; k++) GMT_fclose(fp[k]);
 
-		if (gmtdefs.verbose) fprintf(stderr, "gmtconvert: %ld records passed (input cols = %d; output cols = %d)\n", n_total_read, n_tot_cols, n_out);
+		if (gmtdefs.verbose) fprintf(stderr, "gmtconvert: %ld records passed (input cols = %ld; output cols = %ld)\n", n_total_read, n_tot_cols, n_out);
 	}
 	else {	/* Just cutting out columns */
-		int n_expected_fields, n_fields, n_args;
+		GMT_LONG n_expected_fields, n_fields, n_args;
 		BOOLEAN nofile = TRUE, done = FALSE;
 		FILE *fp = NULL;
 
@@ -447,7 +457,7 @@ int main (int argc, char **argv)
 
 			if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
 				for (i = 0; i < GMT_io.n_header_recs; i++) {
-					GMT_fgets (record_str, BUFSIZ, fp);
+					not_used = GMT_fgets (record_str, BUFSIZ, fp);
 					n_total_read ++;
 					if (!GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) fputs (record_str, GMT_stdout);
 				}
@@ -455,18 +465,21 @@ int main (int argc, char **argv)
 
 			n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ; /* BUFSIZ -> Ascii table, must first find the number of fields */
 			n_fields = GMT_input (fp, &n_expected_fields, &in);
-
+			if (GMT_IO_GAP_CHECKING) GMT_write_segmentheader (GMT_stdout, n_out);
 			n_seg = n_in_seg = 0;
-			while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+			while (!GMT_REC_IS_EOF) {	/* Not yet EOF */
 
-				while (GMT_io.status & GMT_IO_SEGMENT_HEADER && !(GMT_io.status & GMT_IO_EOF)) {
-					if (Ctrl->E.active && (Ctrl->E.mode & 2) && n_in_seg > 0) fprintf (GMT_stdout, "%s", record_str);	/* Previous segments last point */
-					if (Ctrl->S.active) on = (strstr (GMT_io.segment_header, Ctrl->S.pattern) != NULL);	/* If match then we turn output ON */
+				while (GMT_REC_IS_NEW_SEGMENT && !GMT_REC_IS_EOF) {
+					if (Ctrl->E.active && (Ctrl->E.mode & 2) && n_in_seg > 0) fprintf (GMT_stdout, "%s", record_str);		/* Previous segments last point */
+					if (Ctrl->S.active) {	/* See if this header has text matching our search string */
+						match = !strstr (GMT_io.segment_header, Ctrl->S.pattern);
+						on = (Ctrl->S.inverse == !match);	/* If match then we turn output ON */
+					}
 					if (ascii_in_bin_out && n_out == 0)
 						n_hdr_delay++;	/* Unable to write header yet since we dont know the field count... */
 					else if ((Ctrl->L.active || on) && Ctrl->D.active) {
 						out_seg++;
-						sprintf (filename, Ctrl->D.name, out_seg);
+						sprintf (filename, Ctrl->D.name, (int)out_seg);
 						if (fp_out != NULL) GMT_fclose (fp_out);
 						if ((fp_out = GMT_fopen (filename, GMT_io.w_mode)) == NULL ) {
 							fprintf (stderr, "%s: Error creating file %s\n", GMT_program, filename);
@@ -476,14 +489,19 @@ int main (int argc, char **argv)
 					else if (Ctrl->L.active || on)
 						GMT_write_segmentheader (GMT_stdout, n_out);
 					n_seg++;
+					if (on) n_out_seg++;
 					n_in_seg = 0;
 					n_fields = GMT_input (fp,  &n_expected_fields, &in);
 					n_total_read ++;
 				}
+				if (GMT_REC_IS_GAP) {
+					GMT_write_segmentheader (GMT_stdout, n_out);
+					GMT_io.status = 0;	/* Done with gap */
+				}
 
-				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
-					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_total_read);
+				while (!GMT_REC_IS_LINE_BREAK && !GMT_REC_IS_EOF) {	/* Keep going until FALSE or = 2 segment header */
+					if (GMT_REC_IS_ERROR) {
+						fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_total_read);
 						exit (EXIT_FAILURE);
 					}
 					if (n_out == 0) {
@@ -496,7 +514,7 @@ int main (int argc, char **argv)
 						out = (double *) GMT_memory (VNULL, (size_t)n_out, sizeof (double), GMT_program);
 					}
 					if (n_expected_fields < n_out) {
-						fprintf (stderr, "%s: Columns requested (%d) exceed actual (%d) - aborting\n", GMT_program, n_expected_fields, n_out);
+						fprintf (stderr, "%s: Columns requested (%ld) exceed actual (%ld) - aborting\n", GMT_program, n_expected_fields, n_out);
 						exit (EXIT_FAILURE);
 					}
 
@@ -534,8 +552,8 @@ int main (int argc, char **argv)
 
 			if (fp != GMT_stdin) GMT_fclose(fp);
 		}
-		if (gmtdefs.verbose) fprintf(stderr, "%s: %ld records passed (input cols = %d; output cols = %d)\n", GMT_program, n_total_read, n_expected_fields, n_out);
-		if (gmtdefs.verbose && Ctrl->S.active) fprintf (stderr, "%s: Extracted %ld segments\n", argv[0], n_seg);
+		if (gmtdefs.verbose) fprintf(stderr, "%s: %ld records passed (input cols = %ld; output cols = %ld)\n", GMT_program, n_total_read, n_expected_fields, n_out);
+		if (gmtdefs.verbose && Ctrl->S.active) fprintf (stderr, "%s: Extracted %ld from a total of %ld segments\n", argv[0], n_out_seg, n_seg);
 	}
 
 	if (Ctrl->I.active) {	/* Time to write out rows in reverse order */
@@ -561,7 +579,7 @@ void *New_Gmtconvert_Ctrl () {	/* Allocate and initialize a new control structur
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
-	C->F.col = (int *) GMT_memory (VNULL, (size_t)GMTCONVERT_MAX_COL, sizeof (int), GMT_program);
+	C->F.col = (GMT_LONG *) GMT_memory (VNULL, (size_t)GMTCONVERT_MAX_COL, sizeof (GMT_LONG), GMT_program);
 	
 	return ((void *)C);
 }
