@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtdefaults.c,v 1.27 2008/03/24 08:58:31 guru Exp $
+ *	$Id: gmtdefaults.c,v 1.31 2009/09/09 23:27:03 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 
 int main (int argc, char **argv)
 {
-	int i, get = 0;
+	GMT_LONG i, get = 0;
 
 	BOOLEAN error = FALSE, get_sys_defaults = FALSE, get_user_defaults = FALSE;
 
@@ -103,7 +103,7 @@ int main (int argc, char **argv)
 	else {
 		GMT_getdefpath (get, &path);
 		GMT_getdefaults (path);
-		GMT_free (path);
+		free (path);
 	}
 
 	GMT_putdefaults ("-");
