@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grd2cpt.c,v 1.54 2009/01/09 04:02:33 guru Exp $
+ *	$Id: grd2cpt.c,v 1.56 2009/09/09 23:27:03 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -67,7 +67,7 @@ struct GRD2CPT_CTRL {
 	} N;
 	struct Q {	/* -Q[i|o */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 	} Q;
 	struct S {	/* -S<z_start>/<z_stop>/<z_inc> */
 		BOOLEAN active;
@@ -77,7 +77,7 @@ struct GRD2CPT_CTRL {
 	} S;
 	struct T {	/* -T<kind> */
 		BOOLEAN active;
-		int kind; /* -1 symmetric +-zmin, +1 +-zmax, -2 = +-Minx(|zmin|,|zmax|), +2 = +-Max(|zmin|,|zmax|), 0 = min to max [Default] */
+		GMT_LONG kind; /* -1 symmetric +-zmin, +1 +-zmax, -2 = +-Minx(|zmin|,|zmax|), +2 = +-Max(|zmin|,|zmax|), 0 = min to max [Default] */
 	} T;
 	struct Z {	/* -Z */
 		BOOLEAN active;
@@ -92,9 +92,7 @@ int main (int argc, char **argv)
 
 	char *grdfile[mgrd], CPT_lis[BUFSIZ], CPT_file[BUFSIZ], format[BUFSIZ], kind;
 
-	int j, ngrd = 0;
-	
-	GMT_LONG i, nxy, nxyg, nfound, ngood;
+	GMT_LONG j, ngrd = 0, i, nxy, nxyg, nfound, ngood;
 
 	float	*zdata;
 
