@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grd2xyz.c,v 1.52 2008/05/08 07:41:13 guru Exp $
+ *	$Id: grd2xyz.c,v 1.60 2009/09/09 23:27:03 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@ int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, b_only = FALSE, first = TRUE;
 
-	int i, j, k, nx, ny, n_files = 0, n_out;
+	GMT_LONG i, j, k, nx, ny, n_files = 0, n_out;
 	GMT_LONG ij, nm, gmt_ij, n_suppressed = 0, n_total = 0;
 
 	float *z;
@@ -203,31 +203,19 @@ int main (int argc, char **argv)
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[k]);
 
-		nm = ((size_t)grd.nx) * ((size_t)grd.ny);
-		n_total += nm;
-
 		if (e > w && n > s) {	/* Subset */
-			if (fabs (grd.x_max - grd.x_min) < 360.0) {
-				if (w < grd.x_min) { w = grd.x_min; error = TRUE; }
-				if (e > grd.x_min) { e = grd.x_max; error = TRUE; }
-			}
-			if (s < grd.y_min) { s = grd.y_min; error = TRUE; }
-			if (n > grd.y_max) { n = grd.y_max; error = TRUE; }
-			if (error) fprintf (stderr, "%s: Warning: Subset exceeds data domain. Subset reduced to common region.\n", GMT_program);
-			error = FALSE;
 			GMT_err_fail (GMT_adjust_loose_wesn (&w, &e, &s, &n, &grd), "");	/* Make sure w,e,s,n matches header spacing */
 			nx = GMT_get_n (w, e, grd.x_inc, grd.node_offset);
 			ny = GMT_get_n (s, n, grd.y_inc, grd.node_offset);
-
-			z = (float *) GMT_memory (VNULL, (size_t) ((size_t)nx * (size_t)ny), sizeof (float), GMT_program);
-
-			GMT_err_fail (GMT_read_grd (argv[k], &grd, z, w, e, s, n, GMT_pad, FALSE), argv[k]);
+			nm = GMT_get_nm (nx, ny);
 		}
-		else {
-			z = (float *) GMT_memory (VNULL, (size_t) nm, sizeof (float), GMT_program);
+		else
+			nm = GMT_get_nm (grd.nx, grd.ny);
 
-			GMT_err_fail (GMT_read_grd (argv[k], &grd, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), argv[k]);
-		}
+		z = (float *) GMT_memory (VNULL, (size_t) nm, sizeof (float), GMT_program);
+		GMT_err_fail (GMT_read_grd (argv[k], &grd, z, w, e, s, n, GMT_pad, FALSE), argv[k]);
+
+		n_total += nm;
 
 		GMT_err_fail (GMT_set_z_io (&io, &grd), argv[k]);
 
@@ -294,7 +282,7 @@ int main (int argc, char **argv)
 				if (!grd.x_units[0]) strcpy (grd.x_units, "x");
 				if (!grd.y_units[0]) strcpy (grd.y_units, "y");
 				if (!grd.z_units[0]) strcpy (grd.z_units, "z");
-				if (gmtdefs.xy_toggle[0])
+				if (gmtdefs.xy_toggle[GMT_IN])
 					fprintf (GMT_stdout, "%s\t%s\t%s", grd.y_units, grd.x_units, grd.z_units);
 				else
 					fprintf (GMT_stdout, "%s\t%s\t%s", grd.x_units, grd.y_units, grd.z_units);
@@ -311,7 +299,7 @@ int main (int argc, char **argv)
 					n_suppressed++;
 					continue;
 				}
-				out[0] = x[i];	out[1] = y[j];
+				out[GMT_X] = x[i];	out[GMT_Y] = y[j];
 				GMT_output (GMT_stdout, n_out, out);
 			}
 			GMT_free ((void *)x);
