@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdcontour.c,v 1.135 2009/01/09 04:02:33 guru Exp $
+ *	$Id: grdcontour.c,v 1.142 2009/09/11 10:57:37 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ struct GRDCONTOUR_CTRL {
 	} E;
 	struct F {	/* -F<way> */
 		BOOLEAN active;
-		int value;
+		GMT_LONG value;
 	} F;
 	struct G {	/* -G[d|f|n|l|L|x|X]<params> */
 		BOOLEAN active;
@@ -77,11 +77,11 @@ struct GRDCONTOUR_CTRL {
 	} L;
 	struct Q {	/* -Q<cut> */
 		BOOLEAN active;
-		int min;
+		GMT_LONG min;
 	} Q;
 	struct S {	/* -S<smooth> */
 		BOOLEAN active;
-		int value;
+		GMT_LONG value;
 	} S;
 	struct T {	/* -T[+|-][<gap>[c|i|m|p]/<length>[c|i|m|p]][:LH] */
 		BOOLEAN active;
@@ -105,17 +105,17 @@ struct GRDCONTOUR_CTRL {
 struct SAVE {
 	double *x, *y;
 	double cval;
-	int n;
+	GMT_LONG n;
 	struct GMT_PEN pen;
 	BOOLEAN do_it, high;
 };
 
 int main (int argc, char **argv)
 {
-	int j, k, c, n_edges, n_alloc, section, n_contours, side, id;
-	int nx, ny, n_save = 0, got, rgb[3], *edge;
+	GMT_LONG j, k, c, n_edges, n_alloc = 0, section, n_contours, side, id;
+	GMT_LONG nx, ny, n_save = 0, got, nm, i, n, nn, *edge;
 	
-	GMT_LONG nm, i, n, nn;
+	int rgb[3];
 
 	BOOLEAN error = FALSE, interior, begin, subset = FALSE;
 
@@ -134,7 +134,7 @@ int main (int argc, char **argv)
 	struct SAVE *save = NULL;
 	struct GRDCONTOUR_CTRL *Ctrl;
 
-	void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float *grd, double tick_gap, double tick_length, BOOLEAN tick_low, BOOLEAN tick_high, BOOLEAN tick_label, char Llabel, char Hlabel);
+	void sort_and_plot_ticks (struct SAVE *save, GMT_LONG n, struct GRD_HEADER *h, float *grd, double tick_gap, double tick_length, BOOLEAN tick_low, BOOLEAN tick_high, BOOLEAN tick_label, char Llabel, char Hlabel);
 	void GMT_grd_minmax (float *a, struct GRD_HEADER *h, double xyz[2][3]);
 	void adjust_hill_label (struct GMT_CONTOUR *G, struct GRD_HEADER *h, float *z);
 	void *New_Grdcontour_Ctrl (), Free_Grdcontour_Ctrl (struct GRDCONTOUR_CTRL *C);
@@ -152,6 +152,7 @@ int main (int argc, char **argv)
 				case 'B':
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -164,6 +165,7 @@ int main (int argc, char **argv)
 				case 'b':
 				case 'c':
 				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
@@ -233,10 +235,6 @@ int main (int argc, char **argv)
 					Ctrl->L.active = TRUE;
 					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->L.low, &Ctrl->L.high);
 					break;
-				case 'M':	/* with -D, create one multiple line segments */
-					GMT_multisegment (&argv[i][2]);
-					GMT_io.multi_segments[GMT_OUT] = 2;
-					break;
 				case 'N':	/* Backwards compatibility - now done in -A instead */
 					if (argv[i][2])
 						strcpy (Ctrl->contour.unit, &argv[i][2]);
@@ -334,9 +332,9 @@ int main (int argc, char **argv)
 		fprintf (stderr,"grdcontour %s - Contouring of 2-D gridded data sets\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdcontour <grdfile> -C<cont_int> %s\n", GMT_J_OPT);
 		fprintf (stderr, "\t[-A[-|<annot_int>][<labelinfo>] [%s] [-D<dumpfile>] [%s] [-F[l|r]] [%s]\n", GMT_B_OPT, GMT_E_OPT, GMT_CONTG);
-		fprintf (stderr, "\t[-K] [-L<Low/high>] [%s] [-O] [-P] [-Q<cut>] [%s] [-S<smooth>]\n", GMT_Mo_OPT, GMT_Rgeo_OPT);
+		fprintf (stderr, "\t[-K] [-L<Low/high>] [-O] [-P] [-Q<cut>] [%s] [-S<smooth>]\n", GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[-T[+|-][<gap>[c|i|m|p]/<length>[c|i|m|p]][:LH]] [%s] [-V] [-W[+]<type><pen>]\n", GMT_U_OPT);
-		fprintf (stderr, "\t[%s] [%s] [-Z[<fact>[/shift>]][p]] [%s] [%s]\n\n", GMT_X_OPT, GMT_Y_OPT, GMT_bo_OPT, GMT_c_OPT);
+		fprintf (stderr, "\t[%s] [%s] [-Z[<fact>[/shift>]][p]] [%s] [%s] [%s]\n\n", GMT_X_OPT, GMT_Y_OPT, GMT_bo_OPT, GMT_c_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -355,7 +353,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   <labelinfo> controls the specifics of the labels.  Append what you need:\n");
 		GMT_label_syntax (5, 0);
 		GMT_explain_option ('b');
-		fprintf (stderr, "\t-D to Dump contour lines to individual files (but see -M)\n");
+		fprintf (stderr, "\t-D to Dump contour lines to individual files (but see -m)\n");
 		fprintf (stderr, "\t   Append file prefix [contour].  Files will be called <dumpfile>_<cont>_#[_i].xyz|b\n");
 		fprintf (stderr, "\t   where <cont> is the contour value and # is a segment counter.\n");
 		fprintf (stderr, "\t   _i is inserted for interior (closed) contours, with xyz (ascii) or b (binary) as extension.\n");
@@ -367,8 +365,6 @@ int main (int argc, char **argv)
 		GMT_cont_syntax (3, 0);
 		GMT_explain_option ('K');
 		fprintf (stderr, "\t-L only contour inside this range\n");
-		fprintf (stderr, "\t-M Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
-		fprintf (stderr, "\t   whose first character is <flag> ['>'].  This header also has the contour level value\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
 		fprintf (stderr, "\t-Q Do not draw contours with less than <cut> points [Draw all contours]\n");
@@ -396,6 +392,8 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		GMT_explain_option ('c');
 		GMT_explain_option ('f');
+		fprintf (stderr, "\t-m Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
+		fprintf (stderr, "\t   whose first character is <flag> ['>'].  This header also has the contour level value\n");
 		exit (EXIT_FAILURE);
 	}
 
@@ -448,6 +446,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: contours will be written to files contour_*.xyz\n", GMT_program);
 		strcpy (Ctrl->D.file,"contour");
 	}
+	if (GMT_io.multi_segments[GMT_OUT]) GMT_io.multi_segments[GMT_OUT] = 2;
+
 	GMT_z_periodic = Ctrl->Z.periodic;	/* Phase data */
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
@@ -475,7 +475,8 @@ int main (int argc, char **argv)
 
 	/* Determine the wesn to be used to read the grid file */
 
-	if (GMT_grd_setregion (&header, &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
+	if (GMT_grd_setregion (&header, &data_west, &data_east, &data_south, &data_north, BCR_BILINEAR)) {
+		/* No grid to plot; just do empty map and exit */
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: No data within specified region\n", GMT_program);
 		GMT_plotinit (argc, argv);
 		GMT_map_basemap ();
@@ -485,29 +486,11 @@ int main (int argc, char **argv)
 		exit (EXIT_SUCCESS);
 	}
 
-	/* Read data */
-
-	if (subset && header.node_offset) {	/* Possibly extend the subset by 1 row/col since pixel tiles need 4 corners to work */
-		BOOLEAN global;
-		global = (GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (header.x_max, header.x_min));
-		if (global && data_west > project_info.w)
-			data_west -= header.x_inc;
-		else if (GMT_x_to_i (data_west, header.x_min, header.x_inc, header.xy_off, header.nx) > 0)
-			data_west -= header.x_inc;
-		if (global && data_east < project_info.e)
-			data_east += header.x_inc;
-		else if (GMT_x_to_i (data_east, header.x_min, header.x_inc, header.xy_off, header.nx) < (header.nx)) {
-			data_east += header.x_inc;
-			if (data_east > header.x_max) data_east -= header.x_inc;
-		}
-		if (GMT_y_to_j (data_south, header.y_min, header.y_inc, header.xy_off, header.ny) < (header.ny)) data_south -= header.y_inc;
-		if (GMT_y_to_j (data_north, header.y_min, header.y_inc, header.xy_off, header.ny) > 0) data_north += header.y_inc;
-		if (data_south < header.y_min) data_south += header.y_inc;
-	}
-
 	nx = GMT_get_n (data_west, data_east, header.x_inc, header.node_offset);
 	ny = GMT_get_n (data_south, data_north, header.y_inc, header.node_offset);
 	nm = GMT_get_nm (nx, ny);
+
+	/* Read data */
 	
 	grd_original = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 	GMT_err_fail (GMT_read_grd (grdfile, &header, grd_original, data_west, data_east, data_south, data_north, GMT_pad, FALSE), grdfile);
@@ -541,12 +524,6 @@ int main (int argc, char **argv)
 	else
 		aval = header.z_max + 1.0;
 
-	n_alloc = GMT_CHUNK;
-	contour = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-	cont_type = (char *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (char), GMT_program);
-	cont_do_tick = (char *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (char), GMT_program);
-	cont_angle = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-
 	if (Ctrl->C.cpt) {	/* Presumably got a cpt-file */
 		GMT_read_cpt (Ctrl->C.file);
 #ifdef GMT_CPT2	
@@ -556,12 +533,8 @@ int main (int argc, char **argv)
 		}
 #endif
 		n_contours = GMT_n_colors + 1;
-		if (n_contours > n_alloc) {
-			contour = (double *) GMT_memory ((void *)contour, (size_t)n_contours, sizeof (double), GMT_program);
-			cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_contours, sizeof (char), GMT_program);
-			cont_do_tick = (char *) GMT_memory ((void *)cont_do_tick, (size_t)n_contours, sizeof (char), GMT_program);
-			cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_contours, sizeof (double), GMT_program);
-		}
+		(void)GMT_alloc_memory2 ((void **)&contour, (void **)&cont_angle, n_contours, 0, sizeof (double), GMT_program);
+		n_alloc = GMT_alloc_memory2 ((void **)&cont_type, (void **)&cont_do_tick, n_contours, 0, sizeof (char), GMT_program);
 		for (i = c = 0; i < GMT_n_colors; i++) {
 			if (GMT_lut[i].skip) continue;
 			contour[c] = GMT_lut[i].z_low;
@@ -596,19 +569,16 @@ int main (int argc, char **argv)
 		}
 		while (GMT_fgets (line, BUFSIZ, fpc)) {
 			if (line[0] == '#') continue;
+			if (n_contours == n_alloc) {
+				(void)GMT_alloc_memory2 ((void **)&contour, (void **)&cont_angle, n_contours, n_alloc, sizeof (double), GMT_program);
+				n_alloc = GMT_alloc_memory2 ((void **)&cont_type, (void **)&cont_do_tick, n_contours, n_alloc, sizeof (char), GMT_program);
+			}
 			got = sscanf (line, "%lf %c %lf", &contour[n_contours], &cont_type[n_contours], &tmp);
 			if (cont_type[n_contours] == 0) cont_type[n_contours] = 'C';
 			cont_do_tick[n_contours] = (Ctrl->T.active && ((cont_type[n_contours] == 'C') || (cont_type[n_contours] == 'A'))) ? 1 : 0;
 			cont_angle[n_contours] = (got == 3) ? tmp : GMT_d_NaN;
 			if (got == 3) Ctrl->contour.angle_type = 2;	/* Must set this directly if angles are provided */
 			n_contours++;
-			if (n_contours == n_alloc) {
-				n_alloc <<= 1;
-				contour = (double *) GMT_memory ((void *)contour, (size_t)n_alloc, sizeof (double), GMT_program);
-				cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_alloc, sizeof (char), GMT_program);
-				cont_do_tick = (char *) GMT_memory ((void *)cont_do_tick, (size_t)n_alloc, sizeof (char), GMT_program);
-				cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_alloc, sizeof (double), GMT_program);
-			}
 		}
 		GMT_fclose (fpc);
 	}
@@ -617,10 +587,8 @@ int main (int argc, char **argv)
 		max = ceil (header.z_max / Ctrl->C.interval) * Ctrl->C.interval; if (max > header.z_max) max -= Ctrl->C.interval;
 		for (c = irint (min/Ctrl->C.interval), n_contours = 0; c <= irint (max/Ctrl->C.interval); c++, n_contours++) {
 			if (n_contours == n_alloc) {
-				n_alloc <<= 1;
-				contour = (double *) GMT_memory ((void *)contour, (size_t)n_alloc, sizeof (double), GMT_program);
-				cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_alloc, sizeof (char), GMT_program);
-				cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_alloc, sizeof (double), GMT_program);
+				(void)GMT_alloc_memory2 ((void **)&contour, (void **)&cont_angle, n_contours, n_alloc, sizeof (double), GMT_program);
+				n_alloc = GMT_alloc_memory2 ((void **)&cont_type, (void **)&cont_do_tick, n_contours, n_alloc, sizeof (char), GMT_program);
 			}
 			contour[n_contours] = c * Ctrl->C.interval;
 			if (Ctrl->contour.annot && (contour[n_contours] - aval) > GMT_SMALL) aval += Ctrl->A.interval;
@@ -646,17 +614,13 @@ int main (int argc, char **argv)
 		Free_Grdcontour_Ctrl (Ctrl);	/* Deallocate control structure */
 		GMT_end (argc, argv);
 		exit (EXIT_SUCCESS);
-
 	}
 	
 	/* OK, now we know we have contouring to do */
 	
-	contour = (double *) GMT_memory ((void *)contour, (size_t)n_contours, sizeof (double), GMT_program);
-	cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_contours, sizeof (char), GMT_program);
-	cont_do_tick = (char *) GMT_memory ((void *)cont_do_tick, (size_t)n_contours, sizeof (char), GMT_program);
-	cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_contours, sizeof (double), GMT_program);
+	(void)GMT_alloc_memory2 ((void **)&contour, (void **)&cont_angle, 0, n_contours, sizeof (double), GMT_program);
+	n_alloc = GMT_alloc_memory2 ((void **)&cont_type, (void **)&cont_do_tick, 0, n_contours, sizeof (char), GMT_program);
 
-	if (Ctrl->T.active) save = (struct SAVE *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct SAVE), GMT_program);
 	GMT_grd_minmax (grd_original, &header, xyz);
 	if (GMT_contlabel_prep (&Ctrl->contour, xyz)) exit (EXIT_FAILURE);	/* Prep for crossing lines, if any */
 
@@ -667,7 +631,7 @@ int main (int argc, char **argv)
 	 
 	grd = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 	n_edges = header.ny * (int )ceil (header.nx / 16.0);
-	edge = (int *) GMT_memory (VNULL, (size_t)n_edges, sizeof (int), GMT_program);
+	edge = (GMT_LONG *) GMT_memory (VNULL, (size_t)n_edges, sizeof (GMT_LONG), GMT_program);
 
 
 	GMT_plotinit (argc, argv);
@@ -677,6 +641,8 @@ int main (int argc, char **argv)
 
 	for (c = 0; c < n_contours; c++) {	/* For each contour value cval */
 
+		if (Ctrl->L.active && (contour[c] < Ctrl->L.low || contour[c] > Ctrl->L.high)) continue;	/* Outside desired range */
+
 		/* Reset markers and set up new zero-contour*/
 
 		cval = contour[c];
@@ -701,6 +667,7 @@ int main (int argc, char **argv)
 
 		side = 0;
 		begin = TRUE;
+		n_alloc = 0;
 		while (side <= GMT_LAST_CONTOUR_SIDE) {
 			while ((n = GMT_contours (grd, &header, Ctrl->S.value, gmtdefs.interpolant, Ctrl->F.value, &side, edge, begin, &x, &y)) > 0) {
 
@@ -722,8 +689,8 @@ int main (int argc, char **argv)
 						else
 							cont_label[0] = (char)0;
 						if (cont_do_tick[c] && interior) {
-							save[n_save].x = (double *) GMT_memory (VNULL, (size_t)nn, sizeof (double), GMT_program);
-							save[n_save].y = (double *) GMT_memory (VNULL, (size_t)nn, sizeof (double), GMT_program);
+							if (n_save == n_alloc) n_alloc = GMT_alloc_memory ((void **)&save, n_save, n_alloc, sizeof (struct SAVE), GMT_program);
+							(void)GMT_alloc_memory2 ((void **)&save[n_save].x, (void **)&save[n_save].y, nn, 0, sizeof (double), GMT_program);
 							memcpy ((void *)save[n_save].x, (void *)xp, (size_t)(nn * sizeof (double)));
 							memcpy ((void *)save[n_save].y, (void *)yp, (size_t)(nn * sizeof (double)));
 							save[n_save].n = nn;
@@ -731,10 +698,6 @@ int main (int argc, char **argv)
 							save[n_save].do_it = TRUE;
 							save[n_save].cval = cval;
 							n_save++;
-							if (n_save == n_alloc) {
-								n_alloc <<= 1;
-								save = (struct SAVE *) GMT_memory ((void *)save, (size_t)n_alloc, sizeof (struct SAVE), GMT_program);
-							}
 						}
 						GMT_hold_contour (&xp, &yp, nn, cval, cont_label, cont_type[c], cont_angle[c], interior, &Ctrl->contour);
 						GMT_free ((void *)xp);
@@ -753,7 +716,7 @@ int main (int argc, char **argv)
 	if (Ctrl->W.pen[0].texture || Ctrl->W.pen[1].texture) ps_setdash (CNULL, 0);
 
 	if (Ctrl->T.active && n_save) {
-		save = (struct SAVE *) GMT_memory ((void *)save, (size_t)n_save, sizeof (struct SAVE), GMT_program);
+		(void)GMT_alloc_memory ((void **)&save, 0, n_save, sizeof (struct SAVE), GMT_program);
 
 		sort_and_plot_ticks (save, n_save, &header, grd_original, Ctrl->T.spacing, Ctrl->T.length, Ctrl->T.low, Ctrl->T.high, Ctrl->T.label, Ctrl->T.L_label, Ctrl->T.H_label);
 		for (i = 0; i < n_save; i++) {
@@ -796,9 +759,9 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float *grd, double tick_gap, double tick_length, BOOLEAN tick_low, BOOLEAN tick_high, BOOLEAN tick_label, char Llabel, char Hlabel)
+void sort_and_plot_ticks (struct SAVE *save, GMT_LONG n, struct GRD_HEADER *h, float *grd, double tick_gap, double tick_length, BOOLEAN tick_low, BOOLEAN tick_high, BOOLEAN tick_label, char Llabel, char Hlabel)
 {
-	int np, i, j, inside, ix, iy, iy_s, done, n_ticks;
+	GMT_LONG np, i, j, inside, ix, iy, iy_s, done, n_ticks;
 	double x0, y0, add, sign, dx, dy, x_back, y_back, x_front, y_front;
 	double xmin, xmax, ymin, ymax, x_mean = 0.0, y_mean = 0.0, inc, dist, a, small, this_lon, this_lat, sa, ca, *s;
 	char txt[2][2];
@@ -937,7 +900,7 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 
 void GMT_grd_minmax (float *a, struct GRD_HEADER *h, double xyz[2][3])
 {
-	int i, i_min, i_max;
+	GMT_LONG i, i_min, i_max;
 	float z_min, z_max;
 	double half;
 
@@ -964,7 +927,7 @@ void GMT_grd_minmax (float *a, struct GRD_HEADER *h, double xyz[2][3])
 
 void adjust_hill_label (struct GMT_CONTOUR *G, struct GRD_HEADER *h, float *z)
 {	
-	int i, k, i0, j0;
+	GMT_LONG i, k, i0, j0;
 	double nx, ny, x_on, y_on, x_node, y_node, x_node_p, y_node_p, dx, dy, dz, dot, angle;
 	struct GMT_CONTOUR_LINE *C;
 	
