@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdfft.c,v 1.38 2009/01/09 04:02:33 guru Exp $
+ *	$Id: grdfft.c,v 1.48 2009/09/09 23:27:03 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
  *   Note to myself:  users want azimuthal power spectra, and
  *   cross-spectra between two grids.  --whfs
  * 5/17/2006 PW: Added enhanced -F option for Gaussian band-pass.
+ * 6/17/2009 PW: Added enhanced -F option for Butterworth band-pass.
  *
  * Version:	4
  */
@@ -40,8 +41,8 @@
 #endif
 
 struct GRDFFT_CTRL {
-	int n_op_count, n_par;
-	int *operation;
+	GMT_LONG n_op_count, n_par;
+	GMT_LONG *operation;
 	double	*par;
 
 	struct A {	/* -A<azimuth> */
@@ -58,11 +59,11 @@ struct GRDFFT_CTRL {
 	} D;
 	struct E {	/* -E[x_or_y][w] */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 	} E;
 	struct F {	/* -F[x_or_y]<lc>/<lp>/<hp>/<hc> or -F[x_or_y]<lo>/<hi> */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 		double lc, lp, hp, hc;
 	} F;
 	struct G {	/* -G<outfile> */
@@ -101,9 +102,10 @@ struct GRDFFT_CTRL {
 #define DIFFERENTIATE	2
 #define INTEGRATE	3
 #define ISOSTASY	4
-#define FILTER_COS	5
-#define FILTER_EXP	6
-#define SPECTRUM	7
+#define FILTER_EXP	5
+#define FILTER_BW	6
+#define FILTER_COS	7
+#define SPECTRUM	8
 
 /* Macro definition ij_data(i,j) finds the array index to an element
 	containing the real data(i,j) in the padded complex array:  */
@@ -112,7 +114,7 @@ struct GRDFFT_CTRL {
 
 
 GMT_LONG	narray[2];
-int	i, j, k, n, nx2 = 0, ny2 = 0, ndatac, i_data_start, j_data_start;
+GMT_LONG	i, j, k, n, nx2 = 0, ny2 = 0, ndatac, i_data_start, j_data_start;
 
 BOOLEAN	map_units = FALSE, force_narray = FALSE, suggest_narray = FALSE, leave_trend_alone = FALSE, n_user_set = FALSE;
 
@@ -132,14 +134,17 @@ struct F_INFO {
 	double	htaper[3];	/* High taper width for r, x, and y	*/
 	double	llambda[3];	/* Low full-wavelength where Gauss amp = 0.5 for r, x, and y	*/
 	double	hlambda[3];	/* High full-wavelength where Gauss amp = 0.5  for r, x, and y	*/
-	int	do_this[3];	/* T/F this filter wanted for r, x, and y	*/
-	int	set_already;
-	int	kind;		/* 0 = cosine taper, 1 = Gaussian */
+	double	bw_order;	/* Order, N, of Butterworth filter	*/
+	PFD filter;		/* Points to the correct filter function */
+	GMT_LONG	do_this[3];	/* T/F this filter wanted for r, x, and y	*/
+	GMT_LONG	set_already;
+	GMT_LONG	kind;		/* FILTER_EXP, FILTER_BW, FILTER_COS  */
+	GMT_LONG	arg;		/* 0 = Gaussian, 1 = Butterworth, 2 = cosine taper,  */
 } f_info;
 
 struct FFT_SUGGESTION {
-	int	nx;
-	int	ny;
+	GMT_LONG	nx;
+	GMT_LONG	ny;
 	GMT_LONG	worksize;	/* # single-complex elements needed in work array  */
 	GMT_LONG	totalbytes;	/* (8*(nx*ny + worksize))  */
 	double	run_time;
@@ -150,16 +155,19 @@ struct FFT_SUGGESTION {
 float	*datac, *workc;
 double	scale_out = 1.0;
 
+double cosine_weight (double freq, int j), gauss_weight (double freq, int j), bw_weight (double freq, int j);
+
 int main (int argc, char **argv)
 {
-	int op_count = 0, par_count = 0, filter_type = 0;
+	GMT_LONG op_count = 0, par_count = 0, filter_type = 0;
 
 	BOOLEAN error = FALSE, stop, give_wavelength = FALSE;
 	struct GRDFFT_CTRL *Ctrl;
-	int parse_f_string (char *c), count_slashes (char *txt);
+	GMT_LONG parse_f_string (char *c), count_slashes (char *txt);
 	void read_data(struct GRDFFT_CTRL *Ctrl, int argc, char **argv);
-	void do_continuation(double *zlevel), do_azimuthal_derivative(double *azim), do_differentiate(double *par), do_integrate(double *par), do_isostasy(double *par);
-	void do_filter_cos(void), do_filter_exp(void), do_spectrum(double *par, int give_wavelength);
+	void do_continuation(double *zlevel), do_azimuthal_derivative(double *azim), do_differentiate(double *par), do_integrate(double *par);
+	void do_filter(void), do_spectrum(double *par, GMT_LONG give_wavelength), do_isostasy(double *par);
+	double get_filter_weight (GMT_LONG k);
 
 	void *New_Grdfft_Ctrl (), Free_Grdfft_Ctrl (struct GRDFFT_CTRL *C);
 	
@@ -190,7 +198,7 @@ int main (int argc, char **argv)
 			
                                 case 'A':
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = AZIMUTHAL_DERIVATIVE;
 					op_count++;
 					Ctrl->n_par++;
@@ -203,7 +211,7 @@ int main (int argc, char **argv)
                                         break;
                                 case 'C':
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = UP_DOWN_CONTINUE;
 					op_count++;
 					Ctrl->n_par++;
@@ -216,7 +224,7 @@ int main (int argc, char **argv)
                                         break;
                                 case 'D':
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = DIFFERENTIATE;
 					op_count++;
 					Ctrl->n_par++;
@@ -233,7 +241,7 @@ int main (int argc, char **argv)
                                         break;
                                 case 'E':
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = SPECTRUM;
 					op_count++;
 					Ctrl->n_par++;
@@ -255,12 +263,12 @@ int main (int argc, char **argv)
                                 case 'F':
                                 	if (!(f_info.set_already)) {
 						filter_type = count_slashes (&argv[i][2]);
+						f_info.kind = FILTER_EXP + (filter_type - 1);
 						Ctrl->n_op_count++;
-						Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
-                                        	Ctrl->operation[op_count] = (filter_type > 2) ? FILTER_COS : FILTER_EXP;
+						Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
+                                        	Ctrl->operation[op_count] = f_info.kind;
                                         	op_count++;
                                         	f_info.set_already = TRUE;
-						f_info.kind = (filter_type == 1);
                                         }
                                         if (parse_f_string(&argv[i][2])) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: ", GMT_program);
@@ -272,7 +280,7 @@ int main (int argc, char **argv)
                                         break;
                                 case 'I':
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = INTEGRATE;
 					op_count++;
 					Ctrl->n_par++;
@@ -299,7 +307,7 @@ int main (int argc, char **argv)
 					else if (argv[i][2] == 'q' || argv[i][2] == 'Q')
 						suggest_narray = TRUE;
 					else {
-	                                        sscanf(&argv[i][2], "%d/%d", &nx2, &ny2);
+	                                        sscanf(&argv[i][2], "%ld/%ld", &nx2, &ny2);
 						n_user_set = TRUE;
 					}
                                         break;
@@ -310,7 +318,7 @@ int main (int argc, char **argv)
                                 case 'T':
                                         Ctrl->T.active = TRUE;
 					Ctrl->n_op_count++;
-					Ctrl->operation = (int *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(int), GMT_program);
+					Ctrl->operation = (GMT_LONG *) GMT_memory ((void *)Ctrl->operation, (size_t)Ctrl->n_op_count, sizeof(GMT_LONG), GMT_program);
                                         Ctrl->operation[op_count] = ISOSTASY;
 					op_count++;
  					Ctrl->n_par += 5;
@@ -352,12 +360,14 @@ int main (int argc, char **argv)
 		fprintf (stderr,"\t-D Differentiate, i.e., multiply by kr [ * scale].  Use -Dg to get mGal from m]\n");
 		fprintf (stderr,"\t-E Estimate spEctrum of r [x] [y].  Write f, power[f], 1 std dev(power[f]) to stdout.\n");
 		fprintf (stderr,"\t\tAppend w to write wavelength instead of frequency.\n");
-		fprintf (stderr,"\t-F Filter r [x] [y] freq according to one of two kinds of filter specifications:.\n");
+		fprintf (stderr,"\t-F Filter r [x] [y] freq according to one of three kinds of filter specifications:.\n");
 		fprintf (stderr,"\t   a) Cosine band-pass: Append four wavelengths <lc>/<lp>/<hp>/<hc>.\n");
 		fprintf (stderr,"\t      freq outside <lc>/<hc> are cut; inside <lp>/<hp> are passed, rest are tapered.\n");
 		fprintf (stderr,"\t      Replace wavelength by - to skip, e.g.  -F-/-/500/100 is a low-pass filter.\n");
 		fprintf (stderr,"\t   b) Gaussian band-pass: Append two wavelengths <lo>/<hi> where filter amplitudes = 0.5.\n");
 		fprintf (stderr,"\t      Replace wavelength by - to skip, e.g.  -F300/- is a high-pass Gaussian filter.\n");
+		fprintf (stderr,"\t   c) Butterworth band-pass: Append two wavelengths and order <lo>/<hi>/<order> where filter amplitudes = 0.5.\n");
+		fprintf (stderr,"\t      Replace wavelength by - to skip, e.g.  -F300/-/2 is a high-pass 2nd-order Butterworth filter.\n");
 		fprintf (stderr,"\t-I Integrate, i.e., divide by kr [ * scale].  Use -Ig to get m from mGal]\n");
 		fprintf (stderr,"\t-L Leave trend alone.  Do not remove least squares plane from data.  [Default removes plane].\n");
 		fprintf (stderr,"\t-M Map units used.  Convert grid dimensions from degrees to meters.\n");
@@ -414,6 +424,16 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+#ifdef FTEST
+	{
+		double f = 0.0;
+		while (f < 3.0) {
+			printf ("%g\t%g\n", f, f_info.filter (f, 0));	/* Radial filter */
+			f += 0.01;
+		}
+		exit (-1);
+	}
+#endif
 	if (gmtdefs.verbose) fprintf (stderr, "%s: forward FFT...", GMT_program);
 	
 	GMT_fourt (datac, narray, 2, -1, 1, workc);
@@ -446,12 +466,16 @@ int main (int argc, char **argv)
 				par_count += 5;
 				break;
 			case FILTER_COS:
-				if (gmtdefs.verbose) fprintf (stderr, "filter...");
-				do_filter_cos ();
+				if (gmtdefs.verbose) fprintf (stderr, "cosine filter...");
+				do_filter ();
 				break;
 			case FILTER_EXP:
-				if (gmtdefs.verbose) fprintf (stderr, "filter...");
-				do_filter_exp ();
+				if (gmtdefs.verbose) fprintf (stderr, "Gaussian filter...");
+				do_filter ();
+				break;
+			case FILTER_BW:
+				if (gmtdefs.verbose) fprintf (stderr, "Butterworth filter...");
+				do_filter ();
 				break;
 			case SPECTRUM:
 				if (gmtdefs.verbose) fprintf (stderr, "spectrum...");
@@ -489,10 +513,10 @@ int main (int argc, char **argv)
 
 void read_data (struct GRDFFT_CTRL *Ctrl, int argc, char **argv)
 {
-	int	worksize, factors[32];
+	GMT_LONG	worksize, factors[32];
 	double	tdummy, edummy;
-	void	suggest_fft(int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print);
-	void	fourt_stats(int nx, int ny, int *f, double *r, int *s, double *t);
+	void	suggest_fft(GMT_LONG nx, GMT_LONG ny, struct FFT_SUGGESTION *fft_sug, GMT_LONG do_print);
+	void	fourt_stats(GMT_LONG nx, GMT_LONG ny, GMT_LONG *f, double *r, GMT_LONG *s, double *t);
 	void remove_plane(void), taper_edges(void);
 	
 	GMT_err_fail (GMT_read_grd_info (infile, &h), infile);
@@ -531,7 +555,7 @@ void read_data (struct GRDFFT_CTRL *Ctrl, int argc, char **argv)
 	narray[0] = nx2;
 	narray[1] = ny2;
 	fourt_stats(nx2, ny2, factors, &edummy, &worksize, &tdummy);
-	if (gmtdefs.verbose) fprintf(stderr,"%s:  Data dimension %d %d\tFFT dimension %d %d\n", GMT_program,
+	if (gmtdefs.verbose) fprintf(stderr,"%s:  Data dimension %d %d\tFFT dimension %ld %ld\n", GMT_program,
 		h.nx, h.ny, nx2, ny2);
 
 	/* Make an array of floats 2 * nx2 * ny2 for complex data:  */
@@ -582,7 +606,7 @@ void remove_plane (void)
 	spend some multiplications on normalizing the 
 	range of x,y into [-1,1], to avoid roundoff error.  */
 
-	int	one_or_zero;
+	GMT_LONG	one_or_zero;
 	double	x_half_length, one_on_xhl, y_half_length, one_on_yhl;
 	double	sumx2, sumy2, x, y, z;
 
@@ -771,8 +795,8 @@ void do_azimuthal_derivative (double *azim)
 	float	tempr, tempi, fact;
 	double cos_azim, sin_azim;
 
-	cos_azim = cos ((*azim) * D2R);
-	sin_azim = sin ((*azim) * D2R);
+	cos_azim = cosd (*azim);
+	sin_azim = sind (*azim);
 
 	datac[0] = datac[1] = 0.0;
 	for (k = 2; k < ndatac; k+= 2) {
@@ -825,66 +849,22 @@ void do_isostasy (double *par)
 	}
 }
 
-void do_filter_cos (void)
+void do_filter (void)
 {
 	GMT_LONG	k;
-	double	weight, f_wt_cos(GMT_LONG k);
+	double	weight;
+	double get_filter_weight (GMT_LONG k);
 	for (k = 0; k < ndatac; k += 2) {
-		weight = f_wt_cos(k);
+		weight = get_filter_weight(k);
 		datac[k] *= (float)weight;
 		datac[k+1] *= (float)weight;
 	}
 }
 
-void do_filter_exp (void)
-{
-	GMT_LONG	k;
-	double	weight, f_wt_exp(GMT_LONG k);
-	for (k = 0; k < ndatac; k += 2) {
-		weight = f_wt_exp(k);
-		datac[k] *= (float)weight;
-		datac[k+1] *= (float)weight;
-	}
-}
-
-double f_wt_cos (GMT_LONG k)
+double get_filter_weight (GMT_LONG k)
 {
 	double	freq, return_value = 1.0;
 	GMT_LONG	j;
-	
-	/* Always return immediately when zero weight encountered.  */
-	
-	for (j = 0; j < 3; j++) {
-		if (!(f_info.do_this[j])) continue;	/* Only do one of x, y, or r filtering */
-		switch (j) {
-			case 0:
-				freq = modk(k);
-				break;
-			case 1:
-				freq = kx(k);
-				break;
-			default:
-				freq = ky(k);
-				break;
-		}
-		if (freq <= f_info.lc[j] || freq >= f_info.hc[j]) return(0.0);	/* In fully cut range.  Weight is zero.  */
-			
-		if (freq > f_info.lc[j] && freq < f_info.lp[j])
-			return_value *= 0.5 * (1.0 + cos (M_PI * (freq - f_info.lp[j]) * f_info.ltaper[j]));
-		else if (freq > f_info.hp[j] && freq < f_info.hc[j])
-			return_value *= 0.5 * (1.0 + cos (M_PI * (freq - f_info.hp[j]) * f_info.htaper[j]));
-		/* Else freq is in the fully passed range, so weight is multiplied by 1.0  */
-	}
-
-	/* Get here when all weights have been multiplied on return_value:  */
-	
-	return (return_value);
-}
-
-double f_wt_exp (GMT_LONG k)
-{
-	double	freq, return_value = 1.0, lo, hi;
-	GMT_LONG	j;
 		
 	for (j = 0; j < 3; j++) {
 		if (!(f_info.do_this[j])) continue;	/* Only do one of x, y, or r filtering */
@@ -899,28 +879,47 @@ double f_wt_exp (GMT_LONG k)
 				freq = ky(k);
 				break;
 		}
-		lo = (f_info.llambda[j] == -1.0) ? 0.0 : exp (-M_LN2 * pow (freq * f_info.llambda[j], 2.0));
-		hi = (f_info.hlambda[j] == -1.0) ? 1.0 : exp (-M_LN2 * pow (freq * f_info.hlambda[j], 2.0));
-		return_value = hi - lo;
+		return_value = f_info.filter (freq, j);
 	}
 
 	return (return_value);
 }
 
-int count_slashes (char *txt)
+double gauss_weight (double freq, int j) {
+	double hi, lo;
+	lo = (f_info.llambda[j] == -1.0) ? 0.0 : exp (-M_LN2 * pow (freq * f_info.llambda[j], 2.0));	/* Low-pass part */
+	hi = (f_info.hlambda[j] == -1.0) ? 1.0 : exp (-M_LN2 * pow (freq * f_info.hlambda[j], 2.0));	/* Hi-pass given by its complementary low-pass */
+	return (hi - lo);
+}
+
+double bw_weight (double freq, int j) {
+	double hi, lo;
+	lo = (f_info.llambda[j] == -1.0) ? 0.0 : sqrt (1.0 / (1.0 + pow (freq * f_info.llambda[j], f_info.bw_order)));	/* Low-pass part */
+	hi = (f_info.hlambda[j] == -1.0) ? 1.0 : sqrt (1.0 / (1.0 + pow (freq * f_info.hlambda[j], f_info.bw_order)));	/* Hi-pass given by its complementary low-pass */
+	return (hi - lo);
+}
+
+double cosine_weight (double freq, int j) {
+	if (freq <= f_info.lc[j] || freq >= f_info.hc[j]) return(0.0);	/* In fully cut range.  Weight is zero.  */
+	if (freq > f_info.lc[j] && freq < f_info.lp[j]) return (0.5 * (1.0 + cos (M_PI * (freq - f_info.lp[j]) * f_info.ltaper[j])));
+	if (freq > f_info.hp[j] && freq < f_info.hc[j]) return (0.5 * (1.0 + cos (M_PI * (freq - f_info.hp[j]) * f_info.htaper[j])));
+	return (1.0);	/* Freq is in the fully passed range, so weight is multiplied by 1.0  */
+}
+
+GMT_LONG count_slashes (char *txt)
 {
-	int i, n;
+	GMT_LONG i, n;
 	for (i = n = 0; txt[i]; i++) if (txt[i] == '/') n++;
 	return (n);
 }
 
-int parse_f_string (char *c)
+GMT_LONG parse_f_string (char *c)
 {
-	int	i, j, n_tokens, pos, descending;
+	GMT_LONG	i, j, n_tokens, pos, descending;
 	double	fourvals[4];
 	char	line[GMT_LONG_TEXT], p[GMT_LONG_TEXT];
 	
-	/* Syntax is either -F[x|y]lc/hc/lp/hp (Cosine taper) or -F[x|y]lo/hi (Gaussian) */
+	/* Syntax is either -F[x|y]lc/hc/lp/hp (Cosine taper), -F[x|y]lo/hi (Gaussian), or 0F[x|y]lo/hi/order (Butterworth) */
 	
 	strcpy(line, c);
 	i = j = 0;	/* j is Filter type:  r=0, x=1, y=2  */
@@ -948,18 +947,20 @@ int parse_f_string (char *c)
 		}
 		else {
 			if ((sscanf(p, "%lf", &fourvals[n_tokens])) != 1) {
-				fprintf(stderr,"%s:  Cannot read token %d.\n", GMT_program, n_tokens);
+				fprintf(stderr,"%s:  Cannot read token %ld.\n", GMT_program, n_tokens);
 				return(TRUE);
 			}
 		}
 		n_tokens++;
 	}
 	
-	if (!(n_tokens == 2 || n_tokens == 4)) {
-		fprintf(stderr,"%s: -F Cannot find 2 or 4 tokens separated by slashes.\n", GMT_program);
+	if (!(n_tokens == 2 || n_tokens == 3 || n_tokens == 4)) {
+		fprintf(stderr,"%s: -F Cannot find 2-4 tokens separated by slashes.\n", GMT_program);
 		return(TRUE);
 	}
 	descending = TRUE;
+	if (f_info.kind == FILTER_BW && n_tokens == 3) n_tokens = 2;	/* So we dont check the order as a wavelength */
+
 	for (i = 1; i < n_tokens; i++) {
 		if (fourvals[i] == -1.0 || fourvals[i-1] == -1.0) continue;
 		if (fourvals[i] > fourvals[i-1]) descending = FALSE;
@@ -968,7 +969,7 @@ int parse_f_string (char *c)
 		fprintf(stderr,"%s: -F Wavelengths are not in descending order.\n", GMT_program);
 		return(TRUE);
 	}
-	if (f_info.kind == 0) {	/* Cosine band-pass specification */
+	if (f_info.kind == FILTER_COS) {	/* Cosine band-pass specification */
 		if ( (fourvals[0] * fourvals[1]) < 0.0 || (fourvals[2] * fourvals[3]) < 0.0) {
 			fprintf(stderr,"%s: -F Pass/Cut specification error.\n", GMT_program);
 			return(TRUE);
@@ -987,16 +988,24 @@ int parse_f_string (char *c)
 			f_info.hc[j] = (2.0 * M_PI)/fourvals[3];
 			if (fourvals[2] != fourvals[3]) f_info.htaper[j] = 1.0/(f_info.hc[j] - f_info.hp[j]);
 		}
+		f_info.filter = (PFD) cosine_weight;
+	}
+	else if (f_info.kind == FILTER_BW) {	/* Butterworth specification */
+		f_info.llambda[j] = (fourvals[0] == -1.0) ? -1.0 : fourvals[0] / TWO_PI;	/* TWO_PI is used to counteract the 2*pi in the wavenumber */
+		f_info.hlambda[j] = (fourvals[1] == -1.0) ? -1.0 : fourvals[1] / TWO_PI;
+		f_info.bw_order = 2.0 * fourvals[2];
+		f_info.filter = (PFD) bw_weight;
 	}
 	else {	/* Gaussian half-amp specifications */
-		f_info.llambda[j] = fourvals[0] / TWO_PI;	/* TWO_PI is used to counteract the 2*pi in the wavenumber */
-		f_info.hlambda[j] = fourvals[1] / TWO_PI;
+		f_info.llambda[j] = (fourvals[0] == -1.0) ? -1.0 : fourvals[0] / TWO_PI;	/* TWO_PI is used to counteract the 2*pi in the wavenumber */
+		f_info.hlambda[j] = (fourvals[1] == -1.0) ? -1.0 : fourvals[1] / TWO_PI;
+		f_info.filter = (PFD) gauss_weight;
 	}
-	
+	f_info.arg = f_info.kind - FILTER_EXP;
 	return(FALSE);
 }
 
-void do_spectrum (double *par, int give_wavelength)
+void do_spectrum (double *par, GMT_LONG give_wavelength)
 {
 	/* This is modeled on the 1-D case, using the following ideas:
 	 *	In 1-D, we ensemble average over samples of length L = 
@@ -1074,7 +1083,7 @@ void do_spectrum (double *par, int give_wavelength)
 	}
 }
 
-void fourt_stats (int nx, int ny, int *f, double *r, int *s, double *t)
+void fourt_stats (GMT_LONG nx, GMT_LONG ny, GMT_LONG *f, double *r, GMT_LONG *s, double *t)
 {
 	/* Find the proportional run time, t, and rms relative error, r,
 	 * of a Fourier transform of size nx,ny.  Also gives s, the size
@@ -1113,11 +1122,11 @@ void fourt_stats (int nx, int ny, int *f, double *r, int *s, double *t)
 	 * W. H. F. Smith, 26 February 1992.
 	 *  */
 
-	int	n_factors, i, sum2, sumnot2, nnot2;
-	int	nonsymx, nonsymy, nonsym, storage;
+	GMT_LONG	n_factors, i, sum2, sumnot2, nnot2;
+	GMT_LONG	nonsymx, nonsymy, nonsym, storage;
 	GMT_LONG ntotal;
 	double	err_scale, sig_bits = FSIGNIF;
-	int get_prime_factors(GMT_LONG n, int *f), get_non_symmetric_f(int *f, int n);
+	GMT_LONG get_prime_factors(GMT_LONG n, GMT_LONG *f), get_non_symmetric_f(GMT_LONG *f, GMT_LONG n);
 
 	/* Find workspace needed.  First find non_symmetric factors in nx, ny  */
 	n_factors = get_prime_factors((GMT_LONG)nx, f);
@@ -1155,10 +1164,10 @@ void fourt_stats (int nx, int ny, int *f, double *r, int *s, double *t)
 	return;
 } 
 
-int get_non_symmetric_f (int *f, int n)
+GMT_LONG get_non_symmetric_f (GMT_LONG *f, GMT_LONG n)
 {
 	/* Return the product of the non-symmetric factors in f[]  */
-	int	i = 0, j = 1, retval = 1;
+	GMT_LONG	i = 0, j = 1, retval = 1;
 
 	if (n == 1) return(f[0]);
 
@@ -1172,24 +1181,24 @@ int get_non_symmetric_f (int *f, int n)
 	return(retval);
 }
 
-void suggest_fft (int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print)
+void suggest_fft (GMT_LONG nx, GMT_LONG ny, struct FFT_SUGGESTION *fft_sug, GMT_LONG do_print)
 {
-	int	f[64], xstop, ystop;
-	int	nx_best_t, ny_best_t;
-	int	nx_best_e, ny_best_e;
-	int	nx_best_s, ny_best_s;
-        int     nxg, nyg;       /* Guessed by this routine  */
-        int     nx2, ny2, nx3, ny3, nx5, ny5;   /* For powers  */
+	GMT_LONG	f[64], xstop, ystop;
+	GMT_LONG	nx_best_t, ny_best_t;
+	GMT_LONG	nx_best_e, ny_best_e;
+	GMT_LONG	nx_best_s, ny_best_s;
+        GMT_LONG     nxg, nyg;       /* Guessed by this routine  */
+        GMT_LONG     nx2, ny2, nx3, ny3, nx5, ny5;   /* For powers  */
 	double	current_time, best_time, given_time, s_time, e_time;
-	int	current_space, best_space, given_space, e_space, t_space;
+	GMT_LONG	current_space, best_space, given_space, e_space, t_space;
 	double	current_err, best_err, given_err, s_err, t_err;
-	void	fourt_stats(int nx, int ny, int *f, double *r, int *s, double *t);
+	void	fourt_stats(GMT_LONG nx, GMT_LONG ny, GMT_LONG *f, double *r, GMT_LONG *s, double *t);
 
 
 	fourt_stats(nx, ny, f, &given_err, &given_space, &given_time);
 	given_space += nx*ny;
 	given_space *= 8;
-	if (do_print) fprintf(stderr,"%s:  Data dimension\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
+	if (do_print) fprintf(stderr,"%s:  Data dimension\t%ld %ld\ttime factor %.8g\trms error %.8e\tbytes %ld\n", GMT_program,
 		nx, ny, given_time, given_err, given_space);
 
 	best_err = s_err = t_err = given_err;
@@ -1247,11 +1256,11 @@ void suggest_fft (int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print)
 	}
 
 	if (do_print) {
-		fprintf(stderr,"%s:  Highest speed\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
+		fprintf(stderr,"%s:  Highest speed\t%ld %ld\ttime factor %.8g\trms error %.8e\tbytes %ld\n", GMT_program,
 			nx_best_t, ny_best_t, best_time, t_err, t_space);
-		fprintf(stderr,"%s:  Most accurate\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %d\n", GMT_program,
+		fprintf(stderr,"%s:  Most accurate\t%ld %ld\ttime factor %.8g\trms error %.8e\tbytes %ld\n", GMT_program,
 			nx_best_e, ny_best_e, e_time, best_err, e_space);
-		fprintf(stderr,"%s:  Least storage\t%d %d\ttime factor %.8g\trms error %.8e\tbytes %ds\n", GMT_program,
+		fprintf(stderr,"%s:  Least storage\t%ld %ld\ttime factor %.8g\trms error %.8e\tbytes %lds\n", GMT_program,
 			nx_best_s, ny_best_s, s_time, s_err, best_space);
 	}
 	/* Fastest solution:  */
@@ -1279,7 +1288,7 @@ void suggest_fft (int nx, int ny, struct FFT_SUGGESTION *fft_sug, int do_print)
 	return;
 }
 
-int get_prime_factors (GMT_LONG n, int *f)
+GMT_LONG get_prime_factors (GMT_LONG n, GMT_LONG *f)
 {
 	/* Fills the integer array f with the prime factors of n.
 	 * Returns the number of locations filled in f, which is
@@ -1308,7 +1317,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 	GMT_LONG	current_factor;	/* The factor currently being tried  */
 	GMT_LONG	max_factor;	/* Don't try any factors bigger than this  */
-	int	n_factors = 0;	/* Returned; one if n is prime  */
+	GMT_LONG	n_factors = 0;	/* Returned; one if n is prime  */
 	GMT_LONG	two_four_toggle = 0;	/* Used to add 2 or 4 to get next trial factor  */
 	GMT_LONG	ten_twenty_toggle = 0;	/* Used to add 10 or 20 to skip_five  */
 	GMT_LONG	skip_five = 25;	/* Used to skip multiples of 5 in the list  */
@@ -1317,19 +1326,15 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 	/* Initialize m and max_factor  */
 	
-#ifdef __LP64__
-	m = labs(n);
-#else
-	m = abs(n);
-#endif
+	m = GMT_abs(n);
 	if (m < 2) return(0);
-	max_factor = (int)floor(sqrt((double)m));
+	max_factor = (GMT_LONG)floor(sqrt((double)m));
 
 	/* First find the 2s  */
 	current_factor = 2;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1338,7 +1343,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 	current_factor = 3;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1347,7 +1352,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 	current_factor = 5;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1386,7 +1391,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 		while(!(m%current_factor)) {
 			m /= current_factor;
-			f[n_factors] = current_factor;
+			f[n_factors] = (GMT_LONG)current_factor;
 			n_factors++;
 		}
 	}
@@ -1395,7 +1400,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 	if (m > 1) {
 		/* m is an additional prime factor of n  */
-		f[n_factors] = m;
+		f[n_factors] = (GMT_LONG)m;
 		n_factors++;
 	}
 	return (n_factors);
