@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdfilter.c,v 1.69 2009/01/09 04:02:33 guru Exp $
+ *	$Id: grdfilter.c,v 1.73 2009/09/09 23:27:03 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -47,14 +47,14 @@
 struct GRDFILTER_CTRL {
 	struct D {	/* -D<distflag> */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 	} D;
 	struct F {	/* <type>[-]<filter_width>[<mode>] */
 		BOOLEAN active;
 		BOOLEAN highpass;
 		char filter;	/* Character codes for the filter */
 		double width;
-		int mode;
+		GMT_LONG mode;
 	} F;
 	struct G {	/* -G<file> */
 		BOOLEAN active;
@@ -66,14 +66,14 @@ struct GRDFILTER_CTRL {
 	} I;
 	struct N {	/* -Np|i|r */
 		BOOLEAN active;
-		int mode;	/* 0 is default (i), 1 is replace (r), 2 is preserve (p) */
+		GMT_LONG mode;	/* 0 is default (i), 1 is replace (r), 2 is preserve (p) */
 	} N;
 	struct T {	/* -T */
 		BOOLEAN active;
 	} T;
 };
 
-#define IMG2LAT(img) (2.0*atan(exp((img)*D2R))*R2D - 90.0)
+#define IMG2LAT(img) (2.0*atand(exp((img)*D2R))-90.0)
 #define LAT2IMG(lat) (R2D*log(tand(0.5*((lat)+90.0))))
 
 #define GRDFILTER_WIDTH		0
@@ -106,9 +106,9 @@ int main (int argc, char **argv)
 	GMT_LONG	nx_out, ny_out, n_in_median, n_nan = 0, i_west, i_east, j_s_pole, j_n_pole;
 	GMT_LONG	j_origin, i_out, j_out, half_nx, i_orig = 0, n_bad = 0, nm;
 	GMT_LONG	i_in, j_in, ii, jj, i, j, ij_in, ij_out, ij_wt, effort_level;
-	int	filter_type, one_or_zero = 1, GMT_n_multiples = 0;
+	GMT_LONG	filter_type, one_or_zero = 1, GMT_n_multiples = 0;
 	
-	int *i_origin;
+	GMT_LONG *i_origin;
 
 
 	BOOLEAN	error, new_range, fast_way, slow = FALSE, same_grid = FALSE;
@@ -279,7 +279,7 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->T.active, &Ctrl->I.active);
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, NULL, &Ctrl->I.active);
 
 	if (!Ctrl->G.file) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option:  Must specify output file\n", GMT_program);
@@ -401,7 +401,7 @@ int main (int argc, char **argv)
 
 	nm = GMT_get_nm (nx_out, ny_out);
 	output = (float *) GMT_memory (VNULL, (size_t)nm, sizeof(float), GMT_program);
-	i_origin = (int *) GMT_memory (VNULL, (size_t)nx_out, sizeof(int), GMT_program);
+	i_origin = (GMT_LONG *) GMT_memory (VNULL, (size_t)nx_out, sizeof(GMT_LONG), GMT_program);
 	if (!fast_way) x_shift = (double *) GMT_memory (VNULL, (size_t)nx_out, sizeof(double), GMT_program);
 
 	xincnew2 = (one_or_zero) ? 0.0 : 0.5 * Ctrl->I.xinc;
@@ -543,7 +543,7 @@ int main (int argc, char **argv)
 		y_out = north_new - j_out * Ctrl->I.yinc - yincnew2;
 		lat_out = (Ctrl->D.mode == 5) ? IMG2LAT (y_out) : y_out;
 		j_origin = GMT_y_to_j (y_out, h.y_min, h.y_inc, h.xy_off, h.ny);
-		if (Ctrl->D.mode == 3) par[GRDFILTER_X_SCALE] = project_info.KM_PR_DEG * cos (D2R * lat_out);	/* Update flat-earth longitude scale */
+		if (Ctrl->D.mode == 3) par[GRDFILTER_X_SCALE] = project_info.KM_PR_DEG * cosd (lat_out);	/* Update flat-earth longitude scale */
 
 		if (Ctrl->D.mode > 2) {	/* Update max filterweight nodes to deal with at this latitude */
 			y = fabs (lat_out);
@@ -690,7 +690,7 @@ int main (int argc, char **argv)
 	/* At last, that's it!  Output: */
 
 	if (n_nan && gmtdefs.verbose) fprintf (stderr, "%s: Unable to estimate value at %ld nodes, set to NaN\n", GMT_program, n_nan);
-	if (GMT_n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %d multiple modes found\n", GMT_program, GMT_n_multiples);
+	if (GMT_n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %ld multiple modes found\n", GMT_program, GMT_n_multiples);
 
 	h.nx = nx_out;
 	h.ny = ny_out;
