@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdimage.c,v 1.82 2009/01/09 04:02:33 guru Exp $
+ *	$Id: grdimage.c,v 1.109 2009/09/18 04:14:34 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
  * Author:	Paul Wessel
  * Date:	17-SEP-2001
  * Ver:		4
+ *
  */
 
 #include "gmt.h"
@@ -36,10 +37,16 @@ struct GRDIMAGE_CTRL {
 		BOOLEAN active;
 		char *file;
 	} C;
+#ifdef USE_GDAL
+	struct D {	/* -D to read gdal file */
+		BOOLEAN active;
+		BOOLEAN mode;	/* Use info of -R option to reference image */
+	} D;
+#endif
 	struct E {	/* -Ei|<dpi> */
 		BOOLEAN active;
 		BOOLEAN device_dpi;
-		int dpi;
+		GMT_LONG dpi;
 	} E;
 	struct G {	/* -G[f|b]<rgb> */
 		BOOLEAN active;
@@ -62,25 +69,23 @@ struct GRDIMAGE_CTRL {
 	struct S {	/* -S[-]b|c|l|n[/<threshold>] */
 		BOOLEAN active;
 		BOOLEAN antialias;
-		int interpolant;
+		GMT_LONG interpolant;
 		double threshold;
 	} S;
 };
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, do_rgb = FALSE, need_to_project, normal_x, normal_y, resampled = FALSE;
+	BOOLEAN error = FALSE, do_rgb = FALSE, done, need_to_project, normal_x, normal_y, resampled = FALSE;
 
-	unsigned char *bitimage_8 = NULL, *bitimage_24 = NULL;
+	unsigned char *bitimage_8 = NULL, *bitimage_24 = NULL, *rgb_used = NULL;
 
 	char *grdfile[3], *c_method[2] = {"colorimage", "colortiles",};
 
-	int i, j, k, rgb[3], byte, nx_f, ny_f, grid_type = 0;
-	int nx, ny, nx_proj = 0, ny_proj = 0, n_grids = 0, index;
-	int PS_colormask_off = 0, PS_colormask, PS_interpolate;
+	GMT_LONG i, j, k, byte, nx_f, ny_f, nx, ny, nx_proj = 0, ny_proj = 0, index, grid_type = 0, n_grids = 0;
+	GMT_LONG PS_colormask_off = 0, PS_colormask, PS_interpolate, nm, nm2 = 0, node, kk;
+	int rgb[3], try;
 	
-	size_t nm, nm2 = 0, node, kk;
-
 	float *tmp1[3], *tmp2 = NULL, *map[3], *intensity = NULL;
 
 	double  dx, dy, x_side, y_side, x0 = 0.0, y0 = 0.0;
@@ -90,6 +95,13 @@ int main (int argc, char **argv)
 	struct GMT_EDGEINFO edgeinfo;
 	struct GRDIMAGE_CTRL *Ctrl;
 
+#ifdef USE_GDAL
+	BOOLEAN do_indexed = FALSE;
+	unsigned char *r_table = NULL, *g_table = NULL, *b_table = NULL;
+	struct GDALREAD_CTRL *to_gdalread;
+	struct GD_CTRL *from_gdalread;
+#endif
+
 	void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g);
 	void *New_Grdimage_Ctrl (), Free_Grdimage_Ctrl (struct GRDIMAGE_CTRL *C);
 
@@ -129,6 +141,12 @@ int main (int argc, char **argv)
 					Ctrl->C.active = TRUE;
 					Ctrl->C.file = strdup (&argv[i][2]);
 					break;
+#ifdef USE_GDAL
+				case 'D':
+					Ctrl->D.active = TRUE;
+					if (argv[i][2] == 'r') Ctrl->D.mode = TRUE;
+					break;
+#endif
 				case 'E':
 					Ctrl->E.active = TRUE;
 					if (argv[i][2] == 'i')	/* Interpolate image to device resolution */
@@ -238,19 +256,29 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"grdimage %s - Plot grid files in 2-D\n\n", GMT_VERSION);
+#ifdef USE_GDAL
+		fprintf (stderr, "usage: grdimage <grd_z|grd_r grd_g grd_b> %s [%s] [-C<cpt_file>] [-D[r]] [-Ei|<dpi>] [-G[f|b]<rgb>]\n", GMT_J_OPT, GMT_B_OPT);
+#else
 		fprintf (stderr, "usage: grdimage <grd_z|grd_r grd_g grd_b> %s [%s] [-C<cpt_file>] [-Ei|<dpi>] [-G[f|b]<rgb>]\n", GMT_J_OPT, GMT_B_OPT);
+#endif
 		fprintf (stderr, "\t[-I<intensity_file>] [-K] [-M] [-N] [-O] [-P] [-Q] [%s] [-S[-]b|c|l|n[/<threshold>]] [-T]\n", GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[%s] [-V] [%s] [%s] [%s]\n\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr,"\t<grd_z> is data set to be plotted.  Its z-values are in user units and will be\n");
-		fprintf (stderr,"\t  converted to rgb colors via the cpt file.  Alternatively, give three separate\n");
-		fprintf (stderr,"\t  grid files that contain the red, green, and blue components in the 0-255 range.\n");
+		fprintf (stderr, "\t<grd_z> is data set to be plotted.  Its z-values are in user units and will be\n");
+		fprintf (stderr, "\t  converted to rgb colors via the cpt file.  Alternatively, give three separate\n");
+		fprintf (stderr, "\t  grid files that contain the red, green, and blue components in the 0-255 range.\n");
+#ifdef USE_GDAL
+		fprintf (stderr, "\t  If -D is used then <grd_z> is instead expected to be an image.\n");
+#endif
 		GMT_explain_option ('j');
 		fprintf (stderr,"\n\tOPTIONS:\n");
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-C color palette file to convert z to rgb\n");
+#ifdef USE_GDAL
+		fprintf (stderr, "\t-D is used to read an image via GDAL.  Append r to equate image region to -R region.\n");
+#endif
 		fprintf (stderr, "\t-E sets dpi for the projected grid which must be constructed\n");
 		fprintf (stderr, "\t   if -Jx or -Jm is not selected [Default gives same size as input grid]\n");
 		fprintf (stderr, "\t   Give i to do the interpolation in PostScript at device resolution.\n");
@@ -261,7 +289,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-N Do not clip image at the map boundary\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr, "\t-Q use PS Level 3 colormasking for nodes with z = NaN.\n");
+		fprintf (stderr, "\t-Q use PS Level 3 colormasking to make nodes with z = NaN transparent.\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\t-S Determines the interpolation mode (b = B-spline, c = bicubic, l = bilinear,\n");
 		fprintf (stderr, "\t   n = nearest-neighbor) [Default: bicubic]\n");
@@ -287,6 +315,9 @@ int main (int argc, char **argv)
 			error++;
 		}
 	}
+#ifdef USE_GDAL
+	if (Ctrl->D.active) {} else
+#endif
 	if (!Ctrl->C.file && !do_rgb) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify color palette table\n", GMT_program);
 		error++;
@@ -309,20 +340,122 @@ int main (int argc, char **argv)
 	}
 	if (error) exit (EXIT_FAILURE);
 
-	/* Get color palette file */
+#ifdef USE_GDAL
+	if (Ctrl->D.active) {
+		int pad;
+		/* One more test though */
+		if (Ctrl->I.active) {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR cannot use -D and -I options.\n", GMT_program);
+			exit (EXIT_FAILURE);
+		}
+		if (Ctrl->D.mode && !project_info.region_supplied) {
+			fprintf (stderr, "%s: GMT WARNING: -Dr without -R makes no sense. Ignoring -Dr.\n", GMT_program);
+			Ctrl->D.mode = FALSE;
+		}
+
+		/* Allocate new control structures */
+		to_gdalread = (struct GDALREAD_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GDALREAD_CTRL), "New_Gdalread_Ctrl");
+		from_gdalread = (struct GD_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GD_CTRL), "New_Gd_Ctrl");
+		to_gdalread->F.active = 1;
+
+		if (project_info.region_supplied && !Ctrl->D.mode) {
+			char	strR [128]; 
+			sprintf(strR, "-R%.10f/%.10f/%.10f/%.10f", west, east, south, north);
+			to_gdalread->R.region = strR;
+		}
+
+		pad = (project_info.projection != GMT_LINEAR || project_info.xyz_projection[0] == GMT_LOG10 || 
+			project_info.xyz_projection[0] == GMT_POW || project_info.xyz_projection[1] == GMT_LOG10 || 
+			project_info.xyz_projection[1] == GMT_POW || Ctrl->E.dpi > 0);
+		if (pad) {
+			to_gdalread->p.active = 1;
+			to_gdalread->p.pad = 2;
+		}
+
+		if (GMT_gdalread ( grdfile[0], to_gdalread, from_gdalread)) {
+			fprintf (stderr, "%s: ERROR reading file with gdalread.\n", GMT_program);
+			exit (EXIT_FAILURE);
+		}
+
+		if (!from_gdalread->UInt8.active) {
+			fprintf(stderr, "%s: Using data type other than byte (unsigned char) is not implemented\n", GMT_program);
+			exit (EXIT_FAILURE);
+		}
+
+		do_rgb = (from_gdalread->RasterCount == 3);
+		if (do_rgb) n_grids = 3;	/* To be compatible with original algo */
+
+		if (gmtdefs.verbose && from_gdalread->ProjectionRefPROJ4 != CNULL)
+			fprintf (stderr, "Data projection (Proj4 type)\n\t%s\n", from_gdalread->ProjectionRefPROJ4);
+
+		GMT_grd_init (&g_head[i], argc, argv, FALSE);
+		GMT_grd_init (&r_head[i], argc, argv, FALSE);
+		if (!Ctrl->D.mode) {
+			g_head[0].x_min = g_head[1].x_min = g_head[2].x_min = from_gdalread->hdr[0];
+			g_head[0].x_max = g_head[1].x_max = g_head[2].x_max = from_gdalread->hdr[1];
+			g_head[0].y_min = g_head[1].y_min = g_head[2].y_min = from_gdalread->hdr[2];
+			g_head[0].y_max = g_head[1].y_max = g_head[2].y_max = from_gdalread->hdr[3];
+			g_head[0].x_inc = g_head[1].x_inc = g_head[2].x_inc = from_gdalread->hdr[7];
+			g_head[0].y_inc = g_head[1].y_inc = g_head[2].y_inc = from_gdalread->hdr[8];
+		}
+		else {
+			g_head[0].x_min = g_head[1].x_min = g_head[2].x_min = west;
+			g_head[0].x_max = g_head[1].x_max = g_head[2].x_max = east;
+			g_head[0].y_min = g_head[1].y_min = g_head[2].y_min = south;
+			g_head[0].y_max = g_head[1].y_max = g_head[2].y_max = north;
+			g_head[0].x_inc = g_head[1].x_inc = g_head[2].x_inc = (g_head[0].x_max - g_head[0].x_min) / from_gdalread->RasterXsize;
+			g_head[0].y_inc = g_head[1].y_inc = g_head[2].y_inc = (g_head[0].y_max - g_head[0].y_min) / from_gdalread->RasterYsize;
+		}
+		g_head[0].nx = g_head[1].nx = g_head[2].nx = from_gdalread->RasterXsize;
+		g_head[0].ny = g_head[1].ny = g_head[2].ny = from_gdalread->RasterYsize;
+		g_head[0].node_offset = g_head[1].node_offset = g_head[2].node_offset = (int)from_gdalread->hdr[6];
+	}
+#endif
+
+	/* Get/calculate a color palette file */
+
+	if (!do_rgb) {
+		if (Ctrl->C.active)
+			GMT_read_cpt (Ctrl->C.file);
+#ifdef USE_GDAL
+		else if (Ctrl->D.active) {
+			if (from_gdalread->ColorMap == NULL && !strncmp(from_gdalread->ColorInterp,"Gray",4)) {
+    				r_table = (unsigned char *) malloc(256 * sizeof(*r_table));
+				for (i = 0; i < 256; i++) r_table[i] = (unsigned char)i;
+				GMT_gray = TRUE;
+			}
+			else if (from_gdalread->ColorMap != NULL) {
+    				r_table = (unsigned char *) malloc(256 * sizeof(*r_table));
+    				g_table = (unsigned char *) malloc(256 * sizeof(*g_table));
+    				b_table = (unsigned char *) malloc(256 * sizeof(*b_table));
+				for (i = 0; i < 256; i++) {
+					r_table[i] = from_gdalread->ColorMap[i*4];	/* 4 because color table is RGBA */
+					g_table[i] = from_gdalread->ColorMap[i*4 + 1];
+					b_table[i] = from_gdalread->ColorMap[i*4 + 2];
+				}
+				do_indexed = TRUE;		/* Now it will be RGB */
+				GMT_gray = FALSE;
+			}
+		}
+#endif
+	}
 
-	if (!do_rgb) GMT_read_cpt (Ctrl->C.file);
 	PS_interpolate = (Ctrl->E.device_dpi) ? -1 : +1;
 	PS_colormask   = (Ctrl->Q.active) ? -1 : +1;;
 
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Allocates memory and read data file\n", GMT_program);
 
-	for (i = 0; i < n_grids; i++) {
-		GMT_grd_init (&g_head[i], argc, argv, FALSE);
-		GMT_grd_init (&r_head[i], argc, argv, FALSE);
-
-		GMT_err_fail (GMT_read_grd_info (grdfile[i], &g_head[i]), grdfile[i]);
+#ifdef USE_GDAL
+	if (!Ctrl->D.active) {
+#endif
+		for (i = 0; i < n_grids; i++) {
+			GMT_grd_init (&g_head[i], argc, argv, FALSE);
+			GMT_grd_init (&r_head[i], argc, argv, FALSE);
+			GMT_err_fail (GMT_read_grd_info (grdfile[i], &g_head[i]), grdfile[i]);
+		}
+#ifdef USE_GDAL
 	}
+#endif
 	if (do_rgb) {	/* Must ensure all three grids are coregistered */
 		if (!(g_head[0].x_min == g_head[1].x_min && g_head[0].x_min == g_head[2].x_min)) error++;
 		if (!(g_head[0].y_min == g_head[1].y_min && g_head[0].y_min == g_head[2].y_min)) error++;
@@ -346,13 +479,19 @@ int main (int argc, char **argv)
 	}
 
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
+	
+	/* Determine if grid is to be projected */
+
+	need_to_project = (project_info.projection != GMT_LINEAR || project_info.xyz_projection[0] == GMT_LOG10 || project_info.xyz_projection[0] == GMT_POW || project_info.xyz_projection[1] == GMT_LOG10 || project_info.xyz_projection[1] == GMT_POW || Ctrl->E.dpi > 0);
 
 	/* Determine the wesn to be used to read the grid file; or exit if file is outside -R */
 
-	if (GMT_grd_setregion (&g_head[0], &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
+	if (GMT_grd_setregion (&g_head[0], &data_west, &data_east, &data_south, &data_north, need_to_project * Ctrl->S.interpolant)) {
+		/* No grid to plot; just do empty map and exit */
 		GMT_plotinit (argc, argv);
 		GMT_map_basemap ();
 		GMT_plotend ();
+		Free_Grdimage_Ctrl (Ctrl);	/* Deallocate control structure */
 		GMT_end (argc, argv);
 		exit (EXIT_SUCCESS);
 	}
@@ -360,25 +499,49 @@ int main (int argc, char **argv)
 	nx_f = g_head[0].nx;
 	ny_f = g_head[0].ny;
 
-	/* Read data */
-
 	nx = GMT_get_n (data_west, data_east, g_head[0].x_inc, g_head[0].node_offset);
 	ny = GMT_get_n (data_south, data_north, g_head[0].y_inc, g_head[0].node_offset);
 
+#ifdef USE_GDAL
+	if (Ctrl->D.active) {	/* Trust more on info from gdal to make it more stable against pixel vs grid registration troubles */
+		nx = from_gdalread->RasterXsize;
+		ny = from_gdalread->RasterYsize;
+	}
+#endif
+
+#if 0
+	fprintf (stderr, "%g %g %g %g / %g %g\n", g_head[0].x_min, g_head[0].x_max, g_head[0].y_min, g_head[0].y_max, g_head[0].x_inc, g_head[0].y_inc);
+	fprintf (stderr, "%g %g %g %g\n", data_west, data_east, data_south, data_north);
+	fprintf (stderr, "%li %li\n", nx, ny);
+#endif
+
 	GMT_boundcond_init (&edgeinfo);
 
-	need_to_project = (project_info.projection != GMT_LINEAR || project_info.xyz_projection[0] == GMT_LOG10 || project_info.xyz_projection[0] == GMT_POW || project_info.xyz_projection[1] == GMT_LOG10 || project_info.xyz_projection[1] == GMT_POW);
-	if (Ctrl->E.dpi > 0 || need_to_project) {
+	if (need_to_project) {
 		nm = GMT_get_nm (4 + nx, 4 + ny);
 		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
 	}
 	else
 		nm = GMT_get_nm (nx, ny);
 
-	for (i = 0; i < n_grids; i++) {
-		tmp1[i] = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
-		GMT_err_fail (GMT_read_grd (grdfile[i], &g_head[i], tmp1[i], data_west, data_east, data_south, data_north, GMT_pad, FALSE), grdfile[i]);
+	/* Read data */
+
+#ifdef USE_GDAL
+	if (Ctrl->D.active) {
+		for (i = 0; i < n_grids; i++) {
+			tmp1[i] = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
+			for (j = 0; j < nm; j++) tmp1[i][j] = (float)from_gdalread->UInt8.data[j+i*nm];
+		}
 	}
+	else {
+#endif
+		for (i = 0; i < n_grids; i++) {
+			tmp1[i] = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
+			GMT_err_fail (GMT_read_grd (grdfile[i], &g_head[i], tmp1[i], data_west, data_east, data_south, data_north, GMT_pad, FALSE), grdfile[i]);
+		}
+#ifdef USE_GDAL
+	}
+#endif
 
 	/* If given, get intensity file or compute intensities */
 
@@ -402,7 +565,7 @@ int main (int argc, char **argv)
 
 	for (i = 0; i < n_grids; i++) GMT_set_proj_limits (&r_head[i], &g_head[i]);
 
-	if (Ctrl->E.dpi > 0 || need_to_project) {	/* Need to resample the grd file */
+	if (need_to_project) {	/* Need to resample the grd file */
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: project grid files\n", GMT_program);
 
@@ -457,6 +620,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: Evaluate pixel colors\n", GMT_program);
 	}
 
+	if (Ctrl->Q.active) rgb_used = (unsigned char *) GMT_memory (VNULL, (size_t)(256*256*256), sizeof (unsigned char), GMT_program);
 	if (Ctrl->M.active || GMT_gray)
 		bitimage_8 = (unsigned char *) GMT_memory (VNULL, (size_t)nm, sizeof (char), GMT_program);
 	else {
@@ -470,39 +634,79 @@ int main (int argc, char **argv)
 	}
 	normal_x = !(project_info.projection == GMT_LINEAR && !project_info.xyz_pos[0] && !resampled);
 	normal_y = !(project_info.projection == GMT_LINEAR && !project_info.xyz_pos[1] && !resampled);
-	for (j = 0, byte = PS_colormask_off; j < r_head[0].ny; j++) {
-		kk = ((size_t)r_head[0].nx) * ((size_t)(normal_y ? j : r_head[0].ny - j - 1));
-		for (i = 0; i < r_head[0].nx; i++) {	/* Compute rgb for each pixel */
-			node = kk + (normal_x ? i : r_head[0].nx - i - 1);
-			if (do_rgb) {
-				for (k = 0; k < 3; k++) {
-					if (GMT_is_fnan (map[k][node])) {	/* If one is NaN they are all assumed to be NaN */
-						k = 3;
-						memcpy ((void *)rgb, (void *)GMT_bfn[GMT_NAN].rgb, 3 * sizeof (int));
-						index = -1;	/* Ensures no illumination done later */
-					}
-					else {				/* Set color, let index = 0 so illuminate test will work */
-						rgb[k] = irint (map[k][node]);	if (rgb[k] < 0) rgb[k] = 0; else if (rgb[k] > 255) rgb[k] = 255;	/* Clip */
-						index = 0;
+	
+	for (try = 0, done = FALSE; !done && try < 2; try++) {	/* Evaluate colors at least once, or twice if -Q and we need to select another NaN color */
+		for (j = 0, byte = PS_colormask_off; j < r_head[0].ny; j++) {
+			kk = ((size_t)r_head[0].nx) * ((size_t)(normal_y ? j : r_head[0].ny - j - 1));
+			for (i = 0; i < r_head[0].nx; i++) {	/* Compute rgb for each pixel */
+				node = kk + (normal_x ? i : r_head[0].nx - i - 1);
+#ifdef USE_GDAL
+				if (Ctrl->D.active && !do_rgb) {
+					index = -1;	/* Ensures no illumination done later */
+					rgb[0] = r_table[(int)map[0][node]];
+					if (do_indexed) {
+						rgb[1] = g_table[(int)map[0][node]];
+						rgb[2] = b_table[(int)map[0][node]];
 					}
 				}
-			}
-			else
-				index = GMT_get_rgb_from_z (map[0][node], rgb);
-			if (Ctrl->I.active && index != -1) GMT_illuminate (intensity[node], rgb);
+				else
+#endif
+				if (do_rgb) {
+					for (k = 0; k < 3; k++) {
+						if (GMT_is_fnan (map[k][node])) {	/* If one is NaN they are all assumed to be NaN */
+							k = 3;
+							memcpy ((void *)rgb, (void *)GMT_bfn[GMT_NAN].rgb, 3 * sizeof (int));
+							index = -1;	/* Ensures no illumination done later */
+						}
+						else {				/* Set color, let index = 0 so illuminate test will work */
+							rgb[k] = irint (map[k][node]);	if (rgb[k] < 0) rgb[k] = 0; else if (rgb[k] > 255) rgb[k] = 255;	/* Clip */
+							index = 0;
+						}
+					}
+				}
+				else
+					index = GMT_get_rgb_from_z (map[0][node], rgb);
 
-			if (GMT_gray)	/* Color table only has grays, pick r */
-				bitimage_8[byte++] = (unsigned char) rgb[0];
-			else if (Ctrl->M.active)	/* Convert rgb to gray using the GMT_YIQ transformation */
-				bitimage_8[byte++] = (unsigned char) GMT_YIQ (rgb);
-			else {
-				bitimage_24[byte++] = (unsigned char) rgb[0];
-				bitimage_24[byte++] = (unsigned char) rgb[1];
-				bitimage_24[byte++] = (unsigned char) rgb[2];
+				if (Ctrl->I.active && index != -1) GMT_illuminate (intensity[node], rgb);
+
+				if (GMT_gray)	/* Color table only has grays, pick r */
+					bitimage_8[byte++] = (unsigned char) rgb[0];
+				else if (Ctrl->M.active)	/* Convert rgb to gray using the GMT_YIQ transformation */
+					bitimage_8[byte++] = (unsigned char) GMT_YIQ (rgb);
+				else {
+					bitimage_24[byte++] = (unsigned char) rgb[0];
+					bitimage_24[byte++] = (unsigned char) rgb[1];
+					bitimage_24[byte++] = (unsigned char) rgb[2];
+					if (Ctrl->Q.active && index != -1) rgb_used[(rgb[0]*256 + rgb[1])*256+rgb[2]] = TRUE;	/* Keep track of all r/g/b combinations used except for NaN */
+				}
 			}
 		}
-	}
 
+		if (Ctrl->Q.active) {	/* Check that colormasking will work OK */
+			index = (GMT_bfn[GMT_NAN].rgb[0]*256 + GMT_bfn[GMT_NAN].rgb[1])*256+GMT_bfn[GMT_NAN].rgb[2];
+			if (rgb_used[index]) {	/* This r/g/b also appears in the image as a non-NaN color; find a replacement color */
+				for (index = 0, k = -1; k == -1 && index < 256*256*256; index++) if (!rgb_used[index]) k = index;
+				if (k == -1) {
+					fprintf (stderr, "%s: Warning: Colormasking will fail as there is no unused color that can represent transparency\n", GMT_program);
+					done = TRUE;
+				}
+				else {	/* Pick the first unused color and let it play the role of the NaN color for transparency */
+					bitimage_24[0] = (unsigned char)(k >> 16);
+					bitimage_24[1] = (unsigned char)((k >> 8) & 255);
+					bitimage_24[2] = (unsigned char)(k & 255);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: transparency color reset from %d/%d/%d to color %d/%d/%d\n", GMT_program, \
+						GMT_bfn[GMT_NAN].rgb[0], GMT_bfn[GMT_NAN].rgb[1], GMT_bfn[GMT_NAN].rgb[2], (int)bitimage_24[0], (int)bitimage_24[1], (int)bitimage_24[2]);
+					GMT_bfn[GMT_NAN].rgb[0] = (int)bitimage_24[0];
+					GMT_bfn[GMT_NAN].rgb[1] = (int)bitimage_24[1];
+					GMT_bfn[GMT_NAN].rgb[2] = (int)bitimage_24[2];
+				}	
+			}
+		}
+		else
+			done = TRUE;
+	}
+	if (Ctrl->Q.active) GMT_free ((void *)rgb_used);
+	
 	for (i = 0; i < n_grids; i++) GMT_free ((void *)map[i]);
 	if (Ctrl->I.active) GMT_free ((void *)intensity);
 
@@ -527,13 +731,12 @@ int main (int argc, char **argv)
 	if (GMT_gray) for (kk = 0, GMT_b_and_w = TRUE; GMT_b_and_w && kk < nm; kk++) if (!(bitimage_8[kk] == 0 || bitimage_8[kk] == 255)) GMT_b_and_w = FALSE;
 
 	if (GMT_b_and_w) {	/* Can get away with 1 bit image */
-		int nx8, shift, byte, b_or_w, nx_pixels;
-		size_t k8;
+		GMT_LONG nx8, shift, byte, b_or_w, nx_pixels, k8;
 		unsigned char *bit;
 
 		if (gmtdefs.verbose) fprintf (stderr, "[1-bit B/W image]\n");
 
-		nx8 = (int)ceil (r_head[0].nx / 8.0);
+		nx8 = (GMT_LONG)ceil (r_head[0].nx / 8.0);
 		nx_pixels = nx8 * 8;
 		bit = (unsigned char *) GMT_memory (VNULL, (size_t)(nx8 * r_head[0].ny), sizeof (char), GMT_program);
 
@@ -586,6 +789,20 @@ int main (int argc, char **argv)
 
 	Free_Grdimage_Ctrl (Ctrl);	/* Deallocate control structure */
 
+#ifdef USE_GDAL
+	if (Ctrl->D.active) {
+		if (r_table) free((void *) r_table);
+		if (g_table) {
+			free((void *) g_table);
+			free((void *) b_table);
+		}
+		GMT_free ((void *)to_gdalread);
+		GMT_free ((void *)from_gdalread->UInt8.data);
+		if (from_gdalread->ColorMap == NULL) GMT_free((void *) from_gdalread->ColorMap);
+		GMT_free((void *) from_gdalread);
+	}
+#endif
+
 	GMT_end (argc, argv);
 	exit (EXIT_SUCCESS);
 
@@ -599,7 +816,7 @@ void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g)
 	 * nx/ny are set accordingly.  Not that some of these may change
 	 * if GMT_grdproject_init is called at a later stage */
 
-	int i, j;
+	GMT_LONG i, j;
 	BOOLEAN all_lats = FALSE, all_lons = FALSE;
 	double lon, lat, x, y;
 
