@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdinfo.c,v 1.57 2009/01/09 04:02:33 guru Exp $
+ *	$Id: grdinfo.c,v 1.62 2009/09/09 23:27:03 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
  */
 
 #include "gmt.h"
+#include "grdinfo.h"	/* Array with gridtype names */
 
 struct GRDINFO_CTRL {
 	struct C {	/* -C */
@@ -36,7 +37,7 @@ struct GRDINFO_CTRL {
 	} F;
 	struct I {	/* -Idx[/dy] */
 		BOOLEAN active;
-		int status;
+		GMT_LONG status;
 		double xinc, yinc;
 	} I;
 	struct M {	/* -M */
@@ -44,7 +45,7 @@ struct GRDINFO_CTRL {
 	} M;
 	struct L {	/* -L[1|2] */
 		BOOLEAN active;
-		int norm;
+		GMT_LONG norm;
 	} L;
 	struct T {	/* -T<dz> */
 		BOOLEAN active;
@@ -56,9 +57,8 @@ int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, slow = FALSE;
 
-	int nfiles = 0, k, i, n_grds = 0, col, row;
-
-	size_t nm = 0, n_nan = 0, n = 0, ij, ij_min, ij_max;
+	GMT_LONG nfiles = 0, k, i, n_grds = 0, col, row;
+	GMT_LONG nm = 0, n_nan = 0, n = 0, ij, ij_min, ij_max;
 
 	float *a;
 
@@ -124,10 +124,6 @@ int main (int argc, char **argv)
 						Ctrl->L.norm |= 2;
 					else if (argv[i][2] == '1')
 						Ctrl->L.norm |= 1;
-					else {
-						error = TRUE;
-						fprintf (stderr, "%s: GMT SYNTAX ERROR -L option:  Choose either -L1 or -L2\n", GMT_program);
-					}
 					break;
 				case 'T':
 					Ctrl->T.active = TRUE;
@@ -145,7 +141,7 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "grdinfo %s - Extract information from netCDF grid files\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grdinfo <grdfiles> [-C] [-F] [-I[<dx>[/<dy>]]] [-L[1|2]] [-M]\n");
+		fprintf (stderr, "usage: grdinfo <grdfiles> [-C] [-F] [-I[<dx>[/<dy>]]] [-L[0|1|2]] [-M]\n");
 		fprintf (stderr, "	[-T<dz>] [%s]\n", GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -160,6 +156,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   If -C is set then rounding off will occur but no -R string is issued\n");
 		fprintf (stderr, "\t   If no argument is given then the -I<xinc>/<yinc> string is issued\n");
 		fprintf (stderr, "\t   If -I- is given then the grid's -R string is issued\n");
+		fprintf (stderr, "\t-L0 reports range of data by actually reading them (not from header)\n");
 		fprintf (stderr, "\t-L1 reports median and L1-scale of data set\n");
 		fprintf (stderr, "\t-L[2] reports mean, standard deviation, and rms of data set\n");
 		fprintf (stderr, "\t-M searches for the global min and max locations (x0,y0) and (x1,y1)\n");
@@ -213,7 +210,7 @@ int main (int argc, char **argv)
 
 		if (slow) {	/* Must determine the location of global min and max values */
 
-		 	nm= GMT_get_nm (grd.nx, grd.ny);
+		 	nm = GMT_get_nm (grd.nx, grd.ny);
 			a = (float *) GMT_memory ((void *)a, nm, sizeof (float), GMT_program);
 			if (GMT_err_pass (GMT_read_grd (argv[k], &grd, a, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), grd.name)) continue;
 
@@ -238,12 +235,12 @@ int main (int argc, char **argv)
 			}
 
 			n_nan = nm - n;
-			col = ij_min % grd.nx;
-			row = ij_min / grd.nx;
+			col = (GMT_LONG)(ij_min % grd.nx);
+			row = (GMT_LONG)(ij_min / grd.nx);
 			x_min = GMT_i_to_x (col, grd.x_min, grd.x_max, grd.x_inc, grd.xy_off, grd.nx);
 			y_min = GMT_j_to_y (row, grd.y_min, grd.y_max, grd.y_inc, grd.xy_off, grd.ny);
-			col = ij_max % grd.nx;
-			row = ij_max / grd.nx;
+			col = (GMT_LONG)(ij_max % grd.nx);
+			row = (GMT_LONG)(ij_max / grd.nx);
 			x_max = GMT_i_to_x (col, grd.x_min, grd.x_max, grd.x_inc, grd.xy_off, grd.nx);
 			y_max = GMT_j_to_y (row, grd.y_min, grd.y_max, grd.y_inc, grd.xy_off, grd.ny);
 		}
@@ -311,8 +308,14 @@ int main (int argc, char **argv)
 			fprintf (GMT_stdout, "%s: Title: %s\n", grd.name, grd.title);
 			fprintf (GMT_stdout, "%s: Command: %s\n", grd.name, grd.command);
 			fprintf (GMT_stdout, "%s: Remark: %s\n", grd.name, grd.remark);
-			fprintf (GMT_stdout, "%s: %s node registration used\n", grd.name, type[grd.node_offset]);
-			fprintf (GMT_stdout, "%s: Grid file format: %c%c (# %d)\n", grd.name, (int)GMT_grdformats[grd.type][0], (int)GMT_grdformats[grd.type][1], grd.type);
+			if (grd.node_offset == 0 || grd.node_offset == 1)
+				fprintf (GMT_stdout, "%s: %s node registration used\n", grd.name, type[grd.node_offset]);
+			else
+				fprintf (GMT_stdout, "%s: Unknown registration! Probably not a GMT grid\n", grd.name);
+			if (grd.type >= 0 && grd.type < N_GRD_TYPES)
+				fprintf (GMT_stdout, "%s: Grid file format: %c%c (# %ld) %s\n", grd.name, (int)GMT_grdformats[grd.type][0], (int)GMT_grdformats[grd.type][1], grd.type, GMT_grd_type[grd.type-1]);
+			else
+				fprintf (GMT_stdout, "%s: Unrecognized grid file format! Probably not a GMT grid\n", grd.name);
 			if (Ctrl->F.active) {
 				if ((fabs (grd.x_min) < 500.0) && (fabs (grd.x_max) < 500.0) && (fabs (grd.y_min) < 500.0) && (fabs (grd.y_max) < 500.0)) {
 					fprintf (GMT_stdout, "%s: x_min: %.7f\n", grd.name, grd.x_min);
@@ -427,7 +430,7 @@ int main (int argc, char **argv)
 		global_xmax = ceil  (global_xmax / Ctrl->I.xinc) * Ctrl->I.xinc;
 		global_ymin = floor (global_ymin / Ctrl->I.yinc) * Ctrl->I.yinc;
 		global_ymax = ceil  (global_ymax / Ctrl->I.yinc) * Ctrl->I.yinc;
-		fprintf (GMT_stdout, "%d\t", n_grds);
+		fprintf (GMT_stdout, "%ld\t", n_grds);
 		GMT_ascii_output_one (GMT_stdout, global_xmin, 0);	fputc ('\t', GMT_stdout);
 		GMT_ascii_output_one (GMT_stdout, global_xmax, 0);	fputc ('\t', GMT_stdout);
 		GMT_ascii_output_one (GMT_stdout, global_ymin, 1);	fputc ('\t', GMT_stdout);
