@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdpaste.c,v 1.23 2008/03/24 08:58:31 guru Exp $
+ *	$Id: grdpaste.c,v 1.27 2009/09/09 23:27:03 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -43,9 +43,9 @@ int main (int argc, char **argv)
 
 	char *grd_a, *grd_b, format[BUFSIZ];
 
-	int	i, way, one_or_zero, n_in = 0;
+	GMT_LONG	i, way, one_or_zero, n_in = 0;
 	
-	size_t nm;
+	GMT_LONG nm;
 
 	float	*c;
 
@@ -218,8 +218,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, format, Ctrl->G.file, hc.x_min, hc.x_max, hc.y_min, hc.y_max, hc.x_inc, hc.y_inc, hc.nx, hc.ny);
 	}
 
-	nm = ((size_t)hc.nx) * ((size_t)hc.ny);
-	c = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
+	nm = GMT_get_nm (hc.nx, hc.ny);
+	c = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 	hc.node_offset = ha.node_offset;
 	hc.z_scale_factor = ha.z_scale_factor;
 	hc.z_add_offset = ha.z_add_offset;
