@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdproject.c,v 1.43 2008/03/24 08:58:31 guru Exp $
+ *	$Id: grdproject.c,v 1.49 2009/09/09 23:27:03 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ struct GRDPROJECT_CTRL {
 	} D;
 	struct E {	/* -E<dpi> */
 		BOOLEAN active;
-		int dpi;
+		GMT_LONG dpi;
 	} E;
 	struct F {	/* -F */
 		BOOLEAN active;
@@ -62,12 +62,12 @@ struct GRDPROJECT_CTRL {
 	} M;
 	struct N {	/* -N<nx/ny> */
 		BOOLEAN active;
-		int nx, ny;
+		GMT_LONG nx, ny;
 	} N;
 	struct S {	/* -S[-]b|c|l|n[/<threshold>] */
 		BOOLEAN active;
 		BOOLEAN antialias;
-		int interpolant;
+		GMT_LONG interpolant;
 		double threshold;
 	} S;
 };
@@ -78,7 +78,7 @@ int main (int argc, char **argv)
 
 	char *infile, format[BUFSIZ], unit_name[GRD_UNIT_LEN], scale_unit_name[GRD_UNIT_LEN];
 
-	int i, j, unit = 0;
+	GMT_LONG i, j, unit = 0;
 	
 	GMT_LONG nm;
 	
@@ -153,7 +153,7 @@ int main (int argc, char **argv)
 					Ctrl->M.unit = argv[i][2];
 					break;
 				case 'N':
-					sscanf (&argv[i][2], "%d/%d", &Ctrl->N.nx, &Ctrl->N.ny);
+					sscanf (&argv[i][2], "%ld/%ld", &Ctrl->N.nx, &Ctrl->N.ny);
 					if (Ctrl->N.ny == 0) Ctrl->N.ny = Ctrl->N.nx;
 					Ctrl->N.active = TRUE;
 					break;
@@ -230,6 +230,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->D.xinc, &Ctrl->D.yinc, &Ctrl->F.active, &Ctrl->D.active);
+
 	if (!infile) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input file\n", GMT_program);
 		error++;
@@ -289,7 +291,7 @@ int main (int argc, char **argv)
 		}
 		else {			/* Do inverse transformation */
 			double x_c, y_c, lon_t, lat_t;
-			/* Obtain a first crude extimation of the good -R */
+			/* Obtain a first crude estimation of the good -R */
 			x_c = (w + e) / 2.; 		/* mid point of projected coords */
 			y_c = (s + n) / 2.; 
 			if (y_c > 0)
@@ -319,7 +321,7 @@ int main (int argc, char **argv)
 			GMT_parse_common_options (opt_R, &ww, &ee, &ss, &nn);
 			GMT_err_fail (GMT_map_setup (ww, ee, ss, nn), "");
 
-			/* Finaly obtain the good limits */
+			/* Finally obtain the good limits */
 			if (shift_xy) {
 				w -= Ctrl->C.easting;	e -= Ctrl->C.easting;
 				s -= Ctrl->C.northing;	n -= Ctrl->C.northing;
@@ -392,7 +394,7 @@ int main (int argc, char **argv)
 		GMT_err_fail (GMT_read_grd_info (infile, &r_head), infile);
 
 		GMT_boundcond_init (&edgeinfo);
-		nm = ((GMT_LONG)(4 + r_head.nx)) * ((GMT_LONG)(4 + r_head.ny));
+		nm = GMT_get_nm (4 + r_head.nx, 4 + r_head.ny);
 		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
 
 		rect = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
@@ -404,7 +406,8 @@ int main (int argc, char **argv)
 			Ctrl->N.ny = r_head.ny;
 		}
 		GMT_err_fail (GMT_grdproject_init (&g_head, Ctrl->D.xinc, Ctrl->D.yinc, Ctrl->N.nx, Ctrl->N.ny, Ctrl->E.dpi, offset), Ctrl->G.file);
-		geo = (float *) GMT_memory (VNULL, (size_t)(g_head.nx * g_head.ny), sizeof (float), GMT_program);
+		nm = GMT_get_nm (g_head.nx, g_head.ny);
+		geo = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 		if (gmtdefs.verbose) {
 			fprintf (stderr, "%s: Transform ", GMT_program);
 			fprintf (stderr, format, g_head.x_min, g_head.x_max, g_head.y_min, g_head.y_max);
@@ -459,7 +462,7 @@ int main (int argc, char **argv)
 		GMT_err_fail (GMT_read_grd_info (infile, &g_head), infile);
 
 		GMT_boundcond_init (&edgeinfo);
-		nm = ((GMT_LONG)(4 + g_head.nx)) * ((GMT_LONG)(4 + g_head.ny));
+		nm = GMT_get_nm (4 + g_head.nx, 4 + g_head.ny);
 		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
 
 		geo = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
@@ -496,7 +499,7 @@ int main (int argc, char **argv)
 		if (Ctrl->F.active) offset = !offset;	/* Toggle */
 
 		GMT_err_fail (GMT_grdproject_init (&r_head, Ctrl->D.xinc, Ctrl->D.yinc, Ctrl->N.nx, Ctrl->N.ny, Ctrl->E.dpi, offset), Ctrl->G.file);
-		nm = ((GMT_LONG)r_head.nx) * ((GMT_LONG)r_head.ny);
+		nm = GMT_get_nm (r_head.nx, r_head.ny);
 		rect = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 		GMT_grd_project (geo, &g_head, rect, &r_head, &edgeinfo, Ctrl->S.antialias, Ctrl->S.interpolant, Ctrl->S.threshold, FALSE);
 
