@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdtrend.c,v 1.30 2008/05/01 03:00:38 guru Exp $
+ *	$Id: grdtrend.c,v 1.35 2009/09/09 23:27:03 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -52,7 +52,8 @@ Version:	4
 Calls:		uses the QR solution of the Normal
 		equations furnished by Wm. Menke's
 		C routine "gauss".  We gratefully
-		acknowledge this contribution.
+		acknowledge this contribution, now
+		as GMT_gauss in gmt_vector.c
 Revised:	12-JUN-1998 PW, for GMT 3.1
 
 Remarks:
@@ -91,7 +92,7 @@ struct GRDTREND_CTRL {	/* All control options for this program (except common ar
 	struct N {	/* -N[r]<n_model> */
 		BOOLEAN active;
 		BOOLEAN robust;
-		int value;
+		GMT_LONG value;
 	} N;
 	struct T {	/* -T<trend.grd> */
 		BOOLEAN active;
@@ -103,16 +104,13 @@ struct GRDTREND_CTRL {	/* All control options for this program (except common ar
 	} W;
 };
 
-#define GRDTREND_MAX_TABLE_COLS 10	/* Used by Menke routine gauss  */
-
 int main(int argc, char **argv)
 {
 	BOOLEAN	error = FALSE, trivial, weighted;
 
 	char	*i_filename = NULL, format[BUFSIZ];
 
-	int	j, k, ierror = 0, iterations;
-	
+	GMT_LONG	j, k, ierror = 0, iterations;
 	GMT_LONG	i, nxy;
 
 	float	*data;		/* Pointer for array from input grid file  */
@@ -131,15 +129,14 @@ int main(int argc, char **argv)
 	struct GRD_HEADER head_d, head_w;
 	struct GRDTREND_CTRL *Ctrl;
 
-	void	gauss(double *a, double *vec, int n, int nstore, double test, int *ierror, int itriag);		/* QR solution of the Normal equations  */
-	void	set_up_vals(double *val, int nval, double vmin, double vmax, double dv, int pixel_reg);		/* Store x[i], y[j] once for all to save time  */
-	void	grd_trivial_model(float *data, int nx, int ny, double *xval, double *yval, double *gtd, int n_model);	/* Fit trivial models.  See Remarks above.  */
-	void	compute_trend(float *trend, int nx, int ny, double *xval, double *yval, double *gtd, int n_model, double *pstuff);	/* Find trend from a model  */
+	void	set_up_vals(double *val, GMT_LONG nval, double vmin, double vmax, double dv, GMT_LONG pixel_reg);		/* Store x[i], y[j] once for all to save time  */
+	void	grd_trivial_model(float *data, GMT_LONG nx, GMT_LONG ny, double *xval, double *yval, double *gtd, GMT_LONG n_model);	/* Fit trivial models.  See Remarks above.  */
+	void	compute_trend(float *trend, GMT_LONG nx, GMT_LONG ny, double *xval, double *yval, double *gtd, GMT_LONG n_model, double *pstuff);	/* Find trend from a model  */
 	void	compute_resid(float *data, float *trend, float *resid, GMT_LONG nxy);	/* Find residuals from a trend  */
 	void	compute_chisq(float *resid, float *weight, GMT_LONG nxy, double *chisq, double scale);	/* Find Chi-Squared from weighted residuals  */
 	void	compute_robust_weight(float *resid, float *weight, GMT_LONG nxy, double *scale);	/* Find weights from residuals  */
-	void	write_model_parameters(double *gtd, int n_model);	/* Do reports if gmtdefs.verbose == TRUE  */
-	void	load_gtg_and_gtd(float *data, int nx, int ny, double *xval, double *yval, double *pstuff, double *gtg, double *gtd, int n_model, float *weight, int weighted);		/* Fill normal equations matrices  */
+	void	write_model_parameters(double *gtd, GMT_LONG n_model);	/* Do reports if gmtdefs.verbose == TRUE  */
+	void	load_gtg_and_gtd(float *data, GMT_LONG nx, GMT_LONG ny, double *xval, double *yval, double *pstuff, double *gtg, double *gtd, GMT_LONG n_model, float *weight, GMT_LONG weighted);		/* Fill normal equations matrices  */
 	void	*New_Grdtrend_Ctrl (), Free_Grdtrend_Ctrl (struct GRDTREND_CTRL *C);
 
 /* Execution begins here with loop over arguments:  */
@@ -246,7 +243,7 @@ int main(int argc, char **argv)
 	GMT_err_fail (GMT_read_grd_info (i_filename, &head_d), i_filename);
 
 	GMT_grd_init (&head_d, argc, argv, TRUE);
-	nxy = ((GMT_LONG)head_d.nx) * ((GMT_LONG)head_d.ny);
+	nxy = GMT_get_nm (head_d.nx, head_d.ny);
 	data = (float *) GMT_memory (VNULL, (size_t)nxy, sizeof (float), GMT_program);
 	GMT_err_fail (GMT_read_grd (i_filename, &head_d, data, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), i_filename);
 
@@ -312,9 +309,9 @@ int main(int argc, char **argv)
 	else {	/* Problem is not trivial  !!  */
 
 		load_gtg_and_gtd(data, head_d.nx, head_d.ny, xval, yval, pstuff, gtg, gtd, Ctrl->N.value, weight, weighted);
-		gauss(gtg, gtd, Ctrl->N.value, Ctrl->N.value, zero_test, &ierror, 1);
+		GMT_gauss (gtg, gtd, Ctrl->N.value, Ctrl->N.value, zero_test, &ierror, 1);
 		if (ierror) {
-			fprintf (stderr,"%s:  Gauss returns error code %d\n", GMT_program, ierror);
+			fprintf (stderr,"%s:  Gauss returns error code %ld\n", GMT_program, ierror);
 			exit (EXIT_FAILURE);
 		}
 		compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, Ctrl->N.value, pstuff);
@@ -329,9 +326,9 @@ int main(int argc, char **argv)
 				for(k = 0; k < Ctrl->N.value; k++) old[k] = gtd[k];
 				compute_robust_weight(resid, weight, nxy, &scale);
 				load_gtg_and_gtd(data, head_d.nx, head_d.ny, xval, yval, pstuff, gtg, gtd, Ctrl->N.value, weight, weighted);
-				gauss(gtg, gtd, Ctrl->N.value, Ctrl->N.value, zero_test, &ierror, 1);
+				GMT_gauss (gtg, gtd, Ctrl->N.value, Ctrl->N.value, zero_test, &ierror, 1);
 				if (ierror) {
-					fprintf (stderr,"%s:  Gauss returns error code %d\n", GMT_program, ierror);
+					fprintf (stderr,"%s:  Gauss returns error code %ld\n", GMT_program, ierror);
 					exit (EXIT_FAILURE);
 				}
 				compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, Ctrl->N.value, pstuff);
@@ -387,9 +384,9 @@ int main(int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void set_up_vals (double *val, int nval, double vmin, double vmax, double dv, int pixel_reg)
+void set_up_vals (double *val, GMT_LONG nval, double vmin, double vmax, double dv, GMT_LONG pixel_reg)
 {
-	int	i;
+	GMT_LONG	i;
 	double  v, middle, drange, true_min, true_max;
 
 	true_min = (pixel_reg) ? vmin + 0.5 * dv : vmin;
@@ -407,7 +404,7 @@ void set_up_vals (double *val, int nval, double vmin, double vmax, double dv, in
 	return;
 }
 
-void load_pstuff (double *pstuff, int n_model, double x, double y, int newx, int newy)
+void load_pstuff (double *pstuff, GMT_LONG n_model, double x, double y, GMT_LONG newx, GMT_LONG newy)
 {
 	/* If either x or y has changed, compute new Legendre polynomials as needed  */
 
@@ -430,11 +427,11 @@ void load_pstuff (double *pstuff, int n_model, double x, double y, int newx, int
 	return;
 }
 
-void compute_trend (float *trend, int nx, int ny, double *xval, double *yval, double *gtd, int n_model, double *pstuff)
+void compute_trend (float *trend, GMT_LONG nx, GMT_LONG ny, double *xval, double *yval, double *gtd, GMT_LONG n_model, double *pstuff)
 {
-	int	i, j, k;
+	GMT_LONG	i, j, k;
 	GMT_LONG	ij;
-	void	load_pstuff(double *pstuff, int n_model, double x, double y, int newx, int newy);		/* Compute Legendre polynomials of x[i],y[j] as needed  */
+	void	load_pstuff(double *pstuff, GMT_LONG n_model, double x, double y, GMT_LONG newx, GMT_LONG newy);		/* Compute Legendre polynomials of x[i],y[j] as needed  */
 
 	for (ij = 0, j = 0; j < ny; j++) {
 		for (i = 0; i < nx; i++, ij++) {
@@ -462,14 +459,14 @@ void compute_resid (float *data, float *trend, float *resid, GMT_LONG nxy)
 	return;
 }
 
-void grd_trivial_model (float *data, int nx, int ny, double *xval, double *yval, double *gtd, int n_model)
+void grd_trivial_model (float *data, GMT_LONG nx, GMT_LONG ny, double *xval, double *yval, double *gtd, GMT_LONG n_model)
 {
 	/* Routine to fit up elementary polynomial model of grd data, 
 	model = gtd[0] + gtd[1]*x + gtd[2]*y + gtd[3] * x * y,
 	where x,y are normalized to range [-1,1] and there are no
 	NaNs in grid file, and problem is unweighted least squares.  */
 
-	int	i, j;
+	GMT_LONG	i, j;
 	GMT_LONG	ij;
 	double	x2, y2, sumx2 = 0.0, sumy2 = 0.0, sumx2y2 = 0.0;
 
@@ -565,9 +562,9 @@ void compute_robust_weight (float *resid, float *weight, GMT_LONG nxy, double *s
 	return;
 }
 
-void write_model_parameters (double *gtd, int n_model)
+void write_model_parameters (double *gtd, GMT_LONG n_model)
 {
-	int	i;
+	GMT_LONG	i;
 	char	pbasis[10][16];
 	char format[BUFSIZ];
 
@@ -588,7 +585,7 @@ void write_model_parameters (double *gtd, int n_model)
 	return;
 }
 
-void load_gtg_and_gtd (float *data, int nx, int ny, double *xval, double *yval, double *pstuff, double *gtg, double *gtd, int n_model, float *weight, int weighted)
+void load_gtg_and_gtd (float *data, GMT_LONG nx, GMT_LONG ny, double *xval, double *yval, double *pstuff, double *gtg, double *gtd, GMT_LONG n_model, float *weight, GMT_LONG weighted)
 {
 	/* Routine to load the matrix G'G (gtg) and vector G'd (gtd)
 	for the normal equations.  Routine uses indices i,j to refer
@@ -598,9 +595,9 @@ void load_gtg_and_gtd (float *data, int nx, int ny, double *xval, double *yval,
 	loading only lower triangular part of gtg and then filling
 	by symmetry after i,j loop.  */
 
-	int	i, j, k, l, n_used;
+	GMT_LONG	i, j, k, l, n_used;
 	GMT_LONG	ij;
-	void	load_pstuff(double *pstuff, int n_model, double x, double y, int newx, int newy);		/* Compute Legendre polynomials of x[i],y[j] as needed  */
+	void	load_pstuff(double *pstuff, GMT_LONG n_model, double x, double y, GMT_LONG newx, GMT_LONG newy);		/* Compute Legendre polynomials of x[i],y[j] as needed  */
 
 /*	First zero things out to start:  */
 
@@ -666,142 +663,6 @@ void load_gtg_and_gtd (float *data, int nx, int ny, double *xval, double *yval,
 	return;
 }
 
-void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror, int itriag)
-{
- 
-/* subroutine gauss, by william menke */
-/* july 1978 (modified feb 1983, nov 85) */
- 
-/* a subroutine to solve a system of n linear equations in n unknowns*/
-/* where n doesn't exceed GRDTREND_MAX_TABLE_COLS */
-/* gaussian reduction with partial pivoting is used */
-/*      a               (sent, destroyed)       n by n matrix           */
-/*      vec             (sent, overwritten)     n vector, replaced w/ solution*/
-/*      nstore          (sent)                  dimension of a  */
-/*      test            (sent)                  div by zero check number*/
-/*      ierror          (returned)              zero on no error*/
-/*      itriag          (sent)                  matrix triangularized only*/
-/*                                               on TRUE useful when solving*/
-/*                                               multiple systems with same a */
-        static int isub[GRDTREND_MAX_TABLE_COLS], l1;
-        int line[GRDTREND_MAX_TABLE_COLS], iet, ieb, i = 0, j, k, l, j2;
-        double big, testa, b, sum;
-
-        iet=0;  /* initial error flags, one for triagularization*/
-        ieb=0;  /* one for backsolving */
-
-/* triangularize the matrix a*/
-/* replacing the zero elements of the triangularized matrix */
-/* with the coefficients needed to transform the vector vec */
-
-        if (itriag) {   /* triangularize matrix */
- 
-                for( j=0; j<n; j++ ) {      /*line is an array of flags*/
-                        line[j]=0; 
-                        /* elements of a are not moved during pivoting*/
-                        /* line=0 flags unused lines */
-                        }    /*end for j*/
-                        
-                for( j=0; j<n-1; j++ ) {
-                        /*  triangularize matrix by partial pivoting */
-                       big = 0.0; /* find biggest element in j-th column*/
-                                  /* of unused portion of matrix*/
-                       for( l1=0; l1<n; l1++ ) {
-                               if( line[l1]==0 ) {
-                                       testa=(double) fabs((double) (*(a+l1*nstore+j)));
-                                       if (testa>big) {
-                                                i=l1;
-                                                big=testa;
-                                                } /*end if*/
-                                        } /*end if*/
-                                } /*end for l1*/
-                       if( big<=test) {   /* test for div by 0 */
-                               iet=1;
-                               } /*end if*/
- 
-                       line[i]=1;  /* selected unused line becomes used line */
-                       isub[j]=i;  /* isub points to j-th row of tri. matrix */
- 
-                       sum=1.0/(*(a+i*nstore+j)); 
-                                /*reduce matrix towards triangle */
-                       for( k=0; k<n; k++ ) {
-                                if( line[k]==0 ) {
-                                        b=(*(a+k*nstore+j))*sum;
-                                        for( l=j+1; l<n; l++ ) {
-                                               *(a+k*nstore+l)= (*(a+k*nstore+l)) -b*(*(a+i*nstore+l));
-                                               } /*end for l*/
-                                       *(a+k*nstore+j)=b;
-                                        } /*end if*/
-                                } /*end for k*/
-                        } /*end for j*/
- 
-               for( j=0; j<n; j++ ) {
-                        /*find last unused row and set its pointer*/
-                        /*  this row contains the apex of the triangle*/
-                        if( line[j]==0) {
-                                l1=j;   /*apex of triangle*/
-                                isub[n-1]=j;
-                                break;
-                                } /*end if*/
-                        } /*end for j*/
- 
-                } /*end if itriag true*/
-                
-        /*start backsolving*/
-        
-        for( i=0; i<n; i++ ) {  /* invert pointers. line(i) now gives*/
-                                /* row no in triang matrix of i-th row*/
-                                /* of actual matrix */
-                line[isub[i]] = i;
-                } /*end for i*/
- 
-        for( j=0; j<n-1; j++) { /*transform the vector to match triang. matrix*/
-               b=vec[isub[j]];
-               for( k=0; k<n; k++ ) {
-                      if (line[k]>j) {  /* skip elements outside of triangle*/
-                                vec[k]=vec[k]-(*(a+k*nstore+j))*b;
-                                } /*end if*/
-                        } /*end for k*/
-                } /*end for j*/
- 
-      b = *(a+l1*nstore+(n-1));   /*apex of triangle*/
-      if( ((double)fabs( (double) b))<=test) {
-                /*check for div by zero in backsolving*/
-                ieb=2;
-                } /*end if*/
-      vec[isub[n-1]]=vec[isub[n-1]]/b;
- 
-      for( j=n-2; j>=0; j-- ) { /* backsolve rest of triangle*/
-                sum=vec[isub[j]];
-                for( j2=j+1; j2<n; j2++ ) {
-                        sum = sum - vec[isub[j2]] * (*(a+isub[j]*nstore+j2));
-                        } /*end for j2*/
-                        b = *(a+isub[j]*nstore+j);
-               if( ((double)fabs((double)b))<=test) {
-                        /* test for div by 0 in backsolving */
-                        ieb=2;
-                        } /*end if*/
-                vec[isub[j]]=sum/b;   /*solution returned in vec*/
-                } /*end for j*/
-
-/*put the solution vector into the proper order*/
-
-      for( i=0; i<n; i++ ) {    /* reorder solution */
-                for( k=i; k<n; k++ ) {  /* search for i-th solution element */
-                        if( line[k]==i ) {
-                                j=k;
-                                break;
-                                } /*end if*/
-                        } /*end for k*/
-               b = vec[j];       /* swap solution and pointer elements*/
-               vec[j] = vec[i];
-               vec[i] = b;
-               line[j] = line[i];
-                } /*end for i*/
- 
-      *ierror = iet + ieb;   /* set final error flag*/
-}
-
 void *New_Grdtrend_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDTREND_CTRL *C;
 	
