@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: grdview.c,v 1.93 2008/05/01 03:00:38 guru Exp $
+ *	$Id: grdview.c,v 1.107 2009/09/09 23:27:03 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -81,13 +81,13 @@ struct GRDVIEW_CTRL {
 		BOOLEAN outline;
 		BOOLEAN mask;
 		BOOLEAN monochrome;
-		int mode;	/* GRDVIEW_MESH, GRDVIEW_SURF, GRDVIEW_IMAGE */
-		int dpi;
+		GMT_LONG mode;	/* GRDVIEW_MESH, GRDVIEW_SURF, GRDVIEW_IMAGE */
+		GMT_LONG dpi;
 		struct GMT_FILL fill;
 	} Q;
 	struct S {	/* -S<smooth> */
 		BOOLEAN active;
-		int value;
+		GMT_LONG value;
 	} S;
 	struct T {	/* -T[s][o[<pen>] */
 		BOOLEAN active;
@@ -129,14 +129,13 @@ int main (int argc, char **argv)
 	char *topofile;
 	char *c_method[2] = {"colorimage", "colortiles",};
 
-	int 	j, n_edges, max, i_bin, j_bin, i_bin_old, j_bin_old;
-	int	side, way, nx_f, ny_f, nx, ny;
-	int	two, sw, se, nw, ne, id, n4, nk, c, rgb[3], i_start, i_stop;
-	int	j_start, j_stop, i_inc, j_inc, q_set = 0, n_commas, ii, jj, n_out;
-	int	PS_colormask_off = 0, PS_colormask,*edge;
-
-	GMT_LONG bin, i, ij, k, k1, n, nm, nm2, nm_use, mx, my, n_drape = 0, bin_inc[4], ij_inc[4];
+	GMT_LONG j, n_edges, max, i_bin, j_bin, i_bin_old, j_bin_old, nx_f, ny_f, nx, ny;
+	GMT_LONG sw, se, nw, ne, id, n4, nk, c, i_start, i_stop, j_start, j_stop, i_inc, j_inc, ii, jj;
+	GMT_LONG bin, i, ij, k, k1, n, nm, nm2, nm_use, mx, my, n_drape = 0, n_out, bin_inc[4], ij_inc[4];
+	GMT_LONG PS_colormask_off = 0, PS_colormask, n_commas, q_set = 0, two, side, way, *edge;
 
+	int rgb[3];
+	
 	float *grd[3], *zgrd, *intensity = NULL, *topo;
 
 	double cval, x_left, x_right, y_top, y_bottom, small = GMT_SMALL, z_ave, this_intensity = 0.0, *xval, *yval;
@@ -152,18 +151,18 @@ int main (int argc, char **argv)
 	struct GRDVIEW_BIN *binij = NULL;
 	struct GRDVIEW_CTRL *Ctrl;
 
-	void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int cont, GMT_LONG *n);
-	void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int cont, GMT_LONG *n);
+	void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], GMT_LONG cont, GMT_LONG *n);
+	void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], GMT_LONG cont, GMT_LONG *n);
 	double get_z_ave (double v[], double next_up, GMT_LONG n);
-	void add_node (double x[], double y[], double z[], double v[], GMT_LONG *k, int node, double X_vert[], double Y_vert[], float topo[], float zgrd[], GMT_LONG ij, GMT_LONG bin);
-	void paint_it (double x[], double y[], GMT_LONG n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity);
-	double get_intensity (float *intensity, GMT_LONG k, int nx);
+	void add_node (double x[], double y[], double z[], double v[], GMT_LONG *k, GMT_LONG node, double X_vert[], double Y_vert[], float topo[], float zgrd[], GMT_LONG ij, GMT_LONG bin);
+	void paint_it (double x[], double y[], GMT_LONG n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity, BOOLEAN outline);
+	double get_intensity (float *intensity, GMT_LONG k, GMT_LONG nx);
 	struct GRDVIEW_CONT *get_cont_struct (GMT_LONG bin, struct GRDVIEW_BIN *binij, double value);
 	struct GRDVIEW_POINT *get_point (double x, double y);
-	void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEAN draw_plane, double plane_level);
-	int pixel_inside (int ip, int jp, int *ix, int *iy, GMT_LONG bin, GMT_LONG bin_inc[]);
-	int quick_idist(int x1, int y1, int x2, int y2);
-	int get_side (double x, double y, double x_left, double y_bottom, double xinc, double yinc, double dx2, double dy2);
+	void grdview_init_setup (struct GRD_HEADER *header, float *topo, GMT_LONG two, BOOLEAN draw_plane, double plane_level);
+	GMT_LONG pixel_inside (GMT_LONG ip, GMT_LONG jp, GMT_LONG *ix, GMT_LONG *iy, GMT_LONG bin, GMT_LONG bin_inc[]);
+	GMT_LONG quick_idist(GMT_LONG x1, GMT_LONG y1, GMT_LONG x2, GMT_LONG y2);
+	GMT_LONG get_side (double x, double y, double x_left, double y_bottom, double xinc, double yinc, double dx2, double dy2);
 	BOOLEAN GMT_rgb_is_nan_rgb (int rgb[]);
 	void *New_Grdview_Ctrl (), Free_Grdview_Ctrl (struct GRDVIEW_CTRL *C);
 
@@ -174,7 +173,11 @@ int main (int argc, char **argv)
 	topofile = CNULL;
 	GMT_3D_mode = 1;	/* Only do background axis first; do foreground at end */
 	grd[0] = grd[1] = grd[2] = (float *)NULL;
-
+	GMT_grd_init (&header, argc, argv, FALSE);
+	GMT_grd_init (&t_head, argc, argv, FALSE);
+	GMT_grd_init (&i_head, argc, argv, FALSE);
+	for (i = 0; i < 3; i++) GMT_grd_init (&d_head[i], argc, argv, FALSE);
+	
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
@@ -206,7 +209,7 @@ int main (int argc, char **argv)
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
-					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
+					error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'G':
 					Ctrl->G.active = TRUE;
@@ -362,7 +365,7 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "grdview %s - Plot topofiles in 3-D\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grdview <topofile> %s [-B<tickinfo>] [-C<cpt_file>] [-E<azimuth/elevation>]\n", GMT_J_OPT);
+		fprintf (stderr, "usage: grdview <topofile> %s [-B<tickinfo>] [-C<cpt_file>] [%s]\n", GMT_J_OPT, GMT_E_OPT);
 		fprintf (stderr, "\t[-G<drapefile> | -G<grd_r>,<grd_g>,<grd_b>] [-I<intensfile>] [%s] [-K] [-L[<flags>]]\n", GMT_Jz_OPT);
 		fprintf (stderr, "\t[-N<level>[/<r/g/b>]] [-O] [-P] [-Q<type>[g]] [%s]\n", GMT_Rgeoz_OPT);
 		fprintf (stderr, "\t[-S<smooth>] [-T[s][o[<pen>]]] [%s] [-V] [-W<type><pen>]\n\t[%s] [%s] [-Z<z_level>] [%s]\n\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT);
@@ -375,8 +378,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-C Color palette file\n");
-		fprintf (stderr, "\t-E Draw perspective from viewpoint at azimuth <v_az>, elevation <v_el> (degrees).\n");
-		fprintf (stderr, "\t   Default is looking straight down [180/90].\n");
+		GMT_explain_option ('E');
 		fprintf (stderr, "\t-G <drapefile> rather than <topofile> is the data set to color-code.\n");
 		fprintf (stderr, "\t   Use <topofile> as the relief and \'drape\' the image on top.\n");
 		fprintf (stderr, "\t   Note that -Jz and -N always refers to the <topofile>\n");
@@ -533,7 +535,8 @@ int main (int argc, char **argv)
 
 	/* Determine the wesn to be used to read the grid file */
 
-	if (GMT_grd_setregion (&t_head, &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
+	if (GMT_grd_setregion (&t_head, &data_west, &data_east, &data_south, &data_north, BCR_BILINEAR)) {
+		/* No grid to plot; just do empty map and exit */
 		GMT_plotinit (argc, argv);
 		if (project_info.three_D) {
 			ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
@@ -550,31 +553,14 @@ int main (int argc, char **argv)
 
 	/* Read data */
 
-	if (subset) {	/* Possibly extend the subset by 1 row/col since tiles need 4 corners to work */
-		BOOLEAN global;
-		global = (GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (t_head.x_max, t_head.x_min));
-		if (global && data_west > project_info.w)
-			data_west -= t_head.x_inc;
-		else if (GMT_x_to_i (data_west, t_head.x_min, t_head.x_inc, t_head.xy_off, t_head.nx) > 0)
-			data_west -= t_head.x_inc;
-		if (global && data_east < project_info.e)
-			data_east += t_head.x_inc;
-		else if (GMT_x_to_i (data_east, t_head.x_min, t_head.x_inc, t_head.xy_off, t_head.nx) < (t_head.nx)) {
-			data_east += t_head.x_inc;
-			if (data_east > t_head.x_max) data_east -= t_head.x_inc;
-		}
-		if (GMT_y_to_j (data_south, t_head.y_min, t_head.y_inc, t_head.xy_off, t_head.ny) < (t_head.ny)) data_south -= t_head.y_inc;
-		if (GMT_y_to_j (data_north, t_head.y_min, t_head.y_inc, t_head.xy_off, t_head.ny) > 0) data_north += t_head.y_inc;
-		if (data_south < t_head.y_min) data_south += t_head.y_inc;
-	}
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Processing shape file\n", GMT_program);
 
 	nx = GMT_get_n (data_west, data_east, t_head.x_inc, t_head.node_offset);
 	ny = GMT_get_n (data_south, data_north, t_head.y_inc, t_head.node_offset);
 	mx = nx + 2 * two;
 	my = ny + 2 * two;
-	nm = nx * ny;
-	nm2 = mx * my;
+	nm = GMT_get_nm (nx, ny);
+	nm2 = GMT_get_nm (mx, my);
 	topo = (float *) GMT_memory (VNULL, (size_t)nm2, sizeof (float), GMT_program);
 	GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = two;
 	GMT_err_fail (GMT_read_grd (topofile, &t_head, topo, data_west, data_east, data_south, data_north, GMT_pad, FALSE), topofile);
@@ -637,8 +623,8 @@ int main (int argc, char **argv)
 
 	if (get_contours) {	/* Need to find contours */
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Find contours\n", GMT_program);
-		n_edges = header.ny * (int )ceil (header.nx / 16.0);
-		edge = (int *) GMT_memory (VNULL, (size_t)n_edges, sizeof (int), GMT_program);
+		n_edges = header.ny * (GMT_LONG )ceil (header.nx / 16.0);
+		edge = (GMT_LONG *) GMT_memory (VNULL, (size_t)n_edges, sizeof (GMT_LONG), GMT_program);
 		binij = (struct GRDVIEW_BIN *) GMT_memory (VNULL, (size_t)nm, sizeof (struct GRDVIEW_BIN), GMT_program);
 		small = GMT_SMALL * (header.z_max - header.z_min);
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Trace and bin contours...\n", GMT_program);
@@ -668,8 +654,8 @@ int main (int argc, char **argv)
 
 					i_bin_old = j_bin_old = -1;
 					for (i = 1; i < n; i++) {
-						i_bin = (int)floor (((0.5 * (x[i-1] + x[i]) - header.x_min) / header.x_inc) - header.xy_off);
-						j_bin = (int)floor (((header.y_max - 0.5 * (y[i-1] + y[i])) / header.y_inc) - header.xy_off) + 1;
+						i_bin = (GMT_LONG)floor (((0.5 * (x[i-1] + x[i]) - header.x_min) / header.x_inc) - header.xy_off);
+						j_bin = (GMT_LONG)floor (((header.y_max - 0.5 * (y[i-1] + y[i])) / header.y_inc) - header.xy_off) + 1;
 						if (i_bin != i_bin_old || j_bin != j_bin_old) {	/* Entering new bin */
 							bin = j_bin * header.nx + i_bin;
 							this_cont = get_cont_struct (bin, binij, cval);
@@ -784,44 +770,44 @@ int main (int argc, char **argv)
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[0], z_project.corner_y[0], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[1], z_project.corner_y[1], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE);
 			}
 			else {
 				for (i = 0; i < header.nx; i++) GMT_geoz_to_xy (header.x_min + i * header.x_inc + delx, header.y_min + dely, Ctrl->N.level, &xx[i], &yy[i]);
-				ps_line (xx, yy, (GMT_LONG)header.nx, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.nx, 3, TRUE);
 			}
 		}
 		if (!z_project.draw[2])	{	/* Northern side */
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[3], z_project.corner_y[3], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[2], z_project.corner_y[2], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE);
 			}
 			else {
 				for (i = 0; i < header.nx; i++) GMT_geoz_to_xy (header.x_min + i * header.x_inc + delx, header.y_max - dely, Ctrl->N.level, &xx[i], &yy[i]);
-				ps_line (xx, yy, (GMT_LONG)header.nx, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.nx, 3, TRUE);
 			}
 		}
 		if (!z_project.draw[3])	{	/* Western side */
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[0], z_project.corner_y[0], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[3], z_project.corner_y[3], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE);
 			}
 			else {
 				for (j = 0; j < header.ny; j++) GMT_geoz_to_xy (header.x_min + delx, header.y_max - j * header.y_inc - dely, Ctrl->N.level, &xx[j], &yy[j]);
-				ps_line (xx, yy, (GMT_LONG)header.ny, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.ny, 3, TRUE);
 			}
 		}
 		if (!z_project.draw[1])	{	/* Eastern side */
 			if (!project_info.region) {
 				GMT_geoz_to_xy (z_project.corner_x[1], z_project.corner_y[1], Ctrl->N.level, &xx[0], &yy[0]);
 				GMT_geoz_to_xy (z_project.corner_x[2], z_project.corner_y[2], Ctrl->N.level, &xx[1], &yy[1]);
-				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)2, 3, TRUE);
 			}
 			else {
 				for (j = 0; j < header.ny; j++) GMT_geoz_to_xy (header.x_max - delx, header.y_max - j * header.y_inc - dely, Ctrl->N.level, &xx[j], &yy[j]);
-				ps_line (xx, yy, (GMT_LONG)header.ny, 3, TRUE, TRUE);
+				ps_line (xx, yy, (GMT_LONG)header.ny, 3, TRUE);
 			}
 		}
 
@@ -860,6 +846,7 @@ int main (int argc, char **argv)
 		for (j = k = 0; j < header.ny; j++) {
 			for (i = 0; i < header.nx; i++, k++) {	/* Compute rgb for each pixel */
 				if (GMT_is_fnan (topo[k]) && Ctrl->T.skip) continue;
+				if (Ctrl->I.active && Ctrl->T.skip && GMT_is_fnan (intensity[k])) continue;
 				GMT_get_rgb_from_z (topo[k], fill.rgb);
 				if (Ctrl->I.active) GMT_illuminate (intensity[k], fill.rgb);
 				n = GMT_graticule_path (&xx, &yy, 1, xval[i] - dx2, xval[i] + dx2, yval[j] - dy2, yval[j] + dy2);
@@ -873,8 +860,8 @@ int main (int argc, char **argv)
 	}
 	
 	if (Ctrl->Q.mode == GRDVIEW_IMAGE) {	/* compute image */
-		int nx_i, ny_i, kk, ip, jp, min_i, max_i, min_j, max_j, dist, node, nm_i, nm_drape, layers, last_i, last_j, *ix, *iy;
-		int *top_jp = VNULL, *bottom_jp = VNULL, p;
+		GMT_LONG nx_i, ny_i, kk, ip, jp, min_i, max_i, min_j, max_j, dist, node, nm_i, nm_drape, layers, last_i, last_j, p;
+		GMT_LONG *top_jp = VNULL, *bottom_jp = VNULL, *ix, *iy;
 		BOOLEAN done;
 		double xp, yp, sum_w, w, sum_i, x_width, y_width, value;
 		double sum_r, sum_g, sum_b, intval = 0.0, y_drape, *x_drape;
@@ -895,8 +882,8 @@ int main (int argc, char **argv)
 		last_i = nx_i - 1;	last_j = ny_i - 1;
 		if (drape_resample) {
 			nm_drape = d_head[0].nx * d_head[0].ny;
-			ix = (int *) GMT_memory (VNULL, (size_t)nm_drape, sizeof (int), GMT_program);
-			iy = (int *) GMT_memory (VNULL, (size_t)nm_drape, sizeof (int), GMT_program);
+			ix = (GMT_LONG *) GMT_memory (VNULL, (size_t)nm_drape, sizeof (GMT_LONG), GMT_program);
+			iy = (GMT_LONG *) GMT_memory (VNULL, (size_t)nm_drape, sizeof (GMT_LONG), GMT_program);
 			x_drape = (double *) GMT_memory (VNULL, (size_t)d_head[0].nx, sizeof (double), GMT_program);
 			if (Ctrl->I.active) int_drape = (float *) GMT_memory (VNULL, (size_t)nm_drape, sizeof (float), GMT_program);
 			for (i = 0; i < d_head[0].nx; i++) x_drape[i] = GMT_i_to_x (i, d_head[0].x_min, d_head[0].x_max, d_head[0].x_inc, d_head[0].xy_off, d_head[0].nx);
@@ -910,8 +897,8 @@ int main (int argc, char **argv)
 					else {
 						GMT_geoz_to_xy (x_drape[i], y_drape, value, &xp, &yp);
 						/* Make sure ix,iy fall in the range (0,nx_i-1), (0,ny_i-1) */
-						ix[bin] = MAX(0, MIN((int)floor((xp - z_project.xmin) * Ctrl->Q.dpi), last_i));
-						iy[bin] = MAX(0, MIN((int)floor((yp - z_project.ymin) * Ctrl->Q.dpi), last_j));
+						ix[bin] = MAX(0, MIN((GMT_LONG)floor((xp - z_project.xmin) * Ctrl->Q.dpi), last_i));
+						iy[bin] = MAX(0, MIN((GMT_LONG)floor((yp - z_project.ymin) * Ctrl->Q.dpi), last_j));
 					}
 					if (Ctrl->I.active) int_drape[bin] = (float)GMT_get_bcr_z(&i_head, x_drape[i], y_drape, intensity, &edgeinfo, &i_bcr);
 				}
@@ -923,8 +910,8 @@ int main (int argc, char **argv)
 			}
 		}
 		else {
-			ix = (int *) GMT_memory (VNULL, (size_t)nm, sizeof (int), GMT_program);
-			iy = (int *) GMT_memory (VNULL, (size_t)nm, sizeof (int), GMT_program);
+			ix = (GMT_LONG *) GMT_memory (VNULL, (size_t)nm, sizeof (GMT_LONG), GMT_program);
+			iy = (GMT_LONG *) GMT_memory (VNULL, (size_t)nm, sizeof (GMT_LONG), GMT_program);
 			for (j = bin = ij = 0; j < header.ny; j++) {	/* Get projected coordinates converted to pixel locations */
 				ij = (two) ? (j + 2) * mx + two : bin;
 				for (i = 0; i < header.nx; i++, bin++, ij++) {
@@ -934,8 +921,8 @@ int main (int argc, char **argv)
 					else {
 						GMT_geoz_to_xy (xval[i], yval[j], (double)topo[ij], &xp, &yp);
 						/* Make sure ix,iy fall in the range (0,nx_i-1), (0,ny_i-1) */
-						ix[bin] = MAX(0, MIN((int)floor((xp - z_project.xmin) * Ctrl->Q.dpi), last_i));
-						iy[bin] = MAX(0, MIN((int)floor((yp - z_project.ymin) * Ctrl->Q.dpi), last_j));
+						ix[bin] = MAX(0, MIN((GMT_LONG)floor((xp - z_project.xmin) * Ctrl->Q.dpi), last_i));
+						iy[bin] = MAX(0, MIN((GMT_LONG)floor((yp - z_project.ymin) * Ctrl->Q.dpi), last_j));
 					}
 				}
 			}
@@ -946,7 +933,7 @@ int main (int argc, char **argv)
 		/* Allocate image array and set background to PAGE_COLOR */
 
 		if (Ctrl->Q.monochrome) {
-			int gray;
+			char gray;
 
 			nm_i = nx_i * ny_i;
 			layers = 1;
@@ -972,8 +959,8 @@ int main (int argc, char **argv)
 		}
 
 		if (no_nans) {	/* Set up arrays for staircase clippath and initialize them */
-			top_jp = (int *) GMT_memory (VNULL, (size_t)nx_i, sizeof (int), GMT_program);
-			bottom_jp = (int *) GMT_memory (VNULL, (size_t)nx_i, sizeof (int), GMT_program);
+			top_jp = (GMT_LONG *) GMT_memory (VNULL, (size_t)nx_i, sizeof (GMT_LONG), GMT_program);
+			bottom_jp = (GMT_LONG *) GMT_memory (VNULL, (size_t)nx_i, sizeof (GMT_LONG), GMT_program);
 			for (ip = 0; ip < nx_i; ip++) bottom_jp[ip] = ny_i;
 		}
 
@@ -982,7 +969,7 @@ int main (int argc, char **argv)
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Start rasterization\n", GMT_program);
 		for (j = j_start; j != j_stop; j += j_inc) {
 
-			if (gmtdefs.verbose) fprintf (stderr, "%s: Scan line conversion at j-line %.6d\r", GMT_program, j);
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Scan line conversion at j-line %.6ld\r", GMT_program, j);
 
 			for (i = i_start; i != i_stop; i += i_inc) {
 				bin = j * header.nx + i;
@@ -1022,7 +1009,10 @@ int main (int argc, char **argv)
 							}
 							else {		/* Use lookup to get color */
 								GMT_get_rgb_from_z (zgrd[node], rgb);
-								if (GMT_is_fnan (zgrd[node])) bad++;	/* watch out for NaNs */
+								if (GMT_is_fnan (zgrd[node]))	/* watch out for NaNs in the z-data*/
+									bad++;
+								else if (Ctrl->I.active && GMT_is_fnan (intensity[node]))	/* watch out for NaNs in the intensity data*/
+									bad++;
 							}
 							if (bad) continue;	/* We don't want to blend in the (typically) gray NaN colors with the others. */
 							
@@ -1144,7 +1134,7 @@ int main (int argc, char **argv)
 							GMT_setpen (&Ctrl->W.pen[0]);
 							pen_not_set = FALSE;
 						}
-						ps_line (xx, yy, (GMT_LONG)k, 3, FALSE, TRUE);
+						ps_line (xx, yy, (GMT_LONG)k, 3, FALSE);
 					}
 					if (!pen_not_set) GMT_setpen (&Ctrl->W.pen[1]);
 				}
@@ -1154,8 +1144,8 @@ int main (int argc, char **argv)
 		GMT_free ((void *) yval);
 	}
 	else if (Ctrl->Q.mode == GRDVIEW_SURF) {
-		int start_side, entry_side, exit_side, next_side, low, ncont, nw_se_diagonal, check;
-		int corner[2], bad_side[2][2], p, p1, p2, saddle_sign;
+		GMT_LONG start_side, entry_side, exit_side, next_side, low, ncont, nw_se_diagonal, check;
+		GMT_LONG corner[2], bad_side[2][2], p, p1, p2, saddle_sign;
 		double *xcont, *ycont, *zcont, *vcont, X_vert[4], Y_vert[4], saddle_small;
 
 		xcont = (double *) GMT_memory (VNULL, (size_t)max, sizeof (double), GMT_program);
@@ -1181,8 +1171,7 @@ int main (int argc, char **argv)
 					X_vert[0] = X_vert[3] = x_left;	X_vert[1] = X_vert[2] = x_right;
 					Y_vert[0] = Y_vert[1] = y_bottom;	Y_vert[2] = Y_vert[3] = y_top;
 					for (k = 0; k < 4; k++) GMT_geoz_to_xy (X_vert[k], Y_vert[k], 0.0, &xmesh[k], &ymesh[k]);
-					paint_it (xmesh, ymesh, (GMT_LONG)4, GMT_d_NaN, FALSE, Ctrl->Q.monochrome, 0.0);
-					if (Ctrl->Q.outline) ps_patch (xmesh, ymesh, (GMT_LONG)4, GMT_no_rgb, TRUE);
+					paint_it (xmesh, ymesh, (GMT_LONG)4, GMT_d_NaN, FALSE, Ctrl->Q.monochrome, 0.0, Ctrl->Q.outline);
 					continue;
 				}
 
@@ -1285,7 +1274,7 @@ int main (int argc, char **argv)
 									next_side = exit_side;
 								}
 								else if (next_side == exit_side) {	/* Just hook up but reverse */
-									copy_points_bw (x, y, z, v, xcont, ycont, zcont, vcont, ncont, &n)	;
+									copy_points_bw (x, y, z, v, xcont, ycont, zcont, vcont, ncont, &n);
 									next_side = entry_side;
 								}
 								/* Compute the xy from the xyz triplets */
@@ -1295,7 +1284,7 @@ int main (int argc, char **argv)
 
 								/* Now paint the polygon piece */
 
-								paint_it (xx, yy, n, z_ave-saddle_small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
+								paint_it (xx, yy, n, z_ave-saddle_small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity, FALSE);
 
 								/* Reset the anchor points to previous contour */
 
@@ -1327,7 +1316,7 @@ int main (int argc, char **argv)
 
 							/* Now paint the polygon piece */
 
-							paint_it (xx, yy, n, z_ave+saddle_small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
+							paint_it (xx, yy, n, z_ave+saddle_small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity, FALSE);
 
 						} /* End triangular piece */
 
@@ -1393,7 +1382,7 @@ int main (int argc, char **argv)
 
 							/* Now paint the polygon piece */
 
-							paint_it (xx, yy, n, z_ave-small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
+							paint_it (xx, yy, n, z_ave-small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity, FALSE);
 
 							/* Reset the anchor points to previous contour */
 
@@ -1422,7 +1411,7 @@ int main (int argc, char **argv)
 
 						/* Now paint the polygon piece */
 
-						paint_it (xx, yy, n, z_ave+small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
+						paint_it (xx, yy, n, z_ave+small, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity, FALSE);
 
 					} /* End non-saddle case */
 
@@ -1441,7 +1430,7 @@ int main (int argc, char **argv)
 							GMT_setpen (&Ctrl->W.pen[0]);
 							pen_not_set = FALSE;
 						}
-						ps_line (xx, yy, k, 3, FALSE, TRUE);
+						ps_line (xx, yy, k, 3, FALSE);
 					}
 					if (!pen_not_set) GMT_setpen (&Ctrl->W.pen[1]);
 					if (Ctrl->Q.outline) {
@@ -1460,8 +1449,7 @@ int main (int argc, char **argv)
 					/* Now paint the polygon piece */
 
 					for (k = 0; k < 4; k++) GMT_geoz_to_xy (X_vert[k], Y_vert[k], (double)(topo[ij+ij_inc[k]]), &xmesh[k], &ymesh[k]);
-					paint_it (xmesh, ymesh, (GMT_LONG)4, z_ave, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity);
-					if (Ctrl->Q.outline) ps_patch (xmesh, ymesh, (GMT_LONG)4, GMT_no_rgb, TRUE);
+					paint_it (xmesh, ymesh, (GMT_LONG)4, z_ave, Ctrl->I.active, Ctrl->Q.monochrome, this_intensity, Ctrl->Q.outline);
 				}
 			}
 		}
@@ -1594,10 +1582,9 @@ struct GRDVIEW_POINT *get_point (double x, double y)
 	return (point);
 }
 
-void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEAN draw_plane, double plane_level)
+void grdview_init_setup (struct GRD_HEADER *header, float *topo, GMT_LONG two, BOOLEAN draw_plane, double plane_level)
 {
-	int i, j;
-	GMT_LONG ij;
+	GMT_LONG i, j, ij;
 	double xtmp, ytmp, tmp, delx, dely;
 
 	delx = (header->node_offset) ? 0.5 * header->x_inc :0.0;
@@ -1637,15 +1624,15 @@ void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEA
 	}
 }
 
-double get_intensity (float *intensity, GMT_LONG k, int nx)
+double get_intensity (float *intensity, GMT_LONG k, GMT_LONG nx)
 {
 	/* Finds the average intensity for this polygon */
 	return (0.25 * (intensity[k] + intensity[k+1] + intensity[k-nx] + intensity[k-nx+1]));
 }
 
-int pixel_inside (int ip, int jp, int *ix, int *iy, GMT_LONG bin, GMT_LONG bin_inc[])
+GMT_LONG pixel_inside (GMT_LONG ip, GMT_LONG jp, GMT_LONG *ix, GMT_LONG *iy, GMT_LONG bin, GMT_LONG bin_inc[])
 {
-	int i, what;
+	GMT_LONG i, what;
 	double x[6], y[6];
 
 	for (i = 0; i < 4; i++) {
@@ -1657,18 +1644,18 @@ int pixel_inside (int ip, int jp, int *ix, int *iy, GMT_LONG bin, GMT_LONG bin_i
 	return (what);
 }
 
-int quick_idist (int x1, int y1, int x2, int y2)
+GMT_LONG quick_idist (GMT_LONG x1, GMT_LONG y1, GMT_LONG x2, GMT_LONG y2)
 {
 	if ((x2 -= x1) < 0) x2 = -x2;
 	if ((y2 -= y1) < 0) y2 = -y2;
 	return (x2 + y2 - (((x2 > y2) ? y2 : x2) >> 1));
 }
 
-int get_side (double x, double y, double x_left, double y_bottom, double xinc, double yinc, double dx2, double dy2) {
+GMT_LONG get_side (double x, double y, double x_left, double y_bottom, double xinc, double yinc, double dx2, double dy2) {
 	/* Figure out on what side this point sites on */
 
 	double del_x, del_y;
-	int side;
+	GMT_LONG side;
 
 	del_x = x - x_left;
 	if (del_x > dx2) del_x = xinc - del_x;
@@ -1681,8 +1668,8 @@ int get_side (double x, double y, double x_left, double y_bottom, double xinc, d
 	return (side);
 }
 
-void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int ncont, GMT_LONG *n) {
-	int k;
+void copy_points_fw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], GMT_LONG ncont, GMT_LONG *n) {
+	GMT_LONG k;
 	for (k = 0; k < ncont; k++, (*n)++) {
 		x[*n] = xcont[k];
 		y[*n] = ycont[k];
@@ -1691,8 +1678,8 @@ void copy_points_fw (double x[], double y[], double z[], double v[], double xcon
 	}
 }
 
-void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], int ncont, GMT_LONG *n) {
-	int k;
+void copy_points_bw (double x[], double y[], double z[], double v[], double xcont[], double ycont[], double zcont[], double vcont[], GMT_LONG ncont, GMT_LONG *n) {
+	GMT_LONG k;
 	for (k = ncont - 1; k >= 0; k--, (*n)++) {
 		x[*n] = xcont[k];
 		y[*n] = ycont[k];
@@ -1709,7 +1696,7 @@ double get_z_ave (double v[], double next_up, GMT_LONG n) {
 	return (z_ave / n);
 }
 
-void add_node (double x[], double y[], double z[], double v[], GMT_LONG *k, int node, double X_vert[], double Y_vert[], float topo[], float zgrd[], GMT_LONG ij, GMT_LONG bin) {
+void add_node (double x[], double y[], double z[], double v[], GMT_LONG *k, GMT_LONG node, double X_vert[], double Y_vert[], float topo[], float zgrd[], GMT_LONG ij, GMT_LONG bin) {
 	/* Adds a corner node to list of points and increments counter */
 	x[*k] = X_vert[node];
 	y[*k] = Y_vert[node];
@@ -1718,8 +1705,9 @@ void add_node (double x[], double y[], double z[], double v[], GMT_LONG *k, int
 	(*k)++;
 }
 
-void paint_it (double x[], double y[], GMT_LONG n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity) {
-	int index, rgb[3];
+void paint_it (double x[], double y[], GMT_LONG n, double z, BOOLEAN intens, BOOLEAN monochrome, double intensity, BOOLEAN outline) {
+	GMT_LONG index;
+	int rgb[3];
 	struct GMT_FILL *f;
 
 	if (n < 3) return;	/* Need at least 3 points to make a polygon */
@@ -1730,12 +1718,12 @@ void paint_it (double x[], double y[], GMT_LONG n, double z, BOOLEAN intens, BOO
 	/* Now we must paint, with colors or patterns */
 
 	if ((index >= 0 && (f = GMT_lut[index].fill)) || (index < 0 && (f = GMT_bfn[index+3].fill))) {	/* Pattern */
-		GMT_fill (x, y, n, f, FALSE);
+		GMT_fill (x, y, n, f, outline);
 	}
 	else {	/* Solid color/gray */
 		if (intens) GMT_illuminate (intensity, rgb);
 		if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb); /* GMT_YIQ transformation */
-		ps_patch (x, y, n, rgb, FALSE);	/* Contours drawn separately (after this call) if desired */
+		ps_patch (x, y, n, rgb, outline);	/* Contours drawn separately (after this call) if desired */
 	}
 }
 
@@ -1764,7 +1752,7 @@ void *New_Grdview_Ctrl () {	/* Allocate and initialize a new control structure *
 }
 
 void Free_Grdview_Ctrl (struct GRDVIEW_CTRL *C) {	/* Deallocate control structure */
-	int i;
+	GMT_LONG i;
 	if (C->C.file) free ((void *)C->C.file);	
 	for (i = 0; i < 3; i++) if (C->G.file[i]) free ((void *)C->G.file[i]);	
 	if (C->I.file) free ((void *)C->I.file);	
