@@ -1,7 +1,7 @@
-/* $Id: img2mercgrd.c,v 1.30 2009/01/09 04:02:35 guru Exp $
+/* $Id: img2mercgrd.c,v 1.33 2009/09/13 00:52:03 guru Exp $
  *
  * Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- * See COPYING file for copying and redistribution conditions.
+ * See LICENSE.TXT file for copying and redistribution conditions.
  *
  * img2mercgrd.c
  *
@@ -92,17 +92,17 @@ struct IMG2MERCGRD_CTRL {
 
 int main (int argc, char **argv)
 {
-	int	i, navgsq, error = FALSE, irarg = -1, tempint, n_files = 0;
-	int	navg;	/* navg by navg pixels are averaged if navg > 1; else if navg == 1 do nothing */
-	int	iout, jout, iinstart, iinstop, jinstart, jinstop, k, kk, ion, jin, jj, iin, ii, kstart;
-	size_t nm;
+	GMT_LONG	i, navgsq, error = FALSE, irarg = -1, tempint, n_files = 0;
+	GMT_LONG	navg;	/* navg by navg pixels are averaged if navg > 1; else if navg == 1 do nothing */
+	GMT_LONG	iout, jout, iinstart, iinstop, jinstart, jinstop, k, kk, ion, jin, jj, iin, ii, kstart;
+	GMT_LONG nm;
 	GMT_LONG ij;
+	GMT_LONG	*ix;
 	double	west, east, south, north, toplat, botlat, dx, rnavgsq, csum, dsum, left, bottom;
 	struct GRD_HEADER h;
 	struct GMT_IMG_COORD imgcoord;
 	float	*a, empty_val;
 	short int *row;
-	int	*ix;
 	char	infile[BUFSIZ];
 	FILE	*fp = NULL;
 
@@ -325,8 +325,8 @@ int main (int argc, char **argv)
 	north = GMT_img_ypix_to_lat ((double)jinstart, &imgcoord);
 	south = GMT_img_ypix_to_lat ((double)jinstop,  &imgcoord);
 
-	iinstart = navg * (int)floor (west/(dx*navg));
-	iinstop  = navg * (int)ceil  (east/(dx*navg));
+	iinstart = navg * (GMT_LONG)floor (west/(dx*navg));
+	iinstop  = navg * (GMT_LONG)ceil  (east/(dx*navg));
 	/* iinstart <= ipixelcol < iinstop, but modulo all with imgcoord.nx360  */
 	/* Reset left and right edges of user area:  */
 	west = iinstart * dx;
@@ -348,7 +348,7 @@ int main (int argc, char **argv)
 
 	/* Set up header with Mercatorized dimensions assuming -Jm1  */
 	if (Ctrl->C.active) {
-		int equator;
+		GMT_LONG equator;
 		equator = irint (GMT_img_lat_to_ypix (0.0, &imgcoord));
 		h.x_min = iinstart * navg * dx;
 		h.x_max = h.x_min + h.nx * navg * dx;
@@ -389,7 +389,7 @@ int main (int argc, char **argv)
 	/* Now malloc some space for float grd array, integer pixel index, and short integer data buffer.  */
 
 	row = (short int *) GMT_memory (VNULL, (size_t)(navg * imgcoord.nxcol), sizeof (short int), GMT_program);
-	ix = (int *) GMT_memory (VNULL, (size_t)(navgsq * h.nx), sizeof (int), GMT_program);
+	ix = (GMT_LONG *) GMT_memory (VNULL, (size_t)(navgsq * h.nx), sizeof (GMT_LONG), GMT_program);
 	nm = GMT_get_nm (h.nx, h.ny);
 	a = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
 
@@ -433,7 +433,7 @@ int main (int argc, char **argv)
 		}
 		else {
 			if ( (fread((void *)row, sizeof (short int), (size_t)(navg * imgcoord.nxcol), fp) ) != (size_t)(navg * imgcoord.nxcol) ) {
-				fprintf(stderr,"%s:  ERROR:  Read failure at jin = %d.\n", GMT_program, jin);
+				fprintf(stderr,"%s:  ERROR:  Read failure at jin = %ld.\n", GMT_program, jin);
 				exit (EXIT_FAILURE);
 			}
 
