@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *    $Id: psmeca.c,v 1.46 2009/02/07 00:25:45 jluis Exp $
+ *    $Id: psmeca.c,v 1.50 2009/07/11 03:16:31 guru Exp $
  *
  *    Copyright (c) 1996-2009 by G. Patau
  *    Distributed under the GNU Public Licence
@@ -40,8 +40,8 @@ PostScript code is written to stdout.
 int main (int argc, char **argv)
 {
 
-    int     i, symbol = 0, n, ix = 0, iy = 1, n_files = 0, fno, last = 0;
-    int     n_args;
+    GMT_LONG     i, symbol = 0, n, ix = 0, iy = 1, n_files = 0, fno, last = 0;
+    GMT_LONG     n_args;
     GMT_LONG     n_rec = 0;
 
     BOOLEAN    outline = FALSE, p_outline = FALSE, t_outline = FALSE;
@@ -68,7 +68,7 @@ int main (int argc, char **argv)
 
     char string[BUFSIZ], event_title[BUFSIZ], txt_a[GMT_TEXT_LEN];
 
-    char line[BUFSIZ], symbol_type, col[15][GMT_TEXT_LEN], *cpt = CNULL;
+    char line[BUFSIZ], symbol_type, col[15][GMT_TEXT_LEN], *cpt = CNULL, *not_used = NULL;
     
     FILE *fp = NULL;
     
@@ -83,10 +83,10 @@ int main (int argc, char **argv)
     struct M_TENSOR mt;
     struct AXIS T, N, P;
 
-    int n_plane = 0, n_plane_old = 0;
-    int default_justify = 2;
-    int justify = default_justify, form = 0;
-    int new = 1;
+    GMT_LONG n_plane = 0, n_plane_old = 0;
+    GMT_LONG default_justify = 2;
+    GMT_LONG justify = default_justify, form = 0;
+    GMT_LONG new = 1;
 
     double default_fontsize = 9.0;
     double fontsize = default_fontsize;
@@ -97,7 +97,7 @@ int main (int argc, char **argv)
     double size, a_size = GMT_d_NaN;
     double P_x, P_y, T_x, T_y;
     char P_sym_type = 0, T_sym_type = 0;
-    int P_sym = 0, T_sym = 0;
+    GMT_LONG P_sym = 0, T_sym = 0;
 
     argc = GMT_begin (argc, argv);
     
@@ -251,7 +251,7 @@ int main (int argc, char **argv)
                     break;
                 case 'T':
                     transparence = TRUE;
-                    sscanf (&argv[i][2], "%d",&n_plane);
+                    sscanf (&argv[i][2], "%ld",&n_plane);
                     if (strlen(argv[i]) > 4) { /* Set transparent attributes */
                         def_tr_pen = TRUE;
                         if (GMT_getpen (&argv[i][4], &tr_pen)) {
@@ -444,7 +444,7 @@ int main (int argc, char **argv)
         fprintf (stderr,"psmeca %s - Plot seismological symbols on maps\n\n", GMT_VERSION);
         fprintf (stderr,"usage: psmeca <infiles> %s %s\n", GMT_J_OPT, GMT_Rgeo_OPT);
         fprintf (stderr, "   -S<symbol><scale>[/fontsize[/justify/offset/angle/form]]\n");
-        fprintf (stderr, "   [%s] [-C<pen>[P<pointsize>]]\n", GMT_B_OPT);
+        fprintf (stderr, "   [%s] [-C[<pen>][P<pointsize>]]\n", GMT_B_OPT);
         fprintf (stderr, "   [-Ddepmin/depmax] [-E<fill>] [-G<fill>]\n");
         fprintf (stderr, "   [%s] [-K] [-L<pen>] [-M] [-N] [-O] [-P] [-r]\n", GMT_Ho_OPT);
         fprintf (stderr, "   [-Tnplane[/<pen>]] [%s]\n", GMT_U_OPT);
@@ -579,7 +579,7 @@ int main (int argc, char **argv)
             ps_comment (line);
         }
         if (GMT_io.io_header[GMT_IN]) 
-            for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+            for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
         
 
         while (GMT_fgets (line, BUFSIZ, fp)) {
@@ -766,7 +766,7 @@ Definition of scalar moment.
             size = (computed_mw(moment, meca.magms) / 5.) * scale;
 
             get_trans(xy[0], xy[1], &t11, &t12, &t21, &t22);
-            delaz = atan2(t12,t11) / D2R;
+            delaz = atan2d(t12,t11);
 
             if ((read_axis || read_tensor) && !plot_dc) {
 
