@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *    $Id: pspolar.c,v 1.30 2009/01/09 04:02:35 guru Exp $ 
+ *    $Id: pspolar.c,v 1.33 2009/07/18 03:06:50 remko Exp $ 
  *
  *    Copyright (c) 1996-2009 by G. Patau
  *    Distributed under the GNU Public Licence
@@ -38,9 +38,9 @@
 
 int main (int argc, char **argv)
 {
-    int     i, symbol = 0, n, n_files = 0, fno;
-    int     n_args;
-    int     form_s = 0, justify_s = 5;
+    GMT_LONG     i, symbol = 0, n, n_files = 0, fno;
+    GMT_LONG     n_args;
+    GMT_LONG     form_s = 0, justify_s = 5;
     
     BOOLEAN error = FALSE, nofile = TRUE;
     BOOLEAN done, greenwich, label_s = FALSE;
@@ -65,7 +65,7 @@ int main (int argc, char **argv)
     double si, co;
 
     char line[BUFSIZ], symbol_type, col[4][GMT_TEXT_LEN];
-    char pol;
+    char pol, *not_used = NULL;
     char txt_a[GMT_TEXT_LEN],txt_b[GMT_TEXT_LEN],txt_c[GMT_TEXT_LEN], txt_d[GMT_TEXT_LEN];
     char stacode[GMT_TEXT_LEN];
     
@@ -250,7 +250,7 @@ int main (int argc, char **argv)
                 case 'T':       /* Information about label printing */
                         label_s = TRUE;
                         if (strlen(argv[i]) > 2) {
-                             sscanf (&argv[i][2], "%lf/%d/%d/%lf/", &angle_s,
+                             sscanf (&argv[i][2], "%lf/%ld/%ld/%lf/", &angle_s,
                                      &form_s, &justify_s, &fontsize_s);
                         }
                         break;
@@ -316,7 +316,7 @@ int main (int argc, char **argv)
         fprintf (stderr, "        -E Specify color symbol for station in extensive part.\n");
         fprintf (stderr, "           Fill can be either <r/g/b> (each 0-255) for color \n");
         fprintf (stderr, "           or <gray> (0-255) for gray-shade [0].\n");
-        fprintf (stderr, "           Default is light grey.\n");
+        fprintf (stderr, "           Default is light gray.\n");
         fprintf (stderr, "        -e Outline of station symbol in extensive part.\n");
         fprintf (stderr, "           Default is current pen.\n");
         fprintf (stderr, "        -F Specify background color of beach ball. It can be\n");
@@ -404,7 +404,7 @@ int main (int argc, char **argv)
             sprintf (line, "File: %s", argv[fno]);
             ps_comment (line);
         }
-        if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, 512, fp);
+        if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, 512, fp);
         
         GMT_world_map = TRUE;
         
