@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: psvelo.c,v 1.31 2008/03/24 08:58:32 guru Exp $
+ *    $Id: psvelo.c,v 1.36 2009/07/11 03:16:31 guru Exp $
  *
- *    Copyright (c) 1996-2008 by G. Patau
+ *    Copyright (c) 1996-2009 by G. Patau
  *    Distributed under the GNU Public Licence
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
@@ -47,8 +47,8 @@ PostScript code is written to stdout.
 
 int main (int argc, char **argv)
 {
-	int 	i, symbol = 0, n, ix = 0, iy = 1, n_files = 0, fno;
-	int	n_args;
+	GMT_LONG 	i, symbol = 0, n, ix = 0, iy = 1, n_files = 0, fno;
+	GMT_LONG	n_args;
 	GMT_LONG     n_rec = 0;
 
 	BOOLEAN	outline = FALSE;
@@ -75,7 +75,7 @@ int main (int argc, char **argv)
 	double t11 = 1.0, t12 = 0.0, t21 = 0.0, t22 = 1.0;
 	double hl,hw,vw, fontsize = 0.0;
 
-	char station_name[20];
+	char *station_name, *not_used = NULL;
 	char txt_a[GMT_TEXT_LEN], txt_b[GMT_TEXT_LEN], txt_c[GMT_TEXT_LEN];
 	char line[BUFSIZ], symbol_type, col[12][GMT_TEXT_LEN];
 
@@ -85,7 +85,7 @@ int main (int argc, char **argv)
 	struct GMT_PEN pen;
 	struct GMT_FILL fill, efill; /* efill is for uncertainty wedge */
 
-	int justify;
+	GMT_LONG justify;
 
 	argc = GMT_begin (argc, argv);
 
@@ -290,6 +290,7 @@ int main (int argc, char **argv)
 
 	done = FALSE;
 
+	station_name = (char *) GMT_memory (VNULL, (size_t)64, sizeof (char), GMT_program);
 
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 		if (!nofile && argv[fno][0] == '-') continue;
@@ -307,7 +308,7 @@ int main (int argc, char **argv)
 			sprintf (line, "File: %s", argv[fno]);
 			ps_comment (line);
 		}
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, 512, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, 512, fp);
 
 		if (gmtdefs.verbose && (read_ellipse || read_rotated_ellipse))	 
 		  fprintf (stderr, "psvelo: 2-D confidence interval and scaling factor %f %f\n",confidence, conrad);
@@ -346,7 +347,7 @@ int main (int argc, char **argv)
                                         ellipse_convert(sigma_x, sigma_y, corr_xy, conrad, &small_axis, &great_axis, &direction);
 
                                     /* convert to degrees */
-                                       direction = direction / D2R;
+                                       direction = direction * R2D;
                                     }
                         }
                         else if(read_rotated_ellipse) {
@@ -458,6 +459,8 @@ int main (int argc, char **argv)
 	if (fp != stdin) GMT_fclose (fp);
 	}
 
+	GMT_free((void *) station_name);
+
 	if (gmtdefs.verbose) 
 			fprintf (stderr, "psvelo: Number of records read: %li\n", n_rec);
 
