@@ -1,4 +1,4 @@
-/*	$Id: utilmeca.c,v 1.18 2009/02/07 15:45:52 remko Exp $
+/*	$Id: utilmeca.c,v 1.22 2009/05/13 21:06:43 guru Exp $
  *    Copyright (c) 1996-2009 by G. Patau
  *    Distributed under the GNU Public Licence
  *    See README file for copying and redistribution conditions.
@@ -77,7 +77,7 @@ double  ps_mechanism(double x0, double y0, st_me meca, double size, int rgb[3],
     double str, radius, increment;
     double si, co;
 
-    int lineout = 1, i;
+    GMT_LONG lineout = 1, i;
     GMT_LONG npoints;
 
     struct AXIS N_axis;
@@ -292,7 +292,7 @@ double ps_meca(double x0,double y0,st_me meca,double size)
 
 {
 
-    int i;
+    GMT_LONG i;
     GMT_LONG npoints;
 
     double proj_radius();
@@ -302,7 +302,8 @@ double ps_meca(double x0,double y0,st_me meca,double size)
     double str, radius, increment;
     double si, co;
 
-    int no_fill[3], lineout = 1;
+	int no_fill[3];
+	GMT_LONG lineout = 1;
 
     no_fill[0] = -1;
     no_fill[1] = -1;
@@ -330,7 +331,7 @@ double ps_meca(double x0,double y0,st_me meca,double size)
             str += increment;
         }
         npoints = i + 1;
-        ps_line(x, y, npoints, 1, FALSE, FALSE);
+        ps_line(x, y, npoints, 1, FALSE);
 
         i = -1;
         increment = 1.;
@@ -344,19 +345,19 @@ double ps_meca(double x0,double y0,st_me meca,double size)
             str += increment;
         }
         npoints = i + 1;
-        ps_line(x, y, npoints, 1, FALSE, FALSE);
+        ps_line(x, y, npoints, 1, FALSE);
         return(radius_size*2.);
 }
 
 /*********************************************************************/
-double ps_plan(double x0,double y0,st_me meca,double size,int num_of_plane)
+double ps_plan(double x0,double y0,st_me meca,double size,GMT_LONG num_of_plane)
 
 /* Genevieve Patau */
 
 {
 
-    int i;
-    GMT_LONG npoints;
+    GMT_LONG i;
+    GMT_LONG npoints, lineout=1;
 
     double proj_radius();
 
@@ -365,7 +366,7 @@ double ps_plan(double x0,double y0,st_me meca,double size,int num_of_plane)
     double str, radius, increment;
     double si, co;
 
-    int rgb[3], lineout=1;
+    int rgb[3];
 
     rgb[0] = -1;
     rgb[1] = -1;
@@ -394,7 +395,7 @@ double ps_plan(double x0,double y0,st_me meca,double size,int num_of_plane)
                    str += increment;
                }
                npoints = i + 1;
-               ps_line(x, y, npoints, 1, FALSE, FALSE);
+               ps_line(x, y, npoints, 1, FALSE);
                break;
 
         case 2:
@@ -410,7 +411,7 @@ double ps_plan(double x0,double y0,st_me meca,double size,int num_of_plane)
                    str += increment;
                }
                npoints = i + 1;
-               ps_line(x, y, npoints, 1, FALSE, FALSE);
+               ps_line(x, y, npoints, 1, FALSE);
                break;
       }
       return(radius_size*2.);
@@ -494,7 +495,7 @@ double computed_strike1(struct nodal_plane NP1)
         temp = ss * cr;
         temp -= sr *  cs * cd1;
         cp2 = am * temp;
-        str2 = d_atan2(sp2, cp2);
+        str2 = d_atan2d(sp2, cp2);
         str2 = zero_360(str2);
     }
     return(str2);
@@ -516,7 +517,7 @@ double computed_dip1(struct nodal_plane NP1)
     double am = (GMT_IS_ZERO (NP1.rake) ? 1. : NP1.rake / fabs(NP1.rake));
     double dip2;
 
-    dip2 = acos(am * sind(NP1.rake) * sind(NP1.dip)) / D2R;
+    dip2 = acosd(am * sind(NP1.rake) * sind(NP1.dip));
 
     return(dip2);
 }
@@ -550,7 +551,7 @@ double computed_rake1(struct nodal_plane NP1)
     else
         sinrake2 = -am * sd * cs / cd;
 
-    rake2 = d_atan2(sinrake2, -am * sd * ss);
+    rake2 = d_atan2d(sinrake2, -am * sd * ss);
 
     return(rake2);
 }
@@ -578,7 +579,7 @@ double computed_dip2(double str1,double dip1,double str2)
             dip2 = 1000.; /* (only first plane will be plotted) */
     }
     else {
-        dip2 = d_atan2(cosd(dip1), -sind(dip1) * cosdp12);
+        dip2 = d_atan2d(cosd(dip1), -sind(dip1) * cosdp12);
     }
 
     return(dip2);
@@ -610,7 +611,7 @@ double computed_rake2(double str1,double dip1,double str2,double dip2,double fau
     else
         sinrake2 = -fault * sd * cs / cd;
 
-    rake2 = d_atan2(sinrake2, - fault * sd * ss);
+    rake2 = d_atan2d(sinrake2, - fault * sd * ss);
 
     return(rake2);
 }
@@ -645,7 +646,7 @@ double null_axis_dip(double str1,double dip1,double str2,double dip2)
 {
     double den;
 
-    den = asin(sind(dip1) * sind(dip2) * sind(str1 - str2)) / D2R;
+    den = asind(sind(dip1) * sind(dip2) * sind(str1 - str2));
     if (den < 0.)
         den = -den;
     return(den);
@@ -677,9 +678,8 @@ double null_axis_strike(double str1,double dip1,double str2,double dip2)
         cosphn = -cosphn;
         sinphn = -sinphn;
     }
-    phn = d_atan2(sinphn, cosphn);
-    if (phn < 0.)
-        phn += 360.;
+    phn = d_atan2d(sinphn, cosphn);
+    if (phn < 0.) phn += 360.;
     return(phn);
 }
 
@@ -715,7 +715,7 @@ double proj_radius(double str1,double dip1,double str)
 /***************************************************************************************/
 void GMT_momten2axe(struct M_TENSOR mt,struct AXIS *T,struct AXIS *N,struct AXIS *P) {
 	/* This version uses GMT_jacobi and does not suffer from the convert_matrix bug */
-	int j, nrots;
+	GMT_LONG j, nrots;
 	GMT_LONG np = 3;
 	double *a, *d, *b, *z, *v;
 	double az[3], pl[3];
@@ -757,14 +757,14 @@ void GMT_momten2axe(struct M_TENSOR mt,struct AXIS *T,struct AXIS *N,struct AXIS
 }
 
 /***************************************************************************************/
-double ps_tensor(double x0,double y0,double size,struct AXIS T,struct AXIS N,struct AXIS P,int c_rgb[3],int e_rgb[3], int outline, int plot_zerotrace)
+double ps_tensor(double x0,double y0,double size,struct AXIS T,struct AXIS N,struct AXIS P,int c_rgb[3],int e_rgb[3], GMT_LONG outline, GMT_LONG plot_zerotrace)
 {
-    int d, b = 1, m;
-    int i, ii, n = 0, j = 1, j2 = 0, j3 = 0;
+    GMT_LONG d, b = 1, m;
+    GMT_LONG i, ii, n = 0, j = 1, j2 = 0, j3 = 0;
     GMT_LONG npoints;
-    int lineout = 1;
+    GMT_LONG lineout = 1;
     int rgb1[3], rgb2[3];
-    int big_iso = 0;
+    GMT_LONG big_iso = 0;
 
     double a[3], p[3], v[3];
     double vi, iso, f;
@@ -1062,8 +1062,8 @@ void axe2dc(struct AXIS T,struct AXIS P,struct nodal_plane *NP1,struct nodal_pla
      }
      if (p2 < 0.) p2 += PII;
 
-     NP1->dip = d1 / D2R; NP1->str = p1 / D2R;
-     NP2->dip = d2 / D2R; NP2->str = p2 / D2R;
+     NP1->dip = d1 * R2D; NP1->str = p1 * R2D;
+     NP2->dip = d2 * R2D; NP2->str = p2 * R2D;
 
      im = 1;
      if (dp > dt) im = -1;
@@ -1081,21 +1081,21 @@ from nodal plane strikes, dips and rakes.
 */
 
 {
-        int im = 0;
-        int pure_strike_slip = 0;
+        GMT_LONG im = 0;
+        GMT_LONG pure_strike_slip = 0;
         double cd1, sd1, cd2, sd2;
         double cp1, sp1, cp2, sp2;
         double amz, amx, amy, dx, px, dy, py;
         double radius;
 
-        if (fabs(sin(meca.NP1.rake * D2R)) > EPSIL) im = (int) (meca.NP1.rake / fabs(meca.NP1.rake));
-        else if (fabs(sin(meca.NP2.rake * D2R)) > EPSIL) im = (int) (meca.NP2.rake / fabs(meca.NP2.rake));
+        if (fabs(sind(meca.NP1.rake)) > EPSIL) im = (GMT_LONG) (meca.NP1.rake / fabs(meca.NP1.rake));
+        else if (fabs(sind(meca.NP2.rake)) > EPSIL) im = (GMT_LONG) (meca.NP2.rake / fabs(meca.NP2.rake));
         else pure_strike_slip = 1;
 
         size *= 0.5;
 
         if (pure_strike_slip) {
-            if (cos(meca.NP1.rake * D2R) < 0.) {
+            if (cosd(meca.NP1.rake) < 0.) {
                 *pp = zero_360(meca.NP1.str + 45.);
                 *pt = zero_360(meca.NP1.str - 45.);
             }
@@ -1106,20 +1106,20 @@ from nodal plane strikes, dips and rakes.
             *dp = 0.;
             *dt = 0.;
             radius = 0.97;
-            *xp = radius * sin(*pp * D2R) * size + x0;
-            *yp = radius * cos(*pp * D2R) * size + y0;
-            *xt = radius * sin(*pt * D2R) * size + x0;
-            *yt = radius * cos(*pt * D2R) * size + y0;
+            *xp = radius * sind(*pp) * size + x0;
+            *yp = radius * cosd(*pp) * size + y0;
+            *xt = radius * sind(*pt) * size + x0;
+            *yt = radius * cosd(*pt) * size + y0;
         }
         else {
-            cd1 = cos(meca.NP1.dip * D2R) *  M_SQRT2;
-            sd1 = sin(meca.NP1.dip * D2R) *  M_SQRT2;
-            cd2 = cos(meca.NP2.dip * D2R) *  M_SQRT2;
-            sd2 = sin(meca.NP2.dip * D2R) *  M_SQRT2;
-            cp1 = - cos(meca.NP1.str * D2R) *  sd1;
-            sp1 = sin(meca.NP1.str * D2R) *  sd1;
-            cp2 = - cos(meca.NP2.str * D2R) *  sd2;
-            sp2 = sin(meca.NP2.str * D2R) *  sd2;
+            cd1 = cosd(meca.NP1.dip) * M_SQRT2;
+            sd1 = sind(meca.NP1.dip) * M_SQRT2;
+            cd2 = cosd(meca.NP2.dip) * M_SQRT2;
+            sd2 = sind(meca.NP2.dip) * M_SQRT2;
+            cp1 = -cosd(meca.NP1.str) * sd1;
+            sp1 = sind(meca.NP1.str) * sd1;
+            cp2 = -cosd(meca.NP2.str) * sd2;
+            sp2 = sin(meca.NP2.str) * sd2;
 
             amz = - (cd1 + cd2);
             amx = - (sp1 + sp2);
@@ -1173,14 +1173,14 @@ from nodal plane strikes, dips and rakes.
 */
 
 {
-        int im = 0;
-        int pure_strike_slip = 0;
+        GMT_LONG im = 0;
+        GMT_LONG pure_strike_slip = 0;
         double cd1, sd1, cd2, sd2;
         double cp1, sp1, cp2, sp2;
         double amz, amx, amy, dx, px, dy, py;
 
-        if (fabs(sind(meca.NP1.rake)) > EPSIL) im = (int) (meca.NP1.rake / fabs(meca.NP1.rake));
-        else if (fabs(sind(meca.NP2.rake)) > EPSIL) im = (int) (meca.NP2.rake / fabs(meca.NP2.rake));
+        if (fabs(sind(meca.NP1.rake)) > EPSIL) im = (GMT_LONG) (meca.NP1.rake / fabs(meca.NP1.rake));
+        else if (fabs(sind(meca.NP2.rake)) > EPSIL) im = (GMT_LONG) (meca.NP2.rake / fabs(meca.NP2.rake));
         else pure_strike_slip = 1;
 
         if (pure_strike_slip) {
@@ -1235,10 +1235,10 @@ from nodal plane strikes, dips and rakes.
 
         }
 
-        T->str /= D2R;
-        T->dip /= D2R;
-        P->str /= D2R;
-        P->dip /= D2R;
+        T->str *= R2D;
+        T->dip *= R2D;
+        P->str *= R2D;
+        P->dip *= R2D;
 
         N->str = null_axis_strike(T->str, T->dip, P->str, P->dip);
         N->dip = null_axis_dip(T->str, T->dip, P->str, P->dip);
