@@ -1,8 +1,8 @@
 /*
- *	$Id: grdread.c,v 1.10 2008/04/29 21:05:53 guru Exp $
+ *	$Id: grdread.c,v 1.14 2009/09/09 23:27:04 guru Exp $
  *
- *      Copyright (c) 1999-2008 by P. Wessel
- *      See COPYING file for copying and redistribution conditions.
+ *      Copyright (c) 1999-2009 by P. Wessel
+ *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ int grdread (double z_8[], double info[], char *filein, struct GRD_HEADER *grd)
 {
 	/* info contains xmin, xmax, ymin, ymax, zmin, zmax, node-offset, dx, dy */
 
-	int i, j, pad[4];
+	GMT_LONG i, j, pad[4];
 	float *z_4;          /* real array for output */
 
 	pad[0] = pad[1] = pad[2] = pad[3] = 0;
@@ -118,6 +118,7 @@ void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
 
 	/* Read the header */
  
+	GMT_grd_init (&grd, 0, NULL, FALSE);
 	if (GMT_read_grd_info (filein, &grd))
 		mexErrMsgTxt ("grdread: failure to read header\n");
 
