@@ -1,8 +1,8 @@
 /*
- *	$Id: grdwrite.c,v 1.10 2008/04/29 21:05:53 guru Exp $
+ *	$Id: grdwrite.c,v 1.13 2009/09/09 23:27:04 guru Exp $
  *
- *      Copyright (c) 1999-2008 by P. Wessel
- *      See COPYING file for copying and redistribution conditions.
+ *      Copyright (c) 1999-2009 by P. Wessel
+ *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -47,7 +47,7 @@ int grdwrite (double z_8[], double x[], double y[], double info[], char *fileout
 /* ny		:	number of y points */
 /* pix		:	1 if pixel reg, 0 if gridline registered */
 
-	int i, j, i2, pad[4], error = 0;
+	GMT_LONG i, j, i2, pad[4], error = 0;
 	float *z_4;           /* real array for output */
 	struct GRD_HEADER grd; 
 
