@@ -1,5 +1,5 @@
 /*-------------------------------------------------------------------------
- *	$Id: mgd77.h,v 1.104 2009/02/12 04:45:55 guru Exp $
+ *	$Id: mgd77.h,v 1.118 2009/09/13 00:52:03 guru Exp $
  * 
  *    Copyright (c) 2005-2009 by P. Wessel
  *    See README file for copying and redistribution conditions.
@@ -33,7 +33,7 @@
 #define MGD77_N_DATA_EXTENDED	28		/* The 27 plus time */
 #define MGD77_N_NUMBER_FIELDS	24		/* Original 24 numerical data columns in MGD77 */
 #define MGD77_N_STRING_FIELDS	3		/* Original 3 text data columns in MGD77 */
-#define MGD77_N_HEADER_ITEMS	66		/* Number of individual header items in the MGD77 header */
+#define MGD77_N_HEADER_ITEMS	72		/* Number of individual header items in the MGD77 header */
 #define MGD77_N_MAG_RF		18		/* Number of different Mag ref fields so far in MGD77 docs */
 /* The 28 MGD77 standard types (27 original + 1 conglomerate (time)) */
 #define MGD77_RECTYPE		0
@@ -115,6 +115,9 @@
 #define MGD77_BAD_HEADER_RECNO		-1
 #define MGD77_BAD_HEADER_ITEM		-2
 
+#define MGD77_ORIG			0
+#define MGD77_REVISED			1
+
 /* Return error numbers */
 
 #define MGD77_NO_ERROR			0
@@ -238,7 +241,7 @@ struct MGD77_META {	/* Information about a cruise as derived from navigation dat
 };
 
 struct MGD77_HEADER {	
-	struct MGD77_HEADER_PARAMS *mgd77;		/* See MGD-77 Documentation from NGDC for details */
+	struct MGD77_HEADER_PARAMS *mgd77[2];		/* See MGD-77 Documentation from NGDC for details; [0] is ORIG, [1] is REVISED */
 	struct MGD77_META meta;				/* Holds some meta-data derived directly from data records */
 	char *author;					/* Name of author of last creation/modification */
 	char *history;					/* History of creation/modifications */
@@ -251,13 +254,43 @@ struct MGD77_HEADER {
 	struct MGD77_DATA_INFO info[MGD77_N_SETS];	/* Info regarding [0] standard MGD77 columns and [1] any extra columns (max 32 each) */
 };
 
+#ifdef USE_CM4
+typedef int logical;
+struct MGD77_CM4 {	/* For use with cm4field.c and initialized by MGD77_CM4_init () */
+	char *path[3];		/* Paths to the three coefficient files */
+	int unit[3];		/* IDs of logical units */
+	logical load[3];	/* TRUE if the file has been read into memory */
+	logical index[2];	/* Index acquisition flags/ TRUE from file, FALSE from argument */
+	logical gmut;		/* Magnetic dipole universal time (MUT) acquisition flag: TRUE, compute from UT, FALSE, from arg */
+	logical cord;		/* TRUE for geodetic, FALSE for geocentric */
+	logical pred[6];	/* TRUE to compute, FALSE not: 1 = main field, 2 magnetispheric, 3 ionospheric, etc */
+	logical curr;		/* Model J current field prediction flag: TRUE, compute, FALSE not */
+	logical coef;		/* Model coefficient generation flag: TRUE, compute, FALSE not */
+	int nhmf[2];		/* Maximum main field spherical harmonic degree (0 for main field 1, 1 for main field 2) */
+	int nlmf[2];		/* Minimum main field spherical harmonic degree (0 for main field 1, 1 for main field 2) */
+	double mut;		/* Magnetic dipole universal time (hours 0-24), computed from UT and returned as gmut = TRUE */
+	double alt;		/* Altitude [0] */
+	double dst;		/* Linearly interpolated hourly Dst magnetic index, returned from file as index[0] = TRUE */
+	double f107;		/* Linearly interpolated 3-monthly means of absolute F10.7 solar radiation flux value, returned */
+	double bmdl[3][7];	/* Array storing computed B field vectors from various sources (nT) */
+	double jmdl[3][4];	/* Array storing computed J field vectors from certain external sources */
+	double *gmdl;		/* Array storing coefficients from various sources */
+	int perr;		/* Error message print flag: 0 do not print */
+	int oerr;		/* Unit to print to */
+	int cerr;		/* Error return code (0 = normal, 1-49 warning, > 50 fatal */
+};
+#endif
 
 /* We may want to output columns that themselves are not stored in the MGD77[+] files but
  * rather are computed based on data that are stored in the file.  We consider such information
  * as AUXILLARY columns and insert them between the observed columns when needed.  The following
  * structures are used to facilitate this process. */
 
+#ifdef USE_CM4
+#define N_MGD77_AUX	16		/* Number of auxilliary derived columns for MGD77 data, including optional CM4 */
+#else
 #define N_MGD77_AUX	15		/* Number of auxilliary derived columns for MGD77 data */
+#endif
 #define N_GENERIC_AUX	3		/* Number of auxilliary derived columns for general files (dist, azim, vel) */
 
 #define MGD77_AUX_DS	0
@@ -275,6 +308,9 @@ struct MGD77_HEADER {
 #define MGD77_AUX_CT	12
 #define MGD77_AUX_GR	13
 #define MGD77_AUX_ID	14
+#ifdef USE_CM4
+#define MGD77_AUX_CM	15
+#endif
 
 struct MGD77_AUXLIST {
 	char name[MGD77_COL_ABBREV_LEN];
@@ -370,8 +406,11 @@ struct MGD77_CONTROL {
 	int format;					/* 0 if any file format, 1 if MGD77, and 2 if netCDF, 3 if ascii table */
 	/* Format-related issues */
 	int time_format;				/* Either GMT_IS_ABSTIME or GMT_IS_RELTIME */
+	struct GMT_TIME_SYSTEM utime;			/* All the information about the Unix time system */
+	BOOLEAN adjust_time;				/* TRUE if GMT time-system is NOT unix */
 	/* Data use information */
 	BOOLEAN original;				/* TRUE means we want original not revised header attributes */
+	BOOLEAN revised;				/* TRUE means we are working on a MGD77+ file with revised header attributes */
 	BOOLEAN Want_Header_Item[MGD77_N_HEADER_ITEMS];	/* TRUE means print this header item if dump is selected */
 	BOOLEAN use_flags[MGD77_N_SETS];		/* TRUE means programs will use error bitflags (if present) when returning data */
 	BOOLEAN use_corrections[MGD77_N_SETS];		/* TRUE means we will apply correction factors (if present) when reading data */
@@ -431,6 +470,7 @@ extern int MGD77_Read_File (char *file, struct MGD77_CONTROL *F, struct MGD77_DA
 extern int MGD77_Write_File (char *file, struct MGD77_CONTROL *F, struct MGD77_DATASET *S);				/* Write entire file (all columns) */
 extern int MGD77_Read_Header_Record (char *file, struct MGD77_CONTROL *F, struct MGD77_HEADER *H);			/* Read the header record */
 extern int MGD77_Write_Header_Record (char *file, struct MGD77_CONTROL *F, struct MGD77_HEADER *H);			/* Write the header record */
+extern int MGD77_Free_Header_Record (struct MGD77_CONTROL *F, struct MGD77_HEADER *H);					/* Frees up header memory */
 extern int MGD77_Read_Data (char *file, struct MGD77_CONTROL *F, struct MGD77_DATASET *S);				/* Allocate & Read all data (selected columns only); Header already read */
 extern int MGD77_Write_Data (char *file, struct MGD77_CONTROL *F, struct MGD77_DATASET *S);				/* Write all data (all columns); Header already written */
 extern int MGD77_Read_Data_Record (struct MGD77_CONTROL *F, struct MGD77_HEADER *H, double dvals[], char *tvals[]);	/* Read a single data record (selected columns only) */
@@ -438,7 +478,7 @@ extern int MGD77_Write_Data_Record (struct MGD77_CONTROL *F, struct MGD77_HEADER
 extern void MGD77_Free (struct MGD77_DATASET *S);									/* Free memory allocated by MGD77_Read_File/MGD77_Read_Data */
 extern void MGD77_Select_Columns (char *string, struct MGD77_CONTROL *F, int option);					/* Decode the -F option specifying the desired columns */
 extern int MGD77_Get_Column (char *word, struct MGD77_CONTROL *F);							/* Get column number from column name (or -1 if not present) */
-extern int MGD77_Info_from_Abbrev (char *name, struct MGD77_HEADER *H, int *set, int *item);
+extern int MGD77_Info_from_Abbrev (char *name, struct MGD77_HEADER *H, GMT_LONG *set, GMT_LONG *item);
 extern void MGD77_List_Header_Items (struct MGD77_CONTROL *F);
 extern int MGD77_Select_Header_Item (struct MGD77_CONTROL *F, char *item);
 extern int MGD77_Get_Set (char *abbrev);										/* Returns 0 if abbrev is in the MGD77 set, else 1 */
@@ -486,8 +526,21 @@ extern double MGD77_carter_correction (double lon, double lat, double twt_in_mse
 extern int MGD77_igrf10syn (int isv, double date, int itype, double alt, double lon, double lat, double *out);
 extern double MGD77_Theoretical_Gravity (double lon, double lat, int version);
 extern void MGD77_IGF_text (FILE *fp, int version);
-extern double MGD77_Recalc_Mag_Anomaly (double time, double lon, double lat, double obs, BOOLEAN calc_date);
-extern double MGD77_time_to_fyear (double time);
+extern double MGD77_Recalc_Mag_Anomaly_IGRF (struct MGD77_CONTROL *F, double time, double lon, double lat, double obs, BOOLEAN calc_date);
+extern double MGD77_time_to_fyear (struct MGD77_CONTROL *F, double time);
+extern double MGD77_cal_to_fyear (struct GMT_gcal *cal);
+extern BOOLEAN MGD77_fake_times (struct MGD77_CONTROL *F, struct MGD77_HEADER *H, double *lon, double *lat, double *times, GMT_LONG nrec);
+extern double MGD77_utime2time (struct MGD77_CONTROL *F, double unix_time);
+extern double MGD77_time2utime (struct MGD77_CONTROL *F, double gmt_time);
+extern double MGD77_rdc2dt (struct MGD77_CONTROL *F, GMT_cal_rd rd, double secs);
+extern void MGD77_dt2rdc (struct MGD77_CONTROL *F, double t, GMT_cal_rd *rd, double *s);
+extern void MGD77_gcal_from_dt (struct MGD77_CONTROL *F, double t, struct GMT_gcal *cal);
+
+#ifdef USE_CM4 
+extern double MGD77_Calc_CM4 (struct MGD77_CONTROL *F, double time, double lon, double lat, BOOLEAN calc_date, struct MGD77_CM4 *CM4);
+extern double MGD77_Recalc_Mag_Anomaly_CM4 (struct MGD77_CONTROL *F, double time, double lon, double lat, double obs, BOOLEAN calc_date, struct MGD77_CM4 *CM4);
+extern void MGD77_CM4_end (struct MGD77_CM4 *CM4);
+#endif
 
 /* These are called indirectly but remain accessible for specialist programs */
 
@@ -513,5 +566,7 @@ double MGD77_Correction_Rec (struct MGD77_CORRECTION *C, double *value, double *
 void MGD77_Free_Correction (struct MGD77_CORRTABLE **CORR, int n);
 
 #include "mgd77_functions.h"	/* These were created by mgd77netcdfhelper.sh */
+#include "cm4_functions.h"
+extern void MGD77_CM4_init (struct MGD77_CONTROL *F, struct MGD77_CM4 *CM4);
 
 #endif	/* _MGD77_H */
