@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: mgd77convert.c,v 1.43 2009/01/09 04:02:35 guru Exp $
+ *	$Id: mgd77convert.c,v 1.45 2009/06/16 19:08:14 guru Exp $
  *
  *    Copyright (c) 2005-2009 by P. Wessel
  *    See README file for copying and redistribution conditions.
@@ -32,19 +32,19 @@ int main (int argc, char **argv)
 	char file[BUFSIZ], **list, *fcode = "act";
 	char *format_name[MGD77_N_FORMATS] = {"MGD77 ASCII", "MGD77+ netCDF", "ASCII table"};
 
-	struct MGD77_CONTROL In;
+	struct MGD77_CONTROL M;
 	struct MGD77_DATASET *D;
 	
 	argc = GMT_begin (argc, argv);		/* Initialize GMT Machinery */
 	
 	/* Initialize MGD77 output order and other parameters*/
 	
-	MGD77_Init (&In);			/* Initialize MGD77 Machinery */
+	MGD77_Init (&M);			/* Initialize MGD77 Machinery */
 	
 	for (argno = 1; !error && argno < argc; argno++) {	/* Process input options */
 		if (argv[argno][0] != '-') continue;
 		
-		in_out   = 0;	/* 0 for input and 1 for output */
+		in_out   = 0;	/* 0 for input and 1 for output (see -T -F) */
 		code_pos = 2;	/* argv[arg_no][code_pos] holds the character code for format */
 		switch (argv[argno][1]) {
 
@@ -54,15 +54,15 @@ int main (int argc, char **argv)
 				break;
 			
 			case 'L':	/* Determine level of error/warning checking and log destination */
-				In.verbose_level = 0;
+				M.verbose_level = 0;
 				for (i = 2; argv[argno][i]; i++) {
-					if (argv[argno][i] == 'e') In.verbose_level |= 2;
-					if (argv[argno][i] == 'w') In.verbose_level |= 1;
-					if (argv[argno][i] == '+') In.verbose_dest = 1;
+					if (argv[argno][i] == 'e') M.verbose_level |= 2;
+					if (argv[argno][i] == 'w') M.verbose_level |= 1;
+					if (argv[argno][i] == '+') M.verbose_dest = 1;
 				}
 				break;
 			case 'T':
-				in_out = 1;	/* The "to" field is 1 */
+				in_out = 1;	/* The "to" field is 1; the fall-trhough via a missing "break" is intentional */
 				if (argv[argno][code_pos] == '+') force = TRUE, code_pos++;	/* Force overwriting existing files */
 			case 'F':
 				switch (argv[argno][code_pos]) {									
@@ -89,10 +89,13 @@ int main (int argc, char **argv)
 				break;
 			default:		/* Options not recognized */
 				error = TRUE;
+				fprintf (stderr, "%s: Option -%c not recognized!\n", GMT_program, argv[argno][1]);
 				break;
 		}
 	}
 	
+	if (error) exit (EXIT_FAILURE);
+
 	/* Check that the options selected are mutually consistent */
 	
 	if (GMT_give_synopsis_and_exit || argc == 1) {	/* Display usage */
@@ -116,7 +119,7 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	n_paths = MGD77_Path_Expand (&In, argv, argc, &list);	/* Get list of requested IDs */
+	n_paths = MGD77_Path_Expand (&M, argv, argc, &list);	/* Get list of requested IDs */
 
 	if (n_paths == 0) {
 		fprintf(stderr, "%s: ERROR: No cruises given\n", GMT_program);
@@ -146,19 +149,19 @@ int main (int argc, char **argv)
 	for (argno = 0; argno < n_paths; argno++) {		/* Process each ID */
 	
 		D = MGD77_Create_Dataset ();	/* Get data structure w/header */
-		MGD77_Reset (&In);		/* Reset to start fresh for next file */
+		MGD77_Reset (&M);		/* Reset to start fresh for next file */
 
-		In.format  = format[GMT_IN];	/* Set input file's format and read everything into memory */
-		In.original = original;
-		if (original) In.use_corrections[MGD77_M77_SET] = In.use_corrections[MGD77_CDF_SET] = FALSE;	/* Turn off E77 corrections */
-		for (i = 0; i < MGD77_N_FORMATS; i++) MGD77_format_allowed[i] = (In.format == i) ? TRUE : FALSE;	/* Only allow the specified input format */
-		if (MGD77_Open_File (list[argno], &In, MGD77_READ_MODE)) continue;
-		if (MGD77_Read_Header_Record (list[argno], &In, &D->H)) {
+		M.format  = format[GMT_IN];	/* Set input file's format and read everything into memory */
+		M.original = original;
+		if (original) M.use_corrections[MGD77_M77_SET] = M.use_corrections[MGD77_CDF_SET] = FALSE;	/* Turn off E77 corrections */
+		for (i = 0; i < MGD77_N_FORMATS; i++) MGD77_format_allowed[i] = (M.format == i) ? TRUE : FALSE;	/* Only allow the specified input format */
+		if (MGD77_Open_File (list[argno], &M, MGD77_READ_MODE)) continue;
+		if (MGD77_Read_Header_Record (list[argno], &M, &D->H)) {
 			fprintf (stderr, "%s: Error reading header sequence for cruise %s\n", GMT_program, list[argno]);
 			exit (EXIT_FAILURE);
 		}
-		sprintf (file, "%s.%s", In.NGDC_id, MGD77_suffix[format[GMT_OUT]]);
-		if (format[GMT_IN] == format[GMT_OUT] && !(In.path[0] == '/' || In.path[1] == ':')) {
+		sprintf (file, "%s.%s", M.NGDC_id, MGD77_suffix[format[GMT_OUT]]);
+		if (format[GMT_IN] == format[GMT_OUT] && !(M.path[0] == '/' || M.path[1] == ':')) {
 			fprintf(stderr, "%s: Input and Output file have same name! Output file will have extension \".new\" appended\n", GMT_program);
 			strcat (file, ".new");	/* To avoid overwriting original file */
 		}
@@ -166,14 +169,14 @@ int main (int argc, char **argv)
 			if (force) {	/* Must delete the file first */
 				if (remove (file)) {	/* Oops, removal failed */
 					fprintf(stderr, "%s: Unable to remove existing file %s - skipping the conversion\n", GMT_program, file);
-					MGD77_Close_File (&In);
+					MGD77_Close_File (&M);
 					MGD77_Free (D);	/* Free memory allocated by MGD77_Read_File */
 					continue;
 				}
 			}
 			else {	/* Cowardly refuse to do this */
 				fprintf(stderr, "\n%s: Output file already exists.  Use -T+%c to force overwriting\n", GMT_program, fcode[format[GMT_OUT]]);
-				MGD77_Close_File (&In);
+				MGD77_Close_File (&M);
 				MGD77_Free (D);	/* Free memory allocated by MGD77_Read_File */
 				continue;
 			}
@@ -181,29 +184,29 @@ int main (int argc, char **argv)
 		
 		/* OK, now we can read the data set */
 		
-		if (MGD77_Read_Data (list[argno], &In, D)) {
+		if (MGD77_Read_Data (list[argno], &M, D)) {
 			fprintf (stderr, "%s: Error reading data set for cruise %s\n", GMT_program, list[argno]);
 			exit (EXIT_FAILURE);
 		}
-		MGD77_Close_File (&In);
+		MGD77_Close_File (&M);
 		
-		MGD77_Verify_Prep (&In, D);	/* Get key meta-data derived form data records */
+		MGD77_Verify_Prep (&M, D);	/* Get key meta-data derived form data records */
 
-		MGD77_Verify_Header (&In, &(D->H), NULL);	/* Verify the header */
+		MGD77_Verify_Header (&M, &(D->H), NULL);	/* Verify the header */
 	
 		if (format[GMT_IN] == MGD77_FORMAT_CDF && format[GMT_OUT] != MGD77_FORMAT_CDF && (D->H.info[MGD77_CDF_SET].n_col || D->flags[0] || D->flags[1])) {
-			fprintf(stderr, "\n%s: Warning: Input file contains enhanced material that the output file format cannot represent\n", GMT_program);
+			fprintf(stderr, "\n%s: Warning: Mput file contains enhanced material that the output file format cannot represent\n", GMT_program);
 		}
 
 		/* OK, ready to write out converted file */
 		
-		In.format  = format[GMT_OUT];				/* Change the format to the desired output format and write new file in current directory */
-		In.original = TRUE;					/* Always write to original attributes */
-		for (i = 0; i < MGD77_N_FORMATS; i++) MGD77_format_allowed[i] = (In.format == i) ? TRUE : FALSE;	/* Only allow the specified output format */
-		D->H.author = (char *)GMT_memory (VNULL, strlen (In.user)+1, sizeof (char), GMT_program);	/* Allocate space for author */
-		strcpy (D->H.author, In.user);									/* Pass current user login id as author */
+		M.format  = format[GMT_OUT];				/* Change the format to the desired output format and write new file in current directory */
+		M.original = TRUE;					/* Always write to original attributes */
+		for (i = 0; i < MGD77_N_FORMATS; i++) MGD77_format_allowed[i] = (M.format == i) ? TRUE : FALSE;	/* Only allow the specified output format */
+		D->H.author = (char *)GMT_memory (VNULL, strlen (M.user)+1, sizeof (char), GMT_program);	/* Allocate space for author */
+		strcpy (D->H.author, M.user);									/* Pass current user login id as author */
 		if (D->H.history) GMT_free ((void *)D->H.history);	/* Make sure history is blank so it is reset by MGD77_Write_File */
-		if (MGD77_Write_File (file, &In, D)) {
+		if (MGD77_Write_File (file, &M, D)) {
 			fprintf (stderr, "%s: Error writing new file for cruise %s\n", GMT_program, list[argno]);
 			exit (EXIT_FAILURE);
 		}
@@ -221,7 +224,7 @@ int main (int argc, char **argv)
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Converted %d MGD77 files\n", GMT_program, n_cruises);
 	
 	MGD77_Path_Free (n_paths, list);
-	MGD77_end (&In);
+	MGD77_end (&M);
 
 	exit (EXIT_SUCCESS);
 }
