@@ -1,8 +1,8 @@
 /* -------------------------------------------------------------------
- *	$Id: mgd77sniffer.h,v 1.31 2008/04/08 05:24:16 mtchandl Exp $	
- *      See COPYING file for copying and redistribution conditions.
+ *	$Id: mgd77sniffer.h,v 1.36 2009/09/09 23:27:04 guru Exp $	
+ *      See LICENSE.TXT file for copying and redistribution conditions.
  *
- *    Copyright (c) 2004-2008 by P. Wessel and M. T. Chandler
+ *    Copyright (c) 2004-2009 by P. Wessel and M. T. Chandler
  *	File:	mgd77sniffer.h
  *
  *	Include file for mgd77sniffer
@@ -38,8 +38,8 @@
 #define MGD77_FATHOMS_PER_METER     (1.0 / MGD77_METERS_PER_FATHOM)
 #define MGD77_MIN_RLS_BINS          20
 #define MGD77_MIN_RLS_PTS           100
-#define MGD77_N_MAG_RF              13
 #define MGD77_MAX_SEARCH            50
+#define MGD77_DIST_FROM_COAST       100                   /* meters */
 #define MGD77_POS_BIT               (1 << 0)
 #define MGD77_ZERO_BIT              (1 << 1)
 #define MGD77_NEG_BIT               (1 << 2)
