@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: binlegs.c,v 1.17 2009/01/09 04:02:35 guru Exp $
+ *	$Id: binlegs.c,v 1.18 2009/07/11 03:16:31 guru Exp $
  *
  *    Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
@@ -61,13 +61,13 @@ int main (int argc, char **argv)
 	struct INFO *info_head, *this_info, *new_info, *make_info(char *name, int id_no, int flag);
 	struct LEG *this_leg, *make_leg(int id, int flag);
 	
-	char file[BUFSIZ], leg[16], lname[16], line[BUFSIZ];
+	char file[BUFSIZ], leg[16], lname[16], line[BUFSIZ], *c_unused = NULL;
 	char leg_file[BUFSIZ], index_file[BUFSIZ], old_leg_file[BUFSIZ], old_index_file[BUFSIZ], path[BUFSIZ];
 	char error = FALSE, replace = FALSE, delete = FALSE, found_it, verbose = FALSE;
 	
 	FILE *fp = NULL, *fbin, *fleg, *find;
 	
-	int no_of_legs, index, id, bin, free_id, max, flag;
+	int no_of_legs, index, id, bin, free_id, max, flag, not_used = 0;
 	int remove_leg(int leg_id, struct GRID *grid);
 	int i, j, year, last_id;
 	struct GRID grid[64800];
@@ -150,13 +150,13 @@ int main (int argc, char **argv)
 	/* Read geographical leg-info from gmt_index.b file */
 	
 	while ((fread ((void *)(&index), (size_t)4, (size_t)1, fbin)) == 1) {
-		fread ((void *)(&no_of_legs), (size_t)4, (size_t)1, fbin);
+		not_used = fread ((void *)(&no_of_legs), (size_t)4, (size_t)1, fbin);
 		if (grid[index].n_legs == 0) {
 			grid[index].first_leg = make_leg (0, 0);
 			grid[index].last_leg = grid[index].first_leg;
 		}
 		for (i = 0; i < no_of_legs; i++) {
-			fread ((void *)(&id), (size_t)4, (size_t)1, fbin);
+			not_used = fread ((void *)(&id), (size_t)4, (size_t)1, fbin);
 			flag = id & 15;
 			id >>= 4;
 			grid[index].last_leg->next_leg = make_leg (id, flag);
@@ -202,7 +202,7 @@ int main (int argc, char **argv)
 			continue;
 		}
 		if (verbose) fprintf (stderr, "binlegs: Adding leg: %s\n",leg);
-		fgets (line, BUFSIZ, find);
+		c_unused = fgets (line, BUFSIZ, find);
 		sscanf (line, "%d %d", &year, &max);
 		
 		/* If a leg is replaced, then use the same id_no, else increment to get a new one */
@@ -267,9 +267,9 @@ int main (int argc, char **argv)
 	
 	for (bin = 0; bin < 64800; bin++) {
 		if (grid[bin].n_legs > 0) {
-			fwrite ((void *)(&bin), (size_t)4, (size_t)1, fbin);
-			fwrite ((void *)(&grid[bin].n_legs), (size_t)4, (size_t)1, fbin);
-			for (this_leg = grid[bin].first_leg->next_leg; this_leg; this_leg = this_leg->next_leg) fwrite ((void *)(&this_leg->leg_id), (size_t)4, (size_t)1, fbin);
+			not_used = fwrite ((void *)(&bin), (size_t)4, (size_t)1, fbin);
+			not_used = fwrite ((void *)(&grid[bin].n_legs), (size_t)4, (size_t)1, fbin);
+			for (this_leg = grid[bin].first_leg->next_leg; this_leg; this_leg = this_leg->next_leg) not_used = fwrite ((void *)(&this_leg->leg_id), (size_t)4, (size_t)1, fbin);
 		}
 	}
 	fclose (fbin);
