@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: dat2gmt.c,v 1.17 2009/01/09 04:02:35 guru Exp $
+ *	$Id: dat2gmt.c,v 1.18 2009/07/11 03:16:31 guru Exp $
  *
  *    Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
@@ -27,7 +27,7 @@ int main (int argc, char **argv)
 	int leg_year, n_records;
 	int rec, time, mag, top, yr, mo, dy, hr, mi, sc, n_alloc = MEM_SIZE, conv;
 	double lat, lon, grv;
-	char line[BUFSIZ], agency[10], stime[20], sgrv[10], smag[10], stop[10];
+	char line[BUFSIZ], agency[10], stime[20], sgrv[10], smag[10], stop[10], *not_used = NULL;
 	FILE *fpi, *fpo;
 	struct GMTMGG_TIME *gmt;
 	struct GMTMGG_REC *record;
@@ -43,7 +43,7 @@ int main (int argc, char **argv)
 	
 	/* Read first record of file containing start-year, n_records and info */
 	
-	fgets (line, BUFSIZ, fpi);
+	not_used = fgets (line, BUFSIZ, fpi);
 	sscanf (line, "%d %d %s", &leg_year, &n_records, agency);
 	gmt = gmtmgg_init (leg_year);
 	
