@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt2bin.c,v 1.17 2009/01/09 04:02:35 guru Exp $
+ *	$Id: gmt2bin.c,v 1.18 2009/07/11 03:16:31 guru Exp $
  *
  *    Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
@@ -39,7 +39,7 @@
 
 int main (int argc, char **argv)
 {
-	int year, n_records;
+	int year, n_records, not_used = 0;
 	int bindex, error = FALSE;
 	int i, lastb, llat1, llat2, llon1, llon2, newbin, flag;
 	double xedge, yedge, xx1, xx2, yy1, yy2, delx, dely, slope, xedgey, yedgex, rxedge, ryedge;
@@ -93,9 +93,9 @@ int main (int argc, char **argv)
 			fprintf (stderr, "gmt2bin: Cannot open file %s\n", gmtfile);
 			continue;
 		}
-		fread ((void *)&year, (size_t)4, (size_t)1, fp);
-		fread ((void *)&n_records, (size_t)4, (size_t)1, fp);
-		fread ((void *)agency, (size_t)10, (size_t)1, fp);
+		not_used = fread ((void *)&year, (size_t)4, (size_t)1, fp);
+		not_used = fread ((void *)&n_records, (size_t)4, (size_t)1, fp);
+		not_used = fread ((void *)agency, (size_t)10, (size_t)1, fp);
 		
 		record = (struct GMTMGG_REC *) GMT_memory ((void *)record, (size_t)n_records, sizeof (struct GMTMGG_REC), "gmt2bin");
 		
