@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtlist.c,v 1.28 2009/01/09 04:02:35 guru Exp $
+ *	$Id: gmtlist.c,v 1.30 2009/07/11 03:16:31 guru Exp $
  *
  *    Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
@@ -44,8 +44,7 @@
 
 int main (int argc, char **argv)
 {
-	int leg_year, n_records;
-	
+	int leg_year, n_records, not_used = 0;
 	int rec, i, j, argno, id, no[10], nval = 0;
 	int start_time = 0, stop_time = 2000000000, dlon, last_lon = 0;
 	int mon1, day1 = 0, year1, hour1, min1, time, last_lat = 0, mon2, day2 = 0, year2, hour2, min2;
@@ -390,7 +389,7 @@ int main (int argc, char **argv)
 			else {
 				dlon = record.lon - last_lon;
 				if (abs (dlon) > 180000000) dlon = irint(copysign ((double) (360000000 - abs (dlon)), (double)dlon));
-				dx = (double) dlon * cos (0.5e-06*D2R*(double)(record.lat+last_lat));
+				dx = (double) dlon * cosd (0.5e-06*(double)(record.lat+last_lat));
 				dy = (double) (record.lat - last_lat);
 				ds = KMPRDEG * hypot (dx, dy);
 				dt = record.time - last_time;
@@ -446,7 +445,7 @@ int main (int argc, char **argv)
 					case 0:	/* Print out time */
 						if (binary) {
 							val = record.time;
-							fwrite ((void *)&val, sizeof (double), (size_t)1, stdout);
+							not_used = fwrite ((void *)&val, sizeof (double), (size_t)1, stdout);
 						}
 						else if (tsec)
 							printf ("%d", record.time);
@@ -464,78 +463,78 @@ int main (int argc, char **argv)
 					case 1:	/* Print out longitude */
 						if (lon > 180.0 && greenwich) lon -= 360.0;
 						if (binary)
-							fwrite ((void *)&lon, sizeof (double), (size_t)1, stdout);
+							not_used = fwrite ((void *)&lon, sizeof (double), (size_t)1, stdout);
 						else
 							(GMT_io.io_header[GMT_OUT]) ? printf("%.5f", lon) : printf("%9.5f", lon);
 						break;
 					case 2:	/* Print out latitude */
 						if (binary)
-							fwrite ((void *)&lat, sizeof (double), (size_t)1, stdout);
+							not_used = fwrite ((void *)&lat, sizeof (double), (size_t)1, stdout);
 						else
 							(GMT_io.io_header[GMT_OUT]) ? printf("%.5f", lat) : printf("%8.5f", lat);
 						break;
 					case 3:	/* Print out gravity */
 						if (record.gmt[0] == GMTMGG_NODATA)
-							(binary) ? fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
+							(binary) ? not_used = fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
 						else {
 							if (id >= 0) grv -= bin[id]->dc_shift_gmt[0] + bin[id]->drift_rate_gmt[0] * record.time;
 							if (binary)
-								fwrite ((void *)&grv, sizeof (double), (size_t)1, stdout);
+								not_used = fwrite ((void *)&grv, sizeof (double), (size_t)1, stdout);
 							else
 		 						(GMT_io.io_header[GMT_OUT]) ? printf("%.2f", grv) : printf("%9.2f", grv);
 						}
 		 				break;
 					case 4:	/* Print out magnetics */
 						if (record.gmt[1] == GMTMGG_NODATA)
-							(binary) ? fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
+							(binary) ? not_used = fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
 						else {
 							if (id >= 0) mag -= bin[id]->dc_shift_gmt[1] + bin[id]->drift_rate_gmt[1] * record.time;
 							if (binary)
-								fwrite ((void *)&mag, sizeof (double), (size_t)1, stdout);
+								not_used = fwrite ((void *)&mag, sizeof (double), (size_t)1, stdout);
 							else
 		 						(GMT_io.io_header[GMT_OUT]) ? printf("%.1f", mag) : printf("%8.1f", mag);
 						}
 						break;
 					case 5:	/* Print out bathymetry */
 						if (record.gmt[2] == GMTMGG_NODATA)
-							(binary) ? fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
+							(binary) ? not_used = fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
 						else {
 							if (id >= 0) top -= bin[id]->dc_shift_gmt[2] + bin[id]->drift_rate_gmt[2] * record.time;
 							if (binary)
-								fwrite ((void *)&top, sizeof (double), (size_t)1, stdout);
+								not_used = fwrite ((void *)&top, sizeof (double), (size_t)1, stdout);
 							else
 								(GMT_io.io_header[GMT_OUT]) ? printf("%.1f", top) : printf("%8.1f", top);
 						}
 						break;
 					case 6:	/* Print out distance */
 						if (binary)
-							fwrite ((void *)&dist, sizeof (double), (size_t)1, stdout);
+							not_used = fwrite ((void *)&dist, sizeof (double), (size_t)1, stdout);
 						else
 							(GMT_io.io_header[GMT_OUT]) ? printf("%.3f", dist) : printf("%9.3f", dist);
 						break;
 					case 7:	/* Print out heading */
 						if (heading == GMTMGG_NODATA)
-							(binary) ? fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
+							(binary) ? not_used = fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
 						else {
 							if (binary)
-								fwrite ((void *)&heading, sizeof (double), (size_t)1, stdout);
+								not_used = fwrite ((void *)&heading, sizeof (double), (size_t)1, stdout);
 							else
 								(GMT_io.io_header[GMT_OUT]) ? printf("%.1f", heading) : printf("%6.1f", heading);
 						}
 						break;
 					case 8:	/* Print out velocity */
 						if (speed == GMTMGG_NODATA)
-							(binary) ? fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
+							(binary) ? not_used = fwrite ((void *)&GMT_f_NaN, sizeof (double), (size_t)1, stdout) : printf ("NaN");
 						else {
 							if (binary)
-								fwrite ((void *)&speed, sizeof (double), (size_t)1, stdout);
+								not_used = fwrite ((void *)&speed, sizeof (double), (size_t)1, stdout);
 							else
 								(GMT_io.io_header[GMT_OUT]) ? printf("%.2f", speed) : printf("%6.2f", speed);
 						}
 						break;
 					case 9:	/* Print out weight */
 						if (binary)
-							fwrite ((void *)&weight, sizeof (double), (size_t)1, stdout);
+							not_used = fwrite ((void *)&weight, sizeof (double), (size_t)1, stdout);
 
 						else
 							(GMT_io.io_header[GMT_OUT]) ? printf("%g", weight) : printf("%6.2f", weight);
