@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: gmttrack.c,v 1.34 2009/01/09 04:02:35 guru Exp $
+ *	$Id: gmttrack.c,v 1.35 2009/02/25 04:32:59 remko Exp $
  *
  *    Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
@@ -252,7 +252,7 @@ int main (int argc, char **argv)
 			else {
 				dlon = fabs (lon - last_lon);
 				if (dlon > 180.0) dlon = 360.0 - dlon;
-				dx = dlon * cos (0.5 * D2R * (lat+last_lat));
+				dx = dlon * cosd (0.5 * (lat+last_lat));
 				dy = (lat - last_lat);
 				last_lon = lon;
 				last_lat = lat;
@@ -425,7 +425,7 @@ double heading (int rec, int n_records, struct GMTMGG_REC *record)
 	GMT_geo_to_xy (lon, lat, &x2, &y2);
 	lon = record[i1].lon * 1.0e-6;	lat = record[i1].lat * 1.0e-6;
 	GMT_geo_to_xy (lon, lat, &x1, &y1);
-	angle = atan2 (y2 - y1, x2 - x1) * R2D;
+	angle = atan2d (y2 - y1, x2 - x1);
 	if (angle > 90.)
 		angle -= 180;
 	else if (angle < -90.0)
