@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: mgd77togmt.c,v 1.18 2008/05/12 20:31:30 guru Exp $
+ *	$Id: mgd77togmt.c,v 1.25 2009/03/27 23:26:54 guru Exp $
  *
- *    Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
+ *    Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *    See README file for copying and redistribution conditions.
  *--------------------------------------------------------------------*/
 /*
@@ -39,13 +39,16 @@
 #include "gmt.h"
 #include "gmt_mgg.h"
 
-int main (int argc, char **argv)
-{
-	int n_records, *year;
+#define MPRDEG 111.1949e-3
+
+int main (int argc, char **argv) {
+	int n_records, *year, k;
 	int i, rec, n_read, n_files = 0, n_alloc = GMT_CHUNK, leg_year = 0, len;
 	int T_INC = 60, fake_running_time = 0;
-	BOOLEAN use_list = FALSE, set_agency = FALSE;
+	int anom_offset = 0;
+	BOOLEAN use_list = FALSE, set_agency = FALSE, mag_rewind = FALSE, greenwich = FALSE;
 	int t_flag = FALSE, error = FALSE;
+	double cable_len = 0;
 	char file[80], *mfile, *list, agency[10], *legfname, line[BUFSIZ], **mgd77, **prefix;
 	struct GMTMGG_TIME *gmt;
 	struct GMTMGG_REC *record;
@@ -80,9 +83,23 @@ int main (int argc, char **argv)
 				case 'A':
 					strcpy (agency, &argv[i][2]);
 					break;
+				case 'T':			/* Read Total field instead of anomaly and subtract a cte */
+					if (argv[i][2])
+						anom_offset = atoi (&argv[i][2]);
+					else
+						anom_offset = 40000;
+					break;
+				case 'G':
+					greenwich = TRUE;
+					break;
 				case 'V':
 					gmtdefs.verbose = TRUE;
 					break;
+				case 'W':
+					if (argv[i][2])
+						cable_len = atof (&argv[i][2]);
+					mag_rewind = TRUE;
+					break;
 				default:
 					fprintf (stderr, "SYNTAX ERROR:  Unrecognized option -%c\n", argv[i][1]);
 					error = TRUE;
@@ -101,16 +118,33 @@ int main (int argc, char **argv)
 		fprintf (stderr, "SYNTAX ERROR -L option:  Specify -L or the combination -F, -Y\n");
 		error = TRUE;
 	}
+	if (!use_list && !mfile && !legfname) {
+		fprintf (stderr, "SYNTAX ERROR -F option:  When using standard input you must use -F option.\n");
+		error = TRUE;
+	}
 	
 	if (argc == 1 || error) {
-		fprintf (stderr, "usage: mgd77togmt [mgd77file] -F<filename> -Y<leg_year> OR -L<leglist> [-A<10 char agency name>] [NGDC-file -I<time_increment>] [-V]\n");
-		fprintf (stderr, "	-F sets gmtfilename prefix (e.g., without path or .gmt)\n");
-		fprintf (stderr, "	-Y sets start year\n");
+		fprintf (stderr, "usage: mgd77togmt [mgd77file] [-F<filename>] -Y<leg_year> OR -L<leglist> [-A<10 char agency name>] [-G] [NGDC-file -I<time_increment>] [-V] [-T[<offset>]] [-W[<cable_length>]]\n");
+		fprintf (stderr, "	-Y sets start year. If not provided and -L option not used, it tries to get\n");
+		fprintf (stderr, "	   it from header file. The header file must be in the same directory of the\n");
+		fprintf (stderr, "	   main file and must have a name equal to the main but with a .h77 extension.\n");
 		fprintf (stderr, "	-L gives name of a list with several records of <mgd77file> <gmtprefix> <leg_year>\n");
-		fprintf (stderr,"\n\tOPTIONS:\n");
-		fprintf(stderr,"	-A sets optional 10 char info for gmt header.\n");
+		fprintf (stderr, "\n\tOPTIONS:\n");
+		fprintf (stderr, "	-A sets optional 10 char info for gmt header.\n");
+		fprintf (stderr, "	-F sets gmtfilename prefix (e.g., without path or .gmt). If not given, it\n");
+		fprintf (stderr, "	   will be constructed from the mgd77file name plus the .gmt extension.\n");
+		fprintf(stderr,"	-G force geographical longitudes (-180/+180) [Default is 0-360]\n");
 		fprintf (stderr, "	-I sets fake timeincrement for files without time information\n");
-		fprintf(stderr,"	-V runs in verbose mode.\n");
+		fprintf (stderr, "	-T Extracts Total field instead of anomaly. Since F does not hold in a 2 byte int var\n");
+		fprintf (stderr, "	   we subtract a constant [default = 40000] but you can provide another value in <offset>.\n");
+		fprintf (stderr, "	-W Take into account that the magnetometer is not at ship's position.\n");
+		fprintf (stderr, "	   <cable> is magnetometer tow distance [default = 200 meters].\n");
+		fprintf (stderr, "	   If -W only is given (e.g. no <cable>) and like with the -Y option, we try\n");
+		fprintf (stderr, "	   to get the tow distance from the header file. Failing, defaults to 200 meters.\n");
+		fprintf (stderr, "	   Note that this option will throw away the first points whose accumulated.\n");
+		fprintf (stderr, "	   distance since the start of magnetic acquisition is less than cable.\n");
+		fprintf (stderr, "	   length, and likewise for the end of the mag profile.\n");
+		fprintf (stderr, "	-V runs in verbose mode.\n");
 		
 		exit (EXIT_FAILURE);
 	}
@@ -119,13 +153,13 @@ int main (int argc, char **argv)
 
 	if (use_list) {
 	
-		mgd77 = (char **) GMT_memory (VNULL, (size_t)n_alloc, sizeof (char *), "mgd77togmt");
-		prefix = (char **) GMT_memory (VNULL, (size_t)n_alloc, sizeof (char *), "mgd77togmt");
-		year = (int *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (int), "mgd77togmt");
 		if ((fp = fopen (list, "r")) == NULL) {
 			fprintf (stderr, "mgd77togmt: Cannot open file %s!\n", list);
 			exit (EXIT_FAILURE);
 		}
+		mgd77 = (char **) GMT_memory (VNULL, (size_t)n_alloc, sizeof (char *), "mgd77togmt");
+		prefix = (char **) GMT_memory (VNULL, (size_t)n_alloc, sizeof (char *), "mgd77togmt");
+		year = (int *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (int), "mgd77togmt");
 		n_read = 0;
 		while (fgets (line, BUFSIZ, fp)) {
 			mgd77[n_read]  = (char *) GMT_memory (VNULL, (size_t)80, sizeof (char), "mgd77togmt");
@@ -149,6 +183,18 @@ int main (int argc, char **argv)
 		year = (int *) GMT_memory ((void *)year, (size_t)n_files, sizeof (int), "mgd77togmt");
 	}
 	else {
+		int	free_legfname = 0;
+		char	*ext;
+		if (!legfname) {
+			legfname = strdup(mfile);
+			free_legfname = 1;
+			GMT_chop_ext(legfname);
+		}
+		else {		/* Make sure that the legname does not have the .gmt extension */
+			ext = GMT_chop_ext(legfname);
+			if (ext && strcmp(ext,".gmt"))		/* Hoops, removed a wrong extension */
+				strcat(legfname,ext);
+		}
 		mgd77  = (char **) GMT_memory (VNULL, (size_t)1, sizeof (char *), "mgd77togmt");
 		prefix = (char **) GMT_memory (VNULL, (size_t)1, sizeof (char *), "mgd77togmt");
 		year = (int *) GMT_memory (VNULL, (size_t)1, sizeof (int), "mgd77togmt");
@@ -158,10 +204,11 @@ int main (int argc, char **argv)
 		strcpy (prefix[0], legfname);
 		year[0] = leg_year;
 		n_files = 1;
+		if (free_legfname) free ((void *)legfname);
 	}
-	
+
 	n_alloc = GMT_CHUNK;
-	
+
 	for (i = 0; i < n_files; i++) {
 	
 		if (gmtdefs.verbose) fprintf (stderr, "mgd77togmt: Processing file %s\n", mgd77[i]);
@@ -181,6 +228,46 @@ int main (int argc, char **argv)
 			if (gmtdefs.verbose) fprintf (stderr, "mgd77togmt: Agency string set to %s\n", prefix[i]);
 		}
 
+		/* See if we have the header file arround and if yes try to get year and mag tow distance (if needed) */
+		if (!leg_year || (mag_rewind && !cable_len)) {
+			char	*hdr, s_yr[5];
+			FILE	*fph;
+			hdr = strdup(&mgd77[i][0]);
+			GMT_chop_ext(hdr);
+			strcat(hdr,".h77");
+			k = 0;
+			if ((fph = fopen (hdr, "r")) != NULL) {
+				while (fgets (line, BUFSIZ, fph) && k < 13) {
+					k++;
+					if (k == 4 && !leg_year) {
+						strncpy (s_yr, &line[0], (size_t) 4);	s_yr[4] = 0;
+						year[i] = atoi (s_yr);
+						if (gmtdefs.verbose) fprintf (stderr, "\tGot year %d from header file %s\n", year[i], hdr);
+						if (!mag_rewind || cable_len > 0) k = 14;	/* End reading header */
+					}
+					else if (k == 13) {	/* Seek for the magnetometer tow distance */
+						strncpy (s_yr, &line[5], (size_t) 4);	s_yr[4] = 0;
+						if (s_yr[0] == ' ' && s_yr[1] == ' ' && s_yr[2] == ' ' && s_yr[3] == ' ')
+							cable_len = 200;	/* The default tow distance */
+						else
+							cable_len = atof (s_yr);
+						if (cable_len > 500) cable_len = 200;	/* This accounts also for 999 non-values in header */
+						if (gmtdefs.verbose) fprintf (stderr, "\tUsing magnetometer tow distance = %.0f\n", cable_len);
+					}
+				}
+				fclose(fph);
+			}
+			else {
+				if (!leg_year) {
+					fprintf (stderr, "WARNING: Year not provided and companion header file not found. Jumping this file\n");
+					continue;
+				}
+				if (mag_rewind) cable_len = 200;
+			}
+
+			free ((void *) hdr);
+		}
+
 		gmt = gmtmgg_init (year[i]);
 
 		record = (struct GMTMGG_REC *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct GMTMGG_REC), "mgd77togmt");
@@ -193,12 +280,15 @@ int main (int argc, char **argv)
 				fprintf (stderr, "mgd77togmt: Record # %d has incorrect length (%d), skipped\n", n_read, len);
 				continue;
 			}
-			if (!gmtmgg_decode_MGD77 (line, t_flag, &record[rec], &gmt)) {
+			if (!gmtmgg_decode_MGD77 (line, t_flag, &record[rec], &gmt, anom_offset)) {
 				if (t_flag) record[rec].time = (fake_running_time += T_INC);
+				if (greenwich && record[rec].lon > 180000000) record[rec].lon -= 360000000;
 				rec++;
 			}
 			else
 				fprintf (stderr, "mgd77togmt: Trouble decoding record # %d (skipped)\n", n_read);
+
+
 			if (rec == n_alloc) {
 				n_alloc <<= 1;
 				record = (struct GMTMGG_REC *) GMT_memory ((void *)record, (size_t)n_alloc, sizeof (struct GMTMGG_REC), "mgd77togmt");
@@ -227,10 +317,51 @@ int main (int argc, char **argv)
 			fprintf (stderr,"mgd77togmt: Error while writing info-header\n");
 			exit (EXIT_FAILURE);
 		}
-		for (rec = 0; rec < n_records; rec++) {
-			if (fwrite ((void *)(&record[rec]), (size_t)18, (size_t)1, fpo) != (size_t)1) {
-				fprintf (stderr,"mgd77togmt: Error writing data record no %d\n",rec);
-				exit (EXIT_FAILURE);
+
+		if (!mag_rewind) {
+			for (rec = 0; rec < n_records; rec++) {
+				if (fwrite ((void *)(&record[rec]), (size_t)18, (size_t)1, fpo) != (size_t)1) {
+					fprintf (stderr,"mgd77togmt: Error writing data record no %d\n",rec);
+					exit (EXIT_FAILURE);
+				}
+			}
+		}
+		else {			/* At ship's position we'll get the mag reading down the track that is closest to cable length */
+			int	n, dlon, last_lon = 0, last_lat = 0, itmp;
+			double	*ds, dds, dx, dy;
+			ds = (double *) GMT_memory (VNULL, (size_t)n_records, sizeof (double), "mgd77togmt");
+			for (rec = 0; rec < n_records; rec++) {
+				if (rec == 0) {
+					last_lon = record[0].lon;
+					last_lat = record[0].lat;
+					ds[0] = 0.0;
+				}
+				else {
+					dlon = record[rec].lon - last_lon;
+					dx = (double) dlon * cosd (0.5e-06*(double)(record[rec].lat+last_lat));
+					dy = (double) (record[rec].lat - last_lat);
+					ds[rec] = ds[rec-1] + MPRDEG * hypot (dx, dy);
+					last_lon = record[rec].lon;
+					last_lat = record[rec].lat;
+				}
+			}
+
+			for (rec = 0; rec < n_records; rec++) {
+				dds = ds[rec] - cable_len;
+				n = rec;
+				if (dds < 0) {			/* First points (of distance < cable_len) are lost */
+					record[rec].gmt[1] = GMTMGG_NODATA;
+				}
+				else {
+					while ((ds[n] - dds) > 0) n--;
+				}
+				itmp = record[rec].gmt[1];
+				record[rec].gmt[1] = record[n].gmt[1];
+				if (fwrite ((void *)(&record[rec]), (size_t)18, (size_t)1, fpo) != (size_t)1) {
+					fprintf (stderr,"mgd77togmt: Error writing data record no %d\n",rec);
+					exit (EXIT_FAILURE);
+				}
+				record[rec].gmt[1] = itmp;	/* Reset to original to be used when its turn arrives */
 			}
 		}
 		fclose (fpo);
