@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *    $Id: minmax.c,v 1.55 2009/02/02 22:25:51 remko Exp $
+ *    $Id: minmax.c,v 1.62 2009/09/09 23:27:03 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -36,12 +36,12 @@ struct MINMAX_CTRL {	/* All control options for this program (except common args
 	struct E {	/* -E<L|l|H|h><col> */
 		BOOLEAN active;
 		BOOLEAN abs;
-		int mode;
-		int col;
+		GMT_LONG mode;
+		GMT_LONG col;
 	} E;
 	struct I {	/* -Idx[/dy[/<dz>..] */
 		BOOLEAN active;
-		int ncol;
+		GMT_LONG ncol;
 		double inc[BUFSIZ];
 	} I;
 	struct S {	/* -S[x|y] */
@@ -51,7 +51,7 @@ struct MINMAX_CTRL {	/* All control options for this program (except common args
 	struct T {	/* -T<dz>[/<col>] */
 		BOOLEAN active;
 		double inc;
-		int col;
+		GMT_LONG col;
 	} T;
 };
 
@@ -60,11 +60,9 @@ int main (int argc, char **argv)
 	BOOLEAN  error = FALSE, nofile = TRUE, done = FALSE, got_stuff = FALSE, first, give_r_string = FALSE;
 	BOOLEAN  dateline = FALSE, brackets = FALSE, work_on_abs_value, special = FALSE, quad[4] = {FALSE, FALSE, FALSE, FALSE};
 
-	char line[BUFSIZ], file[BUFSIZ], chosen[BUFSIZ];
+	char line[BUFSIZ], file[BUFSIZ], chosen[BUFSIZ], *not_used = NULL;
 
-	int i, j, ncol, n_files = 0, fno, n_args, n_fields, n_expected_fields, quad_no;
-
-	GMT_LONG n;
+	GMT_LONG i, j, ncol, n_files = 0, fno, n_args, n_fields, n_expected_fields, quad_no, n;
 	
 	double *xyzmin, *xyzmax, west, east, south, north, low, high, value, e_min = DBL_MAX, e_max = -DBL_MAX, *in;
 	double xmin1 = 360.0, xmin2 = 360.0, xmax1 = -360.0, xmax2 = -360.0;
@@ -73,7 +71,7 @@ int main (int argc, char **argv)
 
 	struct MINMAX_CTRL *Ctrl;
 
-	int strip_blanks_and_output (double x, int col);
+	GMT_LONG strip_blanks_and_output (double x, GMT_LONG col);
 	void *New_Minmax_Ctrl (), Free_Minmax_Ctrl (struct MINMAX_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -86,9 +84,11 @@ int main (int argc, char **argv)
 				/* Common parameters */
                       
 				case 'H':
+				case 'M':
 				case ':':
   				case 'b':
   				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
@@ -111,7 +111,6 @@ int main (int argc, char **argv)
 							break;
 						case 'H':
 							Ctrl->E.abs = TRUE;
-							break;
 						case 'h':
 							Ctrl->E.mode = +1;
 							break;
@@ -122,9 +121,6 @@ int main (int argc, char **argv)
 					}
 					Ctrl->E.col = atoi (&argv[i][3]);
                                         break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'I':
 					Ctrl->I.active = TRUE;
 					if (argv[i][2] == 'p') special = TRUE;
@@ -138,7 +134,7 @@ int main (int argc, char **argv)
 					break;
 				case 'T':
 					Ctrl->T.active = TRUE;
-					j = sscanf (&argv[i][2], "%lf/%d", &Ctrl->T.inc, &Ctrl->T.col);
+					j = sscanf (&argv[i][2], "%lf/%ld", &Ctrl->T.inc, &Ctrl->T.col);
 					if (j == 1) Ctrl->T.col = 0;
 					break;
 				case 'S':
@@ -163,8 +159,8 @@ int main (int argc, char **argv)
 
 	if (error || GMT_give_synopsis_and_exit) {	/* Because it's ok to give no arguments */
 		fprintf (stderr, "minmax %s - Find extreme values in ASCII tables\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: minmax [files] [-C] [-E<L|l|H|h><col>] [%s] [-I[p]<dx>[/<dy>[/<dz>..]] [%s]\n", GMT_H_OPT, GMT_M_OPT);
-		fprintf (stderr, "\t[-S[x][y]] [-T<dz>[/<col>]] [%s] [%s] [%s]\n", GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
+		fprintf (stderr, "usage: minmax [files] [-C] [-E<L|l|H|h><col>] [%s] [-I[p]<dx>[/<dy>[/<dz>..]]\n", GMT_H_OPT);
+		fprintf (stderr, "\t[-S[x][y]] [-T<dz>[/<col>]] [%s] [%s] [%s] [%s]\n", GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT, GMT_m_OPT);
              
               	if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
  
@@ -178,7 +174,6 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   given determines how many columns are rounded off to the nearest multiple.\n");
 		fprintf (stderr, "\t   If only one increment is given we also use it for the second column (for backwards compatibility).\n");
 		fprintf (stderr, "\t   To override this behaviour, use -Ip<dx>.\n");
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-S adds extra space for error bars. Useful together with -I.\n");
 		fprintf (stderr, "\t   -Sx leaves space for horizontal error bar using value in third (2) column.\n");
 		fprintf (stderr, "\t   -Sy leaves space for vertical error bar using value in third (2) column.\n");
@@ -190,6 +185,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -240,7 +236,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (n_files > 0)
@@ -296,7 +292,7 @@ int main (int argc, char **argv)
 			}
 		}
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		n = ncol = 0;
 		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
@@ -312,7 +308,7 @@ int main (int argc, char **argv)
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 				exit (EXIT_FAILURE);
 			}
 
@@ -324,11 +320,11 @@ int main (int argc, char **argv)
 					exit (EXIT_FAILURE);
 				}
 				if (Ctrl->E.active && Ctrl->E.col >= ncol) {
-	  				fprintf (stderr, "%s: GMT SYNTAX ERROR -E option: Chosen column exceeds column range (0-%i)\n", GMT_program, ncol-1);
+	  				fprintf (stderr, "%s: GMT SYNTAX ERROR -E option: Chosen column exceeds column range (0-%ld)\n", GMT_program, ncol-1);
 					exit (EXIT_FAILURE);
 				}
 				if (Ctrl->T.active && Ctrl->T.col >= ncol) {
-					fprintf (stderr, "%s: GMT SYNTAX ERROR -T option: Chosen column exceeds column range (0-%i)\n", GMT_program, ncol-1);
+					fprintf (stderr, "%s: GMT SYNTAX ERROR -T option: Chosen column exceeds column range (0-%ld)\n", GMT_program, ncol-1);
 					exit (EXIT_FAILURE);
 				}
 				if (Ctrl->T.active) ncol = Ctrl->T.col+1;
@@ -351,7 +347,7 @@ int main (int argc, char **argv)
 
 			/* Decode all fields and update minmax arrays */
 
-			while (dateline && in[0] > 180.0) in[0] -= 360.0;
+			while (dateline && in[GMT_X] > 180.0) in[GMT_X] -= 360.0;
 
 			if (Ctrl->E.active && !GMT_is_dnan (in[Ctrl->E.col])) {
 				value = (work_on_abs_value) ? fabs (in[Ctrl->E.col]) : in[Ctrl->E.col];
@@ -503,11 +499,11 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-int strip_blanks_and_output (double x, int col)
+GMT_LONG strip_blanks_and_output (double x, GMT_LONG col)
 {
 	/* Alternative to GMT_ascii_output_one that strips off leading blanks first */
 
-	int k;
+	GMT_LONG k;
 	char text[GMT_TEXT_LEN];
 
 	GMT_ascii_format_one (text, x, GMT_io.out_col_type[col]);
