@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *    $Id: gmtdigitize.c,v 1.22 2009/01/09 04:02:35 guru Exp $
+ *    $Id: gmtdigitize.c,v 1.28 2009/09/13 13:20:10 remko Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -69,10 +69,10 @@ int main (int argc, char **argv)
 	char *control[4] = {"first", "second", "third", "fourth"};
 	char *corner[4] = {"lower left", "lower right", "upper right", "upper left"};
 	char *name = CNULL, *device = CNULL, *xname[2] = {"longitude", "x-coordinate"};
-	char *yname[2] = {"latitude ", "y-coordinate"};
+	char *yname[2] = {"latitude ", "y-coordinate"}, *not_used = NULL;
 	
-	int i, j, n = 0, n_read = 0, unit = 0, n_expected_fields, n_segments = 0, digunit;
-	int val_pos = 2, key_pos = 2, m_button, LPI = DIG_LPI, type, button;
+	GMT_LONG i, j, n = 0, n_read = 0, unit = 0, n_expected_fields, n_segments = 0, digunit;
+	GMT_LONG val_pos = 2, key_pos = 2, m_button, LPI = DIG_LPI, type, button, i_unused = 0;
 	
 	gid_t gid = 0;
 	uid_t uid = 0;
@@ -88,8 +88,8 @@ int main (int argc, char **argv)
 
 	struct GMTDIGITIZE_CTRL	C;
 	
-	int get_digitize_raw (int digunit, double *xdig, double *ydig, struct GMTDIGITIZE_CTRL *C);
-	int get_digitize_xy (int digunit, double *xmap, double *ymap, struct GMTDIGITIZE_CTRL *C);
+	GMT_LONG get_digitize_raw (GMT_LONG digunit, double *xdig, double *ydig, struct GMTDIGITIZE_CTRL *C);
+	GMT_LONG get_digitize_xy (GMT_LONG digunit, double *xmap, double *ymap, struct GMTDIGITIZE_CTRL *C);
 	FILE *next_file (char *name, int n_segments, char *this_file);
 
 	argc = GMT_begin (argc, argv);
@@ -102,10 +102,12 @@ int main (int argc, char **argv)
 			
 				case 'H':
 				case 'J':
+				case 'M':
 				case 'R':
 				case 'V':
 				case 'b':
 				case 'f':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
@@ -132,9 +134,6 @@ int main (int argc, char **argv)
 					if (argv[i][2] == 'k') output_key = TRUE;
 					if (argv[i][2] == 'v') output_val = TRUE;
 					break;
-				case 'M':               /* Multiple line segments */
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':               /* Multiple line segments */
 					name = &argv[i][2];
 					break;
@@ -152,8 +151,8 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "gmtdigitize %s - Digitizing and inverse map transformation of map x/y coordinates\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: gmtdigitize %s %s\n", GMT_J_OPT, GMT_Rgeo_OPT);
-		fprintf (stderr, "\t[-A] [-C<device>] [-D<limit>] [-F] [-H] [-L<lpi>] [%s] [-N<namestem>] [-S]\n", GMT_Mo_OPT);
-		fprintf (stderr, "\t[-V] [-Zk|v] [%s] [%s]\n\n", GMT_bo_OPT, GMT_f_OPT);
+		fprintf (stderr, "\t[-A] [-C<device>] [-D<limit>] [-F] [-H] [-L<lpi>] [-N<namestem>] [-S]\n");
+		fprintf (stderr, "\t[-V] [-Zk|v] [%s] [%s] [%s]\n\n", GMT_bo_OPT, GMT_f_OPT, GMT_mo_OPT);
 		
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 		
@@ -169,7 +168,6 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   the corners of the map if they are known to the program]\n");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-L sets the lines-pr-inch resolution of the digitizer [%d]\n", DIG_LPI);
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N sets name for output file(s).  If name contains a C-format\n");
 		fprintf (stderr, "\t   for integer (e.g., line_%%d.d) then each segment will be written\n");
 		fprintf (stderr, "\t   to separate files [Default is stdout]\n");
@@ -180,6 +178,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-Zk means append button key id as a final column\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -265,14 +264,14 @@ int main (int argc, char **argv)
 			fprintf (stderr, "\n");
 			for (i = 0; i < 4; i++) {
 				fprintf (stderr, "Please Enter %s of %s point: ", xname[type], control[i]);
-				GMT_fgets (line, BUFSIZ, GMT_stdin);
+				not_used = GMT_fgets (line, BUFSIZ, GMT_stdin);
 				GMT_chop (line);
 				if (!(GMT_scanf (line, GMT_io.in_col_type[0], &LON[i]))) {
 					fprintf (stderr, "%s: Conversion error for %sx [%s]\n", GMT_program, xname[type], line);
 					exit (EXIT_FAILURE);
 				}
 				fprintf (stderr, "Please Enter %s of %s point: ", yname[type], control[i]);
-				GMT_fgets (line, BUFSIZ, GMT_stdin);
+				not_used = GMT_fgets (line, BUFSIZ, GMT_stdin);
 				GMT_chop (line);
 				if (!(GMT_scanf (line, GMT_io.in_col_type[1], &LAT[i]))) {
 					fprintf (stderr, "%s: Conversion error for %s [%s]\n", GMT_program, yname[type], line);
@@ -394,7 +393,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "(Do not start with # - it will be prepended automatically.\n\n");
 		do {
 			fprintf (stderr, "==> Please enter comment records, end with blank line: ");
-			GMT_fgets (line, BUFSIZ, stdin);
+			not_used = GMT_fgets (line, BUFSIZ, stdin);
 			GMT_chop (line);
 			if (line[0] != '\0' && !GMT_io.binary[1]) fprintf (fp, "# %s\n", line);
 		} while (line[0] != '\0');
@@ -420,25 +419,25 @@ int main (int argc, char **argv)
 				if (multi_files) {
 					if (fp) {
 						GMT_fclose (fp);
-						chown (this_file, uid, gid);
+						i_unused = chown (this_file, uid, gid);
 					}
 					fp = next_file (name, n_segments, this_file);
 				}
 
 				if (output_val) {
 					fprintf (stderr, "Enter z-value for next segment: ");
-					GMT_fgets (line, BUFSIZ, GMT_stdin);
+					not_used = GMT_fgets (line, BUFSIZ, GMT_stdin);
 					GMT_chop (line);
 					z_val = atof (line);
 				}
 				if (button == MULTISEG_BUTTON1) {	/* Just write blank multisegment header */
-					sprintf (GMT_io.segment_header, "%c %d\n", GMT_io.EOF_flag[GMT_OUT], n_segments);
+					sprintf (GMT_io.segment_header, "%c %ld\n", GMT_io.EOF_flag[GMT_OUT], n_segments);
 				}
 				else {	/* Ask for what to write out */
 					fprintf (stderr, "Enter segment header: ");
-					GMT_fgets (line, BUFSIZ, GMT_stdin);
+					not_used = GMT_fgets (line, BUFSIZ, GMT_stdin);
 					GMT_chop (line);
-					sprintf (GMT_io.segment_header, "%c %d %s\n", GMT_io.EOF_flag[GMT_OUT], n_segments, line);
+					sprintf (GMT_io.segment_header, "%c %ld %s\n", GMT_io.EOF_flag[GMT_OUT], n_segments, line);
 				}
 				GMT_write_segmentheader (fp, n_expected_fields);
 				if (gmtdefs.verbose) fprintf (stderr, "%s", GMT_io.segment_header);
@@ -484,7 +483,7 @@ int main (int argc, char **argv)
 	}
 	if (multi_files && fp) {
 		GMT_fclose (fp);
-		chown (this_file, uid, gid);
+		i_unused = chown (this_file, uid, gid);
 	}
 	
 	if (gmtdefs.verbose && n_read > 0) {
@@ -492,8 +491,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, format, GMT_program, x_in_min, x_in_max, y_in_min, y_in_max);
 		sprintf (format, "%%s: Output extreme values:  Xmin: %s Xmax: %s Ymin: %s Ymax %s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, format, GMT_program, x_out_min, x_out_max, y_out_min, y_out_max);
-		fprintf (stderr, "%s: Digitized %d points\n", GMT_program, n);
-		if (suppress && n != n_read) fprintf (stderr, "%s: %d fell outside region\n", GMT_program, n_read - n);
+		fprintf (stderr, "%s: Digitized %ld points\n", GMT_program, n);
+		if (suppress && n != n_read) fprintf (stderr, "%s: %ld fell outside region\n", GMT_program, n_read - n);
 	}
 
 	GMT_end (argc, argv);
@@ -501,8 +500,8 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-int get_digitize_raw (int digunit, double *xdig, double *ydig, struct GMTDIGITIZE_CTRL *C) {
-	int i, n, ix, iy;
+GMT_LONG get_digitize_raw (GMT_LONG digunit, double *xdig, double *ydig, struct GMTDIGITIZE_CTRL *C) {
+	GMT_LONG i, n, ix, iy;
 	char buffer[256], button;
 	
 	n = read (digunit, buffer, (size_t)255);
@@ -510,19 +509,19 @@ int get_digitize_raw (int digunit, double *xdig, double *ydig, struct GMTDIGITIZ
 	n--;
 	buffer[n] = 0;
 #ifdef DEBUG
-	fprintf (stderr, "Got %d bytes [%s]\n", n, buffer);
+	fprintf (stderr, "Got %ld bytes [%s]\n", n, buffer);
 #endif
 	for (i = 0; i < n; i++) if (buffer[i] == ',') buffer[i] = ' ';
-	sscanf (buffer, "%d %d %c", &ix, &iy, &button);
+	sscanf (buffer, "%ld %ld %c", &ix, &iy, &button);
 	
 	*xdig = (double)(ix) * C->INV_LPI;	/* Convert from lines per inch to inches */
 	*ydig = (double)(iy) * C->INV_LPI;
 	
-	return ((int)(button - '0'));
+	return ((GMT_LONG)(button - '0'));
 }
 
-int get_digitize_xy (int digunit, double *xmap, double *ymap, struct GMTDIGITIZE_CTRL *C) {
-	int button;
+GMT_LONG get_digitize_xy (GMT_LONG digunit, double *xmap, double *ymap, struct GMTDIGITIZE_CTRL *C) {
+	GMT_LONG button;
 	double x_raw, y_raw;
 	
 	button = get_digitize_raw (digunit, &x_raw, &y_raw, C);
