@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: nc2xy.c,v 1.21 2008/04/26 19:13:40 guru Exp $
+ *	$Id: nc2xy.c,v 1.25 2009/09/09 23:27:04 guru Exp $
  *
- *	Copyright (c) 2006-2008 by R. Scharroo
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 2006-2009 by R. Scharroo
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -30,12 +30,13 @@
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, suppress = FALSE, reverse = FALSE, skip;
+	BOOLEAN error = FALSE, suppress = FALSE, reverse = FALSE, skip, plus_const_col = FALSE;
 	int ncid, varid[NVAR], dimid = 0, ndims;
 	GMT_LONG i, k, n_files = 0, n_out;
 	int n_dimid = -1;
 	size_t n_total = 0, j, n_suppressed = 0;
-	double w, e, s, n, out[NVAR], add_offset[NVAR], scale_factor[NVAR], missing_value[NVAR];
+	double w, e, s, n, out[NVAR+1], add_offset[NVAR], scale_factor[NVAR], missing_value[NVAR];
+	double cte_col = 0.0;
 	char varnm[NVAR][GMT_TEXT_LEN], long_name[GMT_LONG_TEXT], units[GMT_LONG_TEXT];
 	struct GMT_TIME_SYSTEM time_system;
 
@@ -53,6 +54,7 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'f':
+				case 'b':
 				case 'V':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &w, &e, &s, &n);
@@ -60,6 +62,10 @@ int main (int argc, char **argv)
 
 				/* Supplemental options */
 
+				case 'C':
+					cte_col = atof(&argv[i][2]);
+					plus_const_col = TRUE;
+					break;
 				case 'F':
 					sscanf (&argv[i][2], "%[^/]/%[^/]/%[^/]/%[^/]/%[^/]/%[^/]/%[^/]/%[^/]/%[^/]/%[^/]", varnm[0], varnm[1], varnm[2], varnm[3], varnm[4], varnm[5], varnm[6], varnm[7], varnm[8], varnm[9]);
 					break;
@@ -79,7 +85,7 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "nc2xy %s - Converting netCDF column file(s) to ASCII xy data\n\n", GMT_VERSION);
-		fprintf( stderr, "usage: nc2xy <files> [-F<var1>/<var2>/...] [-S[r]] [-V] [%s] > xyfile\n", GMT_f_OPT);
+		fprintf( stderr, "usage: nc2xy <files> [-F<var1>/<var2>/...] [-S[r]] [-V] [%s] [-bo] > xyfile\n", GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -90,6 +96,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Append r to reverse the suppression (only output records with NaNs)\n");
 		GMT_explain_option ('V');
 		GMT_explain_option ('f');
+		GMT_explain_option ('o');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -104,7 +111,6 @@ int main (int argc, char **argv)
 #ifdef SET_IO_MODE
 	GMT_setmode (GMT_OUT);
 #endif
-	GMT_io.binary[GMT_OUT] = FALSE;
 
 	for (k = 1; k < argc; k++) {
 		if (argv[k][0] == '-') continue;	/* Skip the options */
@@ -192,8 +198,9 @@ int main (int argc, char **argv)
 					continue;
 				}
 			}
+			if (plus_const_col) out[i] = cte_col;	/* Extra column with a const value on output */
 			if (skip == reverse)
-				GMT_output (GMT_stdout, n_out, out);
+				GMT_output (GMT_stdout, n_out+plus_const_col, out);
 			else
 				n_suppressed++;
 		}
