@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: project.c,v 1.60 2009/01/09 04:02:34 guru Exp $
+ *	$Id: project.c,v 1.69 2009/09/09 23:27:03 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -98,35 +98,34 @@ struct PROJECT_DATA {
 int main (int argc, char **argv)
 {
 	GMT_LONG	i, n_used, n_total_read, n_total_used = 0, n_alloc = GMT_CHUNK;
-	
-	int	j, k, n_outputs;
-	int	n_files = 0, fno, n_args, n_fields, n_expected_fields;
-	int	n_z = 0, n_items, output_choice[PROJECT_N_FARGS];
+	GMT_LONG	j, k, n_outputs;
+	GMT_LONG	n_files = 0, fno, n_args, n_fields, n_expected_fields;
+	GMT_LONG	n_z = 0, n_items, output_choice[PROJECT_N_FARGS];
 
 	double	xx, yy, cos_theta, sin_theta, sin_lat_to_pole = 1.0;
-	double	theta, d_along, *in, *out = (double *)NULL;
+	double	theta = 0.0, d_along, *in, *out = (double *)NULL;
 
 	double	a[3], b[3], x[3], xt[3], pole[3], center[3], e[9];
 
 	BOOLEAN	dateline = FALSE, error = FALSE, find_new_point = FALSE, first = TRUE, z_first = TRUE, pure_ascii, skip;
-	BOOLEAN greenwich = FALSE, rads = TRUE, nofile = TRUE, done = FALSE, want_z_output = FALSE;
+	BOOLEAN greenwich = FALSE, nofile = TRUE, done = FALSE, want_z_output = FALSE;
 
 	FILE *fp = NULL;
 
 	char	modifier, record_str[BUFSIZ], heading[PROJECT_N_FARGS][GMT_TEXT_LEN];
-	char	txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
+	char	txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], *not_used = NULL;
 
 	struct PROJECT_DATA *p_data;
 	struct PROJECT_CTRL *Ctrl;
 
 	int	compare_distances (const void *point_1, const void *point_2);
-	double	oblique_setup (double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads);
-	void	oblique_transform (double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c, int rads);
-	void	make_euler_matrix (double *p, double *e, double *theta, int rads);
+	double	oblique_setup (double plat, double plon, double *p, double clat, double clon, double *c, GMT_LONG c_given);
+	void	oblique_transform (double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c);
+	void	make_euler_matrix (double *p, double *e, double theta);
 	void	matrix_3v (double *a, double *x, double *b);
 	void	matrix_2v (double *a, double *x, double *b);
-	void	sphere_project_setup (double alat, double alon, double *a, double blat, double blon, double *b, double *azim, double *p, double *c, int two_pts, int rads);
-	void	flat_project_setup (double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, int two_pts, BOOLEAN pole_set);
+	void	sphere_project_setup (double alat, double alon, double *a, double blat, double blon, double *b, double azim, double *p, double *c, GMT_LONG two_pts);
+	void	flat_project_setup (double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, GMT_LONG two_pts, BOOLEAN pole_set);
 	void	copy_text_from_col3 (char *line, char *z_cols);
 	void	*New_Project_Ctrl (), Free_Project_Ctrl (struct PROJECT_CTRL *C);
 
@@ -134,7 +133,7 @@ int main (int argc, char **argv)
 
 	Ctrl = (struct PROJECT_CTRL *) New_Project_Ctrl ();		/* Allocate and initialize defaults in a new control structure */
 
-	memset ((void *)output_choice, 0, PROJECT_N_FARGS * sizeof (int));
+	memset ((void *)output_choice, 0, PROJECT_N_FARGS * sizeof (GMT_LONG));
 
 	/* New in GMT 4: Must process -N before the rest to ensure col_types are set properly (there is no -H here)  */
 	for (i = 1; i < argc; i++) if (!strcmp (argv[i], "-N")) Ctrl->N.active = TRUE;
@@ -151,10 +150,12 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'H':
+				case 'M':
 				case 'V':
 				case ':':
 				case 'b':
 				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
@@ -222,9 +223,6 @@ int main (int argc, char **argv)
 						}
 					}
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N': /* Handled above but still in argv */
 					Ctrl->N.active = TRUE;
 					break;
@@ -267,8 +265,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "project %s - project data onto line or great circle, generate track, or translate coordinates\n\n", GMT_VERSION);
 		fprintf(stderr,"usage:	project [files] -C<ox>/<oy> [-A<azimuth>] [-D<d_or_g>] [-E<bx>/<by>]\n");
 		fprintf(stderr,"\t[-F<flags>] [-G<dist>] [%s] [-L[w][<l_min>/<l_max>]]\n", GMT_H_OPT);
-		fprintf(stderr,"\t[%s] [-N] [-Q] [-S] [-T<px>/<py>] [-V] [-W<w_min>/<w_max>]\n", GMT_M_OPT);
-		fprintf(stderr,"\t[%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_b_OPT, GMT_f_OPT);
+		fprintf(stderr,"\t[-N] [-Q] [-S] [-T<px>/<py>] [-V] [-W<w_min>/<w_max>]\n");
+		fprintf(stderr,"\t[%s] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_b_OPT, GMT_f_OPT, GMT_m_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -306,7 +304,6 @@ int main (int argc, char **argv)
 		fprintf(stderr,"\t  -Lw will use only those points Within the span from C to E (Must have set -E).\n");
 		fprintf(stderr,"\t  -L<l_min>/<l_max> will only use points whose p is [l_min <= p <= l_max].\n");
 		fprintf(stderr,"\t  Default uses all points.  Note p = 0 at C and increases toward E in azim direction.\n");
-		GMT_explain_option ('M');
 		fprintf(stderr,"\t-N means Flat_earth; a cartesian projection is made.  Default is spherical.\n");
 		fprintf(stderr,"\t-Q means convert to Map units, so x,y,r,s are degrees,\n");
 		fprintf(stderr,"\t  while p,q,dist,l_min,l_max,w_min,w_max are km.\n");
@@ -324,6 +321,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -395,7 +393,7 @@ int main (int argc, char **argv)
 	}
 
 	if (n_outputs > PROJECT_N_FARGS) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: Too many output columns selected (%d)\n", GMT_program, n_outputs);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option: Too many output columns selected (%ld)\n", GMT_program, n_outputs);
 		error++;
 	}
 	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
@@ -422,7 +420,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -433,9 +431,9 @@ int main (int argc, char **argv)
 
 	if (n_outputs == 0 && !(Ctrl->G.active) ) {	/* Generate default -F setting (all) */
 		n_outputs = PROJECT_N_FARGS;
-		for (i = 0; i < 2; i++) output_choice[i] = i;
+		for (i = 0; i < 2; i++) output_choice[i] = (int)i;
 		output_choice[2] = -1;
-		for (i = 3; i < n_outputs; i++) output_choice[i] = i-1;
+		for (i = 3; i < n_outputs; i++) output_choice[i] = (int)i - 1;
 		find_new_point = TRUE;
 	}
 
@@ -446,8 +444,9 @@ int main (int argc, char **argv)
 	/* Set up rotation matrix e for flat earth, or pole and center for spherical; get Ctrl->L.min, Ctrl->L.max if stay_within  */
 
 	if (Ctrl->N.active) {
-		flat_project_setup(Ctrl->C.y, Ctrl->C.x, Ctrl->E.y, Ctrl->E.x, Ctrl->T.y, Ctrl->T.x, &Ctrl->A.azimuth, e, Ctrl->E.active, Ctrl->T.active);
-		/* Azimuth is now changed to cartesian theta in radians */
+		theta = Ctrl->A.azimuth;
+		flat_project_setup(Ctrl->C.y, Ctrl->C.x, Ctrl->E.y, Ctrl->E.x, Ctrl->T.y, Ctrl->T.x, &theta, e, Ctrl->E.active, Ctrl->T.active);
+		/* Azimuth (theta) is now cartesian in degrees */
 		if (Ctrl->L.constrain) {
 			Ctrl->L.min = 0.0;
 			xx = Ctrl->E.x - Ctrl->C.x;
@@ -458,16 +457,15 @@ int main (int argc, char **argv)
 	}
 	else {
 		if (Ctrl->T.active) {
-			sin_lat_to_pole = oblique_setup(Ctrl->T.y, Ctrl->T.x, pole, Ctrl->C.y, Ctrl->C.x, center, Ctrl->T.active, rads);
+			sin_lat_to_pole = oblique_setup(Ctrl->T.y, Ctrl->T.x, pole, Ctrl->C.y, Ctrl->C.x, center, Ctrl->T.active);
 		}
 		else {
-			sphere_project_setup(Ctrl->C.y, Ctrl->C.x, a, Ctrl->E.y, Ctrl->E.x, b, &Ctrl->A.azimuth, pole, center, Ctrl->E.active, rads);
+			sphere_project_setup(Ctrl->C.y, Ctrl->C.x, a, Ctrl->E.y, Ctrl->E.x, b, Ctrl->A.azimuth, pole, center, Ctrl->E.active);
 		}
-		/* Azimuth is now changed to radians  */
 		if (Ctrl->L.constrain) {
 			Ctrl->L.min = 0.0;
 			Ctrl->L.max = GMT_dot3v(a,b);
-			Ctrl->L.max = d_acos(Ctrl->L.max) * R2D;
+			Ctrl->L.max = d_acosd(Ctrl->L.max);
 			if (Ctrl->Q.active) Ctrl->L.max *= project_info.KM_PR_DEG;
 		}
 	}
@@ -513,7 +511,7 @@ int main (int argc, char **argv)
 		/* We need to find r,s  */
 
 		if (Ctrl->N.active) {
-			sincos (M_PI_2 + Ctrl->A.azimuth, &sin_theta, &cos_theta);
+			sincosd (90.0 + theta, &sin_theta, &cos_theta);
 			for (i = 0; i < n_used; i++) {
 				p_data[i].a[4] = Ctrl->C.x + p_data[i].a[2] * cos_theta;
 				p_data[i].a[5] = Ctrl->C.y + p_data[i].a[2] * sin_theta;
@@ -522,16 +520,11 @@ int main (int argc, char **argv)
 			}
 		}
 		else {
-			xx = Ctrl->C.x;
-			yy = Ctrl->C.y;
-			GMT_geo_to_cart(&yy, &xx, x, rads);
+			GMT_geo_to_cart(Ctrl->C.y, Ctrl->C.x, x, TRUE);
 			for (i = 0; i < n_used; i++) {
-				theta = p_data[i].a[2] / sin_lat_to_pole;
-				make_euler_matrix(pole, e, &theta, rads);
+				make_euler_matrix(pole, e, p_data[i].a[2] / sin_lat_to_pole);
 				matrix_3v(e,x,xt);
-				GMT_cart_to_geo(&yy, &xx, xt, rads);
-				p_data[i].a[4] = xx;
-				p_data[i].a[5] = yy;
+				GMT_cart_to_geo(&(p_data[i].a[5]), &(p_data[i].a[4]), xt, TRUE);
 				while (greenwich && p_data[i].a[4] < 0.0) p_data[i].a[4] += 360.0;
 				while (dateline && p_data[i].a[4] > 180.0) p_data[i].a[4] -= 360.0;
 			}
@@ -587,14 +580,14 @@ int main (int argc, char **argv)
 			if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 
 			if (GMT_io.io_header[GMT_IN]) {
-				GMT_fgets (record_str, BUFSIZ, fp);
+				not_used = GMT_fgets (record_str, BUFSIZ, fp);
 				sscanf (record_str, "%s %s %s", heading[0], heading[1], heading[6]);
 				if (! (heading[6]) ) strcpy (heading[6],"Z");
 				strcpy (heading[2],"p");
 				strcpy (heading[3],"q");
 				strcpy (heading[4],"r");
 				strcpy (heading[5],"s");
-				for (i = 1; i < GMT_io.n_header_recs; i++) GMT_fgets (record_str, BUFSIZ, fp);
+				for (i = 1; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (record_str, BUFSIZ, fp);
 			}
 
 			n_fields = GMT_input (fp, &n_expected_fields, &in);
@@ -617,12 +610,12 @@ int main (int argc, char **argv)
 
 				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_total_read);
+						fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_total_read);
 						exit (EXIT_FAILURE);
 					}
 
-					xx = in[0];
-					yy = in[1];
+					xx = in[GMT_X];
+					yy = in[GMT_Y];
 
 					n_total_read ++;
 
@@ -632,7 +625,7 @@ int main (int argc, char **argv)
 						matrix_2v (e,x,xt);
 					}
 					else {
-						oblique_transform(yy, xx, &xt[1], &xt[0], pole, center, rads);
+						oblique_transform(yy, xx, &xt[1], &xt[0], pole, center);
 					}
 
 					skip = ((Ctrl->L.active && (xt[0] < Ctrl->L.min || xt[0] > Ctrl->L.max)) || (Ctrl->W.active && (xt[1] < Ctrl->W.min || xt[1] > Ctrl->W.max)));
@@ -655,7 +648,7 @@ int main (int argc, char **argv)
 						}
 						else {
 							p_data[n_used].z = (double *) GMT_memory (VNULL, (size_t)n_z, sizeof (double), GMT_program);
-							memcpy ((void *)p_data[n_used].z, (void *)&in[2], (size_t)(n_z * sizeof(double)));
+							memcpy ((void *)p_data[n_used].z, (void *)&in[GMT_Z], (size_t)(n_z * sizeof(double)));
 						}
 					}
 					n_used++;
@@ -674,7 +667,7 @@ int main (int argc, char **argv)
 				if (find_new_point) {	/* We need to find r,s  */
 
 					if (Ctrl->N.active) {
-						sincos (Ctrl->A.azimuth, &sin_theta, &cos_theta);
+						sincosd (theta, &sin_theta, &cos_theta);
 						for (i = 0; i < n_used; i++) {
 							p_data[i].a[4] = Ctrl->C.x + p_data[i].a[2] * cos_theta;
 							p_data[i].a[5] = Ctrl->C.y + p_data[i].a[2] * sin_theta;
@@ -683,16 +676,11 @@ int main (int argc, char **argv)
 						}
 					}
 					else {
-						xx = Ctrl->C.x;
-						yy = Ctrl->C.y;
-						GMT_geo_to_cart(&yy, &xx, x, rads);
+						GMT_geo_to_cart(Ctrl->C.y, Ctrl->C.x, x, TRUE);
 						for (i = 0; i < n_used; i++) {
-							theta = p_data[i].a[2];
-							make_euler_matrix(pole, e, &theta, rads);
+							make_euler_matrix(pole, e, p_data[i].a[2]);
 							matrix_3v(e,x,xt);
-							GMT_cart_to_geo(&yy, &xx, xt, rads);
-							p_data[i].a[4] = xx;
-							p_data[i].a[5] = yy;
+							GMT_cart_to_geo(&(p_data[i].a[5]), &(p_data[i].a[4]), xt, TRUE);
 							while (greenwich && p_data[i].a[4] < 0.0) p_data[i].a[4] += 360.0;
 							while (dateline && p_data[i].a[4] > 180.0) p_data[i].a[4] -= 360.0;
 						}
@@ -793,7 +781,7 @@ int compare_distances (const void *point_1, const void *point_2)
 		return (0);
 }
 
-double oblique_setup (double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads)
+double oblique_setup (double plat, double plon, double *p, double clat, double clon, double *c, GMT_LONG c_given)
 {
 	/* routine sets up a unit 3-vector p, the pole of an 
 	   oblique projection, given plat, plon, the position 
@@ -806,8 +794,7 @@ double oblique_setup (double plat, double plon, double *p, double clat, double c
 	   N pole.  In either case, a unit 3-vector c is created
 	   which is the directed normal to the plane of the central
 	   meridian, pointing in the positive normal (east) sense.
-	   rads = TRUE if we need to convert plat, plon, clat, clon
-	   from degrees to radians.  */
+	   Latitudes and longitudes are in degrees. */
 
 	double	s[3];  /* s points to the south pole  */
 	double cp, sin_lat_to_pole;
@@ -815,10 +802,10 @@ double oblique_setup (double plat, double plon, double *p, double clat, double c
 	s[0] = s[1] = 0.0;
 	s[2] = -1.0;
 
-	GMT_geo_to_cart(&plat, &plon, p, rads);
+	GMT_geo_to_cart(plat, plon, p, TRUE);
 
 	if (c_given) {	/* s points to user's clat, clon  */
-		GMT_geo_to_cart(&clat, &clon, s, rads);
+		GMT_geo_to_cart(clat, clon, s, TRUE);
 	}
 	GMT_cross3v(p, s, c);
 	GMT_normalize3v(c);
@@ -827,52 +814,42 @@ double oblique_setup (double plat, double plon, double *p, double clat, double c
 	return (sin_lat_to_pole);
 }
 
-void oblique_transform (double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c, int rads)
+void oblique_transform (double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c)
 {
 	/* routine takes the point x at conventional (xlat, xlon) and
 	   computes the transformed coordinates (x_t_lat, x_t_lon) in
 	   an oblique reference frame specified by the unit 3-vectors
 	   p (the pole) and c (the directed normal to the oblique
 	   central meridian).  p and c have been computed earlier by
-	   the routine oblique_setup().  rads = TRUE if lats and lons
-	   are in degrees.  */
+	   the routine oblique_setup().
+	   Latitudes and longitudes are in degrees. */
 
 	double	x[3], p_cross_x[3], temp1, temp2;
 
-	GMT_geo_to_cart(&xlat, &xlon, x, rads);
+	GMT_geo_to_cart(xlat, xlon, x, TRUE);
 
 	temp1 = GMT_dot3v(x,p);
-	*x_t_lat = d_asin(temp1);
+	*x_t_lat = d_asind(temp1);
 
 	GMT_cross3v(p,x,p_cross_x);
 	GMT_normalize3v(p_cross_x);
 
 	temp1 = GMT_dot3v(p_cross_x, c);
 	temp2 = GMT_dot3v(x, c);
-	*x_t_lon = copysign( d_acos(temp1), temp2);
-
-	if (rads) {
-		*x_t_lat *= R2D;
-		*x_t_lon *= R2D;
-	}
+	*x_t_lon = copysign(d_acosd(temp1), temp2);
 }
 
-void make_euler_matrix (double *p, double *e, double *theta, int rads)
+void make_euler_matrix (double *p, double *e, double theta)
 {
 	/* Routine to fill an euler matrix e with the elements
 	   needed to rotate a 3-vector about the pole p through
-	   an angle theta.  p is a unit 3-vector.  If rads = TRUE,
-	   we have to convert theta from degrees into radians before
-	   we use it.  */
+	   an angle theta (in degrees).  p is a unit 3-vector.
+	   Latitudes and longitudes are in degrees. */
 
 	double	cos_theta, sin_theta, one_minus_cos_theta;
 	double	pxsin, pysin, pzsin, temp;
 
-	if (rads) {
-		*theta *= D2R;
-	}
-	cos_theta = cos(*theta);
-	sin_theta = sin(*theta);
+	sincosd (theta, &sin_theta, &cos_theta);
 	one_minus_cos_theta = 1.0 - cos_theta;
 
 	pxsin = p[0] * sin_theta;
@@ -918,7 +895,7 @@ void matrix_2v (double *a, double *x, double *b)
 	b[1] = x[0]*a[2] + x[1]*a[3];
 }
 
-void sphere_project_setup (double alat, double alon, double *a, double blat, double blon, double *b, double *azim, double *p, double *c, int two_pts, int rads)
+void sphere_project_setup (double alat, double alon, double *a, double blat, double blon, double *b, double azim, double *p, double *c, GMT_LONG two_pts)
 {
 	/* routine to initialize a pole vector, p, and a central meridian 
 	   normal vector, c, for use in projecting points onto a great circle.
@@ -940,28 +917,27 @@ void sphere_project_setup (double alat, double alon, double *a, double blat, dou
 	   create p_temp = normalized(a X n), where n is the north pole.
 	   p_temp is then rotated about a through the angle azim to give p.
 	   After p is found, then c = normalized(p X a).
+
+	   Latitudes and longitudes are in degrees.
 	*/
 
 	double	e[9];	/* Euler rotation matrix, if needed  */
-	double neg_azim;
 
 	/* First find p vector  */
 
 	if (two_pts) {
-		GMT_geo_to_cart(&alat, &alon, a, rads);
-		GMT_geo_to_cart(&blat, &blon, b, rads);
+		GMT_geo_to_cart(alat, alon, a, TRUE);
+		GMT_geo_to_cart(blat, blon, b, TRUE);
 		GMT_cross3v(a, b, p);
 		GMT_normalize3v(p);
 	}
 	else {
-		GMT_geo_to_cart(&alat, &alon, a, rads);
+		GMT_geo_to_cart(alat, alon, a, TRUE);
 		b[0] = b[1] = 0.0;	/* set b to north pole  */
 		b[2] = 1.0;
 		GMT_cross3v(a, b, c);	/* use c for p_temp  */
 		GMT_normalize3v(c);
-		neg_azim = -(*azim);
-		make_euler_matrix(a, e, &neg_azim, rads);
-		if (rads) *azim *= D2R;
+		make_euler_matrix(a, e, -azim);
 		matrix_3v(e, c, p);	/* c (p_temp) rotates to p  */
 	}
 
@@ -971,32 +947,28 @@ void sphere_project_setup (double alat, double alon, double *a, double blat, dou
 	GMT_normalize3v(c);
 }
 
-void flat_project_setup (double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, int two_pts, BOOLEAN pole_set)
+void flat_project_setup (double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, GMT_LONG two_pts, BOOLEAN pole_set)
 {
 	/* Sets up stuff for rotation of cartesian 2-vectors, analogous
-	   to the spherical three vector stuff above.  Also change azim
-	   to negative azim.  */
+	   to the spherical three vector stuff above.
+	   Output is the negative cartesian azimuth in degrees.
+	   Latitudes and longitudes are in degrees. */
 
-	if (two_pts) {
-		*azim = M_PI_2 - d_atan2((blat - alat), (blon - alon));
-	}
-	else if (pole_set) {
-		*azim = M_PI - d_atan2((plat - alat), (plon - alon));
-	}
-	else {
-		(*azim) *= D2R;
-	}
+	if (two_pts)
+		*azim = 90.0 - d_atan2d(blat - alat, blon - alon);
+	else if (pole_set)
+		*azim = 180.0 - d_atan2d(plat - alat, plon - alon);
 
-	(*azim) = -(*azim);
-	e[0] = e[3] = cos(*azim);
-	e[1] = sin(*azim);
+	*azim = -(*azim);
+	e[0] = e[3] = cosd(*azim);
+	e[1] = sind(*azim);
 	e[2] = -e[1];
 }
 
 void copy_text_from_col3 (char *line, char *z_cols)
 {	/* returns the input line starting at the 3rd column */
 
-	int i;
+	GMT_LONG i;
 
 	/* First replace any commas with spaces */
 
