@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: psbasemap.c,v 1.47 2009/01/21 18:27:50 remko Exp $
+ *	$Id: psbasemap.c,v 1.49 2009/09/09 23:27:03 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ struct PSBASEMAP_CTRL {
 
 int main (int argc, char **argv)
 {
-	int i;
+	GMT_LONG i;
 
 	BOOLEAN error = FALSE;
 
