@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: psclip.c,v 1.39 2007/03/24 01:42:07 pwessel Exp $
+ *	$Id: psclip.c,v 1.57 2009/09/10 15:13:00 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -41,6 +41,9 @@ struct PSCLIP_CTRL {
 	struct N {	/* -N */
 		BOOLEAN active;
 	} N;
+	struct T {	/* -T */
+		BOOLEAN active;
+	} T;
 	struct Z {	/* -Z<z_level> */
 		BOOLEAN active;
 		double level;
@@ -49,11 +52,12 @@ struct PSCLIP_CTRL {
 
 int main (int argc, char **argv)
 {
-	int i, n, fno, n_alloc, n_args, n_files = 0, n_fields, n_expected_fields = 0;
+	GMT_LONG i, n, n_alloc;
+	GMT_LONG fno, n_args, n_files = 0, n_fields, n_expected_fields = 0;
 
-	BOOLEAN error = FALSE, nofile = TRUE, done, first;
+	BOOLEAN error = FALSE, nofile, done, first;
 
-	char line[BUFSIZ];
+	char line[BUFSIZ], *not_used = NULL;
 
 	double west, east, north, south, x0, y0, *in, *xx, *yy;
 
@@ -79,6 +83,7 @@ int main (int argc, char **argv)
 				case 'H':
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -91,6 +96,7 @@ int main (int argc, char **argv)
 				case 'b':
 				case 'c':
 				case 'f':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
@@ -109,8 +115,8 @@ int main (int argc, char **argv)
 				case 'N':
 					Ctrl->N.active = TRUE;
 					break;
-				case 'M':               /* Multiple line segments */
-					GMT_multisegment (&argv[i][2]);
+				case 'T':
+					Ctrl->T.active = TRUE;
 					break;
 				case 'Z':
 					if (argv[i][2]) {
@@ -136,8 +142,8 @@ int main (int argc, char **argv)
 		fprintf (stderr,"psclip %s - To set up polygonal clip paths\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: psclip -C [-K] [-O]  OR\n");
 		fprintf (stderr, "\tpsclip <xy-files> %s %s [%s]\n", GMT_J_OPT, GMT_Rgeo_OPT, GMT_B_OPT);
-		fprintf (stderr, "\t[-Eaz/el] [%s] [-K] [%s] [-N] [-O] [-P] [%s] [-V]\n", GMT_H_OPT, GMT_Mo_OPT, GMT_U_OPT);
-		fprintf (stderr, "\t[%s] [%s] [-Z<zlevel>] [%s] [%s]\n\t[%s] [%s]\n\n", GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
+		fprintf (stderr, "\t[-Eaz/el] [%s] [-K] [-N] [-O] [-P] [-T] [%s] [-V]\n", GMT_H_OPT, GMT_U_OPT);
+		fprintf (stderr, "\t[%s] [%s] [-Z<zlevel>] [%s] [%s]\n\t[%s] [%s] [%s]\n\n", GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -150,10 +156,10 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
 		GMT_explain_option ('H');
 		GMT_explain_option ('K');
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N uses the outside of the polygons as clip paths\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
+		fprintf (stderr, "\t-T Set of clip path for entire map frame.  No input file is required\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
 		GMT_explain_option ('X');
@@ -164,6 +170,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -178,14 +185,21 @@ int main (int argc, char **argv)
 			error++;
 		}
 	}
+	if (Ctrl->T.active) Ctrl->N.active = TRUE;	/* -T implies -N */
+	if (Ctrl->T.active && n_files) fprintf (stderr, "%s: Warning:  Option -T ignores all input files\n", GMT_program);
+
+	if (Ctrl->N.active && frame_info.plot) {
+		fprintf (stderr, "%s: Warning:  Option -B cannot be used in combination with Options -N or -T. -B is ignored.\n", GMT_program);
+		frame_info.plot = FALSE;
+	}
 
 	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Binary input data cannot have header -H\n", GMT_program);
 		error++;
 	}
         if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 2;
         if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < 2) {
-                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 2 columns\n", GMT_program);
+                fprintf (stderr, "%s: GMT SYNTAX ERROR:  Binary input data (-bi) must have at least 2 columns\n", GMT_program);
 		error++;
 	}
 
@@ -193,7 +207,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	z_project.view_azimuth = Ctrl->E.azimuth;
@@ -203,9 +217,9 @@ int main (int argc, char **argv)
 	if (!project_info.y_off_supplied && GMT_ps.overlay) GMT_ps.y_origin = 0.0;
 
 	if (Ctrl->C.active)
-		GMT_ps.clip_off = TRUE;	/* Signal that this program terminates clipping that initiated prior to this process */
+		GMT_ps.clip = -1;	/* Signal that this program terminates clipping that initiated prior to this process */
 	else
-		GMT_ps.clip_on = TRUE;	/* Signal that this program initiates clipping that wil outlive this process */
+		GMT_ps.clip = +1;	/* Signal that this program initiates clipping that wil outlive this process */
 
 	GMT_plotinit (argc, argv);
 
@@ -215,7 +229,9 @@ int main (int argc, char **argv)
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Done!\n", GMT_program);
 
+		Free_Psclip_Ctrl (Ctrl);	/* Deallocate control structure */
 		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
 	}
 
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
@@ -226,7 +242,12 @@ int main (int argc, char **argv)
 
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
 
-	if (!Ctrl->C.active) {	/* Start new clip_path */
+	if (Ctrl->C.active) {	/* Undo previous clip-path and draw basemap */
+		ps_clipoff ();
+		GMT_map_basemap ();
+	}
+
+	else {	/* Start new clip_path */
 
 		GMT_map_basemap ();
 		/* Allocate arrays holding the contour xy values */
@@ -235,10 +256,10 @@ int main (int argc, char **argv)
 		n_alloc = GMT_CHUNK;
 
 		if (Ctrl->N.active) GMT_map_clip_on (GMT_no_rgb, 1);	/* Must clip map */
-		first = (Ctrl->N.active) ? FALSE : TRUE;
-		done = FALSE;
+		first = !Ctrl->N.active;
+		done = Ctrl->T.active;	/* When -T is used, skip all input */
 		n_args = (argc > 1) ? argc : 2;
-		if (n_files > 0) nofile = FALSE;
+		nofile = (n_files == 0);
 
 		for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 			if (!nofile && argv[fno][0] == '-') continue;
@@ -259,9 +280,9 @@ int main (int argc, char **argv)
 				sprintf (line, "File: %s", argv[fno]);
 				ps_comment (line);
 			}
-			if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+			if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 			if (GMT_io.multi_segments[GMT_IN]) {
-				GMT_fgets (line, BUFSIZ, fp);
+				not_used = GMT_fgets (line, BUFSIZ, fp);
 				if (gmtdefs.verbose) ps_comment (line);
 			}
 			n_fields = GMT_input (fp, &n_expected_fields, &in);
@@ -280,14 +301,14 @@ int main (int argc, char **argv)
 
 				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE */
 					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, 2, n);
+						fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%d) fields near line %ld\n", GMT_program, n_fields, 2, n);
 						exit (EXIT_FAILURE);
 					}
 
-					xx[n] = in[0];	yy[n] = in[1];
+					xx[n] = in[GMT_X];	yy[n] = in[GMT_Y];
 					n++;
 					if (n == n_alloc) {
-						n_alloc += GMT_CHUNK;
+						n_alloc <<= 1;
 						xx = (double *) GMT_memory ((void *)xx, (size_t)n_alloc, sizeof (double), GMT_program);
 						yy = (double *) GMT_memory ((void *)yy, (size_t)n_alloc, sizeof (double), GMT_program);
 					}
@@ -311,17 +332,13 @@ int main (int argc, char **argv)
 			if (fp != GMT_stdin) GMT_fclose (fp);
 		}
 
-		ps_clipon (xx, yy, 0, GMT_no_rgb, 2);	/* Finish up the composite clip path */
+		/* Finish up the composite clip path */
+		ps_clipon (xx, yy, (GMT_LONG)0, GMT_no_rgb, 2 + first);
 
 		GMT_free ((void *)xx);
 		GMT_free ((void *)yy);
 	}
 
-	else {	/* Undo previous clip-path and draw basemap */
-		ps_clipoff ();
-		GMT_map_basemap ();
-	}
-
 	if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);
 
 	GMT_plotend ();
@@ -338,7 +355,7 @@ int main (int argc, char **argv)
 void *New_Psclip_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSCLIP_CTRL *C;
 	
-	C = (struct PSCLIP_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSCLIP_CTRL), "New_Psclip_Ctrl");
+	C = (struct PSCLIP_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSCLIP_CTRL), "New_Psclip_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.azimuth = 180.0;
