@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: pscoast.c,v 1.87 2009/01/16 21:43:11 guru Exp $
+ *	$Id: pscoast.c,v 1.99 2009/09/10 15:13:00 remko Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -54,16 +54,17 @@
 #include "gmt.h"
 #include "pslib.h"
 
+#define LAKE	0
+#define RIVER	1
+
 struct PSCOAST_CTRL {
 	struct A {	/* -A<min_area>[/<min_level>/<max_level>] */
 		BOOLEAN active;
-		int low;	/* Lowest hierarchical level to use [0] */
-		int high;	/* Highest hierarchical level to use [4] */
-		double area;	/* Area of smallest geographical feature to include [0] */
+		struct GMT_SHORE_SELECT info;
 	} A;
 	struct C {	/* -C<fill> */
 		BOOLEAN active;
-		struct GMT_FILL fill;
+		struct GMT_FILL fill[2];	/* lake and riverlake fill */
 	} C;
 	struct D {	/* -D<resolution> */
 		BOOLEAN active;
@@ -81,7 +82,7 @@ struct PSCOAST_CTRL {
 	} G;
 	struct I {	/* -I<feature>[/<pen>] */
 		BOOLEAN active;
-		int use[GMT_N_RLEVELS];
+		GMT_LONG use[GMT_N_RLEVELS];
 		struct GMT_PEN pen[GMT_N_RLEVELS];
 	} I;
 	struct L {	/* -L */
@@ -90,7 +91,7 @@ struct PSCOAST_CTRL {
 	} L;
 	struct N {	/* -N<feature>[/<pen>] */
 		BOOLEAN active;
-		int use[GMT_N_BLEVELS];
+		GMT_LONG use[GMT_N_BLEVELS];
 		struct GMT_PEN pen[GMT_N_BLEVELS];
 	} N;
 	struct Q {	/* -Q */
@@ -105,9 +106,10 @@ struct PSCOAST_CTRL {
 		BOOLEAN active;
 		struct GMT_MAP_ROSE item;
 	} T;
-	struct W {	/* -W<pen> */
+	struct W {	/* -W[<feature>/]<pen> */
 		BOOLEAN active;
-		struct GMT_PEN pen;
+		GMT_LONG use[GMT_MAX_GSHHS_LEVEL];
+		struct GMT_PEN pen[GMT_MAX_GSHHS_LEVEL];
 	} W;
 	struct Z {	/* -Z<z_level> */
 		BOOLEAN active;
@@ -124,7 +126,7 @@ struct PSCOAST_CTRL {
 int main (int argc, char **argv)
 {
 	int i, np, ind, bin, base, anti_bin = -1, level, flag, np_new, k, last_k, err;
-	int level_to_be_painted = 0, direction, start_direction, stop_direction;
+	int level_to_be_painted = 0, direction, start_direction, stop_direction, last_pen_level;
 	int n_blevels = 0, n_rlevels = 0, bin_trouble, lowest_level = 1, n_pieces = 0;
 
 	GMT_LONG n;
@@ -132,7 +134,7 @@ int main (int argc, char **argv)
 	BOOLEAN	error = FALSE, shift = FALSE, need_coast_base, recursive, fill_ocean, fill_land;
 	BOOLEAN	greenwich = FALSE, possibly_donut_hell = FALSE, fill_in_use = FALSE;
 	BOOLEAN clobber_background, paint_polygons = FALSE, donut, dumping = FALSE;
-	BOOLEAN donut_hell = FALSE, world_map_save, did_clip = FALSE, clipping = FALSE;
+	BOOLEAN donut_hell = FALSE, world_map_save, clipping = FALSE;
 
 	double	west = 0.0, east = 0.0, south = 0.0, north = 0.0, bin_x[5], bin_y[5], out[2];
 	double west_border, east_border, anti_lon = 0.0, anti_lat = -90.0, edge = 720.0;
@@ -141,7 +143,7 @@ int main (int argc, char **argv)
 	char *string, comment[GMT_LONG_TEXT];
 	char *shore_resolution[5] = {"full", "high", "intermediate", "low", "crude"};
 
-	struct GMT_FILL fill[5];	/* Colors for (0) water, (1) land, (2) lakes, (3) islands in lakes, (4) lakes in islands in lakes */
+	struct GMT_FILL fill[6];	/* Colors for (0) water, (1) land, (2) lakes, (3) islands in lakes, (4) lakes in islands in lakes, (5) riverlakes */
 	struct GMT_PEN pen;
 	struct GMT_SHORE c;
 	struct GMT_BR b, r;
@@ -167,6 +169,7 @@ int main (int argc, char **argv)
 				case 'B':	/* Get tickmark information */
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -178,6 +181,7 @@ int main (int argc, char **argv)
 				case 'y':
 				case 'c':
 				case 'b':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
@@ -186,14 +190,23 @@ int main (int argc, char **argv)
 
 				case 'A':
 					Ctrl->A.active = TRUE;
-					n = sscanf (&argv[i][2], "%lf/%d/%d", &Ctrl->A.area, &Ctrl->A.low, &Ctrl->A.high);
-					if (n == 1) Ctrl->A.low = 0, Ctrl->A.high = GMT_MAX_GSHHS_LEVEL;
+					GMT_set_levels (&argv[i][2], &Ctrl->A.info);
 					break;
-				case 'C':	/* Lake color */
+				case 'C':	/* Lake colors */
 					Ctrl->C.active = TRUE;
-					if (argv[i][2] && GMT_getfill (&argv[i][2], &Ctrl->C.fill)) {
-						GMT_fill_syntax ('C', " ");
-						error++;
+					if ((argv[i][2] == 'l' || argv[i][2] == 'r') && argv[i][3] == '/') {	/* Specific lake or river-lake fill */
+						k = (argv[i][2] == 'l') ? LAKE : RIVER;
+						if (argv[i][4] && GMT_getfill (&argv[i][4], &Ctrl->C.fill[k])) {
+							GMT_fill_syntax ('C', " ");
+							error++;
+						}
+					}
+					else if (argv[i][2]) {
+						if (GMT_getfill (&argv[i][2], &Ctrl->C.fill[LAKE])) {
+							GMT_fill_syntax ('C', " ");
+							error++;
+						}
+						Ctrl->C.fill[RIVER] = Ctrl->C.fill[LAKE];
 					}
 					break;
 				case 'D':
@@ -218,9 +231,6 @@ int main (int argc, char **argv)
 					Ctrl->L.active = TRUE;
 					error += GMT_getscale (&argv[i][2], &Ctrl->L.item);
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
 					if (!argv[i][2]) {
@@ -278,19 +288,19 @@ int main (int argc, char **argv)
 							for (k = 0; k < GMT_N_RLEVELS; k++) Ctrl->I.pen[k] = pen;
 							break;
 						case 'r':
-							for (k = 0; k < 4; k++) Ctrl->I.use[k] = TRUE;
-							for (k = 0; k < 4; k++) Ctrl->I.pen[k] = pen;
+							for (k = 0; k < GMT_RIV_INTERMITTENT; k++) Ctrl->I.use[k] = TRUE;
+							for (k = 0; k < GMT_RIV_INTERMITTENT; k++) Ctrl->I.pen[k] = pen;
 							break;
 						case 'i':
-							for (k = 4; k < 7; k++) Ctrl->I.use[k] = TRUE;
-							for (k = 4; k < 7; k++) Ctrl->I.pen[k] = pen;
+							for (k = GMT_RIV_INTERMITTENT; k < GMT_RIV_CANALS; k++) Ctrl->I.use[k] = TRUE;
+							for (k = GMT_RIV_INTERMITTENT; k < GMT_RIV_CANALS; k++) Ctrl->I.pen[k] = pen;
 							break;
 						case 'c':
-							for (k = 7; k < 10; k++) Ctrl->I.use[k] = TRUE;
-							for (k = 7; k < 10; k++) Ctrl->I.pen[k] = pen;
+							for (k = GMT_RIV_CANALS; k < GMT_N_RLEVELS; k++) Ctrl->I.use[k] = TRUE;
+							for (k = GMT_RIV_CANALS; k < GMT_N_RLEVELS; k++) Ctrl->I.pen[k] = pen;
 							break;
 						default:
-							k = atoi (&argv[i][2]) - 1;
+							k = atoi (&argv[i][2]);
 							if (k < 0 || k >= GMT_N_RLEVELS) {
 								fprintf (stderr, "%s: GMT SYNTAX ERROR -I option: Feature not in list!\n", GMT_program);
 								error++;
@@ -317,10 +327,30 @@ int main (int argc, char **argv)
 					error += GMT_getrose (&argv[i][2], &Ctrl->T.item);
 					break;
 				case 'W':
-					Ctrl->W.active = TRUE;
-					if (argv[i][2] && GMT_getpen (&argv[i][2], &Ctrl->W.pen)) {
-						GMT_pen_syntax ('W', " ");
-						error++;
+					Ctrl->W.active = TRUE;	/* Want to draw shorelines */
+					if ((argv[i][2] >= '1' && argv[i][2] <= '4') && argv[i][3] == '/') {	/* Specific pen for this feature */
+						k = (GMT_LONG)(argv[i][2] - '1');
+						if (argv[i][4] && GMT_getpen (&argv[i][4], &Ctrl->W.pen[k])) {
+							GMT_pen_syntax ('W', " ");
+							error++;
+						}
+						Ctrl->W.use[k] = TRUE;
+					}
+					else if (argv[i][2]) {	/* Same pen for all features */
+						Ctrl->W.use[0] = TRUE;
+						if (GMT_getpen (&argv[i][2], &Ctrl->W.pen[0])) {
+							GMT_pen_syntax ('W', " ");
+							error++;
+						}
+						else {
+							for (k = 1; k < GMT_MAX_GSHHS_LEVEL; k++) {
+								Ctrl->W.pen[k] = Ctrl->W.pen[0];
+								Ctrl->W.use[k] = TRUE;
+							}
+						}
+					}
+					else {	/* Accept default pen for all features */
+						for (k = 0; k < GMT_MAX_GSHHS_LEVEL; k++) Ctrl->W.use[k] = TRUE;
 					}
 					break;
 				case 'Q':
@@ -353,12 +383,16 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
 		fprintf (stderr,"pscoast %s - Plot continents, shorelines, rivers, and borders on maps\n\n", GMT_VERSION);
-		fprintf (stderr,"usage: pscoast %s %s\n\t[-A<min_area>[/<min_level>/<max_level>]] [%s] [-C[<fill>]]\n", GMT_B_OPT, GMT_J_OPT, GMT_Rgeoz_OPT);
-		fprintf (stderr, "\t[-D<resolution>][+] [%s] [-G[<fill>]] [-I<feature>[/<pen>]] [%s] [-K]\n", GMT_E_OPT, GMT_Jz_OPT);
-		fprintf (stderr, "\t[%s]\n", GMT_SCALE);
-		fprintf (stderr, "\t[%s] [-N<feature>[/<pen>]] [-O] [-P] [-Q] [-S<fill>]\n", GMT_Mo_OPT);
+		fprintf (stderr,"usage: pscoast %s %s [%s] [%s]\n", GMT_B_OPT, GMT_J_OPT, GMT_A_OPT, GMT_Rgeoz_OPT);
+		fprintf (stderr, "\t[-C[<feature>/]<fill>] [-D<resolution>][+] [%s] [-G[<fill>]] [-I<feature>[/<pen>]]\n", GMT_E_OPT);
+		fprintf (stderr, "\t[%s] [-K] [%s]\n", GMT_Jz_OPT, GMT_SCALE);
+		fprintf (stderr, "\t[-N<feature>[/<pen>]] [-O] [-P] [-Q] [-S<fill>]\n");
 		fprintf (stderr, "\t[%s] [%s]\n", GMT_TROSE, GMT_U_OPT);
-		fprintf (stderr, "\t[-V] [-W[<pen>]] [%s] [%s] [-Z<zlevel>] [%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT, GMT_bo_OPT, GMT_c_OPT);
+		fprintf (stderr, "\t[-V] [-W[<feature>/][<pen>]] [%s] [%s] [-Z<zlevel>]\n", GMT_X_OPT, GMT_Y_OPT);
+		fprintf (stderr, "\t[%s] [%s] [%s]\n", GMT_bo_OPT, GMT_c_OPT, GMT_mo_OPT);
+#ifdef DEBUG
+		fprintf (stderr, "\t[-+<bin>]\n");
+#endif
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -366,10 +400,12 @@ int main (int argc, char **argv)
 		GMT_explain_option ('Z');
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-A features smaller than <min_area> (in km^2) or of levels (0-4) outside the min-max levels\n");
-		fprintf (stderr, "\t   will be skipped [0/4 (4 means lake inside island inside lake)]\n");
+		GMT_GSHHS_syntax ('A', "Place limits on coastline features from the GSHHS data base.");
 		GMT_explain_option ('b');
-		GMT_fill_syntax ('C', "Sets separate color for lakes [Default is same as ocean].");
+		GMT_fill_syntax ('C', "Sets separate color for lakes and riverlakes [Default is same as ocean].");
+		fprintf (stderr, "\t   Alternatively, set custom fills below.  Repeat the -C option as needed\n");
+		fprintf (stderr, "\t      l = Lakes\n");
+		fprintf (stderr, "\t      r = River-lakes\n");
 		fprintf (stderr, "\t-D Choose one of the following resolutions:\n");
 		fprintf (stderr, "\t   f - full resolution (may be very slow for large regions)\n");
 		fprintf (stderr, "\t   h - high resolution (may be slow for large regions)\n");
@@ -382,6 +418,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   6) c to issue clip paths for land areas.\n");
 		GMT_pen_syntax ('I', "draws rivers.  Specify feature and optionally append pen [Default is solid line of unit thickness].");
 		fprintf (stderr, "\t   Choose from the features below.  Repeat the -I option as many times as needed\n");
+		fprintf (stderr, "\t      0 = Double-lined rivers (river-lakes)\n");
 		fprintf (stderr, "\t      1 = Permanent major rivers\n");
 		fprintf (stderr, "\t      2 = Additional major rivers\n");
 		fprintf (stderr, "\t      3 = Additional rivers\n");
@@ -392,15 +429,12 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t      8 = Major canals\n");
 		fprintf (stderr, "\t      9 = Minor canals\n");
 		fprintf (stderr, "\t     10 = Irrigation canals\n");
-		fprintf (stderr, "\t      a = All rivers and canals (1-10)\n");
-		fprintf (stderr, "\t      r = All permanent rivers (1-4)\n");
+		fprintf (stderr, "\t      a = All rivers and canals (0-10)\n");
+		fprintf (stderr, "\t      r = All permanent rivers (0-4)\n");
 		fprintf (stderr, "\t      i = All intermittent rivers (5-7)\n");
 		fprintf (stderr, "\t      c = All canals (8-10)\n");
 		GMT_explain_option ('K');
 		GMT_mapscale_syntax ('L', "Draws a simple map scale centered on <lon0>/<lat0>");
-		fprintf (stderr, "\t-M Dump a multisegment ascii (or binary, see -bo) file to standard output.  No plotting occurs\n");
-		fprintf (stderr, "\t   Specify any combination of -W, -I, -N.  Optionally, append 1-char\n");
-		fprintf (stderr, "\t   segment header <flag> [%c]\n", GMT_io.EOF_flag[GMT_OUT]);
 		GMT_pen_syntax ('N', "draws boundaries.  Specify feature and optionally append pen [Default is solid line of unit thickness].");
 		fprintf (stderr, "\t   Choose from the features below.  Repeat the -N option as many times as needed\n");
 		fprintf (stderr, "\t     1 = National boundaries\n");
@@ -415,12 +449,24 @@ int main (int argc, char **argv)
 		GMT_maprose_syntax ('T', "Draws a north-pointing map rose centered on <lon0>/<lat0>");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
-		GMT_pen_syntax ('W', "draw shorelines.  Append pen [Default is solid line of unit thickness].");
+		GMT_pen_syntax ('W', "draw shorelines.  Append pen [Default is solid black line of unit thickness for all levels].");
+		fprintf (stderr, "\t   Alternatively, set custom pens below.  Repeat the -W option as many times as needed\n");
+		fprintf (stderr, "\t      1 = Coastline\n");
+		fprintf (stderr, "\t      2 = Lake shores\n");
+		fprintf (stderr, "\t      3 = Island in lakes shores\n");
+		fprintf (stderr, "\t      4 = Lake in island in lake shores\n");
+		fprintf (stderr, "\t   When feature-specific pens are used, those not set are deactivated\n");
 		GMT_explain_option ('X');
 		fprintf (stderr, "\t-Z For 3-D plots: Set the z-level of map [0]\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
 		GMT_explain_option ('c');
+		fprintf (stderr, "\t-m Dump a multisegment ascii (or binary, see -bo) file to standard output.  No plotting occurs\n");
+		fprintf (stderr, "\t   Specify any combination of -W, -I, -N.  Optionally, append 1-char\n");
+		fprintf (stderr, "\t   segment header <flag> [%c]\n", GMT_io.EOF_flag[GMT_OUT]);
+#ifdef DEBUG
+		fprintf (stderr, "\t-+ Print only a single bin (debug option)\n");
+#endif
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -428,7 +474,7 @@ int main (int argc, char **argv)
 	fill_land = (Ctrl->G.active && !Ctrl->G.clip);
 	fill_ocean = (Ctrl->S.active && !Ctrl->S.clip);
 	if (Ctrl->C.active && !(fill_land || fill_ocean || Ctrl->W.active)) {	/* Just lakes, fix -A */
-		if (Ctrl->A.low < 2) Ctrl->A.low = 2;
+		if (Ctrl->A.info.low < 2) Ctrl->A.info.low = 2;
 	}
 
 	/* Check that the options selected are mutually consistent */
@@ -445,9 +491,11 @@ int main (int argc, char **argv)
 			error++;
 		}
 	}
-	if (Ctrl->W.pen.width < 0.0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -W option:  Pen thickness cannot be negative\n", GMT_program);
-		error++;
+	for (k = 0; k < GMT_MAX_GSHHS_LEVEL; k++) {
+		if (Ctrl->W.pen[k].width < 0.0) {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR -W option:  Pen thickness for feature %d cannot be negative\n", GMT_program, k);
+			error++;
+		}
 	}
 	if (!(fill_land || fill_ocean || Ctrl->C.active || Ctrl->W.active || Ctrl->N.active || Ctrl->I.active || clipping || Ctrl->Q.active)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify at least one of -C, -G, -S, -I, -N, and -W\n", GMT_program);
@@ -481,7 +529,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: Must specify at least one of -I, -N, and -W\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
-
+		
 	if (Ctrl->I.active) {
 		for (k = n_rlevels = 0; k < GMT_N_RLEVELS; k++) {
 			if (!Ctrl->I.use[k]) continue;
@@ -489,7 +537,7 @@ int main (int argc, char **argv)
 				fprintf (stderr, "%s: GMT SYNTAX ERROR -I option:  Pen thickness cannot be negative\n", GMT_program);
 				error++;
 			}
-			Ctrl->I.use[n_rlevels] = k + 1;
+			Ctrl->I.use[n_rlevels] = k;
 			Ctrl->I.pen[n_rlevels] = Ctrl->I.pen[k];
 			n_rlevels++;
 		}
@@ -516,8 +564,10 @@ int main (int argc, char **argv)
 	z_project.view_elevation = Ctrl->E.elevation;
 	fill[0] = Ctrl->S.fill;
 	fill[1] = fill[3] = Ctrl->G.fill;
-	fill[2] = fill[4] = ((Ctrl->C.active) ? Ctrl->C.fill : Ctrl->S.fill);
+	fill[2] = fill[4] = ((Ctrl->C.active) ? Ctrl->C.fill[LAKE] : Ctrl->S.fill);
+	fill[5] = (Ctrl->C.active) ? Ctrl->C.fill[RIVER] : fill[2];
 	need_coast_base = (fill_land || fill_ocean || Ctrl->C.active || Ctrl->W.active || Ctrl->G.clip || Ctrl->S.clip);
+	if (Ctrl->Q.active) need_coast_base = FALSE;	/* Since we just end clipping */
 	clobber_background = (fill_land && fill_ocean);
 	recursive = (((fill_land + (fill_ocean || Ctrl->C.active)) == 1) || clipping);
 	paint_polygons = (fill_land || fill_ocean || Ctrl->C.active) || clipping;
@@ -539,7 +589,7 @@ int main (int argc, char **argv)
 
 	world_map_save = GMT_world_map;
 
-	if (need_coast_base && GMT_init_shore (Ctrl->D.set, &c, project_info.w, project_info.e, project_info.s, project_info.n))  {
+	if (need_coast_base && GMT_init_shore (Ctrl->D.set, &c, project_info.w, project_info.e, project_info.s, project_info.n, &Ctrl->A.info))  {
 		fprintf (stderr, "%s: %s resolution shoreline data base not installed\n", GMT_program, shore_resolution[base]);
 		need_coast_base = FALSE;
 	}
@@ -548,6 +598,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: %s resolution political boundary data base not installed\n", GMT_program, shore_resolution[base]);
 		Ctrl->N.active = FALSE;
 	}
+	if (need_coast_base && gmtdefs.verbose == 2) fprintf (stderr, "GSHHS version %s\n%s\n%s\n", c.version, c.title, c.source);
 
 	if (Ctrl->I.active && GMT_init_br ('r', Ctrl->D.set, &r, project_info.w, project_info.e, project_info.s, project_info.n)) {
 		fprintf (stderr, "%s: %s resolution river data base not installed\n", GMT_program, shore_resolution[base]);
@@ -567,9 +618,9 @@ int main (int argc, char **argv)
 	}
 	else {
 		if (Ctrl->Q.active)
-			GMT_ps.clip_off = TRUE;	/* Signal that this program terminates clipping that initiated prior to this process */
+			GMT_ps.clip = -1;	/* Signal that this program terminates clipping that initiated prior to this process */
 		else if (clipping)
-			GMT_ps.clip_on = TRUE;	/* Signal that this program initiates clipping that wil outlive this process */
+			GMT_ps.clip = +1;	/* Signal that this program initiates clipping that wil outlive this process */
 
 		GMT_plotinit (argc, argv);
 
@@ -591,7 +642,7 @@ int main (int argc, char **argv)
 		if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
 	}
 
-	if (fill_ocean && !Ctrl->C.active) Ctrl->C.fill = fill[4] = Ctrl->S.fill;	/* Use ocean fill by default for lakes */
+	if (fill_ocean && !Ctrl->C.active) Ctrl->C.fill[LAKE] = fill[4] = Ctrl->S.fill;	/* Use ocean fill by default for lakes */
 
 	for (i = 0; i < 5; i++) if (fill[i].use_pattern) fill_in_use = TRUE;
 
@@ -631,8 +682,8 @@ int main (int argc, char **argv)
 	if (clobber_background) {	/* Paint entire map as ocean first, then lay land on top */
 		n = GMT_map_clip_path (&xtmp, &ytmp, &donut);
 		if (donut) {
-			ps_line (xtmp, ytmp, n, 1, TRUE, FALSE);
-			ps_line (&xtmp[n], &ytmp[n], n, -1, TRUE, FALSE);
+			ps_line (xtmp, ytmp, n, 1, TRUE);
+			ps_line (&xtmp[n], &ytmp[n], n, -1, TRUE);
 			GMT_fill (xtmp, ytmp, n, &Ctrl->S.fill, -9);
 		}
 		else
@@ -679,20 +730,21 @@ int main (int argc, char **argv)
 	west_border = floor (project_info.w / c.bsize) * c.bsize;
 	east_border = ceil (project_info.e / c.bsize) * c.bsize;
 
-	if (!dumping && Ctrl->W.active) GMT_setpen (&Ctrl->W.pen);
+	if (!dumping && Ctrl->W.active) GMT_setpen (&Ctrl->W.pen[0]);
 
 	if (Ctrl->Z.active) project_info.z_level = Ctrl->Z.level;
 
 	/* Maximum step size (in degrees) used for interpolation of line segments along great circles */
 	step = gmtdefs.line_step / project_info.x_scale / project_info.M_PR_DEG;
+	last_pen_level = -1;
 
 	for (ind = 0; need_coast_base && ind < c.nb; ind++) {	/* Loop over necessary bins only */
 
 		bin = c.bins[ind];
 #ifdef DEBUG
-		if (Ctrl->debug.bin >= 0 && bin != Ctrl->debug.bin) continue;
+		if (Ctrl->debug.active && bin != Ctrl->debug.bin) continue;
 #endif
-		if ((err = GMT_get_shore_bin (ind, &c, Ctrl->A.area, Ctrl->A.low, Ctrl->A.high))) {
+		if ((err = GMT_get_shore_bin (ind, &c))) {
 			fprintf (stderr, "%s: %s [%s resolution shoreline]\n", GMT_program, GMT_strerror(err), shore_resolution[base]);
 			exit (EXIT_FAILURE);
 		}
@@ -725,17 +777,14 @@ int main (int argc, char **argv)
 
 			/* Assemble one or more segments into polygons */
 
-			if ((np = GMT_assemble_shore (&c, direction, Ctrl->A.low, TRUE, shift, west_border, east_border, &p)) == 0) continue;
+			if ((np = GMT_assemble_shore (&c, direction, TRUE, shift, west_border, east_border, &p)) == 0) continue;
 
 			/* Get clipped polygons in x,y inches that can be plotted */
 
 			np_new = GMT_prep_polygons (&p, np, donut_hell, step, bin_trouble);
 
 			if (clipping) {
-				for (k = level_to_be_painted; k < GMT_MAX_GSHHS_LEVEL - 1; k++) {
-					recursive_clipping (-1, np_new, p, k, k, &n_pieces);
-					did_clip = TRUE;
-				}
+				for (k = level_to_be_painted; k < GMT_MAX_GSHHS_LEVEL - 1; k++) recursive_clipping (-1, np_new, p, k, k, &n_pieces);
 
 				for (k = 0; k < np_new; k++) {	/* Do any remaining interior polygons */
 					if (p[k].n == 0) continue;
@@ -746,20 +795,20 @@ int main (int argc, char **argv)
 						}
 				}
 			}
-			else if (!recursive) {	/* Simply points all polygons as is */
+			else if (!recursive) {	/* Simply paints all polygons as is */
 				for (k = 0; k < np_new; k++) {
 					if (p[k].n == 0 || p[k].level < lowest_level) continue;
 					level = p[k].level;
 					if (donut_hell && GMT_non_zero_winding (anti_x, anti_y, p[k].lon, p[k].lat, p[k].n)) {	/* Antipode inside polygon, must do donut */
 						n = GMT_map_clip_path (&xtmp, &ytmp, &donut);
-						ps_line (xtmp, ytmp, n, 1, TRUE, FALSE);
-						ps_line (p[k].lon, p[k].lat, p[k].n, -1, TRUE, FALSE);
-						GMT_fill (xtmp, ytmp, n, &fill[level], -9);
+						ps_line (xtmp, ytmp, n, 1, TRUE);
+						ps_line (p[k].lon, p[k].lat, p[k].n, -1, TRUE);
+						GMT_fill (xtmp, ytmp, n, &fill[p[k].fid], -9);
 						GMT_free ((void *)xtmp);
 						GMT_free ((void *)ytmp);
 					}
 					else
-						GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[level], FALSE);
+						GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[p[k].fid], FALSE);
 				}
 			}
 			else {	/* Must avoid pointing anything but the polygons inside */
@@ -771,7 +820,7 @@ int main (int argc, char **argv)
 				for (k = 0; k < np_new; k++) {	/* Do any remaining interior polygons */
 					if (p[k].n == 0) continue;
 					if (p[k].level%2 == level_to_be_painted || p[k].level > 2)
-						GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[p[k].level], FALSE);
+						GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[p[k].fid], FALSE);
 				}
 
 			}
@@ -781,21 +830,26 @@ int main (int argc, char **argv)
 		}
 
 		if (Ctrl->W.active && c.ns) {	/* Draw or dump shorelines, no need to assemble polygons */
-			if ((np = GMT_assemble_shore (&c, direction, Ctrl->A.low, FALSE, shift, west_border, east_border, &p)) == 0) continue;
+			if ((np = GMT_assemble_shore (&c, direction, FALSE, shift, west_border, east_border, &p)) == 0) continue;
 
 			for (i = 0; i < np; i++) {
 				if (dumping) {
-					sprintf (GMT_io.segment_header, "%c Shore Bin # %d, Level %d\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
+					sprintf (GMT_io.segment_header, "%c Shore Bin # %d, Level %ld\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
 					GMT_write_segmentheader (GMT_stdout, 2);
 					for (k = 0; k < p[i].n; k++) {
-						out[0] = p[i].lon[k];
-						out[1] = p[i].lat[k];
+						out[GMT_X] = p[i].lon[k];
+						out[GMT_Y] = p[i].lat[k];
 						GMT_output (GMT_stdout, 2, out);
 					}
 				}
-				else {
+				else if (Ctrl->W.use[p[i].level-1]) {
 					if (donut_hell) p[i].n = GMT_fix_up_path (&p[i].lon, &p[i].lat, p[i].n, step, 0);
 					GMT_n_plot = GMT_geo_to_xy_line (p[i].lon, p[i].lat, p[i].n);
+					k = p[i].level - 1;
+					if (k != last_pen_level) {
+						GMT_setpen (&Ctrl->W.pen[k]);
+						last_pen_level = k;
+					}
 					if (GMT_n_plot) GMT_plot_line (GMT_x_plot, GMT_y_plot, GMT_pen, GMT_n_plot);
 				}
 			}
@@ -809,7 +863,10 @@ int main (int argc, char **argv)
 	}
 	if (need_coast_base) GMT_shore_cleanup (&c);
 
-	if (clipping && did_clip) ps_clipon ((double *)NULL, (double *)NULL, 0, GMT_no_rgb, 2);	/* Dummy to end path */
+	if (clipping) {
+		if (n_pieces == 0) ps_clipon ((double *)NULL, (double *)NULL, 0, GMT_no_rgb, 1);	/* Just in case no piece was found */
+		ps_clipon ((double *)NULL, (double *)NULL, 0, GMT_no_rgb, 2);	/* Dummy to end path */
+	}
 
 	if (gmtdefs.verbose) fprintf (stderr, "\n");
 
@@ -838,11 +895,11 @@ int main (int argc, char **argv)
 
 			for (i = 0; i < np; i++) {
 				if (dumping) {
-					sprintf (GMT_io.segment_header, "%c River Bin # %d, Level %d\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
+					sprintf (GMT_io.segment_header, "%c River Bin # %d, Level %ld\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
 					GMT_write_segmentheader (GMT_stdout, 2);
 					for (k = 0; k < p[i].n; k++) {
-						out[0] = p[i].lon[k];
-						out[1] = p[i].lat[k];
+						out[GMT_X] = p[i].lon[k];
+						out[GMT_Y] = p[i].lat[k];
 						GMT_output (GMT_stdout, 2, out);
 					}
 				}
@@ -850,7 +907,7 @@ int main (int argc, char **argv)
 					GMT_n_plot = GMT_geo_to_xy_line (p[i].lon, p[i].lat, p[i].n);
 					if (!GMT_n_plot) continue;
 
-					k = p[i].level - 1;
+					k = p[i].level;
 					if (k != last_k) {
 						GMT_setpen (&Ctrl->I.pen[k]);
 						last_k = k;
@@ -900,11 +957,11 @@ int main (int argc, char **argv)
 
 			for (i = 0; i < np; i++) {
 				if (dumping) {
-					sprintf (GMT_io.segment_header, "%c Border Bin # %d, Level %d\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
+					sprintf (GMT_io.segment_header, "%c Border Bin # %d, Level %ld\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
 					GMT_write_segmentheader (GMT_stdout, 2);
 					for (k = 0; k < p[i].n; k++) {
-						out[0] = p[i].lon[k];
-						out[1] = p[i].lat[k];
+						out[GMT_X] = p[i].lon[k];
+						out[GMT_Y] = p[i].lat[k];
 						GMT_output (GMT_stdout, 2, out);
 					}
 				}
@@ -948,6 +1005,8 @@ int main (int argc, char **argv)
 		GMT_plotend ();
 	}
 
+	if (gmtdefs.verbose) fprintf (stderr, "Done\n");
+	
 	Free_Pscoast_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
@@ -981,7 +1040,7 @@ void recursive_painting (int k0, int np, struct GMT_GSHHS_POL p[], int level, in
 
 		if (p[k].level == start_level) {	/* Done nesting, time to paint the assembled swiss cheese polygon */
 			ps_clipon (p[k].lon, p[k].lat, (GMT_LONG)0, GMT_no_rgb, 4);	/* Dummy call with no points just to end current path */
-			GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[p[k].level], -1);
+			GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[p[k].fid], -1);
 		}
 	}
 }
@@ -1010,7 +1069,7 @@ BOOLEAN add_this_polygon_to_path (int k0, struct GMT_GSHHS_POL *p, int level, in
 {
 	/* Determines if we should add the current polygon pol[k] to the growing path we are constructing */
 
-	int j;
+	GMT_LONG j;
 	if (p[k].level != level) return (FALSE);	/* Sorry, this polygon does not have the correct level */
 	if (k0 == -1) return (TRUE);			/* The first one is always used */
 	/* Must make sure the polygon is inside the mother polygon.  Because numerical noise can trick us we try up to two points */
@@ -1029,21 +1088,19 @@ void *New_Pscoast_Ctrl () {	/* Allocate and initialize a new control structure *
 
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 
-	C->A.high = GMT_MAX_GSHHS_LEVEL;			/* Include all GSHHS levels */
+	C->A.info.high = GMT_MAX_GSHHS_LEVEL;			/* Include all GSHHS levels */
 	C->D.set = 'l';						/* Low-resolution coastline data */
 	C->E.azimuth = 180.0;
 	C->E.elevation = 90.0;
 	GMT_init_fill (&C->S.fill, 255, 255, 255);		/* Default Ocean color */
 	GMT_init_fill (&C->G.fill, 0, 0, 0);			/* Default Land color */
-	GMT_init_fill (&C->C.fill, 255, 255, 255);		/* Default Lake color */
-	for (k = 0; k < GMT_N_RLEVELS; k++) GMT_init_pen (&C->I.pen[k], GMT_PENWIDTH);	/* Default river pens */
-	for (k = 0; k < GMT_N_BLEVELS; k++) GMT_init_pen (&C->N.pen[k], GMT_PENWIDTH);	/* Default border pens */
+	GMT_init_fill (&C->C.fill[LAKE], 255, 255, 255);	/* Default Lake color */
+	GMT_init_fill (&C->C.fill[RIVER], 255, 255, 255);	/* Default Riverlake color */
+	for (k = 0; k < GMT_N_RLEVELS; k++) GMT_init_pen (&C->I.pen[k], GMT_PENWIDTH);		/* Default river pens */
+	for (k = 0; k < GMT_N_BLEVELS; k++) GMT_init_pen (&C->N.pen[k], GMT_PENWIDTH);		/* Default border pens */
+	for (k = 0; k < GMT_MAX_GSHHS_LEVEL; k++) GMT_init_pen (&C->W.pen[k], GMT_PENWIDTH);	/* Default coastline pens */
 	memset ((void *)&C->L.item, 0, sizeof (struct GMT_MAP_SCALE));
 	memset ((void *)&C->T.item, 0, sizeof (struct GMT_MAP_ROSE));
-	GMT_init_pen (&C->W.pen, GMT_PENWIDTH);				/* Default coastline pen */
-#ifdef DEBUG
-	C->debug.bin = -1;
-#endif
 
 	return ((void *)C);
 }
