@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: pscontour.c,v 1.104 2009/01/09 04:02:34 guru Exp $
+ *	$Id: pscontour.c,v 1.113 2009/09/09 23:27:03 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -84,7 +84,7 @@ struct PSCONTOUR_LINE {	/* Beginning and end of straight contour segment */
 };
 
 struct PSCONTOUR {
-	int n_alloc, nl;
+	GMT_LONG n_alloc, nl;
 	double val;
 	double angle;
 	char type;
@@ -104,18 +104,19 @@ struct PSCONTOUR_CHAIN {
 
 int main(int argc, char **argv)
 {
-	int *ind, nx, i, k, k2, k3, node1, node2, c, rgb[3], PSCONTOUR_SUM;
-	int n_alloc, section = 0, *vert, *cind, n_fields, n_contours = 0;
-	int add, close, bad, n_expected_fields, low, high, last_entry, last_exit;
-
-	GMT_LONG ij, n, np;
+	GMT_LONG nx, k2, k3, node1, node2, c, PSCONTOUR_SUM;
+	GMT_LONG n_alloc, section = 0, *vert, *cind, n_fields, n_contours = 0;
+	GMT_LONG add, close, bad, n_expected_fields, last_entry, last_exit;
+	GMT_LONG ij, n, np, k, i, low, high;
+	
+	int rgb[3], *ind = NULL;
 	
 	BOOLEAN error = FALSE, more, skip = FALSE, advance = FALSE;
 
 	double xx[3], yy[3], zz[3], xout[5], yout[5], *in, west, east, south, north;
 	double *xc, *yc, *zc, *x, *y, *z, *xp, *yp, current_contour = -DBL_MAX, xyz[2][3];
 
-	char line[BUFSIZ], cont_label[GMT_LONG_TEXT], format[GMT_LONG_TEXT];
+	char line[BUFSIZ], cont_label[GMT_LONG_TEXT], format[GMT_LONG_TEXT], *not_used = NULL;
 #ifdef TRIANGLE_D
 	char *algorithm = "Shewchuk";
 #else
@@ -127,7 +128,7 @@ int main(int argc, char **argv)
 	struct PSCONTOUR *cont = NULL;
 	struct PSCONTOUR_CTRL *Ctrl;
 
-	int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, int **v, int **cindex);
+	GMT_LONG get_triangle_crossings (double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, GMT_LONG **v, GMT_LONG **cindex);
 	void paint_it (double x[], double y[], GMT_LONG n, double z);
 	void *New_Pscontour_Ctrl (), Free_Pscontour_Ctrl (struct PSCONTOUR_CTRL *C);
 
@@ -147,6 +148,7 @@ int main(int argc, char **argv)
 				case 'H':
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -158,6 +160,7 @@ int main(int argc, char **argv)
 				case 'y':
 				case 'b':
 				case 'c':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
@@ -208,10 +211,6 @@ int main(int argc, char **argv)
 						error++;
 					}
 					break;
-				case 'M':	/* with -D, create one multiple line segments */
-					GMT_multisegment (&argv[i][2]);
-					GMT_io.multi_segments[GMT_OUT] = 2;
-					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
 					break;
@@ -248,9 +247,9 @@ int main(int argc, char **argv)
 		fprintf (stderr,"pscontour %s - Contour xyz-data by triangulation [%s]\n\n", GMT_VERSION, algorithm);
 		fprintf (stderr,"usage: pscontour <xyzfile> -C<cpt_file> %s %s\n", GMT_J_OPT, GMT_Rgeoz_OPT);
 		fprintf (stderr, "\t[-A[-|<annot_int>][<labelinfo>] [%s] [-D<dumpfile>] [%s]\n", GMT_B_OPT, GMT_E_OPT);
-		fprintf (stderr, "\t[%s] [%s] [-I] [%s] [-K] [-L<pen>] [%s] [-N] [-O]\n", GMT_CONTG, GMT_H_OPT, GMT_Jz_OPT, GMT_Mo_OPT);
+		fprintf (stderr, "\t[%s] [%s] [-I] [%s] [-K] [-L<pen>] [-N] [-O]\n", GMT_CONTG, GMT_H_OPT, GMT_Jz_OPT);
  		fprintf (stderr, "\t[-P] [-S] [-T<indexfile>] [-U] [-V] [-W[+]<pen>] [%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT);
-		fprintf (stderr, "\t[%s] [%s] [%s]\n\n", GMT_c_OPT, GMT_t_OPT, GMT_b_OPT);
+		fprintf (stderr, "\t[%s] [%s] [%s] [%s]\n\n", GMT_c_OPT, GMT_t_OPT, GMT_b_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -264,7 +263,7 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t   <labelinfo> controls the specifics of the labels.  Append what you need:\n");
 		GMT_label_syntax (5, 0);
 		GMT_explain_option ('b');
-		fprintf (stderr, "\t-D to Dump contour lines to individual files (but see -M)\n");
+		fprintf (stderr, "\t-D to Dump contour lines to individual files (but see -m)\n");
 		GMT_explain_option ('E');
 		fprintf (stderr, "\t-G Controls placement of labels along contours.  Choose among five algorithms:\n");
 		GMT_cont_syntax (3, 0);
@@ -272,8 +271,6 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t-I Color triangles using the cpt file\n");
 		GMT_explain_option ('K');
 		GMT_pen_syntax ('L', "draws the triangular mesh with the specified pen.");
-		fprintf (stderr, "\t-M Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
-		fprintf (stderr, "\t   whose first character is <flag> ['>'].  This header also has the contour level value\n");
 		fprintf (stderr,"\t-N do NOT clip contours/image at the border [Default clips]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
@@ -292,6 +289,8 @@ int main(int argc, char **argv)
 		fprintf (stderr,"\t   Default is 3 input columns.\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
+		fprintf (stderr, "\t-m Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
+		fprintf (stderr, "\t   whose first character is <flag> ['>'].  This header also has the contour level value\n");
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -332,11 +331,11 @@ int main(int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (GMT_io.binary[GMT_OUT] && GMT_io.multi_segments[GMT_OUT]) {
-		fprintf (stderr, "%s: GMT Warning.  -M for output ignored with -D, -bo\n", GMT_program);
+		fprintf (stderr, "%s: GMT Warning.  -m for output ignored with -D, -bo\n", GMT_program);
 		error++;
 	}
 	z_project.view_azimuth = Ctrl->E.azimuth;
@@ -352,6 +351,7 @@ int main(int argc, char **argv)
 		fprintf (stderr, "%s: contours will be written to file contour\n", GMT_program);
 		strcpy (Ctrl->D.file,"contour");
 	}
+	if (GMT_io.multi_segments[GMT_OUT]) GMT_io.multi_segments[GMT_OUT] = 2;
 
 	GMT_read_cpt (Ctrl->C.file);
 	if (Ctrl->I.active && GMT_continuous) {
@@ -384,7 +384,7 @@ int main(int argc, char **argv)
 	y = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	z = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 
-	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 	xyz[0][2] = DBL_MAX;	xyz[1][2] = -DBL_MAX;
 	n = 0;
@@ -394,20 +394,20 @@ int main(int argc, char **argv)
 	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 			exit (EXIT_FAILURE);
 		}
 
 		if (Ctrl->S.active) {	/* Must check if points are inside plot region */
-			GMT_map_outside (in[0], in[1]);
+			GMT_map_outside (in[GMT_X], in[GMT_Y]);
 			skip = ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1);
 		}
 
-		if (!(skip || GMT_is_dnan (in[2]))) {	/* Unless outside or z = NaN */
+		if (!(skip || GMT_is_dnan (in[GMT_Z]))) {	/* Unless outside or z = NaN */
 
-			x[n] = in[0];
-			y[n] = in[1];
-			z[n] = in[2];
+			x[n] = in[GMT_X];
+			y[n] = in[GMT_Y];
+			z[n] = in[GMT_Z];
 			if (z[n] < xyz[1][2]) {	/* New minimum */
 				xyz[0][0] = x[n];
 				xyz[0][1] = y[n];
@@ -503,7 +503,7 @@ int main(int argc, char **argv)
 			xx[2] = x[ind[k]];	yy[2] = y[ind[k++]];
 
 			if (project_info.three_D) for (c = 0; c < 3; c++) GMT_xy_do_z_to_xy (xx[c], yy[c], project_info.z_level, &xx[c], &yy[c]);
-			ps_line (xx, yy, (GMT_LONG)3, 3, 1, FALSE);
+			ps_line (xx, yy, (GMT_LONG)3, 3, TRUE);
 		}
 	}
 
@@ -661,7 +661,7 @@ int main(int argc, char **argv)
 					cont[c].n_alloc <<= 1;
 					cont[c].L = (struct PSCONTOUR_LINE *) GMT_memory ((void *)cont[c].L, (size_t)cont[c].n_alloc, sizeof (struct PSCONTOUR_LINE), GMT_program);
 				}
-				cont[c].nl = n;
+				cont[c].nl = (int)n;
 			}
 		}
 
@@ -847,13 +847,13 @@ int main(int argc, char **argv)
 }
 
 
-int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, int **v, int **cindex)
+GMT_LONG get_triangle_crossings (double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, GMT_LONG **v, GMT_LONG **cindex)
 {
 	/* This routine finds all the contour crossings for this triangle.  Each contour consists of
 	 * linesegments made up of two points, with coordinates xc, yc, and contour level zc.
 	 */
 	 
-	int i, j, k, k2, i1, nx, n_alloc, *vout, *cind;
+	GMT_LONG i, j, k, k2, i1, nx, n_alloc, *vout, *cind;
 	double xx[3], yy[3], zz[3], zmin, zmax, dz, frac, *xout, *yout, *zout;
 
 	xx[0] = x[ind[0]];	yy[0] = y[ind[0]];	zz[0] = z[ind[0]];
@@ -877,8 +877,8 @@ int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **
 	xout = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	yout = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	zout = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-	vout = (int *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (int), GMT_program);
-	cind = (int *) GMT_memory (VNULL, (size_t)nx, sizeof (int), GMT_program);
+	vout = (GMT_LONG *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (GMT_LONG), GMT_program);
+	cind = (GMT_LONG *) GMT_memory (VNULL, (size_t)nx, sizeof (GMT_LONG), GMT_program);
 
 	/* Fill out array zout which holds the nx contour levels */
 
@@ -931,7 +931,8 @@ int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **
 }
 
 void paint_it (double x[], double y[], GMT_LONG n, double z) {
-	int index, k, rgb[3];
+	GMT_LONG index, k;
+	int rgb[3];
 	struct GMT_FILL *f;
 
 	if (n < 3) return;	/* Need at least 3 points to make a polygon */
