@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: pslib_inc.h,v 1.47 2009/01/14 23:11:36 guru Exp $
+ *	$Id: pslib_inc.h,v 1.51 2009/09/09 23:27:03 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -153,6 +153,7 @@ typedef int BOOLEAN;		/* BOOLEAN used for logical variables */
 #define PSL_RGB			0
 #define PSL_CMYK		1
 #define PSL_HSV			2
+#define PSL_GRAY		3
 
 /*--------------------------------------------------------------------
  *			PSL FUNCTION MACRO DEFINITIONS
@@ -171,8 +172,8 @@ struct PSL {
 	struct INIT {	/* Parameters set by user via ps_plotinit() */
 		char *file;			/* Name of output file (NULL means stdout)	*/
 		char *encoding;			/* The encoding name. e.g. ISO-8859-1		*/
-		int overlay;			/* TRUE skips writing the PS header section	*/
-		int mode;			/* 32 bit-flags, used as follows:
+		PS_LONG overlay;		/* TRUE skips writing the PS header section	*/
+		PS_LONG mode;			/* 32 bit-flags, used as follows:
 			bit 0 : 0 = Landscape, 1 = Portrait,
 			bit 1 : 0 = be silent, 1 = be verbose
 			bit 2 : 0 = bin image, 1 = hex image
@@ -183,11 +184,11 @@ struct PSL {
 			bits 16-17 : (0,1,2) sets the line miter setting
 			bits 18-25 : (8 bits) sets the miter limit
 			bit 31 : 0 = write no comments, 1 = write PS comments to PS file	*/
-		int unit;			/* 0 = cm, 1 = inch, 2 = meter			*/
-		int copies;			/* Number of copies for this plot		*/
+		PS_LONG unit;			/* 0 = cm, 1 = inch, 2 = meter			*/
+		PS_LONG copies;			/* Number of copies for this plot		*/
 		int page_rgb[3];		/* RGB color for background paper [white]	*/
 		double page_size[2];		/* Width and height of paper used in points	*/
-		int dpi;			/* Selected dots per inch			*/
+		PS_LONG dpi;			/* Selected dots per inch			*/
 		double magnify[2];		/* Global scale values [1/1]			*/
 		double origin[2];		/* Origin offset [1/1]				*/
 		struct EPS *eps;		/* structure with Document info			*/
@@ -198,37 +199,36 @@ struct PSL {
 		char rgb_format[64];		/* Same, for RGB color triplets			*/
 		char hsv_format[64];		/* Same, for HSV color triplets	(HSB in PS)	*/
 		char cmyk_format[64];		/* Same, for CMYK color quadruples		*/
-		int font_no;			/* Current font number				*/
-		int linewidth;			/* Current pen thickness			*/
+		PS_LONG font_no;		/* Current font number				*/
+		PS_LONG linewidth;		/* Current pen thickness			*/
 		int rgb[3];			/* Current paint				*/
 		int fill_rgb[3];		/* Current fill					*/
-		int outline;			/* Current outline				*/
-		int offset;			/* Current setdash offset			*/
+		PS_LONG outline;		/* Current outline				*/
+		PS_LONG offset;			/* Current setdash offset			*/
 	} current;
 	struct INTERNAL {	/* Variables used internally only */
 		char *SHAREDIR;			/* Pointer to path of directory with pslib subdirectory */
 		char *USERDIR;			/* Pointer to path of directory with user definitions (~/.gmt) */
 		char *user_image[PSL_N_PATTERNS];	/* Name of user patterns		*/
-		char paint_code[12];		/* Shorthands for setgray, setrgbcolor etc	*/
 		BOOLEAN verbose;		/* TRUE for verbose output, FALSE remains quiet	*/
 		BOOLEAN comments;		/* TRUE for writing comments to output, FALSE strips all comments */
 		BOOLEAN landscape;		/* TRUE = Landscape, FALSE = Portrait		*/
 		BOOLEAN text_init;		/* TRUE after PSL_text.ps has been loaded	*/
-		int ascii;			/* TRUE writes images in ascii, FALSE uses binary	*/
-		int absolute;			/* TRUE will reset origin, FALSE means relative position	*/
-		int eps_format;			/* TRUE makes EPS file, FALSE means PS file	*/
-		int N_FONTS;			/* Total no of fonts;  To add more, modify the file CUSTOM_font_info.d */
-		int compress;			/* Compresses images with (1) RLE or (2) LZW or (0) None */
-		int color_mode;			/* 0 = rgb, 1 = cmyk, 2 = hsv (only 1-2 for images)	*/
-		int line_cap;			/* 0, 1, or 2 for butt, round, or square [butt] */
-		int line_join;			/* 0, 1, or 2 for miter, arc, or bevel [miter] */
-		int miter_limit;		/* Acute angle threshold 0-180; 0 means PS default [0] */
+		PS_LONG ascii;			/* TRUE writes images in ascii, FALSE uses binary	*/
+		PS_LONG absolute;		/* TRUE will reset origin, FALSE means relative position	*/
+		PS_LONG eps_format;		/* TRUE makes EPS file, FALSE means PS file	*/
+		PS_LONG N_FONTS;		/* Total no of fonts;  To add more, modify the file CUSTOM_font_info.d */
+		PS_LONG compress;		/* Compresses images with (1) RLE or (2) LZW or (0) None */
+		PS_LONG color_mode;		/* 0 = rgb, 1 = cmyk, 2 = hsv (only 1-2 for images)	*/
+		PS_LONG line_cap;		/* 0, 1, or 2 for butt, round, or square [butt] */
+		PS_LONG line_join;		/* 0, 1, or 2 for miter, arc, or bevel [miter] */
+		PS_LONG miter_limit;		/* Acute angle threshold 0-180; 0 means PS default [0] */
 		double bb[4];			/* Boundingbox arguments			*/
 		PS_LONG ix, iy;			/* Absolute coordinates of last point		*/
 		double p_width;			/* Paper width in points, set in plotinit();	*/
-		double p_height;			/* Paper height in points, set in plotinit();	*/
+		double p_height;		/* Paper height in points, set in plotinit();	*/
 		PS_LONG length;			/* Image row output byte counter		*/
-		int n_userimages;		/* Number of specified custom patterns		*/
+		PS_LONG n_userimages;		/* Number of specified custom patterns		*/
 		double scale;			/* Must be set through plotinit();		*/
 		double points_pr_unit;		/* # of points pr measure unit (e.g., 72/inch	*/
 		FILE *fp;			/* PS output file pointer. NULL = stdout	*/
@@ -240,7 +240,8 @@ struct PSL {
 		} *font;	/* Pointer to array of font structures 		*/
 		struct PSL_PATTERN {
 			PS_LONG nx, ny;
-			int status, depth, dpi, f_rgb[3], b_rgb[3];
+			PS_LONG status, depth, dpi;
+			int f_rgb[3], b_rgb[3];
 		} pattern[PSL_N_PATTERNS*2];
 	} internal;
 } *PSL;
