@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: psmask.c,v 1.61 2008/05/01 03:00:39 guru Exp $
+ *	$Id: psmask.c,v 1.82 2009/09/09 23:27:03 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * psmask tries to achieve masking using one of two different approachs:
+ * psmask tries to achieve masking using one of two different approaches:
  * The default way of operation is: Instead of painting tiles where there's no
  * data [or where there is data] psmask uses contouring to find the polygons
  * that contain the data [or contain the regions with no data].  For many types
@@ -47,6 +47,7 @@ struct PSMASK_CTRL {
 	struct D {	/* -D<dumpfile> */
 		BOOLEAN active;
 		char *file;
+		GMT_LONG min_pts;
 	} D;
 	struct E {	/* -Eazim/elev */
 		BOOLEAN active;
@@ -78,21 +79,20 @@ struct PSMASK_CTRL {
 
 struct PSMASK_INFO {
 	BOOLEAN first_dump;
-	int p[5], i_off[5], j_off[5], k_off[5], offset;
+	GMT_LONG p[5], i_off[5], j_off[5], k_off[5], offset;
 	unsigned int bit[32];
 };
 
 int main (int argc, char **argv)
 {
 	GMT_LONG ij, n, nm;
-	
-	int i, j, n_edges, di, dj, ii, jj, n_expected_fields, *edge;
-	int section, n_fields, distance_flag = 0;
-
+	GMT_LONG i, j, n_edges, di, dj, ii, jj, n_expected_fields, *edge;
+	GMT_LONG section, n_fields, distance_flag = 0, n_plus = 0, k;
 	GMT_LONG n_read, n_alloc;
+	
 	BOOLEAN error = FALSE, first = TRUE, node_only;
 
-	char line[BUFSIZ], *grd;
+	char line[BUFSIZ], *grd, *not_used = NULL;
 
 	double *in, distance, x0, y0, x1, y1, shrink = 1.0;
 	double xinc2, yinc2, *x, *y;
@@ -103,10 +103,13 @@ int main (int argc, char **argv)
 	struct PSMASK_INFO info;
 	struct PSMASK_CTRL *Ctrl;
 
-	void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], int id, int flag);
-	void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, int id, char *file);
+#ifdef DEBUG
+	BOOLEAN debug = FALSE;
+#endif
+	void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], GMT_LONG id, GMT_LONG flag);
+	void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, GMT_LONG id, char *file);
 	void shrink_clip_contours (double *x, double *y, GMT_LONG n, double w, double e);
-	int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, GMT_LONG *max);
+	GMT_LONG clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, GMT_LONG *edge, GMT_LONG first, double **x, double **y, GMT_LONG *max);
 	void *New_Psmask_Ctrl (), Free_Psmask_Ctrl (struct PSMASK_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -121,6 +124,8 @@ int main (int argc, char **argv)
 	GMT_init_fill (&Ctrl->G.fill, -1, -1, -1);
 	xinc2 = yinc2 = 0.0;
 
+	Ctrl->D.min_pts = 0;
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
@@ -154,11 +159,22 @@ int main (int argc, char **argv)
 				case 'D':	/* Dump the polygons to files */
 					Ctrl->D.active = TRUE;
 					free ((void *)Ctrl->D.file);
+
+					for (n_plus = 0, k = 2; argv[i][k]; k++) {
+						if (argv[i][k] == '+' && argv[i][k+1] == 'n') {
+							Ctrl->D.min_pts = atoi(&argv[i][k + 2]);
+							n_plus = 1;
+							break;
+						}
+					}
+
+					if (n_plus)	/* If extra option rip it before check if there is a prefix */
+						argv[i][k] = '\0';
 					Ctrl->D.file = strdup (&argv[i][2]);
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
-					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
+					error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'F':
 					Ctrl->F.active = TRUE;
@@ -188,6 +204,11 @@ int main (int argc, char **argv)
 				case 'T':
 					Ctrl->T.active = TRUE;
 					break;
+#ifdef DEBUG
+				case 'd':
+					debug = TRUE;
+					break;
+#endif
 				default:
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -205,9 +226,9 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"psmask %s - Masking or clipping of 2-D data sets\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: psmask <xyz-file> %s %s %s\n", GMT_I_OPT, GMT_J_OPT, GMT_Rgeo_OPT);
-		fprintf (stderr, "\t[%s] [-C] -D[-]<file> [-Eaz/el] [-F] [-G<fill>] [%s] [-K] [%s]\n", GMT_B_OPT, GMT_H_OPT, GMT_Mo_OPT);
+		fprintf (stderr, "\t[%s] [-C] -D[-]<file>[+n<n_pts>] [%s] [-F] [-G<fill>] [%s] [-K]\n", GMT_B_OPT, GMT_E_OPT, GMT_H_OPT);
 		fprintf (stderr, "\t[-N] [-O] [-P] [-S<radius>[k]] [-T] [%s] [-V] [%s]\n", GMT_U_OPT, GMT_X_OPT);
-		fprintf (stderr, "\t[%s] [%s] [%s] [%s]\n\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_b_OPT);
+		fprintf (stderr, "\t[%s] [%s] [%s] [%s] [%s] [-n]\n\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_b_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -221,12 +242,12 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-D dumps the clip-paths to files using the prefix <file>_ [mask_]\n");
 		fprintf (stderr, "\t   Ignored if -T is specified.  If prefix begins with - we will instead\n");
 		fprintf (stderr, "\t   use the rest of the name for a single multi-segment file.\n");
-		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t   Append +n<n_pts> to limit the number of points in files to a minimum of n_pts.\n");
+		GMT_explain_option ('E');
 		fprintf (stderr, "\t-F Force pixel registration [Default is gridline registration]\n");
 		GMT_fill_syntax ('G', "Select fill color/pattern [Default is no fill].");
 		GMT_explain_option ('H');
 		GMT_explain_option ('K');
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N will invert the sense of the clipping [or tiling]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
@@ -236,7 +257,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Default is -S0, i.e., only the nearest node is considered reliable\n");
 		fprintf (stderr, "\t   Append k for km (implies -R,-I in degrees), use flat Earth approximation.\n");
 		fprintf (stderr, "\t   Append K for km (implies -R,-I in degrees), use exact geodesic distances.\n");
-		fprintf (stderr, "\t   If the current ELLIPSOID is Sphere then spherical great circle distances are used.\n");
+		fprintf (stderr, "\t   If the current ELLIPSOID is spherical then great circle distances are used.\n");
 		fprintf (stderr, "\t-T will paint tiles.  [Default will trace data outline]\n");
 		fprintf (stderr, "\t   If set you must also specify a color/fill with -G\n");
 		GMT_explain_option ('U');
@@ -249,10 +270,13 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (!Ctrl->C.active) {
 		if (Ctrl->E.elevation <= 0.0 || Ctrl->E.elevation > 90.0) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Elevation must be in 0-90 range\n", GMT_program);
@@ -298,12 +322,26 @@ int main (int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
+	/* Now test also if we must provide a default -J */
+	if (Ctrl->D.active && project_info.projection < 0) {		/* Is this the right way of testing it? */
+		GMT_parse_J_option ("x1d");
+	}
+
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 	if (Ctrl->S.unit == 'k') distance_flag = 1;
 	if (Ctrl->S.unit == 'K') distance_flag = 2;
+	if (distance_flag) {
+		shrink = cosd (0.5 * (h.y_min + h.y_max));
+		di = (GMT_LONG)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.x_inc * shrink) + GMT_CONV_LIMIT);
+		dj = (GMT_LONG)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.y_inc) + GMT_CONV_LIMIT);
+	}
+	else {
+		di = irint (0.5 * Ctrl->S.radius / h.x_inc + GMT_CONV_LIMIT);
+		dj = irint (0.5 * Ctrl->S.radius / h.y_inc + GMT_CONV_LIMIT);
+	}
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
 	if (!Ctrl->C.active) GMT_err_fail (GMT_map_setup (h.x_min, h.x_max, h.y_min, h.y_max), "");
@@ -329,15 +367,8 @@ int main (int argc, char **argv)
 		nm = h.nx * h.ny;
 		grd = (char *) GMT_memory (VNULL, (size_t)nm, sizeof (char), GMT_program);
 
-		if (distance_flag) {
-			shrink = cos (0.5 * (h.y_min + h.y_max) * D2R);
-			di = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.x_inc * shrink));
-			dj = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * h.y_inc));
-		}
-		else {
-			di = irint (0.5 * Ctrl->S.radius / h.x_inc);
-			dj = irint (0.5 * Ctrl->S.radius / h.y_inc);
-		}
+		/* Add GMT_CONV_LIMIT to ensure that special case radius = inc --> irint(0.5) actually rounds to 1 */
+		
 		if (distance_flag == 2 && !GMT_IS_SPHERICAL) distance_flag = 3;	/* Use geodesics */
 		switch (distance_flag) {	/* Take different action depending on how we want distances calculated */
 
@@ -355,7 +386,9 @@ int main (int argc, char **argv)
 			break;
 		}
 		node_only = (di == 0 && dj == 0);
-
+		if (node_only && Ctrl->S.radius > 0.0) {
+			fprintf (stderr, "%s: Warning: Your search radius is too small to have any effect and is ignored.\n", GMT_program);
+		}
 		if (fp == NULL) {
 			fp = GMT_stdin;
 #ifdef SET_IO_MODE
@@ -363,7 +396,7 @@ int main (int argc, char **argv)
 #endif
 		}
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
 		n_read = 0;
@@ -375,16 +408,16 @@ int main (int argc, char **argv)
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 			n_read++;
 
 			/* Determine the node closest to the data point */
 
-			i = GMT_x_to_i (in[0], h.x_min, h.x_inc, h.xy_off, h.nx);
+			i = GMT_x_to_i (in[GMT_X], h.x_min, h.x_inc, h.xy_off, h.nx);
 			if (i < 0 || i >= h.nx) continue;
-			j = GMT_y_to_j (in[1], h.y_min, h.y_inc, h.xy_off, h.ny);
+			j = GMT_y_to_j (in[GMT_Y], h.y_min, h.y_inc, h.xy_off, h.ny);
 			if (j < 0 || j >= h.ny) continue;
 
 			if (node_only) {
@@ -423,14 +456,23 @@ int main (int argc, char **argv)
 		for (i = 0, ij = (h.ny-1) * h.nx; i < h.nx; i++) grd[i] = grd[i+ij] = FALSE;
 		for (j = 0; j < h.ny; j++) grd[j*h.nx] = grd[(j+1)*h.nx-1] = FALSE;
 
+#ifdef DEBUG
+		if (debug) {
+			float *z;
+			z = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
+			for (i = 0; i < nm; i++) z[i] = (float)grd[i];
+			GMT_write_grd ("psmask.grd", &h, z, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE);
+			GMT_free ((void *)z);
+		}
+#endif
 		if (!Ctrl->T.active) {	/* Must trace the outline of ON/OFF values in grd */
 			/* Arrays holding the contour xy values */
 			x = (double *) GMT_memory (VNULL, (size_t)GMT_CHUNK, sizeof (double), GMT_program);
 			y = (double *) GMT_memory (VNULL, (size_t)GMT_CHUNK, sizeof (double), GMT_program);
 			n_alloc = GMT_CHUNK;
 
-			n_edges = h.ny * (int )ceil (h.nx / 16.0);
-			edge = (int *) GMT_memory (VNULL, (size_t)n_edges, sizeof (int), GMT_program);
+			n_edges = h.ny * (GMT_LONG )ceil (h.nx / 16.0);
+			edge = (GMT_LONG *) GMT_memory (VNULL, (size_t)n_edges, sizeof (GMT_LONG), GMT_program);
 
 			GMT_map_basemap ();
 
@@ -440,7 +482,7 @@ int main (int argc, char **argv)
 			first = TRUE;
 			while ((n = clip_contours (&info, grd, &h, xinc2, yinc2, edge, first, &x, &y, &n_alloc)) > 0) {
 				shrink_clip_contours (x, y, n, h.x_min, h.x_max);
-				if (Ctrl->D.active) dump_clip_contours (&info, x, y, n, section, Ctrl->D.file);
+				if (Ctrl->D.active && n > Ctrl->D.min_pts) dump_clip_contours (&info, x, y, n, section, Ctrl->D.file);
 				draw_clip_contours (x, y, n, Ctrl->G.fill.rgb, section, first);
 				first = FALSE;
 				section++;
@@ -453,8 +495,7 @@ int main (int argc, char **argv)
 			GMT_free ((void *)y);
 		}
 		else {	/* Just paint tiles */
-			int start;
-			GMT_LONG n_use, np, plot_n;
+			GMT_LONG start, n_use, np, plot_n;
 			double *grd_x, grd_y, y_bot, y_top, *xx, *yy, *xp, *yp;
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Tiling...\n", GMT_program);
 			grd_x = (double *) GMT_memory (VNULL, (size_t)h.nx, sizeof (double), GMT_program);
@@ -466,7 +507,7 @@ int main (int argc, char **argv)
 				y_top = grd_y + yinc2;
 				ij = GMT_IJ (j, 1, h.nx);
 				for (i = 0; i < h.nx; i++, ij++) {
-					if (((int)grd[ij]) == 0) continue;
+					if (((GMT_LONG)grd[ij]) == 0) continue;
 
 					np = GMT_graticule_path (&xx, &yy, 1, grd_x[i] - xinc2, grd_x[i] + xinc2, y_bot, y_top);
 					plot_n = GMT_clip_to_map (xx, yy, np, &xp, &yp);
@@ -476,21 +517,20 @@ int main (int argc, char **argv)
 					
 					if ((*GMT_will_it_wrap) (xp, yp, plot_n, &start)) {	/* Polygon wraps */
 
-					/* First truncate agains left border */
+						/* First truncate against left border */
 
-						GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, -1);
+						GMT_n_plot = GMT_truncate (xp, yp, plot_n, start, -1);
 						n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &Ctrl->G.fill, FALSE);
 
-						/* Then truncate agains right border */
+						/* Then truncate against right border */
 
-						GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, +1);
+						GMT_n_plot = GMT_truncate (xp, yp, plot_n, start, +1);
 						n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &Ctrl->G.fill, FALSE);
 					}
-					else {
+					else
 						GMT_fill (xp, yp, plot_n, &Ctrl->G.fill, FALSE);
-					}
 					GMT_free ((void *)xp);
 					GMT_free ((void *)yp);
 				}
@@ -523,7 +563,7 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], int id, int flag)
+void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], GMT_LONG id, GMT_LONG flag)
 {
 	GMT_LONG i;
 	double x, y;
@@ -540,17 +580,17 @@ void draw_clip_contours (double *xx, double *yy, GMT_LONG nn, int rgb[], int id,
 	if (project_info.three_D) GMT_2D_to_3D (xx, yy, project_info.z_level, nn);
 
 	if (nn > 0) {
-		sprintf (comment, "Start of clip path sub-segment %d", id);
+		sprintf (comment, "Start of clip path sub-segment %ld", id);
 		ps_comment (comment);
 	}
 	ps_clipon (xx, yy, nn, rgb, flag);
 	if (nn > 0) {
-		sprintf (comment, "End of clip path sub-segment %d", id);
+		sprintf (comment, "End of clip path sub-segment %ld", id);
 		ps_comment (comment);
 	}
 }
 
-void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, int id, char *file)
+void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_LONG nn, GMT_LONG id, char *file)
 {
 	GMT_LONG i;
 	double out[2];
@@ -558,6 +598,7 @@ void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_L
 	FILE *fp;
 
 	if (nn < 2) return;
+	if (nn == 2 && GMT_IS_ZERO (xx[1] - xx[0]) && GMT_IS_ZERO (yy[1] - yy[0])) return;
 
 	if (file[0] == '-') {	/* Want a single multi-segment file */
 		sprintf (fname, "%s", &file[1]);
@@ -570,39 +611,40 @@ void dump_clip_contours (struct PSMASK_INFO *info, double *xx, double *yy, GMT_L
 			exit (EXIT_FAILURE);
 		}
 		info->first_dump = FALSE;
+		GMT_write_segmentheader (fp, 2);
 	}
 	else {
 		if (GMT_io.binary[GMT_OUT])
-			sprintf (fname, "%s_%d.b", file, id);
+			sprintf (fname, "%s_%ld.b", file, id);
 		else
-			sprintf (fname, "%s_%d.xy", file, id);
+			sprintf (fname, "%s_%ld.xy", file, id);
 		if ((fp = GMT_fopen (fname, GMT_io.w_mode)) == NULL) {
 			fprintf (stderr, "%s: Unable to create file %s - exiting\n", GMT_program, fname);
 			exit (EXIT_FAILURE);
 		}
 	}
 	for (i = 0; i < nn; i++) {
-		out[0] = xx[i];	out[1] = yy[i];
+		out[GMT_X] = xx[i];	out[GMT_Y] = yy[i];
 		GMT_output (fp, 2, out);
 	}
 	GMT_fclose (fp);
 }
 
-int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, GMT_LONG *max)
+GMT_LONG clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, GMT_LONG *edge, GMT_LONG first, double **x, double **y, GMT_LONG *max)
 {
 	/* The routine finds the zero-contour in the grd dataset.  it assumes that
 	 * no node has a value exactly == 0.0.  If more than max points are found
 	 * trace_clip_contours will try to allocate more memory in blocks of GMT_CHUNK points
 	 */
 	 
-	static int i0, j0, side;
+	static GMT_LONG i0, j0, side;
 	GMT_LONG ij;
-	int i, j, n = 0, n_edges, edge_word, edge_bit;
+	GMT_LONG i, j, n = 0, n_edges, edge_word, edge_bit;
 	BOOLEAN go_on = TRUE;
-	int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, GMT_LONG *max);
+	GMT_LONG trace_clip_contours (struct PSMASK_INFO *info, char *grd, GMT_LONG *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, GMT_LONG i, GMT_LONG j, GMT_LONG kk, GMT_LONG *max);
 	 
 	 
-	n_edges = h->ny * (int) ceil (h->nx / 16.0);
+	n_edges = h->ny * (GMT_LONG) ceil (h->nx / 16.0);
 	 
 	 /* Reset edge-flags to zero, if necessary */
 	 if (first) {
@@ -623,8 +665,8 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 		for (j = j0; go_on && j < h->ny; j++) {
 			ij = GMT_IJ (j, i0, h->nx);
 			for (i = i0; go_on && i < h->nx-1; i++, ij++) {	/* nx-1 since the last bin starts at nx-2 and ends at nx-1 */
-				edge_word = ij / 32 + info->offset;
-				edge_bit = ij % 32;
+				edge_word = (GMT_LONG)(ij / 32 + info->offset);
+				edge_bit = (GMT_LONG)(ij % 32);
 				if (!(edge[edge_word] & info->bit[edge_bit]) && ((grd[ij]+grd[ij-h->nx]) == 1)) { /* Start tracing contour */
 					*x[0] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
 					*y[0] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
@@ -647,8 +689,8 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 		for (j = j0; go_on && j < h->ny; j++) {
 			ij = GMT_IJ (j, i0, h->nx);
 			for (i = i0; go_on && i < h->nx-1; i++, ij++) {
-				edge_word = ij / 32 + info->offset;
-				edge_bit = ij % 32;
+				edge_word = (GMT_LONG)(ij / 32 + info->offset);
+				edge_bit = (GMT_LONG)(ij % 32);
 				if (!(edge[edge_word] & info->bit[edge_bit]) && ((grd[ij]+grd[ij+1]) == 1)) { /* Start tracing contour */
 					*x[0] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
 					*y[0] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
@@ -666,11 +708,11 @@ int clip_contours (struct PSMASK_INFO *info, char *grd, struct GRD_HEADER *h, do
 	return (n);
 }
 
-int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, GMT_LONG *max)
+GMT_LONG trace_clip_contours (struct PSMASK_INFO *info, char *grd, GMT_LONG *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, GMT_LONG i, GMT_LONG j, GMT_LONG kk, GMT_LONG *max)
 {
-	int n = 1, k, k0, n_cuts, kk_opposite, first_k, more;
-	int edge_word, edge_bit, m;
-	GMT_LONG ij, ij0;
+	GMT_LONG n = 1, k, k0, n_cuts, kk_opposite, first_k, more;
+	GMT_LONG edge_word, edge_bit;
+	GMT_LONG ij, ij0, m;
 	double xk[4], yk[4], x0, y0;
 
 	m = *max - 2;
@@ -692,8 +734,8 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 			/* Skip edge already has been used */
 
 			ij0 = GMT_IJ (j + info->j_off[k], i + info->i_off[k], h->nx);
-			edge_word = ij0 / 32 + info->k_off[k] * info->offset;
-			edge_bit = ij0 % 32;
+			edge_word = (GMT_LONG)(ij0 / 32 + info->k_off[k] * info->offset);
+			edge_bit = (GMT_LONG)(ij0 % 32);
 			if (edge[edge_word] & info->bit[edge_bit]) continue;
 
 			/* Skip if no zero-crossing on this edge */
@@ -733,10 +775,13 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 			*yy = (double *) GMT_memory ((void *)*yy, (size_t)(*max), sizeof (double), "trace_clip_contours");
 		}
 		if (n_cuts == 0) {	/* Close interior contour and return */
-			if (fmod ((*xx[0] - xinc2), h->x_inc) == 0.0)	/* On side 1 or 3 */
-				first_k = ((*xx[0] - x0) == 0.0) ? 3 : 1;
+			/* if (fmod ((*xx[0] - xinc2), h->x_inc) == 0.0) */	/* On side 1 or 3 */
+			if (GMT_IS_ZERO (fmod ((*xx[0] - xinc2), h->x_inc)))	/* On side 1 or 3 */
+				/* first_k = ((*xx[0] - x0) == 0.0) ? 3 : 1; */
+				first_k = GMT_IS_ZERO (*xx[0] - x0) ? 3 : 1;
 			else 	/* On side 0 or 2 */
-				first_k = ((*yy[0] - y0) == 0.0) ? 0 : 2;
+				/* first_k = ((*yy[0] - y0) == 0.0) ? 0 : 2; */
+				first_k = GMT_IS_ZERO (*yy[0] - y0) ? 0 : 2;
 			kk_opposite = (first_k + 2) % 4;
 			if (k0 != kk_opposite) {
 				(*xx)[n] = x0 + 0.5*h->x_inc;
@@ -772,8 +817,8 @@ int trace_clip_contours (struct PSMASK_INFO *info, char *grd, int *edge, struct
 		}
 		if (more) {	/* Mark this edge as used */
 			ij0 = GMT_IJ (j + info->j_off[kk], i + info->i_off[kk], h->nx);
-			edge_word = ij0 / 32 + info->k_off[kk] * info->offset;
-			edge_bit = ij0 % 32;
+			edge_word = (GMT_LONG)(ij0 / 32 + info->k_off[kk] * info->offset);
+			edge_bit = (GMT_LONG)(ij0 % 32);
 			edge[edge_word] |= info->bit[edge_bit];
 		}
 
