@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: psrose.c,v 1.68 2009/01/09 04:02:34 guru Exp $
+ *	$Id: psrose.c,v 1.76 2009/09/09 23:27:03 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -100,13 +100,12 @@ int main (int argc, char **argv)
 	BOOLEAN error = FALSE, find_mean = FALSE, half_only = FALSE, windrose = TRUE;
 	BOOLEAN automatic = FALSE, sector_plot = FALSE;
 
-	char text[BUFSIZ], *file = CNULL, format[BUFSIZ], *choice[2] = {"OFF", "ON"};
+	char text[BUFSIZ], *file = CNULL, format[BUFSIZ], *choice[2] = {"OFF", "ON"}, *not_used = NULL;
 	char txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT], txt_d[GMT_LONG_TEXT];
 
-	int n_bins, bin, n_annot;
-	int status, n_alpha, n_modes, n_fields, n_expected_fields;
-	
-	GMT_LONG i, n = 0, n_alloc = GMT_CHUNK;
+	GMT_LONG n_bins, n_annot;
+	GMT_LONG status, n_alpha, n_modes, n_fields, n_expected_fields;
+	GMT_LONG i, bin, n = 0, n_alloc = GMT_CHUNK;
 
 	double max = 0.0, radius, az, x_origin, y_origin, tmp, one_or_two = 1.0, s, c, half_bin_width;
 	double angle1, angle2, angle, x, y, mean_theta, mean_radius, xr = 0.0, yr = 0.0;
@@ -119,8 +118,8 @@ int main (int argc, char **argv)
 	struct PSROSE_CTRL *Ctrl;
 	struct GMT_FILL f;
 
-	void plot3d (double x, double y, int pen);
-	void arc3d (double x, double y, double r, double a1, double a2, int status);
+	void plot3d (double x, double y, GMT_LONG pen);
+	void arc3d (double x, double y, double r, double a1, double a2, GMT_LONG status);
 	void *New_Psrose_Ctrl (), Free_Psrose_Ctrl (struct PSROSE_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -297,7 +296,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('X');
 		fprintf (stderr, "\t-Z multiply the radii by scale before plotting\n");
 		GMT_explain_option ('c');
-		fprintf (stderr, "\t-: Expect (azimuth,radius) input rather than (radius,azimuth) [%s]\n", choice[gmtdefs.xy_toggle[0]]);
+		fprintf (stderr, "\t-: Expect (azimuth,radius) input rather than (radius,azimuth) [%s]\n", choice[gmtdefs.xy_toggle[GMT_IN]]);
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 2 input columns.\n");
@@ -341,7 +340,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	z_project.view_azimuth = Ctrl->E.azimuth;
@@ -392,7 +391,7 @@ int main (int argc, char **argv)
 	azimuth = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	length = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 
-	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (text, BUFSIZ, fp);
+	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (text, BUFSIZ, fp);
 
 	/* Read data and do some stats */
 
@@ -404,12 +403,12 @@ int main (int argc, char **argv)
 	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n);
 			exit (EXIT_FAILURE);
 		}
 
-		length[n] = in[0];
-		azimuth[n] = in[1];
+		length[n] = in[GMT_X];
+		azimuth[n] = in[GMT_Y];
 
 		if (Ctrl->Z.scale != 1.0) length[n] *= Ctrl->Z.scale;
 
@@ -455,7 +454,7 @@ int main (int argc, char **argv)
 		}
 	}
 
-	mean_theta = R2D * d_atan2 (yr, xr) / one_or_two;
+	mean_theta = d_atan2d (yr, xr) / one_or_two;
 	if (mean_theta < 0.0) mean_theta += 360.0;
 	mean_radius = hypot (xr, yr) / total;
 	if (!Ctrl->S.normalize) mean_radius *= max_radius;
@@ -649,7 +648,7 @@ int main (int argc, char **argv)
 		GMT_get_format (frame_info.axis[0].item[GMT_ANNOT_UPPER].interval, frame_info.axis[0].unit, frame_info.axis[0].prefix, format);
 
 		if (half_only) {
-			int which, no_degree;
+			GMT_LONG which, no_degree;
 			which = (gmtdefs.degree_format >= 100) ? gmt_ring : gmt_degree;
 			no_degree = (gmtdefs.degree_format >= 1000);	/* No, we don't want the degree symbol at all */
 			if (!Ctrl->L.active) {	/* Use default labels */
@@ -723,15 +722,15 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void arc3d (double x, double y, double r, double a1, double a2, int status)
+void arc3d (double x, double y, double r, double a1, double a2, GMT_LONG status)
 {
 	if (project_info.three_D) {	/* Must project and draw line segments */
-		int n;
+		GMT_LONG n;
 		double *xx, *yy;
 
 		n = GMT_get_arc (x, y, r, a1, a2, &xx, &yy);
 		GMT_2D_to_3D (xx, yy, project_info.z_level, (GMT_LONG)n);
-		ps_line (xx, yy, (GMT_LONG)n, status, FALSE, TRUE);
+		ps_line (xx, yy, (GMT_LONG)n, status, FALSE);
 		GMT_free ((void *)xx);
 		GMT_free ((void *)yy);
 	}
@@ -739,7 +738,7 @@ void arc3d (double x, double y, double r, double a1, double a2, int status)
 		ps_arc (x, y, r, a1, a2, status);
 }
 
-void plot3d (double x, double y, int pen)
+void plot3d (double x, double y, GMT_LONG pen)
 {
 	if (project_info.three_D) GMT_xyz_to_xy (x, y, project_info.z_level, &x, &y);
 	ps_plot (x, y, pen);
