@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: pstext.c,v 1.88 2009/01/09 04:02:34 guru Exp $
+ *	$Id: pstext.c,v 1.97 2009/09/09 23:27:03 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  * Date:	21-JAN-1991-2000
  * Version:	2.0 based on old v1.x
  * Version:	3.1 based on old 3.0
- * Version:	3.3: PW: Introducing paragraph mode -M
+ * Version:	3.3: PW: Introducing paragraph mode -m
  * Version:	3.3.4: PW: Input x,y can be in dd:mm[:ss] format
  * Version:	4.2.x: Now can handle @_, @:, @; everywhere and @;colorname; is OK
  *
@@ -72,23 +72,23 @@ struct PSTEXT_CTRL {
 		BOOLEAN active;
 		BOOLEAN outline;
 		BOOLEAN paint;
-		int mode;
+		GMT_LONG mode;
 		struct GMT_PEN pen;
 		struct GMT_FILL fill;
 	} W;
 	struct Z {	/* -Z<z_level> */
 		BOOLEAN active;
-		int mode;	/* 0 for common z_level, 1 for optional z-value in 3rd col for each record */
+		GMT_LONG mode;	/* 0 for common z_level, 1 for optional z-value in 3rd col for each record */
 		double level;
 	} Z;
 };
 
 struct PSTEXT_INFO {
-	int text_justify;
-	int paragraph_font;
-	int block_justify;
-	int boxflag;
-	int space_flag;
+	GMT_LONG text_justify;
+	GMT_LONG paragraph_font;
+	GMT_LONG block_justify;
+	GMT_LONG boxflag;
+	GMT_LONG space_flag;
 	double x_offset, y_offset;	/* Offset from reference point */
 	double line_spacing;
 	double paragraph_width;
@@ -104,12 +104,9 @@ struct PSTEXT_INFO {
 
 int main(int argc, char **argv)
 {
-	int i, j, k, last_font, fno, n_files = 0, n_args, nscan;
-	int ix, iy, *use_font, n_processed = 0, n_paragraphs = 0, pos;
-	
-	GMT_LONG n_words = 0, n_read = 0;
-	
-	size_t n_alloc = 0;
+	GMT_LONG i, j, k, last_font, fno, n_files = 0, n_args, nscan;
+	GMT_LONG ix, iy, *use_font, n_paragraphs = 0, pos;
+	GMT_LONG n_words = 0, n_read = 0, n_processed = 0, n_alloc = 0;
 
 	double xy[2], west = 0.0, east = 0.0, south = 0.0, north = 0.0, plot_x = 0.0, plot_y = 0.0;
 	double z_level = 0.0, tmp, dx, dy, xx[2], yy[2], z_level_orig = 0.0;
@@ -120,7 +117,7 @@ int main(int argc, char **argv)
 
 	char text[BUFSIZ], line[BUFSIZ], this_font[GMT_LONG_TEXT], just_key[5], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
 	char txt_x[GMT_LONG_TEXT], txt_y[GMT_LONG_TEXT], txt_z[GMT_LONG_TEXT], p[BUFSIZ];
-	char **word = (char **)NULL, pjust;
+	char **word = (char **)NULL, pjust, *not_used = NULL;
 
 	FILE *fp = NULL;
 
@@ -149,6 +146,7 @@ int main(int argc, char **argv)
 				case 'H':
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -159,6 +157,7 @@ int main(int argc, char **argv)
 				case 'Y':
 				case 'y':
 				case 'c':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
@@ -211,9 +210,6 @@ int main(int argc, char **argv)
 				case 'L':
 					Ctrl->L.active = TRUE;
 					break;
-				case 'M':               /* Multiple line segments input: Text blocks */
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':	/* Do not clip at border */
 					Ctrl->N.active = TRUE;
 					break;
@@ -304,11 +300,11 @@ int main(int argc, char **argv)
 		fprintf (stderr, "pstext %s - To plot text on maps\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pstext <txtfile> %s %s\n", GMT_J_OPT, GMT_Rgeoz_OPT);
 		fprintf (stderr, "\t[-A] [%s] [-C<dx>/<dy>] [-D[j]<dx>[/<dy>][v[<pen>]] [%s] [-G<color>]\n", GMT_B_OPT, GMT_E_OPT);
-		fprintf (stderr, "\t[%s] [-K] [-L] [%s] [-N] [-O] [-P] [-S<pen>] [%s]\n", GMT_Ho_OPT, GMT_Mo_OPT, GMT_U_OPT);
+		fprintf (stderr, "\t[%s] [-K] [-L] [-N] [-O] [-P] [-S<pen>] [%s]\n", GMT_Ho_OPT, GMT_U_OPT);
 		fprintf (stderr, "\t[-V] [-W[<fill>,][o|O|c|C[<pen>]]] [%s] [%s]\n", GMT_X_OPT, GMT_X_OPT);
-		fprintf (stderr, "\t[-Z[<zlevel>|+]] [%s] [%s]\n\n", GMT_t_OPT, GMT_c_OPT);
+		fprintf (stderr, "\t[-Z[<zlevel>|+]] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_c_OPT, GMT_mo_OPT);
 		fprintf (stderr, "\tReads (x,y,size,angle,fontno,justify,text) from <txtfile> [or stdin]\n");
-		fprintf (stderr, "\tOR (with -M) one or more text paragraphs with formatting info in the segment header.\n");
+		fprintf (stderr, "\tOR (with -m) one or more text paragraphs with formatting info in the segment header.\n");
 		fprintf (stderr, "\tjustify is of the form [T|M|B][L|C|R] (top/middle/bottom/left/center/right)\n");
 		fprintf (stderr, "\tBuilt-in escape sequences:\n");
 		fprintf (stderr, "\t   @~ toggles between current font and Symbol font\n");
@@ -328,7 +324,7 @@ int main(int argc, char **argv)
 			fprintf (stderr, "\n\tFont #	Font Name\n");
 			fprintf (stderr, "\t------------------------------------\n");
 			for (i = 0; i < GMT_N_FONTS; i++)
-				fprintf (stderr, "\t%3d\t%s\n", i, GMT_font[i].name);
+				fprintf (stderr, "\t%3ld\t%s\n", i, GMT_font[i].name);
 		}
 
 		if (Ctrl->L.active || GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -347,10 +343,6 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t-G set the color (red/green/blue (0-255)) for solid text [%d/%d/%d]\n", gmtdefs.basemap_frame_rgb[0], gmtdefs.basemap_frame_rgb[1], gmtdefs.basemap_frame_rgb[2]);
 		GMT_explain_option ('H');
 		GMT_explain_option ('K');
-		GMT_explain_option ('M');
-		fprintf (stderr, "\t   Expects (x y size angle fontno justify linespace parwidth parjust) in segment header\n");
-		fprintf (stderr, "\t   followed by lines with one or more paragraphs of text.\n");
-		fprintf (stderr, "\t   parjust is one of (l)eft, (c)enter, (r)ight, or (j)ustified.\n");
 		fprintf (stderr, "\t-N Do Not clip text that exceeds the map boundaries [Default will clip]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
@@ -361,14 +353,18 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t   Append fill color [none].  To draw outline, append o, O, or c, and optionally a pen [No outline]\n");
 		fprintf (stderr, "\t   Separate the fill color and the outline information by a comma if both are present.\n");
 		fprintf (stderr, "\t   Lower case o will draw rectangular outline\n");
-		fprintf (stderr, "\t   Upper case O will draw rectangle with rounded corners (-M only)\n");
-		fprintf (stderr, "\t   Lower case c will draw concave rectangle (-M only)\n");
-		fprintf (stderr, "\t   Upper case C will draw convex rectangle (-M only)\n");
+		fprintf (stderr, "\t   Upper case O will draw rectangle with rounded corners (-m only)\n");
+		fprintf (stderr, "\t   Lower case c will draw concave rectangle (-m only)\n");
+		fprintf (stderr, "\t   Upper case C will draw convex rectangle (-m only)\n");
 		GMT_explain_option ('X');
 		fprintf (stderr, "\t-Z For 3-D plots: Set the z-level of map [0].  If -Z+ is given then we expect\n");
 		fprintf (stderr, "\t   records to have an optional z value in the 3rd column (i.e., x y z size ...)\n");
 		fprintf (stderr, "\t   Note that -Z+ also sets -N\n");
 		GMT_explain_option ('c');
+		GMT_explain_option ('m');
+		fprintf (stderr, "\t   Expects (x y size angle fontno justify linespace parwidth parjust) in segment header\n");
+		fprintf (stderr, "\t   followed by lines with one or more paragraphs of text.\n");
+		fprintf (stderr, "\t   parjust is one of (l)eft, (c)enter, (r)ight, or (j)ustified.\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
@@ -390,7 +386,7 @@ int main(int argc, char **argv)
 		error++;
 	}
 	if (!(z_project.view_azimuth == 180.0 && z_project.view_elevation == 90.0) && do_paragraphs) {
-		fprintf (stderr, "%s: -E not implemented with -M yet\n", GMT_program);
+		fprintf (stderr, "%s: -E not implemented with -m yet\n", GMT_program);
 		error++;
 	}
 	if (Ctrl->D.dx == 0.0 && Ctrl->D.dy == 0.0 && Ctrl->D.line) {
@@ -412,7 +408,7 @@ int main(int argc, char **argv)
 	
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
-	use_font = (int *) GMT_memory (VNULL, (size_t)GMT_N_FONTS, sizeof (int), GMT_program);
+	use_font = (GMT_LONG *) GMT_memory (VNULL, (size_t)GMT_N_FONTS, sizeof (GMT_LONG), GMT_program);
 	add = !(T.x_offset == 0.0 && T.y_offset == 0.0);
 	if (add && Ctrl->D.justify) T.boxflag |= 64;
 	if (n_files > 0)
@@ -437,7 +433,7 @@ int main(int argc, char **argv)
 	if (!(Ctrl->N.active || Ctrl->Z.mode == 1)) GMT_map_clip_on (GMT_no_rgb, 3);
 
 	last_font = 0;
-	ix = (gmtdefs.xy_toggle[0]);	iy = 1 - ix;
+	ix = (gmtdefs.xy_toggle[GMT_IN]);	iy = 1 - ix;
 
 	/* if (draw_box && project_info.three_D) draw_box = FALSE; */	/* Not implemented yet */
 
@@ -472,7 +468,7 @@ int main(int argc, char **argv)
 		}
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = fgets (line, BUFSIZ, fp);
 
 		while (GMT_fgets (line, BUFSIZ, fp)) {
 
@@ -525,14 +521,10 @@ int main(int argc, char **argv)
 						fprintf (stderr, "%s: Record %ld had bad justification info (set to LB)\n", GMT_program, n_read);
 						T.block_justify = 1;
 					}
-					if (this_font[0] >= '0' && this_font[0] <= '9')	/* Gave a font # */
-						T.paragraph_font = atoi (this_font);
-					else {	/* Gave a font name */
-						T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
-						if (T.paragraph_font == GMT_N_FONTS) {
-							fprintf (stderr, "%s: Record %ld had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
-							T.paragraph_font = 0;
-						}
+					T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
+					if (T.paragraph_font == GMT_N_FONTS) {
+						fprintf (stderr, "%s: Record %ld had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
+						T.paragraph_font = 0;
 					}
 					if (!use_font[T.paragraph_font] && strcmp (gmtdefs.encoding.name, "Standard") != 0) {	/* Must reencode this font */
 						ps_encode_font (T.paragraph_font);
@@ -554,7 +546,7 @@ int main(int argc, char **argv)
 					if (line[0] == 0) {	/* Blank line marked by single NULL character */
 						word[n_words] = (char *) calloc ((size_t)(1), sizeof (char));
 						n_words++;
-						if ((size_t)n_words == n_alloc) {
+						if (n_words == n_alloc) {
 							n_alloc <<= 1;
 							word = (char **) GMT_memory (VNULL, n_alloc, sizeof (char *), GMT_program);
 						}
@@ -563,7 +555,7 @@ int main(int argc, char **argv)
 					while ((GMT_strtok (line, " ", &pos, p))) {
 						word[n_words] = strdup (p);
 						n_words++;
-						if ((size_t)n_words == n_alloc) {
+						if (n_words == n_alloc) {
 							n_alloc <<= 1;
 							word = (char **) GMT_memory (VNULL, n_alloc, sizeof (char *), GMT_program);
 						}
@@ -621,14 +613,10 @@ int main(int argc, char **argv)
 					}
 					xx[1] = plot_x;	yy[1] = plot_y;
 				}
-				if (this_font[0] >= '0' && this_font[0] <= '9')	/* Gave a font # */
-					T.paragraph_font = atoi (this_font);
-				else {	/* Gave a font name */
-					T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
-					if (T.paragraph_font == GMT_N_FONTS) {
-						fprintf (stderr, "%s: Record %ld had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
-						T.paragraph_font = 0;
-					}
+				T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
+				if (T.paragraph_font == GMT_N_FONTS) {
+					fprintf (stderr, "%s: Record %ld had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
+					T.paragraph_font = 0;
 				}
 				n_paragraphs++;
 
@@ -693,9 +681,9 @@ int main(int argc, char **argv)
 
 	if (gmtdefs.verbose) {
 		if (do_paragraphs)
-			fprintf (stderr, "pswords: Plotted %d textblocks\n", n_paragraphs);
+			fprintf (stderr, "pswords: Plotted %ld textblocks\n", n_paragraphs);
 		else
-			fprintf (stderr, "pstext: Plotted %d text-strings\n", n_paragraphs);
+			fprintf (stderr, "pstext: Plotted %ld text-strings\n", n_paragraphs);
 	}
 
 	GMT_free ((void *)use_font);
@@ -709,9 +697,8 @@ int main(int argc, char **argv)
 
 void GMT_putwords (double x, double y, char **text, GMT_LONG n_words, struct PSTEXT_INFO *T) {
 	char *boxpen_texture = CNULL, *vecpen_texture = CNULL;
-	int boxpen_width, boxpen_offset, boxpen_rgb[3];
-	int vecpen_width, vecpen_offset, vecpen_rgb[3];
-	int i;
+	GMT_LONG boxpen_width, boxpen_offset, vecpen_width, vecpen_offset, i;
+	int boxpen_rgb[3], vecpen_rgb[3];
 	double dx, dy;
 
 	boxpen_texture = GMT_convertpen (&T->boxpen, &boxpen_width, &boxpen_offset, boxpen_rgb);
