@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: pswiggle.c,v 1.56 2009/01/09 04:02:34 guru Exp $
+ *	$Id: pswiggle.c,v 1.67 2009/09/16 01:30:12 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -89,14 +89,13 @@ int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, nofile = TRUE, paint_wiggle, done;
 
-	char line[BUFSIZ], *units = CNULL, txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
+	char line[BUFSIZ], *units = CNULL, txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], *not_used = NULL;
 
 	GMT_LONG i, j, n, n_alloc = GMT_CHUNK;
+	GMT_LONG k, n_files = 0, fno, n_args, n_expected_fields, n_fields, wantx, wanty;
 
-	int k, n_files = 0, fno, n_args, n_expected_fields, n_fields, wantx, wanty;
-
-	double west = 0.0, east = 360.0, south = 0.0, north = 0.0, x_2, y_2;
-	double dx, dy, dz, ds, *in, *xx, *yy, *zz, *lon, *lat, *z, start_az, stop_az;
+	double west = 0.0, east = 360.0, south = 0.0, north = 0.0, x_2, y_2, d2m = 1.0;
+	double dx, dy, dz, ds, *in, *xx, *yy, *zz, *lon, *lat, *z, start_az, stop_az, fix_az;
 
 	FILE *fp = NULL;
 
@@ -120,6 +119,7 @@ int main (int argc, char **argv)
 				case 'H':
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -132,6 +132,7 @@ int main (int argc, char **argv)
 				case 'b':
 				case 'c':
 				case 'f':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
@@ -166,9 +167,6 @@ int main (int argc, char **argv)
 					Ctrl->I.value = atof (&argv[i][2]);
 					Ctrl->I.active = TRUE;
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
 					break;
@@ -229,8 +227,8 @@ int main (int argc, char **argv)
 		fprintf (stderr,"pswiggle %s - Plot xyz-series along tracks\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pswiggle <xyz-files> %s %s -Z<scale>\n", GMT_J_OPT, GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[-A<azimuth>] [%s] [-C<center>] [-D<gap>] [%s] [-G<fill>] [%s]\n", GMT_B_OPT, GMT_E_OPT, GMT_Ho_OPT);
-		fprintf (stderr, "\t[-I<az>] -K] [%s] [-N] [-O] [-P] [-S[x]<lon0>/<lat0>/<length>/<units>] [-T<trackpen>]\n", GMT_Mo_OPT);
-		fprintf (stderr, "\t[%s] [-V] [-W<outlinepen>] [%s] [%s]\n\t[%s] [%s] [%s] [%s]\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
+		fprintf (stderr, "\t[-I<az>] -K [-N] [-O] [-P] [-S[x]<lon0>/<lat0>/<length>/<units>] [-T<trackpen>]\n");
+		fprintf (stderr, "\t[%s] [-V] [-W<outlinepen>] [%s] [%s]\n\t[%s] [%s] [%s] [%s]] [%s]\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -248,7 +246,6 @@ int main (int argc, char **argv)
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-I set fixed projection azimuths for wiggles\n");
 		GMT_explain_option ('K');
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N Fill negative wiggles instead [Default is positive]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
@@ -266,6 +263,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 3 input columns\n");
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -300,7 +298,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	z_project.view_azimuth = Ctrl->E.azimuth;
@@ -308,7 +306,10 @@ int main (int argc, char **argv)
 
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
 
-	if (GMT_IS_MAPPING) Ctrl->D.value *= 1000.0;    /* Distance in meters */
+	if (GMT_IS_MAPPING) {
+		Ctrl->D.value *= 1000.0;    /* Distance in meters */
+		d2m = TWO_PI * project_info.EQ_RAD / 360.0;
+	}
 
 	GMT_plotinit (argc, argv);
 
@@ -341,7 +342,7 @@ int main (int argc, char **argv)
 	/* Now convert angles to radians and keep it that way */
 	start_az = (Ctrl->A.value - 90.0) * D2R;
 	stop_az  = (Ctrl->A.value + 90.0) * D2R;
-	Ctrl->I.value *= D2R;
+	fix_az = Ctrl->I.value * D2R;
 
 	GMT_map_clip_on (GMT_no_rgb, 3);
 
@@ -379,9 +380,9 @@ int main (int argc, char **argv)
 			ps_comment (line);
 		}
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 		if (GMT_io.multi_segments[GMT_IN]) {
-			GMT_fgets (line, BUFSIZ, fp);
+			not_used = GMT_fgets (line, BUFSIZ, fp);
 			if (gmtdefs.verbose) ps_comment (line);
 		}
 		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3;
@@ -398,13 +399,13 @@ int main (int argc, char **argv)
 			n = 0;
 			while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 					exit (EXIT_FAILURE);
 				}
 
-				lon[n] = in[0];
-				lat[n] = in[1];
-				z[n] = in[2] - Ctrl->C.value;
+				lon[n] = in[GMT_X];
+				lat[n] = in[GMT_Y];
+				z[n] = in[GMT_Z] - Ctrl->C.value;
 				n++;
 				if (n >= n_alloc) {
 					n_alloc <<= 1;
@@ -427,8 +428,8 @@ int main (int argc, char **argv)
 				dx = lon[i] - lon[i-1];
 				dy = lat[i] - lat[i-1];
 				if (GMT_IS_MAPPING) {	/* Distance in meters */
-					dx *= cos (0.5 * (lat[i] + lat[i-1]) * D2R);
-					ds = hypot (dx, dy) * project_info.M_PR_DEG;
+					dx *= cosd (0.5 * (lat[i] + lat[i-1]));
+					ds = d2m * hypot (dx, dy);
 				}
 				else
 					ds = hypot (dx, dy);
@@ -437,12 +438,12 @@ int main (int argc, char **argv)
 
 				if (j > 0 && (ds > Ctrl->D.value || GMT_is_dnan (z[i]))) {	/* Data gap, plot what we have */
 					paint_wiggle = (Ctrl->G.active && ((Ctrl->N.active && zz[j-1] <= 0.0) || (!Ctrl->N.active && zz[j-1] >= 0.0)));
-					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, Ctrl->I.value, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
+					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, fix_az, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
 					j = 0;
 				}
 				else if (j >= PSWIGGLE_MAX_POINTS) {
 					paint_wiggle = (Ctrl->G.active && ((Ctrl->N.active && zz[j-1] <= 0.0) || (!Ctrl->N.active && zz[j-1] >= 0.0)));
-					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, Ctrl->I.value, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
+					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, fix_az, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
 					xx[0] = xx[j-1];
 					yy[0] = yy[j-1];
 					zz[0] = zz[j-1];
@@ -455,7 +456,7 @@ int main (int argc, char **argv)
 					zz[j] = 0.0;
 					j++;
 					paint_wiggle = (Ctrl->G.active && ((Ctrl->N.active && zz[j-2] <= 0.0) || (!Ctrl->N.active && zz[j-2] >= 0.0)));
-					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, Ctrl->I.value, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
+					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, fix_az, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
 					xx[0] = xx[j-1];
 					yy[0] = yy[j-1];
 					zz[0] = zz[j-1];
@@ -469,7 +470,7 @@ int main (int argc, char **argv)
 	
 			if (j > 1) {
 				paint_wiggle = (Ctrl->G.active && ((Ctrl->N.active && zz[j-1] <= 0.0) || (!Ctrl->N.active && zz[j-1] >= 0.0)));
-				plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, Ctrl->I.value, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
+				plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, fix_az, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
 			}
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
@@ -499,7 +500,7 @@ int main (int argc, char **argv)
 
 void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, double start_az, double stop_az, BOOLEAN fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, BOOLEAN paint_wiggle, BOOLEAN negative, BOOLEAN outline, BOOLEAN track)
 {
-	double dx, dy, len, az = 0.0, s, c, x_inc, y_inc;
+	double dx, dy, len, az = 0.0, s = 0.0, c = 0.0, x_inc, y_inc;
 	GMT_LONG n = 0, i;
 	static char *name[2] = {"Positive", "Negative"}, comment[GMT_LONG_TEXT];
 
@@ -560,7 +561,7 @@ void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, d
 		sprintf (comment, "Wiggle line");
 		ps_comment (comment);
 		GMT_setpen (pen_o);
-		ps_line (&GMT_x_plot[1], &GMT_y_plot[1], np, 3, FALSE, TRUE);
+		ps_line (&GMT_x_plot[1], &GMT_y_plot[1], np, 3, FALSE);
 	}
 
 	if (track) {	/* Finally draw track line */
@@ -568,7 +569,7 @@ void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, d
 		sprintf (comment, "Track line");
 		ps_comment (comment);
 		GMT_setpen (pen_t);
-		ps_line (x, y, np, 3, FALSE, TRUE);
+		ps_line (x, y, np, 3, FALSE);
 	}
 }
 
@@ -593,7 +594,7 @@ void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLE
 	GMT_xyz_to_xy (x0, y0 - dy, 0.0, &xx[1], &yy[1]);
 	GMT_xyz_to_xy (x0, y0 + dy, 0.0, &xx[2], &yy[2]);
 	GMT_xyz_to_xy (x0 + gmtdefs.map_scale_height, y0 + dy, 0.0, &xx[3], &yy[3]);
-	ps_line (xx, yy, (GMT_LONG)4, 3, FALSE, TRUE);
+	ps_line (xx, yy, (GMT_LONG)4, 3, FALSE);
 	off = ((gmtdefs.map_scale_height > 0.0) ? gmtdefs.tick_length : 0.0) + gmtdefs.annot_offset[0];
 	GMT_text3D (x0 + off, y0, project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], txt, 0.0, 5, 0);
 }
