@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: sample1d.c,v 1.42 2008/03/24 08:58:32 guru Exp $
+ *	$Id: sample1d.c,v 1.52 2009/09/09 23:27:04 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 struct SAMPLE1D_CTRL {
 	struct F {	/* -Fl|a|c */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 	} F;
 	struct I {	/* -I<inc> */
 		BOOLEAN active;
@@ -42,7 +42,7 @@ struct SAMPLE1D_CTRL {
 	} I;
 	struct T {	/* -T<time_col> */
 		BOOLEAN active;
-		int col;
+		GMT_LONG col;
 	} T;
 	struct N {	/* -N<knotfile> */
 		BOOLEAN active;
@@ -56,8 +56,7 @@ struct SAMPLE1D_CTRL {
 
 int main (int argc, char **argv)
 {
-	int result, n_col = 0, n_fields, n_expected_fields, n_files = 0, fno, n_args, n_req;
-	
+	GMT_LONG result, n_col = 0, n_fields, n_expected_fields, n_files = 0, fno, n_args, n_req;
 	GMT_LONG i, j, k, n, m, n_alloc = 0, m_alloc, rows = 1, m_supplied = 0;
 
 	BOOLEAN error = FALSE, *nan_flag = VNULL, nofile = TRUE, done = FALSE;
@@ -65,7 +64,7 @@ int main (int argc, char **argv)
 	double *t_supplied_out = VNULL, *t_out, *ttime, *data, **col = VNULL, **out = VNULL;
 	double tt, low_t, high_t, *in, *dout = VNULL;
 
-	char line[BUFSIZ], type[3];
+	char line[BUFSIZ], type[3], *not_used = NULL;
 
 	FILE *fp = NULL, *fpt = NULL;
 
@@ -86,9 +85,11 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'H':
+				case 'M':
 				case 'V':
 				case 'b':
 				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
@@ -99,19 +100,15 @@ int main (int argc, char **argv)
 					Ctrl->F.active = TRUE;
 					switch (argv[i][2]) {
 						case 'l':
-						case 'L':
 							Ctrl->F.mode = 0;
 							break;
 						case 'a':
-						case 'A':
 							Ctrl->F.mode = 1;
 							break;
 						case 'c':
-						case 'C':
 							Ctrl->F.mode = 2;
 							break;
 						case 'n':
-						case 'N':
 							Ctrl->F.mode = 3;
 							break;
 						default:	/* Use GMT defaults */
@@ -126,9 +123,6 @@ int main (int argc, char **argv)
 					Ctrl->T.active = TRUE;
 					Ctrl->T.col = atoi (&argv[i][2]);
 					break;
-				case 'M':               /* Multiple line segments */
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':
 					Ctrl->N.file = strdup (&argv[i][2]);
 					Ctrl->N.active = TRUE;
@@ -166,8 +160,8 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
 		fprintf (stderr,"sample1d %s - Resampling of 1-D data sets\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: sample1d <infile(s)> [-Fl|a|c|n] [%s] [-I<t_inc>] [%s] [-N<knotfile>]\n", GMT_H_OPT, GMT_M_OPT);
-		fprintf (stderr, "\t[-S<xstart>] [-T<time_col>] [-V] [%s] [%s]\n\n", GMT_b_OPT, GMT_f_OPT);
+		fprintf (stderr, "usage: sample1d <infile(s)> [-Fl|a|c|n] [%s] [-I<t_inc>] [-N<knotfile>]\n", GMT_H_OPT);
+		fprintf (stderr, "\t[-S<xstart>] [-T<time_col>] [-V] [%s] [%s] [%s]\n\n", GMT_b_OPT, GMT_f_OPT, GMT_m_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -182,7 +176,6 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   [Default is -F%c]\n", type[Ctrl->F.mode]);
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-I <x_inc> sets equidistant grid interval [x1 - x0]\n");
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N <knotfile> is an ASCII table with the desired time positions in column 0\n");
 		fprintf (stderr, "\t   Overrides the -I and -S settings.  If none of -I, -S, and -N is set\n");
 		fprintf (stderr, "\t   then <tstart> = first input point, <t_inc> = (t[1] - t[0])\n");
@@ -195,6 +188,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -203,8 +197,6 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -T option: Column number cannot be negative\n", GMT_program);
 		error++;
 	}
-	GMT_io.skip_if_NaN[0] = GMT_io.skip_if_NaN[1] = FALSE;	/* Turn off default GMT NaN-handling */
-	GMT_io.skip_if_NaN[Ctrl->T.col] = TRUE;				/* ... But disallow NaN in "time" column */
 	if (Ctrl->N.active && Ctrl->I.active) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR: Specify only one of -N and -S\n", GMT_program);
 		error++;
@@ -220,7 +212,7 @@ int main (int argc, char **argv)
         if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 2;
 	n_req = (Ctrl->T.col >= 2) ? Ctrl->T.col + 1 : 2;
         if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < n_req) {
-                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %d columns\n", GMT_program, n_req);
+                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %ld columns\n", GMT_program, n_req);
 		error++;
 	}
 	if (Ctrl->N.active && (fpt = GMT_fopen (Ctrl->N.file, "r")) == NULL) {
@@ -232,7 +224,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -240,6 +232,8 @@ int main (int argc, char **argv)
 #endif
 
 	gmtdefs.interpolant = Ctrl->F.mode;
+	GMT_io.skip_if_NaN[GMT_X] = GMT_io.skip_if_NaN[GMT_Y] = FALSE;	/* Turn off default GMT NaN-handling for (x,y) which is not the case here */
+	GMT_io.skip_if_NaN[Ctrl->T.col] = TRUE;				/* ... But disallow NaN in "time" column */
 	
 	t_out = (double *)NULL;
 
@@ -247,7 +241,7 @@ int main (int argc, char **argv)
 		m_alloc = GMT_CHUNK;
 		t_supplied_out = (double *) GMT_memory (VNULL, (size_t)m_alloc, sizeof (double), GMT_program);
 		m = 0;
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fpt);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fpt);
 		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
 		n_fields = GMT_input (fpt, &n_expected_fields, &in);
 		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
@@ -255,7 +249,7 @@ int main (int argc, char **argv)
 				n_fields = GMT_input (fpt, &n_expected_fields, &in);
 			}
 			if (GMT_io.status & GMT_IO_EOF) continue;	/* At EOF */
-			t_supplied_out[m] = in[0];
+			t_supplied_out[m] = in[GMT_X];
 			m++;
 			if (m == m_alloc) {	/* Get more memory */
 				m_alloc <<= 1;
@@ -296,7 +290,7 @@ int main (int argc, char **argv)
 
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
-				GMT_fgets (line, BUFSIZ, fp);
+				not_used = GMT_fgets (line, BUFSIZ, fp);
 				if (GMT_io.io_header[GMT_OUT]) fprintf (GMT_stdout, "%s", line);
 			}
 		}
@@ -318,14 +312,14 @@ int main (int argc, char **argv)
 
 			while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, i);
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, i);
 					exit (EXIT_FAILURE);
 				}
 
 				if (n_col == 0) {	/* Allocate memory first time around */
 					n_col = n_expected_fields;
 					if (Ctrl->T.col >= n_col) {
-						fprintf (stderr, "%s: time_col = %d exceeds range of columns (%d)!\n", GMT_program, Ctrl->T.col, n_col);
+						fprintf (stderr, "%s: time_col = %ld exceeds range of columns (%ld)!\n", GMT_program, Ctrl->T.col, n_col);
 						exit (EXIT_FAILURE);
 					}
 					dout = (double *) GMT_memory (VNULL, (size_t)n_col, sizeof (double), GMT_program);
@@ -337,12 +331,8 @@ int main (int argc, char **argv)
 				}
 
 				for (j = 0; j < n_col; j++) {
-					if (GMT_is_dnan (in[j])) {
-						nan_flag[j] = TRUE;
-						col[j][n] = GMT_d_NaN;
-					}
-					else
-						col[j][n] = in[j];
+					if (GMT_is_dnan (in[j])) nan_flag[j] = TRUE;
+					col[j][n] = in[j];
 				}
 				n++;
 
@@ -413,7 +403,7 @@ int main (int argc, char **argv)
 
 				out[j] = (double *) GMT_memory (VNULL, (size_t)m, sizeof (double), GMT_program);
 
-				if (nan_flag[j]) {	/* NaN's present, need "clean" time and data columns */
+				if (nan_flag[j] && !gmtdefs.nan_is_gap) {	/* NaN's present, need "clean" time and data columns */
 
 					ttime = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
 					data = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
