@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: splitxyz.c,v 1.44 2009/01/09 04:02:34 guru Exp $
+ *	$Id: splitxyz.c,v 1.50 2009/09/09 23:27:04 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -79,21 +79,21 @@ struct  DATA    {
 int main(int argc, char **argv)
 {
 	GMT_LONG     i, ndata, n_alloc, nprofiles, begin, end;
-	int	j, k, n_required_fields, output_choice[SPLITXYZ_N_OUTPUT_CHOICES], n_expected_fields, n_outputs = 0, n_fields;
+	GMT_LONG	j, k, n_required_fields, output_choice[SPLITXYZ_N_OUTPUT_CHOICES], n_expected_fields, n_outputs = 0, n_fields;
 
 	BOOLEAN	error = FALSE, ok, hilow = FALSE, z_selected = FALSE;
 
 	double	dy, dx, last_d, last_c, last_s, csum, ssum, this_c, this_s, dotprod;
 	double	mean_azim, fwork[SPLITXYZ_F_RES], *in, out[SPLITXYZ_N_OUTPUT_CHOICES];
 
- 	char    buffer[BUFSIZ], filename[BUFSIZ];
+ 	char    buffer[BUFSIZ], filename[BUFSIZ], *not_used = NULL;
 
 	struct  DATA *data;
 	struct SPLITXYZ_CTRL *Ctrl;
 
 	FILE    *fp = NULL, *fpout;
 
-	void	filterz(struct DATA *data, GMT_LONG ndata, double z_filter, double *fwork, int hilow), filterxy_setup(double *fwork), filterxy(struct DATA *data, GMT_LONG begin, GMT_LONG end, double xy_filter, double *fwork, int hilow);
+	void	filterz(struct DATA *data, GMT_LONG ndata, double z_filter, double *fwork, GMT_LONG hilow), filterxy_setup(double *fwork), filterxy(struct DATA *data, GMT_LONG begin, GMT_LONG end, double xy_filter, double *fwork, GMT_LONG hilow);
 	void *New_Splitxyz_Ctrl (), Free_Splitxyz_Ctrl (struct SPLITXYZ_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -101,7 +101,7 @@ int main(int argc, char **argv)
 	Ctrl = (struct SPLITXYZ_CTRL *)New_Splitxyz_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	fpout = GMT_stdout;
-	memset ((void *)output_choice, 0, SPLITXYZ_N_OUTPUT_CHOICES * sizeof (int));
+	memset ((void *)output_choice, 0, SPLITXYZ_N_OUTPUT_CHOICES * sizeof (GMT_LONG));
 
         for (i = 1; i < argc; i++) {
                 if (argv[i][0] == '-') {
@@ -283,7 +283,7 @@ int main(int argc, char **argv)
 	else
 		n_expected_fields = n_required_fields;
 	if (GMT_io.binary[GMT_IN] && n_expected_fields < n_required_fields) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %d columns\n", GMT_program, n_required_fields);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %ld columns\n", GMT_program, n_required_fields);
 		error++;
 	}
 
@@ -291,7 +291,7 @@ int main(int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -328,8 +328,8 @@ int main(int argc, char **argv)
 	}
 	if (n_outputs == 0) {	/* Generate default -Q setting (all) */
 		n_outputs = 5 - Ctrl->Z.active;
-		for (i = 0; i < 2; i++) output_choice[i] = i;
-		for (i = 2; i < n_outputs; i++) output_choice[i] = i + Ctrl->Z.active;
+		for (i = 0; i < 2; i++) output_choice[i] = (int)i;
+		for (i = 2; i < n_outputs; i++) output_choice[i] = (int)i + Ctrl->Z.active;
 
 	}
 
@@ -350,12 +350,12 @@ int main(int argc, char **argv)
 
         data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 
-	if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
+	if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (buffer, BUFSIZ, fp);
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, i);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, i);
 			exit (EXIT_FAILURE);
 
 		}
@@ -365,15 +365,15 @@ int main(int argc, char **argv)
                         data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
                 }
 		if (!Ctrl->S.active) {
-			data[ndata].val[0] = in[0];
-			data[ndata].val[1] = in[1];
-			if (!Ctrl->Z.active) data[ndata].val[2] = in[2];
+			data[ndata].val[0] = in[GMT_X];
+			data[ndata].val[1] = in[GMT_Y];
+			if (!Ctrl->Z.active) data[ndata].val[2] = in[GMT_Z];
 			if (ndata) {
 				dy = (data[ndata].val[1] - data[ndata-1].val[1]);
 				dx = (data[ndata].val[0] - data[ndata-1].val[0]);
 				if (Ctrl->M.active) {
 					dy *= project_info.KM_PR_DEG;
-					dx *= (project_info.KM_PR_DEG * cos (0.5 * (data[ndata].val[1] + data[ndata-1].val[1]) * D2R) );
+					dx *= (project_info.KM_PR_DEG * cosd (0.5 * (data[ndata].val[1] + data[ndata-1].val[1])));
 				}
 				if (dy == 0.0 && dx == 0.0) {
 					data[ndata].val[3] = data[ndata-1].val[3];
@@ -390,9 +390,9 @@ int main(int argc, char **argv)
 			ndata++;
 		}
 		else {
-			data[ndata].val[0] = in[0];
-			data[ndata].val[1] = in[1];
-			data[ndata].val[2] = in[2];
+			data[ndata].val[0] = in[GMT_X];
+			data[ndata].val[1] = in[GMT_Y];
+			data[ndata].val[2] = in[GMT_Z];
 			data[ndata].val[3] = in[3];
 			data[ndata].val[4] = D2R * (90.0 - in[4]);	/* Angles are stored as CCW angles in radians */
 			ndata++;
@@ -525,7 +525,7 @@ int main(int argc, char **argv)
 }
 
 
-void filterz (struct DATA *data, GMT_LONG ndata, double z_filter, double *fwork, int hilow)
+void filterz (struct DATA *data, GMT_LONG ndata, double z_filter, double *fwork, GMT_LONG hilow)
 {
 
 	double	half_width, sum, dt, tmp;
@@ -584,7 +584,7 @@ void filterxy_setup (double *fwork)
 	return;
 }
 
-void filterxy (struct DATA *data, GMT_LONG begin, GMT_LONG end, double xy_filter, double *fwork, int hilow)
+void filterxy (struct DATA *data, GMT_LONG begin, GMT_LONG end, double xy_filter, double *fwork, GMT_LONG hilow)
 {
 	GMT_LONG	i, j, k, istart, istop;
 	double	half_width, dt, sum;
