@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: backtracker.c,v 1.41 2009/01/09 04:02:35 guru Exp $
+ *	$Id: backtracker.c,v 1.47 2009/09/12 23:52:00 guru Exp $
  *
  *   Copyright (c) 1999-2009 by P. Wessel
  *
@@ -64,21 +64,20 @@
 
 int main (int argc, char **argv)
 {
-	struct EULER *p;		/* Pointer to array of stage poles */
+	struct EULER *p;			/* Pointer to array of stage poles */
 
 	GMT_LONG n_points;			/* Number of data points read */
 	GMT_LONG n_chunk;			/* Total length or array returned by libeuler functions */
 	GMT_LONG n_track;			/* Number of points in a track segment */
 	GMT_LONG n_stages = 0;			/* Number of stage poles */
 	GMT_LONG n_segments;			/* Number of path segments written out */
-	GMT_LONG n_skipped = 0;		/* Number of points skipped because t < 0 */
-	int track_limit = 0;		/* Flag for limiting the output track extent [no limit] */
-	int n_args;
-	int n_files = 0, fno;
-	int n_fields, n_expected_fields;
+	GMT_LONG n_skipped = 0;			/* Number of points skipped because t < 0 */
+	GMT_LONG track_limit = 0;		/* Flag for limiting the output track extent [no limit] */
+	GMT_LONG n_args;
+	GMT_LONG n_files = 0, fno;
+	GMT_LONG n_fields, n_expected_fields;
 	GMT_LONG n_read = 0;
-	int n_out;
-
+	GMT_LONG n_out;
 	GMT_LONG i, j, k;			/* Misc. counters */
 
 	BOOLEAN error = FALSE;		/* Set to TRUE if arguments are inconsistent */
@@ -117,6 +116,7 @@ int main (int argc, char **argv)
 	char dir[8];			/* From or To */
 	char conf_flag = 0;		/* t for time, a for angle, blank for nothing */
 	char txt_a[GMT_TEXT_LEN], txt_b[GMT_TEXT_LEN];
+	char *not_used = NULL;
 
 	FILE *fp = NULL;		/* File pointer for input data */
 	FILE *fpo = NULL;		/* File pointer for output data */
@@ -141,8 +141,10 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'H':
+				case 'M':
 				case 'V':
 				case ':':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], NULL, NULL, NULL, NULL);
 					break;
@@ -216,10 +218,6 @@ int main (int argc, char **argv)
 					error += GMT_verify_expectations (GMT_io.in_col_type[1], GMT_scanf_arg (txt_b, GMT_io.in_col_type[1], &rot_lat), txt_b);
 					break;
 
-				case 'M':               /* Multiple line segments */
-					GMT_multisegment (&argv[i][2]);
-					break;
-
 				case 'Q':	/* Fixed age for all points */
 					t_fix = atof (&argv[i][2]);
 					fix_all = TRUE;
@@ -262,7 +260,7 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "%s %s - Forward and backward flowlines and hotspot tracks\n\n", GMT_program, SPOTTER_VERSION);
 		fprintf (stderr, "usage: %s [infile(s)] -E<euler.d> OR -eplon/plat/prot [-A[young/old]] [-C] [-Df|b] [%s] [-Lf|b<d_km>]\n", GMT_program, GMT_H_OPT);
-		fprintf (stderr, "\t[%s] [-N<upper_age>] [-Q<t_fix>] [-S<stem>] [-T<t_zero>] [-V] [-W] [%s] [%s]\n\n", GMT_M_OPT, GMT_t_OPT, GMT_b_OPT);
+		fprintf (stderr, "\t[-N<upper_age>] [-Q<t_fix>] [-S<stem>] [-T<t_zero>] [-V] [-W] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_b_OPT, GMT_m_OPT);
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\tinfiles (in ASCII or binary) has 3 or more columns.  If no file(s) is given, standard input is read.\n");
@@ -281,7 +279,6 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-Lf Compute flowline for seamounts of unknown but maximum age [Default projects single points]\n");
 		fprintf (stderr, "\t    If no <d_km> is given, the start/stop points for each stage are returned\n");
 		fprintf (stderr, "\t    If B and F is used instead, stage id is returned as z-value [Default is predicted ages]\n");
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N extends earliest stage pole back to <upper_age> [no extension]\n");
 		fprintf (stderr, "\t-Q Assigned a fixed age to all input points.\n");
 		fprintf (stderr, "\t-S writes tracks to individual files <stem>.# instead of to stdout (requires -L)\n");
@@ -299,7 +296,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Default is 3 input columns\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('.');
-		fprintf (stderr, "\t   Output produced by -Lf|t can be plotted with psxy using the -M option\n");
+		fprintf (stderr, "\t   Output produced by -Lf|t can be plotted with psxy using the -m option\n");
+		GMT_explain_option ('m');
 		exit (EXIT_FAILURE);
 	}
 
@@ -321,7 +319,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (single_rotation) {	/* Get rotation matrix R */
@@ -377,7 +375,7 @@ int main (int argc, char **argv)
 
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
-				GMT_fgets (buffer, BUFSIZ, fp);
+				not_used = GMT_fgets (buffer, BUFSIZ, fp);
 				if (first && !GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) fprintf (GMT_stdout, "%s", buffer);
 			}
 			first = FALSE;
@@ -393,15 +391,15 @@ int main (int argc, char **argv)
 			if (GMT_io.status & GMT_IO_EOF) continue;
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 				continue;
 			}
 			
 			if (!out) out = (double *) GMT_memory (VNULL, (size_t)MAX(n_out, n_expected_fields), sizeof (double), GMT_program);
 
 			if (single_rotation) {	/* Simple reconstruction, then exit */
-				GMT_geo_to_cart (&in[GMT_Y], &in[GMT_X], x, TRUE);	/* Get x-vector */
-				spotter_matrix_vect_mult (R, x, y);				/* Rotate the x-vector */
+				GMT_geo_to_cart (in[GMT_Y], in[GMT_X], x, TRUE);	/* Get x-vector */
+				spotter_matrix_vect_mult (R, x, y);			/* Rotate the x-vector */
 				GMT_cart_to_geo (&out[GMT_Y], &out[GMT_X], y, TRUE);	/* Recover lon lat representation; TRUE to get degrees */
 				memcpy ((void *)&out[GMT_Z], (void *)&in[GMT_Z], (n_fields - 2) * sizeof (double));
 				GMT_output (fpo, n_fields, out);
