@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: hotspotter.c,v 1.41 2009/01/15 01:22:55 guru Exp $
+ *	$Id: hotspotter.c,v 1.44 2009/09/12 23:52:00 guru Exp $
  *
  *   Copyright (c) 1999-2009 by P. Wessel
  *
@@ -114,21 +114,19 @@ int main (int argc, char **argv)
 	GMT_LONG n_stages;		/* Number of stage rotations (poles) */
 	GMT_LONG n_chunk;		/* Number of path values returned by libspotter functions */
 	GMT_LONG n_track;		/* Number of points along a single flowline */
-	GMT_LONG node_x_width;	/* Number of x-nodes covered by the seamount in question (y-dependent) */
-	GMT_LONG node_y_width;	/* Number of y-nodes covered by the seamount */
+	GMT_LONG node_x_width;		/* Number of x-nodes covered by the seamount in question (y-dependent) */
+	GMT_LONG node_y_width;		/* Number of y-nodes covered by the seamount */
 	GMT_LONG n_args;		/* Command line argument counter */
-	int n_files = 0;	/* Number of input files on the command line */
-	GMT_LONG node;		/* The current node index */
-	int one_or_zero;	/* Will be 1 or 0 depending on grid format (-F) */
-	int n_fields, n_expected_fields;
+	GMT_LONG n_files = 0;		/* Number of input files on the command line */
+	GMT_LONG node;			/* The current node index */
+	GMT_LONG one_or_zero;		/* Will be 1 or 0 depending on grid format (-F) */
+	GMT_LONG n_fields, n_expected_fields;
 	GMT_LONG n_read = 0;		/* Number of records read */
-
 	GMT_LONG n_nodes;		/* Number of nodes in the output CVA grid */
 
 	/* Misc. counters */
 
-	GMT_LONG i, j, kx, ky, m, ii, jj, i0, j0, k0;
-	int dummy[4], fno;
+	GMT_LONG i, j, kx, ky, m, ii, jj, i0, j0, k0, dummy[4], fno;
 
 	BOOLEAN	error = FALSE;		/* TRUE when arguments are wrong */
 	BOOLEAN nofile = TRUE;		/* TRUE when data are to be read from stdin */
@@ -166,6 +164,7 @@ int main (int argc, char **argv)
 	char *euler_file = CNULL;	/* Input name of file with stage (Euler) rotations */
 	char line[BUFSIZ];		/* Read buffer */
 	char *processed_node;		/* Pointer to array with TRUE/FALSE values for each grid node */
+	char *not_used = NULL;
 
 	FILE *fp;			/* Input data file pointer */
 
@@ -363,7 +362,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (n_files > 0)
@@ -392,7 +391,7 @@ int main (int argc, char **argv)
 
 		if (gmtdefs.verbose) (nofile) ? fprintf (stderr, "%s: Reading stdin\n", GMT_program) : fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 			n_read++;
@@ -404,7 +403,7 @@ int main (int argc, char **argv)
 			if (GMT_io.status & GMT_IO_EOF) continue;
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 				continue;
 			}
 
