@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: libspotter.c,v 1.44 2008/05/12 22:35:47 guru Exp $
+ *	$Id: libspotter.c,v 1.58 2009/09/05 01:14:03 guru Exp $
  *
- *   Copyright (c) 1999-2008 by P. Wessel
+ *   Copyright (c) 1999-2009 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -50,14 +50,15 @@
 /* Internal functions */
 
 void matrix_to_pole (double T[3][3], double *plon, double *plat, double *w);
-void matrix_transpose (double At[3][3], double A[3][3]);
-void matrix_mult (double a[3][3], double b[3][3], double c[3][3]);
 void make_rot_matrix_sub (double E[3], double w, double R[3][3]);
 void reverse_rotation_order (struct EULER *p, GMT_LONG n);
 void xyw_to_struct_euler (struct EULER *p, double lon[], double lat[], double w[], GMT_LONG n, BOOLEAN stages, BOOLEAN convert);
 void set_I_matrix (double R[3][3]);
 BOOLEAN must_do_track (GMT_LONG sideA[], GMT_LONG sideB[]);
 void set_inout_sides (double x, double y, double wesn[], GMT_LONG sideXY[2]);
+void record_to_covar (struct EULER *e, double K[8]);
+void covar_of_transpose (double A[3][3], double C[3][3], double Ct[3][3]);
+void spotter_set_M (double lon, double lat, double M[3][3]);
 
 void spotter_finite_to_fwstages (struct EULER p[], GMT_LONG n, BOOLEAN finite_rates, BOOLEAN stage_rates);
 
@@ -72,8 +73,10 @@ int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in,
 	FILE *fp;
 	struct EULER *e;
 	char  buffer[BUFSIZ];
-	GMT_LONG n, nf, i = 0, j, k, n_alloc = GMT_SMALL_CHUNK;
-	double last_t, K[8];
+	GMT_LONG n, nf, i = 0, k, n_alloc = GMT_SMALL_CHUNK;
+	double K[9];
+	int spotter_comp_stage (const void *p_1, const void *p_2);
+	int spotter_comp_finite (const void *p_1, const void *p_2);
 
 	e = (struct EULER *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct EULER), "libspotter");
 
@@ -84,28 +87,25 @@ int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in,
 
 	if (flowline) finite_out = TRUE;	/* Override so we get finite poles for conversion to forward stage poles at the end */
 
-	last_t = (finite_in) ? 0.0 : DBL_MAX;
 	while (GMT_fgets (buffer, 512, fp) != NULL) { /* Expects lon lat t0 t1 ccw-angle */
 		if (buffer[0] == '#' || buffer[0] == '\n') continue;
 
-		if (finite_in) {	/* The record formats is: lon lat t0 [t1] omega [covar] */
-			nf = sscanf (buffer, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",
-				&e[i].lon, &e[i].lat, &e[i].t_start, &e[i].t_stop, &e[i].omega, &K[0], &K[1], &K[2], &K[3], &K[4], &K[5], &K[6], &K[7]);
-			if (nf == 4 || 12) {	/* Got lon lat t0 omega [covars], must shift the K's by one */
-				for (k = 7; k > 0; k--) K[k] = K[k-1];
+		if (finite_in) {	/* The minimalist record formats is: lon lat t0 [t1] omega [covar] */
+			nf = sscanf (buffer, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",
+				&e[i].lon, &e[i].lat, &e[i].t_start, &e[i].t_stop, &e[i].omega, &K[0], &K[1], &K[2], &K[3], &K[4], &K[5], &K[6], &K[7], &K[8]);
+			if (! (nf == 4 || nf == 5 || nf == 13 || nf == 14)) {
+				fprintf (stderr, "libspotter: ERROR: Rotation file format must be lon lat t0 [t1] omega [k_hat a b c d e f g df]\n");
+				GMT_exit (EXIT_FAILURE);
+			}
+			if (nf == 4 || nf == 13) {	/* Got lon lat t0 omega [covars], must shift the K's by one */
+				for (k = 8; k > 0; k--) K[k] = K[k-1];
 				K[0] = e[i].omega;
 				e[i].omega = e[i].t_stop;
 				e[i].t_stop = 0.0;
 			}
-			if (nf > 5) { /* [K = covars] is stored as [k_hat a b c d e f g] */
-				e[i].k_hat   = K[0];
-				e[i].C[0][0] = K[1];
-				e[i].C[0][1] = e[i].C[1][0] = K[2];
-				e[i].C[0][2] = e[i].C[2][0] = K[4];
-				e[i].C[1][1] = K[3];
-				e[i].C[1][2] = e[i].C[2][1] = K[5];
-				e[i].C[2][2] = K[6];
-				for (k = 0; k < 3; k++) for (j = 0; j < 3; j++) e[i].C[k][j] *= (K[7] / K[0]);
+			if (nf > 5) { /* [K = covars] is stored as [k_hat a b c d e f g df] */
+				if (K[8] == 0.0) K[8] = 10000.0;	/* No d.f. given */
+				record_to_covar (&e[i], K);
 			}
 		}
 		else {	/* Stage rotations */
@@ -118,20 +118,6 @@ int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in,
 			GMT_exit (EXIT_FAILURE);
 		}
 		e[i].duration = e[i].t_start - e[i].t_stop;
-		if (finite_in) {
-			if (e[i].t_start < last_t) {
-				fprintf (stderr, "libspotter: ERROR: Finite rotations must go from youngest to oldest\n");
-				GMT_exit (EXIT_FAILURE);
-			}
-			last_t = e[i].t_start;
-		}
-		else {
-			if (e[i].t_stop > last_t) {
-				fprintf (stderr, "libspotter: ERROR: Stage rotations must go from oldest to youngest\n");
-				GMT_exit (EXIT_FAILURE);
-			}
-			last_t = e[i].t_stop;
-		}
 		e[i].omega /= e[i].duration;	/* Convert to opening rate */
 
 		e[i].omega_r = e[i].omega * D2R;
@@ -146,11 +132,16 @@ int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in,
 	}
 	GMT_fclose (fp);
 
+	/* Sort the rotations to make sure they are in the expected order */
+	
 	n = i;
-
-	if (finite_in && !finite_out) spotter_finite_to_stages (e, n, TRUE, TRUE);	/* Convert finite poles to backward stage poles */
-	if (!finite_in && finite_out) spotter_stages_to_finite (e, n, TRUE, TRUE);	/* Convert backward stage poles to finite poles */
-
+	if (finite_in)
+		qsort ((void *)e, (size_t)n, sizeof (struct EULER), spotter_comp_finite);
+	else
+		qsort ((void *)e, (size_t)n, sizeof (struct EULER), spotter_comp_stage);
+	
+	if (finite_in && !finite_out) spotter_finite_to_stages (e, n, TRUE, TRUE);	/* Convert total reconstruction poles to forward stage poles */
+	if (!finite_in && finite_out) spotter_stages_to_finite (e, n, TRUE, TRUE);	/* Convert forward stage poles to total reconstruction poles */
 
 	e = (struct EULER *) GMT_memory ((void *)e, (size_t)n, sizeof (struct EULER), "libspotter");
 
@@ -173,6 +164,82 @@ int spotter_init (char *file, struct EULER **p, int flowline, BOOLEAN finite_in,
 	return (n);
 }
 
+/* Sort functions used to order the rotations */
+
+int spotter_comp_stage (const void *p_1, const void *p_2)
+{
+	/* Returns -1 if rotation pointed to by p_1 is older that point_2,
+	   +1 if the reverse it true, and 0 if they are equal
+	*/
+	struct EULER *point_1, *point_2;
+
+	point_1 = (struct EULER *)p_1;
+	point_2 = (struct EULER *)p_2;
+
+	if (point_1->t_start > point_2->t_start) return (-1);
+	if (point_1->t_start < point_2->t_start) return (1);
+	return (0);
+}
+
+int spotter_comp_finite (const void *p_1, const void *p_2)
+{
+	/* Returns -1 if rotation pointed to by p_1 is older that point_2,
+	   +1 if the reverse it true, and 0 if they are equal
+	*/
+	struct EULER *point_1, *point_2;
+
+	point_1 = (struct EULER *)p_1;
+	point_2 = (struct EULER *)p_2;
+
+	if (point_1->t_start < point_2->t_start) return (-1);
+	if (point_1->t_start > point_2->t_start) return (1);
+	return (0);
+}
+/* hotspot_init: Reads a file with hotspot information and returns pointer to
+ * array of structures */
+
+int spotter_hotspot_init (char *file, struct HOTSPOT **p)
+{
+	FILE *fp;
+	struct HOTSPOT *e;
+	char buffer[BUFSIZ], create, fit, plot;
+	int i = 0, n;
+	size_t n_alloc = GMT_CHUNK;
+	double P[3];
+
+	if ((fp = GMT_fopen (file, "r")) == NULL) {
+		fprintf (stderr, "%s: Cannot open file %s - aborts\n", "libspotter", file);
+		exit (EXIT_FAILURE);
+	}
+
+	e = (struct HOTSPOT *) GMT_memory (VNULL, n_alloc, sizeof (struct HOTSPOT), "libspotter");
+
+	while (GMT_fgets (buffer, 512, fp) != NULL) {
+		if (buffer[0] == '#' || buffer[0] == '\n') continue;
+		n = sscanf (buffer, "%lf %lf %s %d %lf %lf %lf %c %c %c %s", &e[i].lon, &e[i].lat, e[i].abbrev, &e[i].id, &e[i].radius, &e[i].t_off, &e[i].t_on, &create, &fit, &plot, e[i].name);
+		if (n == 3) e[i].id = i;	/* Minimal lon, lat, abbrev */
+		if (n >= 10) {
+			e[i].create = (create == 'Y');
+			e[i].fit = (fit == 'Y');
+			e[i].plot = (plot == 'Y');
+		}
+		GMT_geo_to_cart (e[i].lat, e[i].lon, P, TRUE);
+		e[i].x = P[0];
+		e[i].y = P[1];
+		e[i].z = P[2];
+		i++;
+		if ((size_t)i == n_alloc) {
+			n_alloc <<= 1;
+			e = (struct HOTSPOT *) GMT_memory ((void *)e, n_alloc, sizeof (struct HOTSPOT), "libspotter");
+		}
+	}
+	GMT_fclose (fp);
+	e = (struct HOTSPOT *) GMT_memory ((void *)e, (size_t)i, sizeof (struct HOTSPOT), "libspotter");
+	*p = e;
+
+	return (i);
+}
+
 /* spotter_backtrack: Given a seamount location and age, trace the
  *	hotspot-track between this seamount and a seamount of 
  *	age t_zero.  For t_zero = 0 this means the hotspot
@@ -380,15 +447,10 @@ BOOLEAN must_do_track (GMT_LONG sideA[], GMT_LONG sideB[]) {
 	if (sideA[0] == 0 && sideA[1] == 0) return (TRUE);
 	if (sideB[0] == 0 && sideB[1] == 0) return (TRUE);
 	/* Now check if the two points may cut a corner */
-#ifdef __LP64__
-	dx = labs (sideA[0] - sideB[0]);
-	dy = labs (sideA[1] - sideB[1]);
-#else
-	dx = abs (sideA[0] - sideB[0]);
-	dy = abs (sideA[1] - sideB[1]);
-#endif
+	dx = GMT_abs (sideA[0] - sideB[0]);
+	dy = GMT_abs (sideA[1] - sideB[1]);
 	if (dx && dy) return (TRUE);
-	if (dx == 2 || dy == 2) return (TRUE);	/* COuld cut across the box */
+	if (dx == 2 || dy == 2) return (TRUE);	/* Could cut across the box */
 	return (FALSE);
 }
 
@@ -602,11 +664,11 @@ void spotter_finite_to_fwstages (struct EULER p[], GMT_LONG n, BOOLEAN finite_ra
 	t_old = 0.0;
 	for (i = 0; i < n; i++) {
 		if (finite_rates) p[i].omega *= p[i].duration;			/* Convert opening rate to opening angle */
-		make_rot_matrix (p[i].lon, p[i].lat, -p[i].omega, R_old);	/* Make rotation matrix from rotation parameters, take transpose by passing -omega */
-		matrix_mult (R_young, R_old, R_stage);				/* This is R_stage = R_young * R_old^t */
+		spotter_make_rot_matrix (p[i].lon, p[i].lat, -p[i].omega, R_old);	/* Make rotation matrix from rotation parameters, take transpose by passing -omega */
+		spotter_matrix_mult (R_young, R_old, R_stage);				/* This is R_stage = R_young * R_old^t */
 		matrix_to_pole (R_stage, &elon[i], &elat[i], &ew[i]);		/* Get rotation parameters from matrix */
 		if (elon[i] > 180.0) elon[i] -= 360.0;				/* Adjust lon */
-		matrix_transpose (R_young, R_old);				/* Set R_young = (R_old^t)^t = R_old */
+		spotter_matrix_transpose (R_young, R_old);				/* Set R_young = (R_old^t)^t = R_old */
 		p[i].t_stop = t_old;
 		t_old = p[i].t_start;
 	}
@@ -648,11 +710,11 @@ void spotter_finite_to_stages (struct EULER p[], GMT_LONG n, BOOLEAN finite_rate
 	t_old = 0.0;
 	for (i = 0; i < n; i++) {
 		if (finite_rates) p[i].omega *= p[i].duration;			/* Convert opening rate to opening angle */
-		make_rot_matrix (p[i].lon, p[i].lat, p[i].omega, R_old);	/* Get rotation matrix from pole and angle */
-		matrix_mult (R_young, R_old, R_stage);				/* This is R_stage = R_young^t * R_old */
+		spotter_make_rot_matrix (p[i].lon, p[i].lat, p[i].omega, R_old);	/* Get rotation matrix from pole and angle */
+		spotter_matrix_mult (R_young, R_old, R_stage);				/* This is R_stage = R_young^t * R_old */
 		matrix_to_pole (R_stage, &elon[i], &elat[i], &ew[i]);		/* Get rotation parameters from matrix */
 		if (elon[i] > 180.0) elon[i] -= 360.0;				/* Adjust lon */
-		matrix_transpose (R_young, R_old);				/* Sets R_young = transpose (R_old) for next round */
+		spotter_matrix_transpose (R_young, R_old);				/* Sets R_young = transpose (R_old) for next round */
 		p[i].t_stop = t_old;
 		t_old = p[i].t_start;
 	}
@@ -693,8 +755,8 @@ void spotter_stages_to_finite (struct EULER p[], GMT_LONG n, BOOLEAN finite_rate
 
 	for (i = 0; i < n; i++) {
 		if (stage_rates) p[i].omega *= p[i].duration;				/* Convert opening rate to opening angle */
-		make_rot_matrix (p[i].lon, p[i].lat, p[i].omega, R_stage);		/* Make matrix from rotation parameters */
-		matrix_mult (R_old, R_stage, R_young);					/* Set R_young = R_old * R_stage */
+		spotter_make_rot_matrix (p[i].lon, p[i].lat, p[i].omega, R_stage);		/* Make matrix from rotation parameters */
+		spotter_matrix_mult (R_old, R_stage, R_young);					/* Set R_young = R_old * R_stage */
 		memcpy ((void *)R_old, (void *)R_young, (size_t)(9 * sizeof (double)));	/* Set R_old = R_young for next time around */
 		matrix_to_pole (R_young, &elon[i], &elat[i], &ew[i]);			/* Get rotation parameters from matrix */
 		if (elon[i] > 180.0) elon[i] -= 360.0;					/* Adjust lon */
@@ -724,18 +786,14 @@ void spotter_add_rotations (struct EULER a[], GMT_LONG n_a, struct EULER b[], GM
 
 	struct EULER *a2, *b2, *c2;
 	double *t, t_min, t_max, Ra[3][3], Rb[3][3], Rab[3][3], lon, lat, w, sign_a, sign_b;
+	double tmp[3][3], RaT[3][3], Ca[3][3], Cb[3][3];
 	GMT_LONG i, j, k, n_k = 0;
 	BOOLEAN a_ok = TRUE, b_ok = TRUE;
 
 	sign_a = (n_a > 0) ? +1.0 : -1.0;
 	sign_b = (n_b > 0) ? +1.0 : -1.0;
-#ifdef __LP64__
-	n_a = labs (n_a);
-	n_b = labs (n_b);
-#else
-	n_a = abs (n_a);
-	n_b = abs (n_b);
-#endif
+	n_a = GMT_abs (n_a);
+	n_b = GMT_abs (n_b);
 	/* Allocate more than we need, must likely */
 
 	t = (double *) GMT_memory (VNULL, (size_t)(n_a + n_b), sizeof (double), "libspotter");
@@ -811,9 +869,9 @@ void spotter_add_rotations (struct EULER a[], GMT_LONG n_a, struct EULER b[], GM
 	spotter_stages_to_finite (b2, n_k, FALSE, TRUE);
 
 	for (i = 0; i < n_k; i++) {	/* Add each pair of rotations */
-		make_rot_matrix (a2[i].lon, a2[i].lat, sign_a * a2[i].omega, Ra);
-		make_rot_matrix (b2[i].lon, b2[i].lat, sign_b * b2[i].omega, Rb);
-		matrix_mult (Rb, Ra, Rab);	/* Rot a + Rot b = RB * Ra ! */
+		spotter_make_rot_matrix (a2[i].lon, a2[i].lat, sign_a * a2[i].omega, Ra);
+		spotter_make_rot_matrix (b2[i].lon, b2[i].lat, sign_b * b2[i].omega, Rb);
+		spotter_matrix_mult (Rb, Ra, Rab);	/* Rot a + Rot b = RB * Ra ! */
 		matrix_to_pole (Rab, &lon, &lat, &w);
 		c2[i].lon = lon;
 		c2[i].lat = lat;
@@ -821,6 +879,34 @@ void spotter_add_rotations (struct EULER a[], GMT_LONG n_a, struct EULER b[], GM
 		c2[i].t_stop  = 0.0;
 		c2[i].duration = c2[i].t_start;
 		c2[i].omega = w / c2[i].duration;	/* Return rates again */
+		if (a2[i].has_cov && b2[i].has_cov) {	/* May compute combined covariance matrix, assuming khats = 1 */
+			double fa, fb;
+			c2[i].df = a2[i].df + b2[i].df;
+			if (a2[i].k_hat != 1.0 || b2[i].k_hat != 1.0) {	/* Kappas are not one, use pooled estimate */
+				c2[i].k_hat = (c2[i].df / (a2[i].df / a2[i].k_hat + b2[i].df / b2[i].k_hat));
+				fa = a2[i].k_hat / c2[i].k_hat;
+				fb = b2[i].k_hat / c2[i].k_hat;
+			}
+			else
+				c2[i].k_hat = fa = fb = 1.0;
+
+			spotter_matrix_transpose (RaT, Ra);
+			if (sign_a < 0.0)
+				spotter_cov_of_inverse (&a2[i], Ca);
+			else
+				memcpy ((void *)Ca, (void *)a2[i].C, 9*sizeof (double));
+			if (sign_b < 0.0)
+				spotter_cov_of_inverse (&b2[i], Cb);
+			else
+				memcpy ((void *)Cb, (void *)b2[i].C, 9*sizeof (double));
+			spotter_matrix_mult (Cb, Ra, tmp);
+			spotter_matrix_mult (RaT, tmp, c2[i].C);
+			for (k = 0; k < 3; k++) for (j = 0; j < 3; j++) c2[i].C[k][j] *= fb;
+			for (k = 0; k < 3; k++) for (j = 0; j < 3; j++) tmp[k][j] = fa * Ca[k][j];
+			spotter_matrix_add (c2[i].C, tmp, c2[i].C);
+			c2[i].has_cov = TRUE;
+			c2[i].g = MIN(a2[i].g, b2[i].g);
+		}
 	}
 	GMT_free ((void *)a2);
 	GMT_free ((void *)b2);
@@ -848,7 +934,7 @@ double spotter_t2w (struct EULER a[], GMT_LONG n, double t)
 	return (w);
 }
 
-void make_rot_matrix (double lonp, double latp, double w, double R[3][3])
+void spotter_make_rot_matrix (double lonp, double latp, double w, double R[3][3])
 {
 /*	lonp, latp	Euler pole in degrees
  *	w		angular rotation in degrees
@@ -858,7 +944,7 @@ void make_rot_matrix (double lonp, double latp, double w, double R[3][3])
 
 	double E[3];
 
-        GMT_geo_to_cart (&latp, &lonp, E, TRUE);
+        GMT_geo_to_cart (latp, lonp, E, TRUE);
 	make_rot_matrix_sub (E, w, R);
 }
 
@@ -903,7 +989,7 @@ void make_rot0_matrix (double lonp, double latp, double R[3][3], double E[])
 	 *	R		the rotation matrix without terms depending on omega
 	 */
 
-        GMT_geo_to_cart (&latp, &lonp, E, TRUE);
+        GMT_geo_to_cart (latp, lonp, E, TRUE);
 
 	R[0][0] = E[0] * E[0];
 	R[0][1] = E[0] * E[1];
@@ -942,7 +1028,7 @@ void set_rot_angle (double w, double R[3][3], double E[])
 	R[2][2] = R[2][2] * c + cos_w;
 }
 
-void matrix_mult (double a[3][3], double b[3][3], double c[3][3])
+void spotter_matrix_mult (double a[3][3], double b[3][3], double c[3][3])
 {	/* C = A * B */
 	int i, j, k;
 
@@ -954,14 +1040,14 @@ void matrix_mult (double a[3][3], double b[3][3], double c[3][3])
 	}
 }
 
-void matrix_vect_mult (double a[3][3], double b[3], double c[3])
+void spotter_matrix_vect_mult (double a[3][3], double b[3], double c[3])
 {	/* c = A * b */
 	int i, j;
 
 	for (i = 0; i < 3; i++) for (j = 0, c[i] = 0.0; j < 3; j++) c[i] += a[i][j] * b[j];
 }
 
-void matrix_transpose (double At[3][3], double A[3][3])
+void spotter_matrix_transpose (double At[3][3], double A[3][3])
 {
 	/* Computes the matrix transpose */
 
@@ -973,6 +1059,18 @@ void matrix_transpose (double At[3][3], double A[3][3])
 	}
 }
 
+void spotter_matrix_add (double A[3][3], double B[3][3], double C[3][3])
+{
+	/* Computes the matrix addition */
+
+	int i, j;
+	for (j = 0; j < 3; j++) {
+		for (i = 0; i < 3; i++) {
+			C[i][j] = A[i][j] + B[i][j];
+		}
+	}
+}
+
 void matrix_to_pole (double T[3][3], double *plon, double *plat, double *w)
 {
 	double T13_m_T31, T32_m_T23, T21_m_T12, L, H, tr;
@@ -985,10 +1083,10 @@ void matrix_to_pole (double T[3][3], double *plon, double *plat, double *w)
 	H = sqrt (H);
 	tr = T[0][0] + T[1][1] + T[2][2];
 
-	*plon = atan2 (T13_m_T31, T32_m_T23) * R2D;
+	*plon = atan2d (T13_m_T31, T32_m_T23);
 	if (*plon < 0.0) (*plon) += 360.0;
-	*plat = atan2 (T21_m_T12, H) * R2D;
-	*w = atan2 (L, (tr - 1.0)) * R2D;
+	*plat = atan2d (T21_m_T12, H);
+	*w = atan2d (L, tr - 1.0);
 	if (*plat < 0.0) {	/* Make N hemisphere pole */
 		*plat = -(*plat);
 		*(plon) += 180.0;
@@ -1025,8 +1123,8 @@ void xyw_to_struct_euler (struct EULER *p, double lon[], double lat[], double w[
 		p[i].omega = w[i];
 		if (convert) p[i].omega /= p[i].duration;	/* Convert opening angle to opening rate */
 		p[i].omega_r = p[i].omega * D2R;
-		p[i].sin_lat = sin (p[i].lat * D2R);
-		p[i].cos_lat = cos (p[i].lat * D2R);
+		p[i].sin_lat = sind (p[i].lat);
+		p[i].cos_lat = cosd (p[i].lat);
 		p[i].lon_r = p[i].lon * D2R;
 		p[i].lat_r = p[i].lat * D2R;
 	}
@@ -1039,7 +1137,7 @@ void set_I_matrix (double R[3][3])
 	R[0][0] = R[1][1] = R[2][2] = 1.0;
 }
 
-int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT_LONG np, char flag, double out[])
+int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT_LONG np, char flag, BOOLEAN forward, double out[])
 {
 	/* Given time and rotation parameters, calculate uncertainty in the
 	 * reconstructed point in the form of a confidence ellipse.  To follow
@@ -1049,11 +1147,13 @@ int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT
 	 * context (the error in a reconstructed point along the chain) we are
 	 * actually using the inverse rotation R^t (negative opening angle).  For
 	 * that rotation, the covariance matrix is R * cov(r) * R^t.
+	 * forward is TRUE if we rotate from past to now and FALSE if we
+	 * rotate from now to the past (e.g., move a hotspot up the chain).
 	 */
 
 	GMT_LONG matrix_dim = 3L;
-	int i, j, k, kk = 3, nrots;
-	double R[3][3], x[3], y[3], M[3][3], MtRt[3][3], Rt[3][3], RM[3][3], cov[3][3], tmp[3][3], C[9];
+	GMT_LONG i, j, k, kk = 3, nrots;
+	double R[3][3], x[3], y[3], M[3][3], RMt[3][3], Rt[3][3], MRt[3][3], cov[3][3], tmp[3][3], C[9];
 	double z_unit_vector[3], EigenValue[3], EigenVector[9], work1[3], work2[3], x_in_plane[3], y_in_plane[3];
 	double x_comp, y_comp, w;
 
@@ -1062,40 +1162,39 @@ int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT
 	for (i = 0, k = -1; k < 0 && i < np; i++) if (GMT_IS_ZERO (p[i].t_start - t)) k = i;
 	if (k == -1) return (1);	/* Did not match finite rotation time */
 
-	/* Generate R, the rotation matrix.  This is actually R^t since w is -ve */
-
-	w = p[k].omega * p[k].duration;
-	make_rot_matrix (p[k].lon, p[k].lat, -w, R);
-
 	/* Make M(x), the skew-symmetric matrix needed to compute cov of rotated point */
 
-        GMT_geo_to_cart (&lat, &lon, x, TRUE);
-	M[0][0] = M[1][1] = M[2][2] = 0.0;
-	M[0][1] = -x[2];
-	M[0][2] = x[1];
-	M[1][0] = x[2];
-	M[1][2] = -x[0];
-	M[2][0] = -x[1];
-	M[2][1] = x[0];
+	spotter_set_M (lon, lat, M);
 
-	/* Since we are using the inverse rotation (-ve w) we must rotate the covariance: cov(r^t) = R cov(r) R^t.
-	   Here, R actually contains R^t so we need the original R (which we will call R^t) as well. */
+	w = p[k].omega * p[k].duration;
+	if (forward) w = -w;	/* Want the inverse rotation */
+	spotter_make_rot_matrix (p[k].lon, p[k].lat, w, R);
+	spotter_matrix_transpose (Rt, R);			/* Get the transpose of R^t */
+	if (!forward) {		/* Rotate the point into the present */
+		memcpy ((void *)cov, p[k].C, 9*sizeof (double));	/* The rotation's covarience matrix */
+	}
+	else {	/* Use inverse rotation to rotate the point from the present to past rotations */
+		/* We change the sign of w so then R is actually R^t */
+		/* Since we are using the inverse rotation we must first get the cov matrix of the
+		   inverse rotation: cov(r^t) = R cov(r) R^t.
+		   Here, R actually contains R^t so we need the original R (which we will call R^t) as well. */
 
-	matrix_mult (p[k].C, R, tmp);			/* Calculate the cov(r) *R^t product */
-	matrix_transpose (Rt, R);			/* Get the transpose of R^t which is R*/
-	matrix_mult (Rt, tmp, cov);			/* cov(r^t) = R^t cov(r) R */
+		spotter_matrix_mult (p[k].C, R, tmp);			/* Calculate the cov(r) *R^t product */
+		spotter_matrix_mult (Rt, tmp, cov);			/* cov(r^t) = R^t cov(r) R */
+	}
 
-	/* Calculate cov(y) = (R * M) * cov_R * (R * M)^T */
+	/* Calculate cov(y) = R * M^T * cov_R * M * R^T */
 
-	matrix_mult (R, M, RM);			/* Calculate the R * M product */
-	matrix_transpose (MtRt, RM);		/* Get the transpose (R*M)^T = M^T * R^T */
-	matrix_mult (cov, MtRt, tmp);		/* Get C * M^T * R^T */
-	matrix_mult (RM, tmp, M);		/* Finally get R * M * C * M^T * R^T, store in M */
+	spotter_matrix_mult (M, Rt, MRt);		/* Calculate the M * R^T product */
+	spotter_matrix_transpose (RMt, MRt);		/* Get the transpose (M*R^T)^T = R * M^T */
+	spotter_matrix_mult (cov, MRt, tmp);		/* Get C * M * R^T */
+	spotter_matrix_mult (RMt, tmp, M);		/* Finally get R * M * C * M^T * R^T, store result in M */
 
 	for (i = 0; i < 3; i++) for (j = 0; j < 3; j++) C[3*i+j] = M[i][j];	/* Reformat to 1-D format for GMT_jacobi */
 
 	/* Get projected point y = R*x */
 
+	GMT_geo_to_cart (lat, lon, x, TRUE);
 	for (i = 0; i < 3; i++) y[i] = R[i][0] * x[0] + R[i][1] * x[1] + R[i][2] * x[2];
         GMT_cart_to_geo (&out[1], &out[0], y, TRUE);
 	if (flag == 't')
@@ -1114,7 +1213,7 @@ int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT
 	GMT_cross3v (y, x_in_plane, y_in_plane);	/* Local y-axis in plane normal to mean pole */
 	x_comp = GMT_dot3v (EigenVector, x_in_plane);	/* x-component of major axis in tangent plane */
 	y_comp = GMT_dot3v (EigenVector, y_in_plane);	/* y-component of major axis in tangent plane */
-	out[kk] = fmod (360.0 + (90.0 - atan2 (y_comp, x_comp) * R2D), 360.0);	/* Azimuth of major axis */
+	out[kk] = fmod (360.0 + (90.0 - atan2d (y_comp, x_comp)), 360.0);	/* Azimuth of major axis */
 	if (out[kk] > 180.0) out[kk] -= 180.0;
 	out[++kk] = sqrt (EigenValue[0]) * EQ_RAD * SQRT_CHI2;
 	out[++kk] = sqrt (EigenValue[1]) * EQ_RAD * SQRT_CHI2;
@@ -1122,3 +1221,65 @@ int spotter_conf_ellipse (double lon, double lat, double t, struct EULER *p, GMT
 	return (0);
 }
 
+void spotter_set_M (double lon, double lat, double M[3][3])
+{	/* Just initializes the M(x), the skew-symmetric matrix needed to compute cov of rotated point */
+	double x[3];
+        GMT_geo_to_cart (lat, lon, x, TRUE);	/* Get Cartesian vector for this point */
+	M[0][0] = M[1][1] = M[2][2] = 0.0;
+	M[0][1] = -x[2];
+	M[0][2] = x[1];
+	M[1][0] = x[2];
+	M[1][2] = -x[0];
+	M[2][0] = -x[1];
+	M[2][1] = x[0];
+}
+
+void spotter_covar_to_record (struct EULER *e, double K[])
+{
+	/* Translates an Euler covariance matrix to the 9 values needed for printout
+	 * covariance matrix is stored as [k_hat a b c d e f g df] */
+	
+	int k;
+	K[0] = e->k_hat;
+	K[7] = e->g;
+	K[8] = e->df;
+	K[1] = e->C[0][0];
+	K[2] = e->C[0][1];
+	K[4] = e->C[0][2];
+	K[3] = e->C[1][1];
+	K[5] = e->C[1][2];
+	K[6] = e->C[2][2];
+	for (k = 1; k < 7; k++) K[k] *= (e->k_hat / e->g);
+}
+
+void record_to_covar (struct EULER *e, double K[])
+{
+	/* Translates the 9 values read from plate motion file [k_hat a b c d e f g df]
+	 * into the Euler covariance matrix */
+	
+	int k, j;
+	e->has_cov = TRUE;
+	e->k_hat   = K[0];
+	e->g       = K[7];
+	e->df	   = K[8];
+	e->C[0][0] = K[1];
+	e->C[0][1] = e->C[1][0] = K[2];
+	e->C[0][2] = e->C[2][0] = K[4];
+	e->C[1][1] = K[3];
+	e->C[1][2] = e->C[2][1] = K[5];
+	e->C[2][2] = K[6];
+	for (k = 0; k < 3; k++) for (j = 0; j < 3; j++) e->C[k][j] *= (e->g / e->k_hat);
+}
+
+void spotter_cov_of_inverse (struct EULER *e, double Ct[3][3])
+{	/* If A and cov(u) is a rotation and its covariance matrix and
+	 * let A' and cov(v) be the inverse rotation and its covariacne matrix,
+	 * then cov(v) = A*cov(u)*A' */
+	
+	double A[3][3], At[3][3], tmp[3][3];
+	
+	spotter_make_rot_matrix (e->lon, e->lat, e->omega, A);
+	spotter_matrix_transpose (At, A);	/* Get A' */
+	spotter_matrix_mult (e->C, At, tmp);	/* Calculate the cov(u)*A' product */
+	spotter_matrix_mult (A, tmp, Ct);	/* Calculate the cov(v) = A*cov(u)*A' product */
+}
