@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: originator.c,v 1.42 2008/05/12 22:35:47 guru Exp $
+ *	$Id: originator.c,v 1.52 2009/09/12 23:52:00 guru Exp $
  *
- *   Copyright (c) 2000-2008 by P. Wessel
+ *   Copyright (c) 2000-2009 by P. Wessel
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -90,44 +90,41 @@
 
 #define KM_PR_RAD (R2D * KM_PR_DEG)
 
-struct HOTSPOT {
-	double lon, lat;	/* Current location of hot spot */
+struct HOTSPOT_ORIGINATOR {
+	struct HOTSPOT *h;	/* Pointer to regular HOTSPOT structure */
+	/* Extra variables needed for this program */
 	double np_dist;		/* Distance to nearest point on the current flowline */
 	double np_sign;		/* "Sign" of this distance (see code) */
 	double np_time;		/* Predicted time at nearest point */
 	double np_lon;		/* Longitude of nearest point on the current flowline */
 	double np_lat;		/* Latitude  of nearest point on the current flowline */
 	int nearest;		/* Point id of current flowline node points closest to hotspot */
-	int id;			/* Hot spot id flag */
 	int stage;		/* Stage to which seamount belongs */
-	char name[20];		/* Name of hotspot */
-	char abbrev[4];		/* 3-char abbreviation of hotspot name */
 };
 
 int main (int argc, char **argv)
 {
-	int x, y, n_max_spots, fno, n_files = 0, n_args, n_input = 5;
-	int n_fields, n_expected_fields, n_best_hs = 1, tdz_output = 0, n_out = 0;
+	GMT_LONG x, y, n_max_spots, fno, n_files = 0, n_args, n_input = 5;
+	GMT_LONG n_fields, n_expected_fields, n_best_hs = 1, tdz_output = 0, n_out = 0;
 	GMT_LONG  i, j, k, n, kk, ns, nh, nc, np, n_read, n_skipped = 0;
 
-	BOOLEAN	error = FALSE, done, nofile = TRUE, truncate_ages = FALSE, first = TRUE, better;
+	BOOLEAN	error = FALSE, done, nofile = TRUE, truncate_ages = FALSE, first = TRUE, better, use_ID = FALSE;
 	BOOLEAN finite = FALSE, degree = FALSE;		/* fitite is TRUE if stage pole file contains finite rotation poles instead */
 
-	double x_smt, y_smt, z_smt, r_smt, t_smt, *c, *in, d_km = 5.0, upper_age = 180.0, dist, plon, plat, out[5];
+	double x_smt, y_smt, z_smt, r_smt, t_smt, *c, *in, d_km = 5.0, upper_age = 180.0, dist, out[5];
 	double max_dist = 1.0e100, hx_dist, hx_dist_km, dist_NA, dist_NX, del_dist, dt = 0.0, A[3], H[3], N[3], X[3];
 	double dlon, fix_r, fix_t;
 
-	char *hsfile = CNULL, *efile = CNULL, age[GMT_TEXT_LEN];
+	char *hsfile = CNULL, *efile = CNULL, age[GMT_TEXT_LEN], *not_used = NULL;
 
 	FILE *fp;
 
 	struct EULER *p;
-	struct HOTSPOT *hotspot, *hot;
+	struct HOTSPOT *orig_hotspot;
+	struct HOTSPOT_ORIGINATOR *hotspot, *hot;
 
-	int hotspot_init (char *file, struct HOTSPOT **p);
 	int comp_hs (const void *p1, const void *p2);
 
-
 	argc = GMT_begin (argc, argv);
 
 	GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
@@ -206,6 +203,9 @@ int main (int argc, char **argv)
 				case 'W':
 					max_dist = atof (&argv[i][2]);
 					break;
+				case 'Z':
+					use_ID = TRUE;
+					break;
 				default:
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -222,26 +222,26 @@ int main (int argc, char **argv)
 	}
         if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_input;
         if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < n_input) {
-                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %d columns\n", GMT_program, n_input);
+                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %ld columns\n", GMT_program, n_input);
 		error++;
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "%s %s - Associate seamounts with hotspot point sources\n\n", GMT_program, GMT_VERSION);
 		fprintf (stderr, "usage: %s [<xyfiles>] -E<euler_file> -F<hotspot_file> [-C] [-D<d_km>]\n", GMT_program);
-		fprintf (stderr, "	[-H] [-L[flag]] [-N<upper_age>] [-Qr/t] [-S<n_hs>] [-T] [-V] [-W<maxdist>] [%s]\n\n", GMT_t_OPT);
+		fprintf (stderr, "	[-H] [-L[flag]] [-N<upper_age>] [-Qr/t] [-S<n_hs>] [-T] [-V] [-W<maxdist>] [-Z] [%s]\n\n", GMT_t_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\txyfiles is one or more seamount (x,y,z,r,t) files\n");
-		fprintf (stderr, "\t-E Specify file name for Euler poles.\n");
+		fprintf (stderr, "\t-E specifies the rotations to be used (see man page for format)\n\n");
 		fprintf (stderr, "\t-F Specify file name for hotspot locations.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-C The file given with -E contains finite rotation poles [Default is stage poles]\n");
 		fprintf (stderr, "\t-D set sampling interval in km along tracks [5].\n");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-L Output information for closest approach for nearest hotspot only (ignores -S).\n");
-		fprintf (stderr, "\t   -Lt gives (omega, dist, z) [Default].\n");
+		fprintf (stderr, "\t   -Lt gives (time, dist, z) [Default].\n");
 		fprintf (stderr, "\t   -Lw gives (omega, dist, z).\n");
 		fprintf (stderr, "\t   -Ll gives (lon, lat, time, dist, z).\n");
 		fprintf (stderr, "\t   dist is in km; use upper case T,W,L to get dist in spherical degrees.\n");
@@ -252,6 +252,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t-W Only report seamounts whose closest encounter to a hotspot is less than <maxdist> km\n");
 		fprintf (stderr, "\t   [Default reports for all seamounts] \n");
+		fprintf (stderr, "\t-Z Write hotspot ID number rather than hotspot TAG\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
@@ -278,16 +279,22 @@ int main (int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
-	nh = hotspot_init (hsfile, &hotspot);
+	nh = spotter_hotspot_init (hsfile, &orig_hotspot);
 	if (n_best_hs <= 0 || n_best_hs > nh) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -S:  Give value between 1 and %ld\n", GMT_program, nh);
 		exit (EXIT_FAILURE);
 	}
 	n_max_spots = MIN (n_best_hs, nh);
 
+	hotspot = (struct HOTSPOT_ORIGINATOR *) GMT_memory (VNULL, (size_t)nh, sizeof (struct HOTSPOT_ORIGINATOR), GMT_program);
+	for (i = 0; i < nh; i++) {
+		hotspot[i].h = &orig_hotspot[i];	/* Point to the original hotspot structures */
+		hotspot[i].np_dist = 1.0e100;
+	}
+	
 	ns = spotter_init (efile, &p, TRUE, finite, FALSE, &upper_age, gmtdefs.verbose);
 
-	hot = (struct HOTSPOT *) GMT_memory (VNULL, (size_t)nh, sizeof (struct HOTSPOT), GMT_program);
+	hot = (struct HOTSPOT_ORIGINATOR *) GMT_memory (VNULL, (size_t)nh, sizeof (struct HOTSPOT_ORIGINATOR), GMT_program);
 
 	x = (gmtdefs.xy_toggle[1]) ? 1 : 0;	y = 1 - x;		/* Set up which columns have x and y on output */
 
@@ -318,7 +325,7 @@ int main (int argc, char **argv)
 		n_read = 0;
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
-				GMT_fgets (GMT_io.segment_header, BUFSIZ, fp);
+				not_used = GMT_fgets (GMT_io.segment_header, BUFSIZ, fp);
 				if (first && !GMT_io.binary[GMT_OUT] && GMT_io.io_header[GMT_OUT]) fprintf (GMT_stdout, "%s", GMT_io.segment_header);
 				n_read++;
 			}
@@ -335,7 +342,7 @@ int main (int argc, char **argv)
 			if (GMT_io.status & GMT_IO_EOF) continue;
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld (skipped)\n", GMT_program, n_fields, n_expected_fields, n_read);
 				continue;
 			}
 			if (n_input == 3) {	/* set constant r,t values */
@@ -372,11 +379,11 @@ int main (int argc, char **argv)
 
 			np = (GMT_LONG) c[0];
 
-			memcpy ((void *)hot, (void *)hotspot, nh * sizeof (struct HOTSPOT));
+			memcpy ((void *)hot, (void *)hotspot, nh * sizeof (struct HOTSPOT_ORIGINATOR));
 
 			for (kk = 0, k = 1; kk < np; kk++, k += 3) {	/* For this seamounts track */
 				for (j = 0; j < nh; j++) {	/* For all hotspots */
-					dist = GMT_great_circle_dist (hot[j].lon, hot[j].lat, R2D * c[k], R2D * c[k+1]);
+					dist = GMT_great_circle_dist (hot[j].h->lon, hot[j].h->lat, R2D * c[k], R2D * c[k+1]);
 					if (!degree) dist *= KM_PR_DEG;
 					if (dist < hot[j].np_dist) {
 						hot[j].np_dist = dist;
@@ -386,16 +393,15 @@ int main (int argc, char **argv)
 			}
 			for (j = 0; j < nh; j++) {
 
-				plon = hot[j].lon;	plat = hot[j].lat;
-				GMT_geo_to_cart (&plat, &plon, H, TRUE);	/* 3-D Cartesian vector of this hotspot */
+				GMT_geo_to_cart (hot[j].h->lat, hot[j].h->lon, H, TRUE);	/* 3-D Cartesian vector of this hotspot */
 
 				/* Fine-tune the nearest point by considering intermediate points along greatcircle between knot points */
 
 				k = 3 * hot[j].nearest + 1;			/* Corresponding index for x into the (x,y,t) array c */
-				GMT_geo_to_cart (&c[k+1], &c[k], N, FALSE);	/* 3-D vector of nearest node to this hotspot */
+				GMT_geo_to_cart (c[k+1], c[k], N, FALSE);	/* 3-D vector of nearest node to this hotspot */
 				better = FALSE;
 				if (hot[j].nearest > 0) {	/* There is a point along the flowline before the nearest node */
-					GMT_geo_to_cart (&c[k-2], &c[k-3], A, FALSE);	/* 3-D vector of end of this segment */
+					GMT_geo_to_cart (c[k-2], c[k-3], A, FALSE);	/* 3-D vector of end of this segment */
 					if (GMT_great_circle_intersection (A, N, H, X, &hx_dist) == 0) {	/* X is between A and N */
 						hx_dist_km = d_acos (hx_dist) * KM_PR_RAD;
 						if (hx_dist_km < hot[j].np_dist) {	/* This intermediate point is even closer */
@@ -410,7 +416,7 @@ int main (int argc, char **argv)
 					}
 				}
 				if (hot[j].nearest < (np-1) ) {	/* There is a point along the flowline after the nearest node */
-					GMT_geo_to_cart (&c[k+4], &c[k+3], A, FALSE);	/* 3-D vector of end of this segment */
+					GMT_geo_to_cart (c[k+4], c[k+3], A, FALSE);	/* 3-D vector of end of this segment */
 					if (GMT_great_circle_intersection (A, N, H, X, &hx_dist) == 0) {	/* X is between A and N */
 						hx_dist_km = d_acos (hx_dist) * KM_PR_RAD;
 						if (hx_dist_km < hot[j].np_dist) {	/* This intermediate point is even closer */
@@ -437,7 +443,7 @@ int main (int argc, char **argv)
 				 * x-axis and y-axis is normal to that, flowlines whose closest approach point's longitude is
 				 * further east are said to have negative distance. */
 				 
-				dlon = fmod (hot[j].lon - hot[j].np_lon, 360.0);
+				dlon = fmod (hot[j].h->lon - hot[j].np_lon, 360.0);
 				if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
 				hot[j].np_sign = copysign (1.0, dlon);
 				 
@@ -449,7 +455,7 @@ int main (int argc, char **argv)
 				if (hot[j].stage == 0) hot[j].stage++;
 			}
 
-			if (nh > 1) qsort ((void *)hot, (size_t)nh, sizeof(struct HOTSPOT), comp_hs);
+			if (nh > 1) qsort ((void *)hot, (size_t)nh, sizeof(struct HOTSPOT_ORIGINATOR), comp_hs);
 
 			if (hot[0].np_dist < max_dist) {
 				if (tdz_output == 1) {	/* Want time, dist, z output */
@@ -458,7 +464,7 @@ int main (int argc, char **argv)
 					out[2] = z_smt;
 					GMT_output (GMT_stdout, n_out, out);
 				}
-				else if (tdz_output == 2) {	/* Want time, dist, z output */
+				else if (tdz_output == 2) {	/* Want omega, dist, z output */
 					out[0] = spotter_t2w (p, ns, hot[0].np_time);
 					out[1] = hot[0].np_dist * hot[0].np_sign;
 					out[2] = z_smt;
@@ -472,13 +478,16 @@ int main (int argc, char **argv)
 					out[4] = z_smt;
 					GMT_output (GMT_stdout, n_out, out);
 				}
-				else {	/* COnventional originator output */
+				else {	/* Conventional originator output */
 					if (t_smt == 180.0)
 						strcpy (age, "NaN");
 					else
 						sprintf (age, "%g", t_smt);
 					fprintf (GMT_stdout, "%g\t%g\t%g\t%g\t%s", in[x], in[y], z_smt, r_smt, age);
-					for (j = 0; j < n_max_spots; j++) fprintf (GMT_stdout, "\t%s\t%d\t%g\t%g", hot[j].abbrev, hot[j].stage, hot[j].np_time, hot[j].np_dist);
+					if (use_ID)
+						for (j = 0; j < n_max_spots; j++) fprintf (GMT_stdout, "\t%d\t%d\t%g\t%g", hot[j].h->id, hot[j].stage, hot[j].np_time, hot[j].np_dist);
+					else
+						for (j = 0; j < n_max_spots; j++) fprintf (GMT_stdout, "\t%s\t%d\t%g\t%g", hot[j].h->abbrev, hot[j].stage, hot[j].np_time, hot[j].np_dist);
 					fprintf (GMT_stdout, "\n");
 				}
 			}
@@ -493,6 +502,7 @@ int main (int argc, char **argv)
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Working on seamount # %5ld\n", GMT_program, n);
 
 	GMT_free ((void *)hotspot);
+	GMT_free ((void *)orig_hotspot);
 	GMT_free ((void *)hot);
 	GMT_free ((void *)p);
 
@@ -503,44 +513,11 @@ int main (int argc, char **argv)
 
 int comp_hs (const void *p1, const void *p2)
 {
-	struct HOTSPOT *a, *b;
+	struct HOTSPOT_ORIGINATOR *a, *b;
 
-	a = (struct HOTSPOT *) p1;
-	b = (struct HOTSPOT *) p2;
+	a = (struct HOTSPOT_ORIGINATOR *) p1;
+	b = (struct HOTSPOT_ORIGINATOR *) p2;
 	if (a->np_dist < b->np_dist) return (-1);
 	if (a->np_dist > b->np_dist) return (1);
 	return (0);
 }
-
-int hotspot_init (char *file, struct HOTSPOT **p)
-{
-	FILE *fp;
-	struct HOTSPOT *e;
-	char buffer[BUFSIZ];
-	int i = 0;
-	size_t n_alloc = GMT_CHUNK;
-
-	if ((fp = GMT_fopen (file, "r")) == NULL) {
-		fprintf (stderr, "%s: Cannot open file %s - aborts\n", GMT_program, file);
-		exit (EXIT_FAILURE);
-	}
-
-	e = (struct HOTSPOT *) GMT_memory (VNULL, n_alloc, sizeof (struct HOTSPOT), GMT_program);
-
-	while (GMT_fgets (buffer, 512, fp) != NULL) {
-		if (buffer[0] == '#' || buffer[0] == '\n') continue;
-		sscanf (buffer, "%lf %lf %s %s", &e[i].lon, &e[i].lat, e[i].abbrev, e[i].name);
-		e[i].id = i;
-		e[i].np_dist = 1.0e100;
-		i++;
-		if ((size_t)i == n_alloc) {
-			n_alloc <<= 1;
-			e = (struct HOTSPOT *) GMT_memory ((void *)e, n_alloc, sizeof (struct HOTSPOT), GMT_program);
-		}
-	}
-	GMT_fclose (fp);
-	e = (struct HOTSPOT *) GMT_memory ((void *)e, (size_t)i, sizeof (struct HOTSPOT), GMT_program);
-	*p = e;
-
-	return (i);
-}
