@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: surface.c,v 1.54 2008/05/01 03:00:39 guru Exp $
+ *	$Id: surface.c,v 1.67 2009/09/09 23:27:04 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -107,11 +107,11 @@ struct SURFACE_CTRL {
 		BOOLEAN active;
 		char *low, *high;
 		double min, max;
-		int lmode, hmode;
+		GMT_LONG lmode, hmode;
 	} L;
 	struct N {	/* -N<max_iterations> */
 		BOOLEAN active;
-		int value;
+		GMT_LONG value;
 	} N;
 	struct Q {	/* -Q */
 		BOOLEAN active;
@@ -156,28 +156,28 @@ struct SURFACE_INFO {	/* Control structure for surface setup and execution */
 					 * I means just interpolate from larger grid */
 	char format[BUFSIZ];
 	char *low_file, *high_file;	/* Pointers to grids with low and high limits, if selected */
-	int grid, old_grid;		/* Node spacings  */
-	int n_fact;			/* Number of factors in common (ny-1, nx-1) */
-	int factors[32];		/* Array of common factors */
-	int long_verbose;
-	int set_low;			/* 0 unconstrained,1 = by min data value, 2 = by user value */
-	int set_high;			/* 0 unconstrained,1 = by max data value, 2 = by user value */
-	size_t n_alloc;
-	GMT_LONG npoints;			/* Number of data points */
+	GMT_LONG grid, old_grid;	/* Node spacings  */
+	GMT_LONG n_fact;		/* Number of factors in common (ny-1, nx-1) */
+	GMT_LONG factors[32];		/* Array of common factors */
+	GMT_LONG long_verbose;
+	GMT_LONG set_low;		/* 0 unconstrained,1 = by min data value, 2 = by user value */
+	GMT_LONG set_high;		/* 0 unconstrained,1 = by max data value, 2 = by user value */
+	GMT_LONG n_alloc;
+	GMT_LONG npoints;		/* Number of data points */
 	GMT_LONG ij_sw_corner, ij_se_corner,ij_nw_corner, ij_ne_corner;
-	GMT_LONG n_empty;			/* No of unconstrained nodes at initialization  */
-	GMT_LONG nx;				/* Number of nodes in x-dir. */
-	GMT_LONG ny;				/* Number of nodes in y-dir. (Final grid) */
+	GMT_LONG n_empty;		/* No of unconstrained nodes at initialization  */
+	GMT_LONG nx;			/* Number of nodes in x-dir. */
+	GMT_LONG ny;			/* Number of nodes in y-dir. (Final grid) */
 	GMT_LONG nxny;			/* Total number of grid nodes without boundaries  */
 	GMT_LONG mx;
 	GMT_LONG my;
 	GMT_LONG mxmy;			/* Total number of grid nodes with boundaries  */
-	GMT_LONG block_nx;			/* Number of nodes in x-dir for a given grid factor */
-	GMT_LONG block_ny;			/* Number of nodes in y-dir for a given grid factor */
-	GMT_LONG max_iterations;		/* Max iter per call to iterate */
+	GMT_LONG block_nx;		/* Number of nodes in x-dir for a given grid factor */
+	GMT_LONG block_ny;		/* Number of nodes in y-dir for a given grid factor */
+	GMT_LONG max_iterations;	/* Max iter per call to iterate */
 	GMT_LONG total_iterations;
 	GMT_LONG grid_east;
-	GMT_LONG offset[25][12];		/* Indices of 12 nearby points in 25 cases of edge conditions  */
+	GMT_LONG offset[25][12];	/* Indices of 12 nearby points in 25 cases of edge conditions  */
 	BOOLEAN constrained;		/* TRUE if set_low or set_high is TRUE */
 	float *lower, *upper;		/* arrays for minmax values, if set */
 	float *u;			/* Pointer to grid array */
@@ -207,34 +207,34 @@ struct SURFACE_INFO {	/* Control structure for surface setup and execution */
 };
 
 struct SURFACE_SUGGESTION {	/* Used to find top ten list of faster grid dimensions  */
-	int nx;
-	int ny;
+	GMT_LONG nx;
+	GMT_LONG ny;
 	double factor;	/* Speed up by a factor of factor  */
 };
 
 int compare_points (const void *point_1v, const void *point_2v);
 GMT_LONG gcd_euclid (GMT_LONG a, GMT_LONG b);	/* Finds the greatest common divisor  */
-int get_prime_factors (GMT_LONG n, int *f);
-int iterate (struct SURFACE_INFO *C, int mode);
+GMT_LONG get_prime_factors (GMT_LONG n, GMT_LONG *f);
+GMT_LONG iterate (struct SURFACE_INFO *C, GMT_LONG mode);
 void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform);
 
 int main (int argc, char **argv)
 {
 	char	modifier;
 
-	int	i, error = FALSE, n_files = 0;
+	GMT_LONG	i, error = FALSE, n_files = 0, key;
 
 	FILE	*fp_in = NULL;	/* File pointer  */
 	
 	struct SURFACE_INFO C;
 	struct SURFACE_CTRL *Ctrl;
 	
-	void suggest_sizes_for_surface (int factors[], int nx, int ny);
+	void suggest_sizes_for_surface (GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny);
 	void set_grid_parameters (struct SURFACE_INFO *C);
 	void read_data (FILE *fp, struct SURFACE_INFO *C);
 	void throw_away_unusables (struct SURFACE_INFO *C);
 	void remove_planar_trend (struct SURFACE_INFO *C);
-	void rescale_z_values (struct SURFACE_INFO *C);
+	GMT_LONG rescale_z_values (struct SURFACE_INFO *C);
 	void smart_divide (struct SURFACE_INFO *C);
 	void set_offset (struct SURFACE_INFO *C);
 	void set_index (struct SURFACE_INFO *C);
@@ -446,6 +446,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, NULL, &Ctrl->I.active);
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
@@ -484,7 +486,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	load_parameters (&C, Ctrl);	/* Pass parameters from parsing control to surface INFO structure */
@@ -492,21 +494,27 @@ int main (int argc, char **argv)
 	GMT_RI_prepare (&C.h);	/* Ensure -R -I consistency and set nx, ny */
 	GMT_err_fail (GMT_grd_RI_verify (&C.h, 1), Ctrl->G.file);
 
+	if (C.h.nx < 4 || C.h.ny < 4) {
+		fprintf (stderr, "%s: GMT ERROR.  Grid must have at least 4 nodes in each direction (you have %d by %d) - abort.\n", GMT_program, C.h.nx, C.h.ny);
+		exit (EXIT_FAILURE);
+	}
+	
 	C.relax_old = 1.0 - C.relax_new;
 
 	C.nx = (GMT_LONG)C.h.nx;
 	C.ny = (GMT_LONG)C.h.ny;
-	C.nxny = C.nx * C.ny;
+	C.nxny = GMT_get_nm (C.nx, C.ny);
+	
 	C.mx = C.nx + 4;
 	C.my = C.ny + 4;
-	C.mxmy = C.mx * C.my;
+	C.mxmy = GMT_get_nm (C.mx, C.my);
 	C.r_xinc = 1.0 / C.h.x_inc;
 	C.r_yinc = 1.0 / C.h.y_inc;
 	GMT_Surface_Global.x_min = C.h.x_min;
 	GMT_Surface_Global.y_min = C.h.y_min;
 
 	/* New stuff here for v4.3:  Check out the grid dimensions:  */
-	C.grid = gcd_euclid (C.nx-1, C.ny-1);
+	C.grid = (int)gcd_euclid (C.nx-1, C.ny-1);
 
 	if (gmtdefs.verbose || Ctrl->Q.active) {
 		sprintf (C.format, "%s: W: %s E: %s S: %s N: %s nx: %%ld ny: %%ld\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
@@ -531,12 +539,27 @@ int main (int argc, char **argv)
 	read_data (fp_in, &C);
 	throw_away_unusables (&C);
 	remove_planar_trend (&C);
-	rescale_z_values (&C);
+	key = rescale_z_values (&C);
+	
+	if (key == 1) {	/* Data lies exactly on a plane; just return the plane grid */
+		GMT_free ((void *) C.data);
+		C.u = (float *) GMT_memory (VNULL, (size_t)C.mxmy, sizeof(float), GMT_program);
+		C.ij_sw_corner = 2 * C.my + 2;			/*  Corners of array of actual data  */
+		replace_planar_trend (&C);
+		write_output (&C, Ctrl->G.file);
+		GMT_free ((void *) C.u);
+		Free_Surface_Ctrl (Ctrl);	/* Deallocate control structure */
+
+		GMT_end (argc, argv);
+
+		exit (EXIT_SUCCESS);
+	}
+	
 	load_constraints (&C, TRUE);
 
 	/* Set up factors and reset grid to first value  */
 
-	C.grid = gcd_euclid (C.nx-1, C.ny-1);
+	C.grid = (int)gcd_euclid (C.nx-1, C.ny-1);
 	C.n_fact = get_prime_factors ((GMT_LONG)C.grid, C.factors);
 	set_grid_parameters (&C);
 	while (C.block_nx < 4 || C.block_ny < 4) {
@@ -642,7 +665,7 @@ void set_coefficients (struct SURFACE_INFO *C)
 void set_offset (struct SURFACE_INFO *C)
 {
 	GMT_LONG	add_w[5], add_e[5], add_s[5], add_n[5], add_w2[5], add_e2[5], add_s2[5], add_n2[5];
-	int	i, j, kase;
+	GMT_LONG	i, j, kase;
 
 	add_w[0] = -C->my; add_w[1] = add_w[2] = add_w[3] = add_w[4] = -C->grid_east;
 	add_w2[0] = -2 * C->my;  add_w2[1] = -C->my - C->grid_east;  add_w2[2] = add_w2[3] = add_w2[4] = -2 * C->grid_east;
@@ -1018,9 +1041,9 @@ void new_initialize_grid (struct SURFACE_INFO *C)
 void read_data (FILE *fp_in, struct SURFACE_INFO *C)
 {
 	GMT_LONG	i, j, k, kmax = 0, kmin = 0;
-	int n_fields, n_expected_fields;
+	GMT_LONG n_fields, n_expected_fields;
 	double	*in, zmin = DBL_MAX, zmax = -DBL_MAX;
-	char	buffer[BUFSIZ];
+	char	buffer[BUFSIZ], *not_used = NULL;
 
 	C->data = (struct SURFACE_DATA *) GMT_memory (VNULL, (size_t)C->n_alloc, sizeof(struct SURFACE_DATA), GMT_program);
 
@@ -1029,30 +1052,30 @@ void read_data (FILE *fp_in, struct SURFACE_INFO *C)
 	k = 0;
 	C->z_mean = 0;
 	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3;
-	if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp_in);
+	if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (buffer, BUFSIZ, fp_in);
 
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, k);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, k);
 			exit (EXIT_FAILURE);
 		}
 
-		if (GMT_is_dnan (in[2])) continue;
+		if (GMT_is_dnan (in[GMT_Z])) continue;
 
-		i = (GMT_LONG)floor(((in[0]-C->h.x_min)*C->r_grid_xinc) + 0.5);
+		i = (GMT_LONG)floor(((in[GMT_X]-C->h.x_min)*C->r_grid_xinc) + 0.5);
 		if (i < 0 || i >= C->block_nx) continue;
-		j = (GMT_LONG)floor(((in[1]-C->h.y_min)*C->r_grid_yinc) + 0.5);
+		j = (GMT_LONG)floor(((in[GMT_Y]-C->h.y_min)*C->r_grid_yinc) + 0.5);
 		if (j < 0 || j >= C->block_ny) continue;
 
 		C->data[k].index = i * C->block_ny + j;
-		C->data[k].x = (float)in[0];
-		C->data[k].y = (float)in[1];
-		C->data[k].z = (float)in[2];
-		if (zmin > in[2]) zmin = in[2], kmin = k;
-		if (zmax < in[2]) zmax = in[2], kmax = k;
+		C->data[k].x = (float)in[GMT_X];
+		C->data[k].y = (float)in[GMT_Y];
+		C->data[k].z = (float)in[GMT_Z];
+		if (zmin > in[GMT_Z]) zmin = in[GMT_Z], kmin = k;
+		if (zmax < in[GMT_Z]) zmax = in[GMT_Z], kmax = k;
 		k++;
-		C->z_mean += in[2];
+		C->z_mean += in[GMT_Z];
 		if (k == (GMT_LONG)C->n_alloc) {
 			C->n_alloc <<= 1;
 			C->data = (struct SURFACE_DATA *) GMT_memory ((void *)C->data, (size_t)C->n_alloc, sizeof(struct SURFACE_DATA), GMT_program);
@@ -1120,7 +1143,7 @@ void write_output (struct SURFACE_INFO *C, char *grdfile)
 	if (C->set_high) GMT_free ((void *)C->upper);
 }
 
-int iterate (struct SURFACE_INFO *C, int mode)
+GMT_LONG iterate (struct SURFACE_INFO *C, GMT_LONG mode)
 {
 
 	GMT_LONG	i, j, k, ij, kase, briggs_index, ij_v2;
@@ -1361,7 +1384,7 @@ int iterate (struct SURFACE_INFO *C, int mode)
 	if (gmtdefs.verbose && !C->long_verbose) fprintf (stderr, C->format,
 		C->grid, C->mode_type[mode], iteration_count, max_change, current_limit, C->total_iterations);
 
-	return (iteration_count);
+	return ((int)iteration_count);
 }
 
 void check_errors (struct SURFACE_INFO *C) {
@@ -1617,7 +1640,7 @@ void throw_away_unusables (struct SURFACE_INFO *C)
 	}
 }
 
-void rescale_z_values (struct SURFACE_INFO *C)
+GMT_LONG rescale_z_values (struct SURFACE_INFO *C)
 {
 	GMT_LONG	i;
 	double	ssz = 0.0;
@@ -1628,9 +1651,10 @@ void rescale_z_values (struct SURFACE_INFO *C)
 
 	C->z_scale = sqrt (ssz / C->npoints);
 
-	if (C->z_scale == 0.0) {
-		fprintf (stderr, "%s: WARNING: Input data lie exactly on a plane - no solution (aborting).\n", GMT_program);
+	if (C->z_scale < GMT_CONV_LIMIT) {
+		fprintf (stderr, "%s: WARNING: Input data lie exactly on a plane.\n", GMT_program);
 		C->r_z_scale = C->z_scale = 1.0;
+		return (1);	/* Flag to tell the main to just write out the plane */
 	}
 	else
 		C->r_z_scale = 1.0 / C->z_scale;
@@ -1638,6 +1662,7 @@ void rescale_z_values (struct SURFACE_INFO *C)
 	for (i = 0; i < C->npoints; i++) C->data[i].z *= (float)C->r_z_scale;
 
 	if (C->converge_limit == 0.0) C->converge_limit = 0.001 * C->z_scale; /* i.e., 1 ppt of L2 scale */
+	return (0);
 }
 
 void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
@@ -1662,7 +1687,7 @@ void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 		}
 		if (transform) {
 			for (j = ij = 0; j < C->ny; j++) {
-				yy = C->ny - j - 1;
+				yy = (double)(C->ny - j - 1);
 				for (i = 0; i < C->nx; i++, ij++) {
 					if (GMT_is_fnan (C->lower[ij])) continue;
 					C->lower[ij] -= (float)(C->plane_c0 + C->plane_c1 * i + C->plane_c2 * yy);
@@ -1686,7 +1711,7 @@ void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 		}
 		if (transform) {
 			for (j = ij = 0; j < C->ny; j++) {
-				yy = C->ny - j - 1;
+				yy = (double)(C->ny - j - 1);
 				for (i = 0; i < C->nx; i++, ij++) {
 					if (GMT_is_fnan (C->upper[ij])) continue;
 					C->upper[ij] -= (float)(C->plane_c0 + C->plane_c1 * i + C->plane_c2 * yy);
@@ -1698,7 +1723,7 @@ void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 	}
 }
 
-double guess_surface_time (int factors[], int nx, int ny)
+double guess_surface_time (GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny)
 {
 	/* Routine to guess a number proportional to the operations
 	 * required by surface working on a user-desired grid of
@@ -1726,9 +1751,9 @@ double guess_surface_time (int factors[], int nx, int ny)
 	 * W. H. F. Smith, 26 Feb 1992.  */
 
 	GMT_LONG	gcd;		/* Current value of the gcd  */
-	int	nxg, nyg;	/* Current value of the grid dimensions  */
-	int	nfactors = 0;	/* Number of prime factors of current gcd  */
-	int	factor;		/* Currently used factor  */
+	GMT_LONG	nxg, nyg;	/* Current value of the grid dimensions  */
+	GMT_LONG	nfactors = 0;	/* Number of prime factors of current gcd  */
+	GMT_LONG	factor;		/* Currently used factor  */
 	/* Doubles are used below, even though the values will be integers,
 		because the multiplications might reach sizes of O(n**3)  */
 	double	t_sum;		/* Sum of values of T at each grid cycle  */
@@ -1738,8 +1763,8 @@ double guess_surface_time (int factors[], int nx, int ny)
 	gcd = gcd_euclid((GMT_LONG)nx, (GMT_LONG)ny);
 	if (gcd > 1) {
 		nfactors = get_prime_factors(gcd, factors);
-		nxg = nx/gcd;
-		nyg = ny/gcd;
+		nxg = (GMT_LONG)(nx/gcd);
+		nyg = (GMT_LONG)(ny/gcd);
 		if (nxg < 3 || nyg < 3) {
 			factor = factors[nfactors - 1];
 			nfactors--;
@@ -1768,7 +1793,7 @@ double guess_surface_time (int factors[], int nx, int ny)
 	return(t_sum);
 }
 
-void suggest_sizes_for_surface (int factors[], int nx, int ny)
+void suggest_sizes_for_surface (GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny)
 {
 	/* Calls guess_surface_time for a variety of trial grid
 	 * sizes, where the trials are highly composite numbers
@@ -1782,14 +1807,14 @@ void suggest_sizes_for_surface (int factors[], int nx, int ny)
 	 *
 	 * W. H. F. Smith, 26 Feb 1992.  */
 
-	double	guess_surface_time(int factors[], int nx, int ny);
+	double	guess_surface_time(GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny);
 	double	users_time;	/* Time for user's nx, ny  */
 	double	current_time;	/* Time for current nxg, nyg  */
-	int	i;
-	int	nxg, nyg;	/* Guessed by this routine  */
-	int	nx2, ny2, nx3, ny3, nx5, ny5;	/* For powers  */
-	int	xstop, ystop;	/* Set to 2*nx, 2*ny  */
-	int	n_sug = 0;	/* N of suggestions found  */
+	GMT_LONG	i;
+	GMT_LONG	nxg, nyg;	/* Guessed by this routine  */
+	GMT_LONG	nx2, ny2, nx3, ny3, nx5, ny5;	/* For powers  */
+	GMT_LONG	xstop, ystop;	/* Set to 2*nx, 2*ny  */
+	GMT_LONG	n_sug = 0;	/* N of suggestions found  */
 	int	compare_sugs(const void *point_1, const void *point_2);	/* Sort suggestions decreasing  */
 	struct SURFACE_SUGGESTION *sug = NULL;
 
@@ -1828,7 +1853,7 @@ void suggest_sizes_for_surface (int factors[], int nx, int ny)
 	if (n_sug) {
 		qsort((void *)sug, (size_t)n_sug, sizeof(struct SURFACE_SUGGESTION), compare_sugs);
 		for (i = 0; i < n_sug && i < 10; i++) {
-			fprintf (stderr, "%s:  HINT:  Choosing nx = %d, ny = %d might cut run time by a factor of %.8g\n",
+			fprintf (stderr, "%s:  HINT:  Choosing nx = %ld, ny = %ld might cut run time by a factor of %.8g\n",
 				GMT_program, sug[i].nx, sug[i].ny, sug[i].factor);
 		}
 		GMT_free ((void *)sug);
@@ -1850,7 +1875,7 @@ int compare_sugs (const void *point_1, const void *point_2)
 		return(0);
 }
 
-int get_prime_factors (GMT_LONG n, int *f)
+GMT_LONG get_prime_factors (GMT_LONG n, GMT_LONG *f)
 {
 	/* Fills the integer array f with the prime factors of n.
 	 * Returns the number of locations filled in f, which is
@@ -1879,7 +1904,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 	GMT_LONG	current_factor;	/* The factor currently being tried  */
 	GMT_LONG	max_factor;	/* Don't try any factors bigger than this  */
-	int	n_factors = 0;	/* Returned; one if n is prime  */
+	GMT_LONG	n_factors = 0;	/* Returned; one if n is prime  */
 	GMT_LONG	two_four_toggle = 0;	/* Used to add 2 or 4 to get next trial factor  */
 	GMT_LONG	ten_twenty_toggle = 0;	/* Used to add 10 or 20 to skip_five  */
 	GMT_LONG	skip_five = 25;	/* Used to skip multiples of 5 in the list  */
@@ -1887,19 +1912,15 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 
 	/* Initialize m and max_factor  */
-#ifdef __LP64__
-	m = labs(n);
-#else
-	m = abs(n);
-#endif
+	m = GMT_abs(n);
 	if (m < 2) return(0);
-	max_factor = (int)floor(sqrt((double)m));
+	max_factor = (GMT_LONG)floor(sqrt((double)m));
 
 	/* First find the 2s  */
 	current_factor = 2;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1908,7 +1929,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 	current_factor = 3;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1917,7 +1938,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 	current_factor = 5;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1956,7 +1977,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 		while(!(m%current_factor)) {
 			m /= current_factor;
-			f[n_factors] = current_factor;
+			f[n_factors] = (GMT_LONG)current_factor;
 			n_factors++;
 		}
 	}
@@ -1965,7 +1986,7 @@ int get_prime_factors (GMT_LONG n, int *f)
 
 	if (m > 1) {
 		/* m is an additional prime factor of n  */
-		f[n_factors] = m;
+		f[n_factors] = (GMT_LONG)m;
 		n_factors++;
 	}
 	return (n_factors);
@@ -1985,7 +2006,7 @@ GMT_LONG gcd_euclid (GMT_LONG a, GMT_LONG b)
 	 *
 	 * Walter H. F. Smith, 25 Feb 1992, after D. E. Knuth, vol. II  */
 
-	int	u,v,r;
+	GMT_LONG u,v,r;
 
 	u = MAX(IABS(a), IABS(b));
 	v = MIN(IABS(a), IABS(b));
