@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: trend1d.c,v 1.39 2008/03/24 08:58:32 guru Exp $
+ *	$Id: trend1d.c,v 1.44 2009/09/09 23:27:04 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -125,8 +125,8 @@ struct TREND1D_CTRL {
 	struct N {	/* -N[f]<n_model>[r] */
 		BOOLEAN active;
 		BOOLEAN robust;
-		int mode;
-		int value;
+		GMT_LONG mode;
+		GMT_LONG value;
 	} N;
 	struct W {	/* -W */
 		BOOLEAN active;
@@ -146,14 +146,13 @@ struct	TREND1D_DATA {
 
 int main (int argc, char **argv)
 {
-	int i, j, k,n_outputs, n_model, significant, rank, n_req;
-	
+	GMT_LONG i, j, k,n_outputs, n_model, significant, rank, n_req;
 	GMT_LONG  n_data, np;
 	BOOLEAN	error = FALSE, weighted_output = FALSE;
 
 	double	*gtg, *v, *gtd, *lambda, *workb, *workz, *c_model, *o_model, *w_model, *work;	/* Arrays  */
 	double	xmin, xmax, c_chisq, o_chisq, w_chisq, scale = 1.0, prob;
-	double	get_chisq(struct TREND1D_DATA *data, GMT_LONG n_data, int n_model);
+	double	get_chisq(struct TREND1D_DATA *data, GMT_LONG n_data, GMT_LONG n_model);
 
 	char	format[BUFSIZ];
 
@@ -162,17 +161,17 @@ int main (int argc, char **argv)
 	struct	TREND1D_DATA *data;
 	struct TREND1D_CTRL *Ctrl;
 
-	void read_data(struct TREND1D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, int weighted_input, double **work, FILE *fp);
-	void write_output(struct TREND1D_DATA *data, GMT_LONG n_data, char *output_choice, int n_outputs), transform_x(struct TREND1D_DATA *data, GMT_LONG n_data, int model_type, double xmin, double xmax);
-	void untransform_x(struct TREND1D_DATA *data, GMT_LONG n_data, int model_type, double xmin, double xmax);
+	void read_data(struct TREND1D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, GMT_LONG weighted_input, double **work, FILE *fp);
+	void write_output(struct TREND1D_DATA *data, GMT_LONG n_data, char *output_choice, GMT_LONG n_outputs), transform_x(struct TREND1D_DATA *data, GMT_LONG n_data, GMT_LONG model_type, double xmin, double xmax);
+	void untransform_x(struct TREND1D_DATA *data, GMT_LONG n_data, GMT_LONG model_type, double xmin, double xmax);
 	void recompute_weights(struct TREND1D_DATA *data, GMT_LONG n_data, double *work, double *scale);
 	void allocate_array_space(GMT_LONG np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model);
 	void free_the_memory(double *gtg, double *v, double *gtd, double *lambda, double *workb, double *workz, double *c_model, double *o_model, double *w_model, struct TREND1D_DATA *data, double *work);
-	void calc_m_and_r(struct TREND1D_DATA *data, GMT_LONG n_data, double *model, int n_model, int m_type, double *grow);
-	void move_model_a_to_b(double *model_a, double *model_b, int n_model, double *chisq_a, double *chisq_b);
-	void load_gtg_and_gtd(struct TREND1D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, int n_model, GMT_LONG mp, int m_type);
-	void solve_system(double *gtg, double *gtd, double *model, int n_model, GMT_LONG mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir);
-	void GMT_cheb_to_pol (double c[], int n, double a, double b);
+	void calc_m_and_r(struct TREND1D_DATA *data, GMT_LONG n_data, double *model, GMT_LONG n_model, GMT_LONG m_type, double *grow);
+	void move_model_a_to_b(double *model_a, double *model_b, GMT_LONG n_model, double *chisq_a, double *chisq_b);
+	void load_gtg_and_gtd(struct TREND1D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, GMT_LONG n_model, GMT_LONG mp, GMT_LONG m_type);
+	void solve_system(double *gtg, double *gtd, double *model, GMT_LONG n_model, GMT_LONG mp, double *lambda, double *v, double *b, double *z, double c_no, GMT_LONG *ir);
+	void GMT_cheb_to_pol (double c[], GMT_LONG n, double a, double b);
 	void *New_Trend1d_Ctrl (), Free_Trend1d_Ctrl (struct TREND1D_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -302,7 +301,7 @@ int main (int argc, char **argv)
 	n_req = (Ctrl->W.active) ? 3 : 2;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < n_req) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %d columns\n", GMT_program, n_req);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %ld columns\n", GMT_program, n_req);
 		error++;
 	}
 	for (k = n_outputs = 0; k < TREND1D_N_OUTPUT_CHOICES && Ctrl->F.col[k]; k++) {
@@ -324,7 +323,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -479,12 +478,12 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void read_data (struct TREND1D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, int weighted_input, double **work, FILE *fp)
+void read_data (struct TREND1D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, GMT_LONG weighted_input, double **work, FILE *fp)
 {
-	int	n_alloc = GMT_CHUNK, n_expected_fields, n_fields;
+	GMT_LONG	n_alloc = GMT_CHUNK, n_expected_fields, n_fields;
 	GMT_LONG i;
 	double	*in;
-	char	buffer[BUFSIZ];
+	char	buffer[BUFSIZ], *not_used = NULL;
 
 	if (fp == NULL) {
 		fp = GMT_stdin;
@@ -494,20 +493,20 @@ void read_data (struct TREND1D_DATA **data, GMT_LONG *n_data, double *xmin, doub
 	}
 	(*data) = (struct TREND1D_DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct TREND1D_DATA), GMT_program);
 
-	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
+	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (buffer, BUFSIZ, fp);
 	i = 0;
 	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2 + weighted_input;
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, i);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, i);
 			exit (EXIT_FAILURE);
 		}
 
-		(*data)[i].x = in[0];
-		(*data)[i].y = in[1];
-		(*data)[i].w = (weighted_input) ? in[2] : 1.0;
+		(*data)[i].x = in[GMT_X];
+		(*data)[i].y = in[GMT_Y];
+		(*data)[i].w = (weighted_input) ? in[GMT_Z] : 1.0;
 
 		if (i) {
 			if (*xmin > (*data)[i].x) *xmin = (*data)[i].x;
@@ -550,10 +549,10 @@ void allocate_array_space (GMT_LONG np, double **gtg, double **v, double **gtd,
 	*w_model = (double *) GMT_memory (VNULL, (size_t)np, sizeof(double), GMT_program);
 }
 
-void write_output (struct TREND1D_DATA *data, GMT_LONG n_data, char *output_choice, int n_outputs)
+void write_output (struct TREND1D_DATA *data, GMT_LONG n_data, char *output_choice, GMT_LONG n_outputs)
 {
 	GMT_LONG	i;
-	int j;
+	GMT_LONG j;
 	double out[5];
 
 	for (i = 0; i < n_data; i++) {
@@ -595,7 +594,7 @@ void free_the_memory (double *gtg, double *v, double *gtd, double *lambda, doubl
 	GMT_free ((void *)gtg);
 }
 
-void transform_x (struct TREND1D_DATA *data, GMT_LONG n_data, int model_type, double xmin, double xmax)
+void transform_x (struct TREND1D_DATA *data, GMT_LONG n_data, GMT_LONG model_type, double xmin, double xmax)
 {
 	GMT_LONG	i;
 	double	offset, scale;
@@ -612,7 +611,7 @@ void transform_x (struct TREND1D_DATA *data, GMT_LONG n_data, int model_type, do
 	}
 }
 
-void untransform_x (struct TREND1D_DATA *data, GMT_LONG n_data, int model_type, double xmin, double xmax)
+void untransform_x (struct TREND1D_DATA *data, GMT_LONG n_data, GMT_LONG model_type, double xmin, double xmax)
 {
 	GMT_LONG	i;
 	double	offset, scale;
@@ -629,7 +628,7 @@ void untransform_x (struct TREND1D_DATA *data, GMT_LONG n_data, int model_type,
 	}
 }
 
-double get_chisq (struct TREND1D_DATA *data, GMT_LONG n_data, int n_model)
+double get_chisq (struct TREND1D_DATA *data, GMT_LONG n_data, GMT_LONG n_model)
 {
 	GMT_LONG	i, nu;
 	double	chi = 0.0;
@@ -683,7 +682,7 @@ void recompute_weights (struct TREND1D_DATA *data, GMT_LONG n_data, double *work
 	}
 }
 
-void load_g_row (double x, int n, double *gr, int m)
+void load_g_row (double x, GMT_LONG n, double *gr, GMT_LONG m)
       	  	/* Current data position, appropriately normalized.  */
    	  	/* Number of model parameters, and elements of gr[]  */
       	     	/* Elements of row of G matrix.  */
@@ -693,7 +692,7 @@ void load_g_row (double x, int n, double *gr, int m)
 		G matrix (Menke notation), where x is the ith datum's
 		abscissa.  */
 
-	int	j, k;
+	GMT_LONG	j, k;
 
 	if (n) {
 
@@ -734,13 +733,13 @@ void load_g_row (double x, int n, double *gr, int m)
 	}
 }
 
-void calc_m_and_r (struct TREND1D_DATA *data, GMT_LONG n_data, double *model, int n_model, int m_type, double *grow)
+void calc_m_and_r (struct TREND1D_DATA *data, GMT_LONG n_data, double *model, GMT_LONG n_model, GMT_LONG m_type, double *grow)
 {
 	/*	model[n_model] holds solved coefficients of m_type model.
 		grow[n_model] is a vector for a row of G matrix.  */
 
 	GMT_LONG	i;
-	int j;
+	GMT_LONG j;
 	for (i = 0; i < n_data; i++) {
 		load_g_row(data[i].x, n_model, grow, m_type);
 		data[i].m = 0.0;
@@ -751,21 +750,21 @@ void calc_m_and_r (struct TREND1D_DATA *data, GMT_LONG n_data, double *model, in
 	}
 }
 
-void move_model_a_to_b (double *model_a, double *model_b, int n_model, double *chisq_a, double *chisq_b)
+void move_model_a_to_b (double *model_a, double *model_b, GMT_LONG n_model, double *chisq_a, double *chisq_b)
 {
-	int	i;
+	GMT_LONG	i;
 	for(i = 0; i<  n_model; i++) {
 		model_b[i] = model_a[i];
 	}
 	*chisq_b = *chisq_a;
 }
 
-void load_gtg_and_gtd (struct TREND1D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, int n_model, GMT_LONG mp, int m_type)
+void load_gtg_and_gtd (struct TREND1D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, GMT_LONG n_model, GMT_LONG mp, GMT_LONG m_type)
 {
    	/* mp is row dimension of gtg  */
 
 	GMT_LONG	i;
-	int j, k;
+	GMT_LONG j, k;
 	double	wy;
 
 	/* First zero the contents for summing:  */
@@ -800,10 +799,10 @@ void load_gtg_and_gtd (struct TREND1D_DATA *data, GMT_LONG n_data, double *gtg,
 	}
 }
 
-void solve_system (double *gtg, double *gtd, double *model, int n_model, GMT_LONG mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir)
+void solve_system (double *gtg, double *gtd, double *model, GMT_LONG n_model, GMT_LONG mp, double *lambda, double *v, double *b, double *z, double c_no, GMT_LONG *ir)
 {
 
-	int	i, j, k, rank = 0, nrots;
+	GMT_LONG	i, j, k, rank = 0, nrots;
 	GMT_LONG n, m;
 	double	c_test, temp_inverse_ij;
 
@@ -833,13 +832,13 @@ void solve_system (double *gtg, double *gtd, double *model, int n_model, GMT_LON
 	}
 }
 
-void GMT_cheb_to_pol (double c[], int n, double a, double b)
+void GMT_cheb_to_pol (double c[], GMT_LONG n, double a, double b)
 {
 	/* Convert from Chebyshev coefficients used on a t =  [-1,+1] interval
 	 * to polynomial coefficients on the original x = [a b] interval.
 	 * Modified from Numerical Miracles, ...eh Recipes */
 	 
-	 int j, k;
+	 GMT_LONG j, k;
 	 double sv, cnst, fac;
 	 double *d, *dd;
 	 
