@@ -1,8 +1,8 @@
 /*--------------------------------------------------------------------
- *	$Id: trend2d.c,v 1.41 2009/01/09 04:02:34 guru Exp $
+ *	$Id: trend2d.c,v 1.45 2009/09/09 23:27:04 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
- *	See COPYING file for copying and redistribution conditions.
+ *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
@@ -85,7 +85,7 @@ struct TREND2D_CTRL {
 	struct N {	/* -N<n_model>[r] */
 		BOOLEAN active;
 		BOOLEAN robust;
-		int value;
+		GMT_LONG value;
 	} N;
 	struct W {	/* -W */
 		BOOLEAN active;
@@ -103,14 +103,14 @@ struct	TREND2D_DATA {
 
 int main(int argc, char **argv)
 {
-	int	i, j, k, n_outputs, n_model, significant, rank, n_req, np;
+	GMT_LONG	i, j, k, n_outputs, n_model, significant, rank, n_req, np;
 	GMT_LONG	n_data;
 
 	BOOLEAN	error = FALSE, weighted_output = FALSE;
 
 	double	*gtg, *v, *gtd, *lambda, *workb, *workz, *c_model, *o_model, *w_model, *work;	/* Arrays  */
 	double	xmin, xmax, ymin, ymax, c_chisq, o_chisq, w_chisq, scale = 1.0, prob;
-	double	get_chisq(struct TREND2D_DATA *data, GMT_LONG n_data, int n_model);
+	double	get_chisq(struct TREND2D_DATA *data, GMT_LONG n_data, GMT_LONG n_model);
 
 	char format[BUFSIZ];
 
@@ -119,17 +119,17 @@ int main(int argc, char **argv)
 	struct	TREND2D_DATA *data;
 	struct TREND2D_CTRL *Ctrl;
 
-	void read_data(struct TREND2D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, double *ymin, double *ymax, int weighted_input, double **work, FILE *fp);
-	void write_output(struct TREND2D_DATA *data, GMT_LONG n_data, char *output_choice, int n_outputs);
+	void read_data(struct TREND2D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, double *ymin, double *ymax, GMT_LONG weighted_input, double **work, FILE *fp);
+	void write_output(struct TREND2D_DATA *data, GMT_LONG n_data, char *output_choice, GMT_LONG n_outputs);
 	void transform_x(struct TREND2D_DATA *data, GMT_LONG n_data, double xmin, double xmax, double ymin, double ymax);
 	void untransform_x(struct TREND2D_DATA *data, GMT_LONG n_data, double xmin, double xmax, double ymin, double ymax);
 	void recompute_weights(struct TREND2D_DATA *data, GMT_LONG n_data, double *work, double *scale);
 	void allocate_array_space(GMT_LONG np, double **gtg, double **v, double **gtd, double **lambda, double **workb, double **workz, double **c_model, double **o_model, double **w_model);
 	void free_the_memory(double *gtg, double *v, double *gtd, double *lambda, double *workb, double *workz, double *c_model, double *o_model, double *w_model, struct TREND2D_DATA *data, double *work);
-	void calc_m_and_r(struct TREND2D_DATA *data, GMT_LONG n_data, double *model, int n_model, double *grow);
-	void move_model_a_to_b(double *model_a, double *model_b, int n_model, double *chisq_a, double *chisq_b);
-	void load_gtg_and_gtd(struct TREND2D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, int n_model, int mp);
-	void solve_system(double *gtg, double *gtd, double *model, int n_model, int mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir);
+	void calc_m_and_r(struct TREND2D_DATA *data, GMT_LONG n_data, double *model, GMT_LONG n_model, double *grow);
+	void move_model_a_to_b(double *model_a, double *model_b, GMT_LONG n_model, double *chisq_a, double *chisq_b);
+	void load_gtg_and_gtd(struct TREND2D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, GMT_LONG n_model, GMT_LONG mp);
+	void solve_system(double *gtg, double *gtd, double *model, GMT_LONG n_model, GMT_LONG mp, double *lambda, double *v, double *b, double *z, double c_no, GMT_LONG *ir);
 	void *New_Trend2d_Ctrl (), Free_Trend2d_Ctrl (struct TREND2D_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -253,7 +253,7 @@ int main(int argc, char **argv)
 	n_req = (Ctrl->W.active) ? 4 : 3;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] < n_req) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %d columns\n", GMT_program, n_req);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %ld columns\n", GMT_program, n_req);
 		error++;
 	}
 
@@ -272,14 +272,14 @@ int main(int argc, char **argv)
                 error++;
         }
 	if (n_outputs > TREND2D_N_OUTPUT_CHOICES) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option.  Too many output columns specified (%d)\n", GMT_program, n_outputs);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option.  Too many output columns specified (%ld)\n", GMT_program, n_outputs);
 		error++;
 	}
 	if (error) exit (EXIT_FAILURE);
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -433,12 +433,12 @@ int main(int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void read_data (struct TREND2D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, double *ymin, double *ymax, int weighted_input, double **work, FILE *fp)
+void read_data (struct TREND2D_DATA **data, GMT_LONG *n_data, double *xmin, double *xmax, double *ymin, double *ymax, GMT_LONG weighted_input, double **work, FILE *fp)
 {
 
 	GMT_LONG	i, n_alloc = GMT_CHUNK;
-	int n_expected_fields, n_fields;
-	char	buffer[BUFSIZ];
+	GMT_LONG n_expected_fields, n_fields;
+	char	buffer[BUFSIZ], *not_used = NULL;
 	double	*in;
 
 	if (fp == NULL) {
@@ -449,20 +449,20 @@ void read_data (struct TREND2D_DATA **data, GMT_LONG *n_data, double *xmin, doub
 	}
 	(*data) = (struct TREND2D_DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct TREND2D_DATA), GMT_program);
 
-	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
+	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (buffer, BUFSIZ, fp);
 	i = 0;
 	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3 + weighted_input;
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, i);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, i);
 			exit (EXIT_FAILURE);
 		}
 
-		(*data)[i].x = in[0];
-		(*data)[i].y = in[1];
-		(*data)[i].z = in[2];
+		(*data)[i].x = in[GMT_X];
+		(*data)[i].y = in[GMT_Y];
+		(*data)[i].z = in[GMT_Z];
 		(*data)[i].w = (weighted_input) ? in[3] : 1.0;
 
 		if (i) {
@@ -508,10 +508,10 @@ void allocate_array_space (GMT_LONG np, double **gtg, double **v, double **gtd,
 	*w_model = (double *) GMT_memory (VNULL, (size_t)np, sizeof(double), GMT_program);
 }
 
-void write_output (struct TREND2D_DATA *data, GMT_LONG n_data, char *output_choice, int n_outputs)
+void write_output (struct TREND2D_DATA *data, GMT_LONG n_data, char *output_choice, GMT_LONG n_outputs)
 {
 	GMT_LONG	i;
-	int j;
+	GMT_LONG j;
 	double out[6];
 
 	for (i = 0; i < n_data; i++) {
@@ -590,7 +590,7 @@ void untransform_x (struct TREND2D_DATA *data, GMT_LONG n_data, double xmin, dou
 	}
 }
 
-double get_chisq (struct TREND2D_DATA *data, GMT_LONG n_data, int n_model)
+double get_chisq (struct TREND2D_DATA *data, GMT_LONG n_data, GMT_LONG n_model)
 {
 	GMT_LONG	i, nu;
 	double	chi = 0.0;
@@ -644,7 +644,7 @@ void recompute_weights (struct TREND2D_DATA *data, GMT_LONG n_data, double *work
 	}
 }
 
-void load_g_row (double x, double y, int n, double *gr)
+void load_g_row (double x, double y, GMT_LONG n, double *gr)
       	     	/* Current data position, appropriately normalized.  */
    	  	/* Number of model parameters, and elements of gr[]  */
       	     	/* Elements of row of G matrix.  */
@@ -653,7 +653,7 @@ void load_g_row (double x, double y, int n, double *gr)
 		G matrix (Menke notation), where x,y is the ith datum's
 		location.  */
 
-	int	j;
+	GMT_LONG	j;
 
 	j = 0;
 	while (j < n) {
@@ -693,13 +693,12 @@ void load_g_row (double x, double y, int n, double *gr)
 	}
 }
 
-void calc_m_and_r (struct TREND2D_DATA *data, GMT_LONG n_data, double *model, int n_model, double *grow)
+void calc_m_and_r (struct TREND2D_DATA *data, GMT_LONG n_data, double *model, GMT_LONG n_model, double *grow)
 {
 	/*	model[n_model] holds solved coefficients of m_type model.
 		grow[n_model] is a vector for a row of G matrix.  */
 
-	GMT_LONG	i;
-	int j;
+	GMT_LONG	i, j;
 	for (i = 0; i < n_data; i++) {
 		load_g_row(data[i].x, data[i].y, n_model, grow);
 		data[i].m = 0.0;
@@ -710,23 +709,23 @@ void calc_m_and_r (struct TREND2D_DATA *data, GMT_LONG n_data, double *model, in
 	}
 }
 
-void move_model_a_to_b (double *model_a, double *model_b, int n_model, double *chisq_a, double *chisq_b)
+void move_model_a_to_b (double *model_a, double *model_b, GMT_LONG n_model, double *chisq_a, double *chisq_b)
 {
-	int	i;
+	GMT_LONG	i;
 	for(i = 0; i<  n_model; i++) {
 		model_b[i] = model_a[i];
 	}
 	*chisq_b = *chisq_a;
 }
 
-void load_gtg_and_gtd (struct TREND2D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, int n_model, int mp)
+void load_gtg_and_gtd (struct TREND2D_DATA *data, GMT_LONG n_data, double *gtg, double *gtd, double *grow, GMT_LONG n_model, GMT_LONG mp)
       	    	      
       			  
    			    	/* mp is row dimension of gtg  */
 {
 
 	GMT_LONG	i;
-	int j, k;
+	GMT_LONG j, k;
 	double	wz;
 
 	/* First zero the contents for summing:  */
@@ -761,10 +760,10 @@ void load_gtg_and_gtd (struct TREND2D_DATA *data, GMT_LONG n_data, double *gtg,
 	}
 }
 
-void solve_system (double *gtg, double *gtd, double *model, int n_model, int mp, double *lambda, double *v, double *b, double *z, double c_no, int *ir)
+void solve_system (double *gtg, double *gtd, double *model, GMT_LONG n_model, GMT_LONG mp, double *lambda, double *v, double *b, double *z, double c_no, GMT_LONG *ir)
 {
 
-	int	i, j, k, rank = 0, nrots;
+	GMT_LONG	i, j, k, rank = 0, nrots;
 	GMT_LONG n, m;
 	double	c_test, temp_inverse_ij;
 
