@@ -1,8 +1,8 @@
 /*-----------------------------------------------------------------
- *	$Id: x2sys_get.c,v 1.40 2008/05/12 22:35:47 guru Exp $
+ *	$Id: x2sys_get.c,v 1.47 2009/09/18 02:54:20 guru Exp $
  *
- *      Copyright (c) 1999-2008 by P. Wessel
- *      See COPYING file for copying and redistribution conditions.
+ *      Copyright (c) 1999-2009 by P. Wessel
+ *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 int main (int argc, char **argv)
 {
 	char *TAG = CNULL, *fflags = CNULL, *nflags = CNULL, *fx = NULL;
-	char *y_match = NULL, *n_match = NULL, line[BUFSIZ];
+	char *y_match = NULL, *n_match = NULL, line[BUFSIZ], *p = NULL;
 	
 	double x, y;
 
@@ -43,8 +43,9 @@ int main (int argc, char **argv)
 	BOOLEAN error = FALSE, no_suffix = FALSE, center = FALSE, x_setup = FALSE;
 	BOOLEAN y_ok, n_ok, first, *include = NULL, swap = FALSE;
 
-	int i, j, k, kk, combo = 0, ij, n_tracks_found, n_tracks, bit, start_j, start_i, stop_j, stop_i;
+	int combo = 0, ij, n_tracks_found, n_tracks, bit;
 	int id1, id2, item, n_flags = 0, n_pairs, missing = 0, *ids_in_bin = NULL;
+	GMT_LONG i, j, k, kk, start_j, start_i, stop_j, stop_i;
 	
 	unsigned int *matrix = NULL;
 
@@ -148,11 +149,11 @@ int main (int argc, char **argv)
 
 	/* Read existing track-information from <ID>_tracks.d file */
 
-	x2sys_err_fail (x2sys_bix_read_tracks (TAG, &B, 1, &n_tracks), "");
+	x2sys_err_fail (x2sys_bix_read_tracks (s, &B, 1, &n_tracks), "");
 
 	/* Read geographical track-info from <ID>_index.b file */
 
-	x2sys_err_fail (x2sys_bix_read_index (TAG, &B, swap), "");
+	x2sys_err_fail (x2sys_bix_read_index (s, &B, swap), "");
 
 	if (x_setup) {
 		n_flags = (int)ceil (n_tracks / 32.0);
@@ -165,6 +166,7 @@ int main (int argc, char **argv)
 			while (fgets (line, BUFSIZ, fp)) {
 				GMT_chop (line);	/* Get rid of [CR]LF */
 				if (line[0] == '#' || line[0] == '\0') continue;
+				if ((p = strchr (line, '.'))) line[(int)(p-line)] = '\0';	/* Remove extension */
 				k = find_leg (line, &B, n_tracks);	/* Return track id # for this leg */
 				if (k == -1) {
 					fprintf (stderr, "%s: Warning: Leg %s not in the data base\n", GMT_program, line);
@@ -172,7 +174,7 @@ int main (int argc, char **argv)
 				}
 				include[k] = TRUE;
 			}
-			GMT_fclose (fp);
+			fclose (fp);
 		}
 		else {	/* Use all */
 			for (k = 0; k < n_tracks; k++) include[k] = TRUE;
@@ -189,10 +191,11 @@ int main (int argc, char **argv)
 
 	x2sys_err_fail (x2sys_bix_get_ij (west, south, &start_i, &start_j, &B, &j), "");
 	x2sys_err_fail (x2sys_bix_get_ij (east, north, &stop_i, &stop_j, &B, &j), "");
+	if (B.periodic && stop_i < start_i) stop_i += B.nx_bin;	/* Deal with longitude periodicity */
 
 	for (j = start_j; j <= stop_j; j++) {
 		for (i = start_i; i <= stop_i; i++) {
-			ij = j * B.nx_bin + i;
+			ij = j * B.nx_bin + (i % B.nx_bin);	/* Since i may exceed nx_bin due to longitude periodicity */
 			if (B.base[ij].n_tracks == 0) continue;
 
 			for (k = kk = 0, track = B.base[ij].first_track->next_track, first = TRUE; first && k < B.base[ij].n_tracks; k++, track = track->next_track) {
