@@ -1,8 +1,8 @@
 /*-----------------------------------------------------------------
- *	$Id: x2sys_put.c,v 1.29 2008/05/12 22:35:47 guru Exp $
+ *	$Id: x2sys_put.c,v 1.40 2009/09/09 23:27:05 guru Exp $
  *
- *      Copyright (c) 1999-2008 by P. Wessel
- *      See COPYING file for copying and redistribution conditions.
+ *      Copyright (c) 1999-2009 by P. Wessel
+ *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -41,18 +41,20 @@ int main (int argc, char **argv)
 	struct X2SYS_BIX_TRACK_INFO *this_info, *new_info;
 	struct X2SYS_BIX_TRACK *this_track;
 
-	char track[GMT_TEXT_LEN], line[BUFSIZ];
+	char track[GMT_TEXT_LEN], line[BUFSIZ], *c_unused = NULL;
 	char track_file[BUFSIZ], index_file[BUFSIZ], old_track_file[BUFSIZ], old_index_file[BUFSIZ];
 	char track_path[BUFSIZ], index_path[BUFSIZ], old_track_path[BUFSIZ], old_index_path[BUFSIZ];
 
-	BOOLEAN error = FALSE, replace = FALSE, delete = FALSE, found_it, swap = FALSE;
+	BOOLEAN error = FALSE, replace = FALSE, delete = FALSE, found_it, skip, swap = FALSE;
 
 	FILE *fp = NULL, *fbin, *ftrack;
 
 	int index, id, bin, free_id, max_flag, flag;
 	int i, last_id, bit, total_flag;
+	size_t s_unused = 0;
 
 	int x2sys_bix_remove_track (int track_id, struct X2SYS_BIX *B);
+	struct X2SYS_BIX_TRACK_INFO * x2sys_bix_find_track (char *track, BOOLEAN *found_it, struct X2SYS_BIX *B);
 
 	argc = GMT_begin (argc, argv);
 	
@@ -62,19 +64,21 @@ int main (int argc, char **argv)
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-				case 'D':	/* Remove all traces of these tracks from the database */
-					delete = TRUE;
-					replace = TRUE;
-					break;
 				case 'V':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], NULL, NULL, NULL, NULL);
 					break;
+				case 'D':	/* Remove all traces of these tracks from the database */
+					delete = TRUE;
+					break;
+				case 'F':	/* Force update of existing tracks if new data are found */
+					replace = TRUE;
+					break;
 				case 'T':
 					TAG = &argv[i][2];
 					break;
 				case 'S':
-					swap = TRUE;	/* Swap option for index.b reading */
+					swap = TRUE;	/* Swap option for index.b reading [Obsolete but left for backwardness] */
 					break;
 				default:
 					error = TRUE;
@@ -90,7 +94,7 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || error || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "x2sys_put %s - Update track index database from track bin file\n\n", X2SYS_VERSION);
-		fprintf (stderr, "usage: x2sys_put [<info.tbf>]  -T<TAG> [-S] [-V]\n\n");
+		fprintf (stderr, "usage: x2sys_put [<info.tbf>] -T<TAG> [-D] [-F] [-V]\n\n");
 		fprintf (stderr, "	<info.tbf> is one track bin file from x2sys_binlist [Default reads stdin]\n");
 		fprintf (stderr, "	-T <TAG> is the system tag for this compilation\n");
 
@@ -98,15 +102,26 @@ int main (int argc, char **argv)
 
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf(stderr,"	-D will remove the listed tracks  [Default will add to database]\n");
+		fprintf(stderr,"	-F will force updates to earlier entries for a track with new information\n");
+		fprintf(stderr,"	   [Default refuses to process tracks already in the database]\n");
+#ifdef OBSOLETE
 		fprintf (stderr, "	-S Byte swap binary records during read [no swapping]\n");
+#endif
 		GMT_explain_option ('V');
 		exit (EXIT_FAILURE);
 	}
 
-	if (fp == NULL) fp = GMT_stdin;	/* Read stdin instead */
+	if (delete && replace) {
+		fprintf (stderr, "%s: ERROR: Only specify one of -D and -F\n", GMT_program);
+		exit (EXIT_FAILURE);
+	}
+	
+	if (replace) delete = TRUE;	/* Ironic, given previous if-test, but that is how the logic below works */
+	
+	if (fp == NULL) fp = GMT_stdin;	/* No file given; read stdin instead */
 
-	GMT_fgets (line, BUFSIZ, fp);	/* Got the first record from the track binindex file */
-	if (strncmp (&line[2], TAG, strlen(TAG))) {
+	c_unused = GMT_fgets (line, BUFSIZ, fp);	/* Got the first record from the track binindex file */
+	if (strncmp (&line[2], TAG, strlen(TAG))) {	/* Hard check to see if the TAG matches what we says it should be */
 		fprintf (stderr,"%s: The TAG specified (%s) does not match the one in the .tbf file (%s)\n", GMT_program, TAG, &line[2]);
 		exit (EXIT_FAILURE);
 	}
@@ -121,74 +136,91 @@ int main (int argc, char **argv)
 
 	/* Read existing track-information from <ID>_tracks.d file */
 
-	 x2sys_err_fail (x2sys_bix_read_tracks (TAG, &B, 0, &last_id), "");
+	 x2sys_err_fail (x2sys_bix_read_tracks (s, &B, 0, &last_id), "");
 	 last_id--;
 
 	/* Read geographical track-info from <ID>_index.b file */
 
-	x2sys_err_fail (x2sys_bix_read_index (TAG, &B, swap), "");
+	x2sys_err_fail (x2sys_bix_read_index (s, &B, swap), "");
 
 	/* Ok, now we can start reading new info */
 
 #ifdef DEBUG
 	GMT_memtrack_off (GMT_mem_keeper);
 #endif
-	GMT_fgets (line, BUFSIZ, fp);
+	c_unused = GMT_fgets (line, BUFSIZ, fp);
 	while (line[0] == '>') {	/* Next segment */
 		sscanf (line, "> %s", track);
-		for (this_info = B.head; this_info->next_info && strcmp (this_info->next_info->trackname, track) < 0; this_info = this_info->next_info);
-		free_id = 0;
-		found_it = (this_info->next_info != (struct X2SYS_BIX_TRACK_INFO *)NULL);
-		if (found_it && !strcmp (this_info->next_info->trackname, track)) {
-			if (gmtdefs.verbose) fprintf (stderr, "%s: Removing existing information for track: %s\n", GMT_program, track);
-			free_id = x2sys_bix_remove_track (this_info->next_info->track_id, &B);
-		}
-		if (delete) {	/* Give message and go back and read next track */
-			if (found_it) {
+		
+		/* Determine if this track is already in the data base */
+		
+		this_info = x2sys_bix_find_track (track, &found_it, &B);	/* Returns found_it = TRUE if found */
+		
+		/* In either case, this_info now points to the previous track so that this_info->next is the found track OR
+		 * it is the point after which a new track should be inserted */
+		
+		free_id = 0;	/* Default is to add a new track entry */
+		if (found_it) {	/* This track already exists in the database */
+			if (delete) {	/* Here we wish to delete it (and possibly replace the contents) */
+				if (gmtdefs.verbose) fprintf (stderr, "%s: Removing existing information for track: %s\n", GMT_program, track);
+				free_id = x2sys_bix_remove_track (this_info->next_info->track_id, &B);
 				fprintf (stderr, "%s: track %s removed\n", GMT_program, track);
 				this_info->next_info = this_info->next_info->next_info;
+				skip = !replace;	/* If we are not replacing the info then we skip the new info */
+			}
+			else {	/* Refuse to process tracks already in the database without the delete[and replace] options set */
+				if (gmtdefs.verbose) fprintf (stderr, "%s: Track already in database (skipped): %s\n", GMT_program, track);
+				skip = TRUE;
 			}
-			else
-				fprintf (stderr, "%s: track %s was not found in the database!\n", GMT_program, track);
 		}
-
-		/* Read the tbf information for this track */
-
-		if (delete) {	/* Just wind past this segment */
-			while (line[0] != '>') GMT_fgets (line, BUFSIZ, fp);
+		else if (delete) {	/* Here we did not found the track: Give message and go back and read next track information */
+			fprintf (stderr, "%s: track %s was not found in the database!\n", GMT_program, track);
+			skip = TRUE;
 		}
-		else {
+		else	/* Get here when we wish to add a new track not in the database */
+			skip = FALSE;
+
+		if (skip) {	/* Just wind past this segment */
+			c_unused = GMT_fgets (line, BUFSIZ, fp);
+			while (line[0] != '>' && (GMT_fgets (line, BUFSIZ, fp) != NULL));
+		}
+		else {	/* Read the tbf information for this track */
+
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Adding track: %s\n", GMT_program, track);
 
 			/* If a track is replaced, then use the same id_no, else increment to get a new one */
 
 			id = (free_id) ? free_id : ++last_id;
-			if (!free_id) {
+			if (!free_id) {	/* Must create a new entry */
 				new_info = x2sys_bix_make_entry (track, id, 0);
 				new_info->next_info = this_info->next_info;
 				this_info->next_info = new_info;
 				this_info = new_info;
 			}
-			else
+			else	/* Reuse the previously deleted entry */
 				this_info = this_info->next_info;
 
 			total_flag = 0;
 			while (GMT_fgets (line, BUFSIZ, fp) && line[0] != '>') {
-				sscanf (line, "%*s %*s %d %d", &index, &flag);
-				if (flag > max_flag) {
+				i = sscanf (line, "%*s %*s %d %d", &index, &flag);
+				if (i != 2) {	/* Could not decode the index and the flag entries */
+					fprintf (stderr, "%s: Error processing record for track %s [%s]\n", GMT_program, track, line);
+					exit (EXIT_FAILURE);
+				}
+				else if (flag > max_flag) {
 					fprintf (stderr, "%s: data flag (%d) exceed maximum (%d) for track %s!\n", GMT_program, flag, max_flag, track);
 					exit (EXIT_FAILURE);
 				}
-				if (B.base[index].n_tracks == 0) {
+				if (B.base[index].n_tracks == 0) {	/* First track to cross this bin */
 					B.base[index].first_track = x2sys_bix_make_track (0, 0);
 					B.base[index].last_track = B.base[index].first_track;
 				}
 				B.base[index].last_track->next_track = x2sys_bix_make_track (id, flag);
 				B.base[index].last_track = B.base[index].last_track->next_track;
 				B.base[index].n_tracks++;
-				total_flag |= flag;
+				total_flag |= flag;	/* Accumulate flags for the entire track */
 			}
-			this_info->flag = total_flag;
+			this_info->flag = total_flag;	/* Store the track flags here */
 		}
 	}
 	if (fp != GMT_stdin) GMT_fclose (fp);
@@ -198,8 +230,8 @@ int main (int argc, char **argv)
 
 	/* Done, now we must rewrite the <ID>_index.b and <ID>_tracks.d files */
 
-	sprintf (track_file, "%s_tracks.d", TAG);
-	sprintf (index_file, "%s_index.b",  TAG);
+	sprintf (track_file, "%s%c%s_tracks.d", TAG, DIR_DELIM, TAG);
+	sprintf (index_file, "%s%c%s_index.b",  TAG, DIR_DELIM, TAG);
 	x2sys_path (track_file, track_path);
 	x2sys_path (index_file, index_path);
 
@@ -208,10 +240,12 @@ int main (int argc, char **argv)
 	x2sys_path (old_track_file, old_track_path);
 	x2sys_path (old_index_file, old_index_path);
 
+	remove (old_track_path);	/* First delete old files */
 	if (rename (track_path, old_track_path)) {
-		fprintf (stderr, "%s: Rename failed for %s. Aborting!\n", GMT_program, track_path);
+		fprintf (stderr, "%s: Rename failed for %s\t%s. Aborting %d!\n", GMT_program, track_path, old_track_path, i);
 		exit (EXIT_FAILURE);
 	}
+	remove (old_index_path);	/* First delete old files */
 	if (rename (index_path, old_index_path)) {
 		fprintf (stderr, "%s: Rename failed for %s. Aborts!\n", GMT_program, index_path);
 		exit (EXIT_FAILURE);
@@ -235,11 +269,11 @@ int main (int argc, char **argv)
 	for (bin = 0; bin < B.nm_bin; bin++) {
 		if (B.base[bin].n_tracks == 0) continue;
 
-		fwrite ((void *)(&bin), (size_t)4, (size_t)1, fbin);
-		fwrite ((void *)(&B.base[bin].n_tracks), (size_t)4, (size_t)1, fbin);
+		s_unused = fwrite ((void *)(&bin), (size_t)4, (size_t)1, fbin);
+		s_unused = fwrite ((void *)(&B.base[bin].n_tracks), (size_t)4, (size_t)1, fbin);
 		for (this_track = B.base[bin].first_track->next_track; this_track; this_track = this_track->next_track) {
-			fwrite ((void *)(&this_track->track_id), (size_t)4, (size_t)1, fbin);
-			fwrite ((void *)(&this_track->track_flag), (size_t)4, (size_t)1, fbin);
+			s_unused = fwrite ((void *)(&this_track->track_id), (size_t)4, (size_t)1, fbin);
+			s_unused = fwrite ((void *)(&this_track->track_flag), (size_t)4, (size_t)1, fbin);
 		}
 	}
 	fclose (fbin);
@@ -256,7 +290,7 @@ int main (int argc, char **argv)
 
 int x2sys_bix_remove_track (int track_id, struct X2SYS_BIX *B)
 {
-	/* Remove all traces of the track with id track_id from structure tree */
+	/* Remove all traces of the track with id track_id from structure tree for all bins */
 
 	struct X2SYS_BIX_TRACK *track, *skip_track;
 	int bin;
@@ -266,15 +300,26 @@ int x2sys_bix_remove_track (int track_id, struct X2SYS_BIX *B)
 
 		for (track = B->base[bin].first_track; track->next_track && track->next_track->track_id != track_id; track = track->next_track);	/* Finds the track or end-of-list */
 
-		if (track->next_track) {	/* Ok, found it. Lets remove it from the list */
-			skip_track = track->next_track;
+		if (track->next_track) {	/* Ok, found it. Lets remove it from this bin's list */
+			skip_track = track->next_track;	/* These 3 lines sets the next points to skip the item to be removed */
 			track->next_track = skip_track->next_track;
 			skip_track->next_track = NULL;
-			B->base[bin].n_tracks--;
-			if (!track->next_track) B->base[bin].last_track = track;
-			GMT_free ((void *) skip_track);
-			if (B->base[bin].n_tracks == 0) GMT_free ((void *)B->base[bin].first_track);
+			B->base[bin].n_tracks--;	/* One less entry for this bin */
+			if (!track->next_track) B->base[bin].last_track = track;	/* Update the last track in case we just removed it */
+			GMT_free ((void *) skip_track);	/* Remove memory associated with the track to be removed */
+			if (B->base[bin].n_tracks == 0) GMT_free ((void *)B->base[bin].first_track);	/* OK, that was the only track in this bin, apparently */
 		}
 	}
 	return (track_id);
 }
+
+struct X2SYS_BIX_TRACK_INFO * x2sys_bix_find_track (char *track, BOOLEAN *found_it, struct X2SYS_BIX *B)
+{	/* Looks for given track in data base and if found returns pointer to the track before it and sets found_it to TRUE.
+	 * I.e., the track is actually this_info->next_info.  If not found set found_it to FALSE and return pointer where
+	 * this track should be inserted */
+	
+	struct X2SYS_BIX_TRACK_INFO *this_info;
+	for (this_info = B->head; this_info->next_info && strcmp (this_info->next_info->trackname, track) < 0; this_info = this_info->next_info);
+	*found_it = (this_info->next_info != (struct X2SYS_BIX_TRACK_INFO *)NULL && !strcmp (this_info->next_info->trackname, track));
+	return (this_info);
+}
